# Generated from antlr/KotlinParser.g4 by ANTLR 4.9.2
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\u00ad")
        buf.write("\u0d80\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\4\27\t\27\4\30\t\30\4\31")
        buf.write("\t\31\4\32\t\32\4\33\t\33\4\34\t\34\4\35\t\35\4\36\t\36")
        buf.write("\4\37\t\37\4 \t \4!\t!\4\"\t\"\4#\t#\4$\t$\4%\t%\4&\t")
        buf.write("&\4\'\t\'\4(\t(\4)\t)\4*\t*\4+\t+\4,\t,\4-\t-\4.\t.\4")
        buf.write("/\t/\4\60\t\60\4\61\t\61\4\62\t\62\4\63\t\63\4\64\t\64")
        buf.write("\4\65\t\65\4\66\t\66\4\67\t\67\48\t8\49\t9\4:\t:\4;\t")
        buf.write(";\4<\t<\4=\t=\4>\t>\4?\t?\4@\t@\4A\tA\4B\tB\4C\tC\4D\t")
        buf.write("D\4E\tE\4F\tF\4G\tG\4H\tH\4I\tI\4J\tJ\4K\tK\4L\tL\4M\t")
        buf.write("M\4N\tN\4O\tO\4P\tP\4Q\tQ\4R\tR\4S\tS\4T\tT\4U\tU\4V\t")
        buf.write("V\4W\tW\4X\tX\4Y\tY\4Z\tZ\4[\t[\4\\\t\\\4]\t]\4^\t^\4")
        buf.write("_\t_\4`\t`\4a\ta\4b\tb\4c\tc\4d\td\4e\te\4f\tf\4g\tg\4")
        buf.write("h\th\4i\ti\4j\tj\4k\tk\4l\tl\4m\tm\4n\tn\4o\to\4p\tp\4")
        buf.write("q\tq\4r\tr\4s\ts\4t\tt\4u\tu\4v\tv\4w\tw\4x\tx\4y\ty\4")
        buf.write("z\tz\4{\t{\4|\t|\4}\t}\4~\t~\4\177\t\177\4\u0080\t\u0080")
        buf.write("\4\u0081\t\u0081\4\u0082\t\u0082\4\u0083\t\u0083\4\u0084")
        buf.write("\t\u0084\4\u0085\t\u0085\4\u0086\t\u0086\4\u0087\t\u0087")
        buf.write("\4\u0088\t\u0088\4\u0089\t\u0089\4\u008a\t\u008a\4\u008b")
        buf.write("\t\u008b\4\u008c\t\u008c\4\u008d\t\u008d\4\u008e\t\u008e")
        buf.write("\4\u008f\t\u008f\4\u0090\t\u0090\4\u0091\t\u0091\4\u0092")
        buf.write("\t\u0092\4\u0093\t\u0093\4\u0094\t\u0094\4\u0095\t\u0095")
        buf.write("\4\u0096\t\u0096\4\u0097\t\u0097\4\u0098\t\u0098\4\u0099")
        buf.write("\t\u0099\4\u009a\t\u009a\4\u009b\t\u009b\4\u009c\t\u009c")
        buf.write("\4\u009d\t\u009d\4\u009e\t\u009e\4\u009f\t\u009f\4\u00a0")
        buf.write("\t\u00a0\4\u00a1\t\u00a1\4\u00a2\t\u00a2\4\u00a3\t\u00a3")
        buf.write("\4\u00a4\t\u00a4\4\u00a5\t\u00a5\4\u00a6\t\u00a6\4\u00a7")
        buf.write("\t\u00a7\4\u00a8\t\u00a8\4\u00a9\t\u00a9\4\u00aa\t\u00aa")
        buf.write("\4\u00ab\t\u00ab\4\u00ac\t\u00ac\4\u00ad\t\u00ad\4\u00ae")
        buf.write("\t\u00ae\3\2\5\2\u015e\n\2\3\2\7\2\u0161\n\2\f\2\16\2")
        buf.write("\u0164\13\2\3\2\7\2\u0167\n\2\f\2\16\2\u016a\13\2\3\2")
        buf.write("\3\2\3\2\7\2\u016f\n\2\f\2\16\2\u0172\13\2\3\2\3\2\3\3")
        buf.write("\5\3\u0177\n\3\3\3\7\3\u017a\n\3\f\3\16\3\u017d\13\3\3")
        buf.write("\3\7\3\u0180\n\3\f\3\16\3\u0183\13\3\3\3\3\3\3\3\3\3\3")
        buf.write("\3\7\3\u018a\n\3\f\3\16\3\u018d\13\3\3\3\3\3\3\4\3\4\6")
        buf.write("\4\u0193\n\4\r\4\16\4\u0194\3\5\3\5\3\5\7\5\u019a\n\5")
        buf.write("\f\5\16\5\u019d\13\5\3\5\3\5\7\5\u01a1\n\5\f\5\16\5\u01a4")
        buf.write("\13\5\3\5\3\5\6\5\u01a8\n\5\r\5\16\5\u01a9\3\5\3\5\3\5")
        buf.write("\5\5\u01af\n\5\3\5\7\5\u01b2\n\5\f\5\16\5\u01b5\13\5\3")
        buf.write("\6\3\6\3\6\5\6\u01ba\n\6\5\6\u01bc\n\6\3\7\7\7\u01bf\n")
        buf.write("\7\f\7\16\7\u01c2\13\7\3\b\3\b\3\b\3\b\3\b\5\b\u01c9\n")
        buf.write("\b\3\b\5\b\u01cc\n\b\3\t\3\t\3\t\3\n\3\n\5\n\u01d3\n\n")
        buf.write("\3\13\5\13\u01d6\n\13\3\13\3\13\7\13\u01da\n\13\f\13\16")
        buf.write("\13\u01dd\13\13\3\13\3\13\7\13\u01e1\n\13\f\13\16\13\u01e4")
        buf.write("\13\13\3\13\5\13\u01e7\n\13\3\13\7\13\u01ea\n\13\f\13")
        buf.write("\16\13\u01ed\13\13\3\13\3\13\7\13\u01f1\n\13\f\13\16\13")
        buf.write("\u01f4\13\13\3\13\3\13\3\f\3\f\3\f\3\f\3\f\5\f\u01fd\n")
        buf.write("\f\3\r\5\r\u0200\n\r\3\r\3\r\3\r\7\r\u0205\n\r\f\r\16")
        buf.write("\r\u0208\13\r\5\r\u020a\n\r\3\r\5\r\u020d\n\r\3\r\7\r")
        buf.write("\u0210\n\r\f\r\16\r\u0213\13\r\3\r\3\r\7\r\u0217\n\r\f")
        buf.write("\r\16\r\u021a\13\r\3\r\5\r\u021d\n\r\3\r\7\r\u0220\n\r")
        buf.write("\f\r\16\r\u0223\13\r\3\r\5\r\u0226\n\r\3\r\7\r\u0229\n")
        buf.write("\r\f\r\16\r\u022c\13\r\3\r\3\r\7\r\u0230\n\r\f\r\16\r")
        buf.write("\u0233\13\r\3\r\5\r\u0236\n\r\3\r\7\r\u0239\n\r\f\r\16")
        buf.write("\r\u023c\13\r\3\r\5\r\u023f\n\r\3\r\7\r\u0242\n\r\f\r")
        buf.write("\16\r\u0245\13\r\3\r\3\r\7\r\u0249\n\r\f\r\16\r\u024c")
        buf.write("\13\r\3\r\5\r\u024f\n\r\3\16\5\16\u0252\n\16\3\16\3\16")
        buf.write("\7\16\u0256\n\16\f\16\16\16\u0259\13\16\5\16\u025b\n\16")
        buf.write("\3\16\3\16\3\17\3\17\7\17\u0261\n\17\f\17\16\17\u0264")
        buf.write("\13\17\3\17\3\17\7\17\u0268\n\17\f\17\16\17\u026b\13\17")
        buf.write("\3\17\3\17\3\20\3\20\7\20\u0271\n\20\f\20\16\20\u0274")
        buf.write("\13\20\3\20\3\20\7\20\u0278\n\20\f\20\16\20\u027b\13\20")
        buf.write("\3\20\3\20\7\20\u027f\n\20\f\20\16\20\u0282\13\20\3\20")
        buf.write("\7\20\u0285\n\20\f\20\16\20\u0288\13\20\3\20\7\20\u028b")
        buf.write("\n\20\f\20\16\20\u028e\13\20\3\20\5\20\u0291\n\20\5\20")
        buf.write("\u0293\n\20\3\20\7\20\u0296\n\20\f\20\16\20\u0299\13\20")
        buf.write("\3\20\3\20\3\21\5\21\u029e\n\21\3\21\5\21\u02a1\n\21\3")
        buf.write("\21\7\21\u02a4\n\21\f\21\16\21\u02a7\13\21\3\21\3\21\3")
        buf.write("\21\7\21\u02ac\n\21\f\21\16\21\u02af\13\21\3\21\3\21\7")
        buf.write("\21\u02b3\n\21\f\21\16\21\u02b6\13\21\3\21\3\21\7\21\u02ba")
        buf.write("\n\21\f\21\16\21\u02bd\13\21\3\21\5\21\u02c0\n\21\3\22")
        buf.write("\3\22\7\22\u02c4\n\22\f\22\16\22\u02c7\13\22\3\22\3\22")
        buf.write("\7\22\u02cb\n\22\f\22\16\22\u02ce\13\22\3\22\7\22\u02d1")
        buf.write("\n\22\f\22\16\22\u02d4\13\22\3\23\3\23\3\23\3\23\5\23")
        buf.write("\u02da\n\23\3\24\3\24\3\24\3\25\7\25\u02e0\n\25\f\25\16")
        buf.write("\25\u02e3\13\25\3\25\7\25\u02e6\n\25\f\25\16\25\u02e9")
        buf.write("\13\25\3\25\3\25\3\26\3\26\5\26\u02ef\n\26\3\26\7\26\u02f2")
        buf.write("\n\26\f\26\16\26\u02f5\13\26\3\26\3\26\7\26\u02f9\n\26")
        buf.write("\f\26\16\26\u02fc\13\26\3\26\3\26\3\27\3\27\7\27\u0302")
        buf.write("\n\27\f\27\16\27\u0305\13\27\3\27\3\27\7\27\u0309\n\27")
        buf.write("\f\27\16\27\u030c\13\27\3\27\3\27\7\27\u0310\n\27\f\27")
        buf.write("\16\27\u0313\13\27\3\27\7\27\u0316\n\27\f\27\16\27\u0319")
        buf.write("\13\27\3\27\7\27\u031c\n\27\f\27\16\27\u031f\13\27\3\27")
        buf.write("\5\27\u0322\n\27\3\27\7\27\u0325\n\27\f\27\16\27\u0328")
        buf.write("\13\27\3\27\3\27\3\30\5\30\u032d\n\30\3\30\7\30\u0330")
        buf.write("\n\30\f\30\16\30\u0333\13\30\3\30\3\30\7\30\u0337\n\30")
        buf.write("\f\30\16\30\u033a\13\30\3\30\3\30\7\30\u033e\n\30\f\30")
        buf.write("\16\30\u0341\13\30\3\30\5\30\u0344\n\30\3\31\3\31\7\31")
        buf.write("\u0348\n\31\f\31\16\31\u034b\13\31\3\31\3\31\7\31\u034f")
        buf.write("\n\31\f\31\16\31\u0352\13\31\3\31\3\31\7\31\u0356\n\31")
        buf.write("\f\31\16\31\u0359\13\31\3\31\7\31\u035c\n\31\f\31\16\31")
        buf.write("\u035f\13\31\3\32\7\32\u0362\n\32\f\32\16\32\u0365\13")
        buf.write("\32\3\32\3\32\7\32\u0369\n\32\f\32\16\32\u036c\13\32\3")
        buf.write("\32\3\32\7\32\u0370\n\32\f\32\16\32\u0373\13\32\3\32\3")
        buf.write("\32\3\33\3\33\5\33\u0379\n\33\7\33\u037b\n\33\f\33\16")
        buf.write("\33\u037e\13\33\3\34\3\34\3\34\3\34\5\34\u0384\n\34\3")
        buf.write("\35\3\35\7\35\u0388\n\35\f\35\16\35\u038b\13\35\3\35\3")
        buf.write("\35\3\36\5\36\u0390\n\36\3\36\3\36\7\36\u0394\n\36\f\36")
        buf.write("\16\36\u0397\13\36\3\36\3\36\7\36\u039b\n\36\f\36\16\36")
        buf.write("\u039e\13\36\3\36\5\36\u03a1\n\36\3\36\7\36\u03a4\n\36")
        buf.write("\f\36\16\36\u03a7\13\36\3\36\3\36\7\36\u03ab\n\36\f\36")
        buf.write("\16\36\u03ae\13\36\3\36\5\36\u03b1\n\36\3\36\7\36\u03b4")
        buf.write("\n\36\f\36\16\36\u03b7\13\36\3\36\5\36\u03ba\n\36\3\37")
        buf.write("\3\37\7\37\u03be\n\37\f\37\16\37\u03c1\13\37\3\37\3\37")
        buf.write("\7\37\u03c5\n\37\f\37\16\37\u03c8\13\37\3\37\3\37\7\37")
        buf.write("\u03cc\n\37\f\37\16\37\u03cf\13\37\3\37\7\37\u03d2\n\37")
        buf.write("\f\37\16\37\u03d5\13\37\3\37\7\37\u03d8\n\37\f\37\16\37")
        buf.write("\u03db\13\37\3\37\5\37\u03de\n\37\5\37\u03e0\n\37\3\37")
        buf.write("\7\37\u03e3\n\37\f\37\16\37\u03e6\13\37\3\37\3\37\3 \5")
        buf.write(" \u03eb\n \3 \3 \7 \u03ef\n \f \16 \u03f2\13 \3 \3 \7")
        buf.write(" \u03f6\n \f \16 \u03f9\13 \3 \5 \u03fc\n \3!\5!\u03ff")
        buf.write("\n!\3!\3!\7!\u0403\n!\f!\16!\u0406\13!\3!\5!\u0409\n!")
        buf.write("\3!\7!\u040c\n!\f!\16!\u040f\13!\3!\3!\7!\u0413\n!\f!")
        buf.write("\16!\u0416\13!\3!\3!\5!\u041a\n!\3!\7!\u041d\n!\f!\16")
        buf.write("!\u0420\13!\3!\3!\7!\u0424\n!\f!\16!\u0427\13!\3!\3!\7")
        buf.write("!\u042b\n!\f!\16!\u042e\13!\3!\3!\7!\u0432\n!\f!\16!\u0435")
        buf.write("\13!\3!\5!\u0438\n!\3!\7!\u043b\n!\f!\16!\u043e\13!\3")
        buf.write("!\5!\u0441\n!\3!\7!\u0444\n!\f!\16!\u0447\13!\3!\5!\u044a")
        buf.write("\n!\3\"\3\"\3\"\7\"\u044f\n\"\f\"\16\"\u0452\13\"\3\"")
        buf.write("\5\"\u0455\n\"\3#\7#\u0458\n#\f#\16#\u045b\13#\3#\7#\u045e")
        buf.write("\n#\f#\16#\u0461\13#\3#\3#\7#\u0465\n#\f#\16#\u0468\13")
        buf.write("#\3#\3#\7#\u046c\n#\f#\16#\u046f\13#\3#\5#\u0472\n#\3")
        buf.write("$\3$\7$\u0476\n$\f$\16$\u0479\13$\3$\3$\7$\u047d\n$\f")
        buf.write("$\16$\u0480\13$\3$\3$\7$\u0484\n$\f$\16$\u0487\13$\3$")
        buf.write("\7$\u048a\n$\f$\16$\u048d\13$\3$\7$\u0490\n$\f$\16$\u0493")
        buf.write("\13$\3$\5$\u0496\n$\3$\7$\u0499\n$\f$\16$\u049c\13$\3")
        buf.write("$\3$\3%\5%\u04a1\n%\3%\3%\7%\u04a5\n%\f%\16%\u04a8\13")
        buf.write("%\3%\5%\u04ab\n%\3%\7%\u04ae\n%\f%\16%\u04b1\13%\3%\3")
        buf.write("%\7%\u04b5\n%\f%\16%\u04b8\13%\3%\3%\5%\u04bc\n%\3%\7")
        buf.write("%\u04bf\n%\f%\16%\u04c2\13%\3%\3%\5%\u04c6\n%\3%\7%\u04c9")
        buf.write("\n%\f%\16%\u04cc\13%\3%\5%\u04cf\n%\3%\7%\u04d2\n%\f%")
        buf.write("\16%\u04d5\13%\3%\3%\7%\u04d9\n%\f%\16%\u04dc\13%\3%\3")
        buf.write("%\5%\u04e0\n%\5%\u04e2\n%\3%\6%\u04e5\n%\r%\16%\u04e6")
        buf.write("\3%\5%\u04ea\n%\3%\7%\u04ed\n%\f%\16%\u04f0\13%\3%\5%")
        buf.write("\u04f3\n%\3%\7%\u04f6\n%\f%\16%\u04f9\13%\3%\5%\u04fc")
        buf.write("\n%\3%\5%\u04ff\n%\3%\5%\u0502\n%\3%\7%\u0505\n%\f%\16")
        buf.write("%\u0508\13%\3%\5%\u050b\n%\3%\5%\u050e\n%\5%\u0510\n%")
        buf.write("\3&\3&\7&\u0514\n&\f&\16&\u0517\13&\3&\3&\3\'\5\'\u051c")
        buf.write("\n\'\3\'\3\'\7\'\u0520\n\'\f\'\16\'\u0523\13\'\3\'\3\'")
        buf.write("\7\'\u0527\n\'\f\'\16\'\u052a\13\'\3\'\3\'\7\'\u052e\n")
        buf.write("\'\f\'\16\'\u0531\13\'\3\'\3\'\7\'\u0535\n\'\f\'\16\'")
        buf.write("\u0538\13\'\3\'\5\'\u053b\n\'\3\'\7\'\u053e\n\'\f\'\16")
        buf.write("\'\u0541\13\'\3\'\5\'\u0544\n\'\3(\5(\u0547\n(\3(\3(\7")
        buf.write("(\u054b\n(\f(\16(\u054e\13(\3(\3(\7(\u0552\n(\f(\16(\u0555")
        buf.write("\13(\3(\3(\7(\u0559\n(\f(\16(\u055c\13(\3(\5(\u055f\n")
        buf.write("(\3(\7(\u0562\n(\f(\16(\u0565\13(\3(\3(\7(\u0569\n(\f")
        buf.write("(\16(\u056c\13(\3(\3(\7(\u0570\n(\f(\16(\u0573\13(\3(")
        buf.write("\5(\u0576\n(\3(\7(\u0579\n(\f(\16(\u057c\13(\3(\3(\5(")
        buf.write("\u0580\n(\3)\3)\7)\u0584\n)\f)\16)\u0587\13)\3)\3)\7)")
        buf.write("\u058b\n)\f)\16)\u058e\13)\3)\3)\7)\u0592\n)\f)\16)\u0595")
        buf.write("\13)\3)\7)\u0598\n)\f)\16)\u059b\13)\3)\7)\u059e\n)\f")
        buf.write(")\16)\u05a1\13)\3)\5)\u05a4\n)\5)\u05a6\n)\3)\7)\u05a9")
        buf.write("\n)\f)\16)\u05ac\13)\3)\3)\3*\5*\u05b1\n*\3*\3*\7*\u05b5")
        buf.write("\n*\f*\16*\u05b8\13*\3*\3*\7*\u05bc\n*\f*\16*\u05bf\13")
        buf.write("*\3*\5*\u05c2\n*\3+\3+\7+\u05c6\n+\f+\16+\u05c9\13+\3")
        buf.write("+\3+\7+\u05cd\n+\f+\16+\u05d0\13+\3+\5+\u05d3\n+\3,\3")
        buf.write(",\7,\u05d7\n,\f,\16,\u05da\13,\3,\3,\7,\u05de\n,\f,\16")
        buf.write(",\u05e1\13,\3,\3,\3-\5-\u05e6\n-\3-\3-\7-\u05ea\n-\f-")
        buf.write("\16-\u05ed\13-\3-\3-\7-\u05f1\n-\f-\16-\u05f4\13-\3-\3")
        buf.write("-\7-\u05f8\n-\f-\16-\u05fb\13-\3-\5-\u05fe\n-\3-\7-\u0601")
        buf.write("\n-\f-\16-\u0604\13-\3-\5-\u0607\n-\3.\5.\u060a\n.\3.")
        buf.write("\3.\7.\u060e\n.\f.\16.\u0611\13.\3.\3.\7.\u0615\n.\f.")
        buf.write("\16.\u0618\13.\3.\3.\7.\u061c\n.\f.\16.\u061f\13.\3.\5")
        buf.write(".\u0622\n.\3.\7.\u0625\n.\f.\16.\u0628\13.\3.\5.\u062b")
        buf.write("\n.\3/\3/\7/\u062f\n/\f/\16/\u0632\13/\3/\3/\3\60\3\60")
        buf.write("\7\60\u0638\n\60\f\60\16\60\u063b\13\60\3\60\5\60\u063e")
        buf.write("\n\60\3\60\7\60\u0641\n\60\f\60\16\60\u0644\13\60\3\60")
        buf.write("\3\60\7\60\u0648\n\60\f\60\16\60\u064b\13\60\3\60\5\60")
        buf.write("\u064e\n\60\3\60\7\60\u0651\n\60\f\60\16\60\u0654\13\60")
        buf.write("\3\60\3\60\3\61\3\61\7\61\u065a\n\61\f\61\16\61\u065d")
        buf.write("\13\61\3\61\3\61\7\61\u0661\n\61\f\61\16\61\u0664\13\61")
        buf.write("\3\61\7\61\u0667\n\61\f\61\16\61\u066a\13\61\3\61\7\61")
        buf.write("\u066d\n\61\f\61\16\61\u0670\13\61\3\61\5\61\u0673\n\61")
        buf.write("\3\62\3\62\7\62\u0677\n\62\f\62\16\62\u067a\13\62\5\62")
        buf.write("\u067c\n\62\3\62\3\62\7\62\u0680\n\62\f\62\16\62\u0683")
        buf.write("\13\62\3\62\5\62\u0686\n\62\3\62\7\62\u0689\n\62\f\62")
        buf.write("\16\62\u068c\13\62\3\62\5\62\u068f\n\62\3\63\5\63\u0692")
        buf.write("\n\63\3\63\3\63\3\63\3\63\5\63\u0698\n\63\3\64\3\64\5")
        buf.write("\64\u069c\n\64\3\65\3\65\5\65\u06a0\n\65\3\65\7\65\u06a3")
        buf.write("\n\65\f\65\16\65\u06a6\13\65\3\65\6\65\u06a9\n\65\r\65")
        buf.write("\16\65\u06aa\3\66\3\66\3\67\3\67\7\67\u06b1\n\67\f\67")
        buf.write("\16\67\u06b4\13\67\3\67\3\67\7\67\u06b8\n\67\f\67\16\67")
        buf.write("\u06bb\13\67\3\67\7\67\u06be\n\67\f\67\16\67\u06c1\13")
        buf.write("\67\38\38\78\u06c5\n8\f8\168\u06c8\138\38\58\u06cb\n8")
        buf.write("\39\59\u06ce\n9\39\39\59\u06d2\n9\3:\6:\u06d5\n:\r:\16")
        buf.write(":\u06d6\3;\3;\7;\u06db\n;\f;\16;\u06de\13;\3;\5;\u06e1")
        buf.write("\n;\3<\3<\7<\u06e5\n<\f<\16<\u06e8\13<\3<\3<\7<\u06ec")
        buf.write("\n<\f<\16<\u06ef\13<\5<\u06f1\n<\3<\3<\7<\u06f5\n<\f<")
        buf.write("\16<\u06f8\13<\3<\3<\7<\u06fc\n<\f<\16<\u06ff\13<\3<\3")
        buf.write("<\3=\3=\7=\u0705\n=\f=\16=\u0708\13=\3=\3=\5=\u070c\n")
        buf.write("=\3=\7=\u070f\n=\f=\16=\u0712\13=\3=\3=\7=\u0716\n=\f")
        buf.write("=\16=\u0719\13=\3=\3=\5=\u071d\n=\7=\u071f\n=\f=\16=\u0722")
        buf.write("\13=\3=\7=\u0725\n=\f=\16=\u0728\13=\3=\5=\u072b\n=\3")
        buf.write("=\7=\u072e\n=\f=\16=\u0731\13=\3=\3=\3>\3>\7>\u0737\n")
        buf.write(">\f>\16>\u073a\13>\3>\3>\7>\u073e\n>\f>\16>\u0741\13>")
        buf.write("\3>\3>\3?\5?\u0746\n?\3?\3?\3?\5?\u074b\n?\3@\3@\7@\u074f")
        buf.write("\n@\f@\16@\u0752\13@\3@\3@\5@\u0756\n@\3@\7@\u0759\n@")
        buf.write("\f@\16@\u075c\13@\3@\3@\3A\3A\3A\3A\7A\u0764\nA\fA\16")
        buf.write("A\u0767\13A\5A\u0769\nA\3A\5A\u076c\nA\3B\3B\7B\u0770")
        buf.write("\nB\fB\16B\u0773\13B\3B\3B\3B\3B\5B\u0779\nB\3C\3C\3C")
        buf.write("\7C\u077e\nC\fC\16C\u0781\13C\3D\3D\5D\u0785\nD\3E\3E")
        buf.write("\7E\u0789\nE\fE\16E\u078c\13E\3E\3E\7E\u0790\nE\fE\16")
        buf.write("E\u0793\13E\3E\3E\3F\3F\3F\5F\u079a\nF\3G\3G\7G\u079e")
        buf.write("\nG\fG\16G\u07a1\13G\3G\3G\7G\u07a5\nG\fG\16G\u07a8\13")
        buf.write("G\3G\3G\5G\u07ac\nG\3G\3G\3G\3G\7G\u07b2\nG\fG\16G\u07b5")
        buf.write("\13G\3G\5G\u07b8\nG\3H\3H\7H\u07bc\nH\fH\16H\u07bf\13")
        buf.write("H\3H\3H\3H\3H\7H\u07c5\nH\fH\16H\u07c8\13H\3H\3H\5H\u07cc")
        buf.write("\nH\3I\3I\7I\u07d0\nI\fI\16I\u07d3\13I\3I\5I\u07d6\nI")
        buf.write("\3I\7I\u07d9\nI\fI\16I\u07dc\13I\3I\3I\7I\u07e0\nI\fI")
        buf.write("\16I\u07e3\13I\3I\3I\3I\3I\3J\3J\3J\3J\3J\3J\5J\u07ef")
        buf.write("\nJ\3J\7J\u07f2\nJ\fJ\16J\u07f5\13J\3J\3J\3K\3K\7K\u07fb")
        buf.write("\nK\fK\16K\u07fe\13K\3K\5K\u0801\nK\3L\6L\u0804\nL\rL")
        buf.write("\16L\u0805\3L\5L\u0809\nL\3M\3M\3N\3N\7N\u080f\nN\fN\16")
        buf.write("N\u0812\13N\3N\3N\7N\u0816\nN\fN\16N\u0819\13N\3N\7N\u081c")
        buf.write("\nN\fN\16N\u081f\13N\3O\3O\7O\u0823\nO\fO\16O\u0826\13")
        buf.write("O\3O\3O\7O\u082a\nO\fO\16O\u082d\13O\3O\7O\u0830\nO\f")
        buf.write("O\16O\u0833\13O\3P\3P\3P\7P\u0838\nP\fP\16P\u083b\13P")
        buf.write("\3P\3P\7P\u083f\nP\fP\16P\u0842\13P\3Q\3Q\3Q\7Q\u0847")
        buf.write("\nQ\fQ\16Q\u084a\13Q\3Q\3Q\7Q\u084e\nQ\fQ\16Q\u0851\13")
        buf.write("Q\3R\3R\7R\u0855\nR\fR\16R\u0858\13R\3S\3S\3S\7S\u085d")
        buf.write("\nS\fS\16S\u0860\13S\3S\3S\3S\3S\7S\u0866\nS\fS\16S\u0869")
        buf.write("\13S\3S\3S\7S\u086d\nS\fS\16S\u0870\13S\3T\3T\7T\u0874")
        buf.write("\nT\fT\16T\u0877\13T\3T\3T\7T\u087b\nT\fT\16T\u087e\13")
        buf.write("T\3T\3T\7T\u0882\nT\fT\16T\u0885\13T\3U\3U\3U\3V\3V\3")
        buf.write("V\7V\u088d\nV\fV\16V\u0890\13V\3V\3V\7V\u0894\nV\fV\16")
        buf.write("V\u0897\13V\3W\3W\3W\7W\u089c\nW\fW\16W\u089f\13W\3W\7")
        buf.write("W\u08a2\nW\fW\16W\u08a5\13W\3X\3X\3X\7X\u08aa\nX\fX\16")
        buf.write("X\u08ad\13X\3X\3X\7X\u08b1\nX\fX\16X\u08b4\13X\3Y\3Y\3")
        buf.write("Y\7Y\u08b9\nY\fY\16Y\u08bc\13Y\3Y\3Y\7Y\u08c0\nY\fY\16")
        buf.write("Y\u08c3\13Y\3Z\3Z\7Z\u08c7\nZ\fZ\16Z\u08ca\13Z\3Z\3Z\7")
        buf.write("Z\u08ce\nZ\fZ\16Z\u08d1\13Z\3Z\3Z\7Z\u08d5\nZ\fZ\16Z\u08d8")
        buf.write("\13Z\3[\7[\u08db\n[\f[\16[\u08de\13[\3[\3[\3\\\3\\\3\\")
        buf.write("\3\\\7\\\u08e6\n\\\f\\\16\\\u08e9\13\\\5\\\u08eb\n\\\3")
        buf.write("]\3]\7]\u08ef\n]\f]\16]\u08f2\13]\3^\3^\3^\3^\3^\5^\u08f9")
        buf.write("\n^\3_\3_\3_\3_\3_\5_\u0900\n_\3`\3`\7`\u0904\n`\f`\16")
        buf.write("`\u0907\13`\3`\3`\7`\u090b\n`\f`\16`\u090e\13`\3`\3`\3")
        buf.write("a\3a\5a\u0914\na\3b\3b\7b\u0918\nb\fb\16b\u091b\13b\3")
        buf.write("b\3b\7b\u091f\nb\fb\16b\u0922\13b\3b\3b\3c\3c\3c\5c\u0929")
        buf.write("\nc\3d\3d\7d\u092d\nd\fd\16d\u0930\13d\3d\3d\7d\u0934")
        buf.write("\nd\fd\16d\u0937\13d\3d\3d\7d\u093b\nd\fd\16d\u093e\13")
        buf.write("d\3d\7d\u0941\nd\fd\16d\u0944\13d\3d\7d\u0947\nd\fd\16")
        buf.write("d\u094a\13d\3d\5d\u094d\nd\3d\7d\u0950\nd\fd\16d\u0953")
        buf.write("\13d\3d\3d\3e\3e\7e\u0959\ne\fe\16e\u095c\13e\3e\3e\3")
        buf.write("e\5e\u0961\ne\3f\5f\u0964\nf\3f\5f\u0967\nf\3f\3f\5f\u096b")
        buf.write("\nf\3g\7g\u096e\ng\fg\16g\u0971\13g\3g\5g\u0974\ng\3g")
        buf.write("\7g\u0977\ng\fg\16g\u097a\13g\3g\3g\3h\3h\7h\u0980\nh")
        buf.write("\fh\16h\u0983\13h\3h\3h\7h\u0987\nh\fh\16h\u098a\13h\3")
        buf.write("h\3h\7h\u098e\nh\fh\16h\u0991\13h\3h\7h\u0994\nh\fh\16")
        buf.write("h\u0997\13h\3h\7h\u099a\nh\fh\16h\u099d\13h\3h\5h\u09a0")
        buf.write("\nh\3h\7h\u09a3\nh\fh\16h\u09a6\13h\3h\3h\3i\3i\7i\u09ac")
        buf.write("\ni\fi\16i\u09af\13i\3i\3i\7i\u09b3\ni\fi\16i\u09b6\13")
        buf.write("i\3i\3i\7i\u09ba\ni\fi\16i\u09bd\13i\3i\7i\u09c0\ni\f")
        buf.write("i\16i\u09c3\13i\3i\7i\u09c6\ni\fi\16i\u09c9\13i\3i\5i")
        buf.write("\u09cc\ni\3i\7i\u09cf\ni\fi\16i\u09d2\13i\5i\u09d4\ni")
        buf.write("\3i\3i\3j\5j\u09d9\nj\3j\7j\u09dc\nj\fj\16j\u09df\13j")
        buf.write("\3j\3j\7j\u09e3\nj\fj\16j\u09e6\13j\3j\3j\7j\u09ea\nj")
        buf.write("\fj\16j\u09ed\13j\5j\u09ef\nj\3j\5j\u09f2\nj\3j\7j\u09f5")
        buf.write("\nj\fj\16j\u09f8\13j\3j\3j\3k\3k\3k\3k\3k\3k\3k\3k\3k")
        buf.write("\3k\3k\3k\3k\3k\5k\u0a0a\nk\3l\3l\7l\u0a0e\nl\fl\16l\u0a11")
        buf.write("\13l\3l\3l\7l\u0a15\nl\fl\16l\u0a18\13l\3l\3l\3m\3m\7")
        buf.write("m\u0a1e\nm\fm\16m\u0a21\13m\3m\3m\7m\u0a25\nm\fm\16m\u0a28")
        buf.write("\13m\3m\3m\7m\u0a2c\nm\fm\16m\u0a2f\13m\3m\7m\u0a32\n")
        buf.write("m\fm\16m\u0a35\13m\3m\7m\u0a38\nm\fm\16m\u0a3b\13m\3m")
        buf.write("\5m\u0a3e\nm\3m\7m\u0a41\nm\fm\16m\u0a44\13m\5m\u0a46")
        buf.write("\nm\3m\3m\3n\3n\3o\3o\5o\u0a4e\no\3p\3p\3p\7p\u0a53\n")
        buf.write("p\fp\16p\u0a56\13p\3p\3p\3q\3q\3q\3q\7q\u0a5e\nq\fq\16")
        buf.write("q\u0a61\13q\3q\3q\3r\3r\3s\3s\7s\u0a69\ns\fs\16s\u0a6c")
        buf.write("\13s\3s\3s\7s\u0a70\ns\fs\16s\u0a73\13s\3s\3s\3t\3t\3")
        buf.write("u\3u\7u\u0a7b\nu\fu\16u\u0a7e\13u\3u\3u\7u\u0a82\nu\f")
        buf.write("u\16u\u0a85\13u\3u\3u\3v\3v\7v\u0a8b\nv\fv\16v\u0a8e\13")
        buf.write("v\3v\5v\u0a91\nv\3v\7v\u0a94\nv\fv\16v\u0a97\13v\3v\3")
        buf.write("v\7v\u0a9b\nv\fv\16v\u0a9e\13v\5v\u0aa0\nv\3v\3v\7v\u0aa4")
        buf.write("\nv\fv\16v\u0aa7\13v\3v\3v\3w\3w\7w\u0aad\nw\fw\16w\u0ab0")
        buf.write("\13w\3w\3w\7w\u0ab4\nw\fw\16w\u0ab7\13w\3w\7w\u0aba\n")
        buf.write("w\fw\16w\u0abd\13w\3w\7w\u0ac0\nw\fw\16w\u0ac3\13w\3w")
        buf.write("\5w\u0ac6\nw\3x\3x\3x\7x\u0acb\nx\fx\16x\u0ace\13x\3x")
        buf.write("\3x\7x\u0ad2\nx\fx\16x\u0ad5\13x\3x\5x\u0ad8\nx\5x\u0ada")
        buf.write("\nx\3y\3y\7y\u0ade\ny\fy\16y\u0ae1\13y\3y\3y\7y\u0ae5")
        buf.write("\ny\fy\16y\u0ae8\13y\3y\3y\5y\u0aec\ny\3y\7y\u0aef\ny")
        buf.write("\fy\16y\u0af2\13y\3y\3y\7y\u0af6\ny\fy\16y\u0af9\13y\3")
        buf.write("y\3y\7y\u0afd\ny\fy\16y\u0b00\13y\3y\5y\u0b03\ny\3y\7")
        buf.write("y\u0b06\ny\fy\16y\u0b09\13y\3y\5y\u0b0c\ny\3y\7y\u0b0f")
        buf.write("\ny\fy\16y\u0b12\13y\3y\5y\u0b15\ny\3z\3z\5z\u0b19\nz")
        buf.write("\3{\3{\7{\u0b1d\n{\f{\16{\u0b20\13{\3{\3{\7{\u0b24\n{")
        buf.write("\f{\16{\u0b27\13{\3{\3{\7{\u0b2b\n{\f{\16{\u0b2e\13{\5")
        buf.write("{\u0b30\n{\3{\7{\u0b33\n{\f{\16{\u0b36\13{\3{\5{\u0b39")
        buf.write("\n{\3|\3|\3}\3}\3}\7}\u0b40\n}\f}\16}\u0b43\13}\3}\3}")
        buf.write("\7}\u0b47\n}\f}\16}\u0b4a\13}\3}\3}\5}\u0b4e\n}\3}\3}")
        buf.write("\5}\u0b52\n}\3}\5}\u0b55\n}\3~\3~\7~\u0b59\n~\f~\16~\u0b5c")
        buf.write("\13~\3~\3~\7~\u0b60\n~\f~\16~\u0b63\13~\3~\3~\7~\u0b67")
        buf.write("\n~\f~\16~\u0b6a\13~\3~\3~\7~\u0b6e\n~\f~\16~\u0b71\13")
        buf.write("~\3~\3~\5~\u0b75\n~\3~\7~\u0b78\n~\f~\16~\u0b7b\13~\3")
        buf.write("~\5~\u0b7e\n~\3~\7~\u0b81\n~\f~\16~\u0b84\13~\3~\3~\7")
        buf.write("~\u0b88\n~\f~\16~\u0b8b\13~\3~\3~\5~\u0b8f\n~\3~\5~\u0b92")
        buf.write("\n~\3\177\3\177\7\177\u0b96\n\177\f\177\16\177\u0b99\13")
        buf.write("\177\3\177\7\177\u0b9c\n\177\f\177\16\177\u0b9f\13\177")
        buf.write("\3\177\3\177\7\177\u0ba3\n\177\f\177\16\177\u0ba6\13\177")
        buf.write("\3\177\3\177\7\177\u0baa\n\177\f\177\16\177\u0bad\13\177")
        buf.write("\3\177\3\177\7\177\u0bb1\n\177\f\177\16\177\u0bb4\13\177")
        buf.write("\5\177\u0bb6\n\177\3\177\3\177\3\177\3\u0080\3\u0080\7")
        buf.write("\u0080\u0bbd\n\u0080\f\u0080\16\u0080\u0bc0\13\u0080\3")
        buf.write("\u0080\5\u0080\u0bc3\n\u0080\3\u0080\7\u0080\u0bc6\n\u0080")
        buf.write("\f\u0080\16\u0080\u0bc9\13\u0080\3\u0080\3\u0080\7\u0080")
        buf.write("\u0bcd\n\u0080\f\u0080\16\u0080\u0bd0\13\u0080\3\u0080")
        buf.write("\3\u0080\7\u0080\u0bd4\n\u0080\f\u0080\16\u0080\u0bd7")
        buf.write("\13\u0080\7\u0080\u0bd9\n\u0080\f\u0080\16\u0080\u0bdc")
        buf.write("\13\u0080\3\u0080\7\u0080\u0bdf\n\u0080\f\u0080\16\u0080")
        buf.write("\u0be2\13\u0080\3\u0080\3\u0080\3\u0081\3\u0081\7\u0081")
        buf.write("\u0be8\n\u0081\f\u0081\16\u0081\u0beb\13\u0081\3\u0081")
        buf.write("\3\u0081\7\u0081\u0bef\n\u0081\f\u0081\16\u0081\u0bf2")
        buf.write("\13\u0081\3\u0081\7\u0081\u0bf5\n\u0081\f\u0081\16\u0081")
        buf.write("\u0bf8\13\u0081\3\u0081\7\u0081\u0bfb\n\u0081\f\u0081")
        buf.write("\16\u0081\u0bfe\13\u0081\3\u0081\5\u0081\u0c01\n\u0081")
        buf.write("\3\u0081\7\u0081\u0c04\n\u0081\f\u0081\16\u0081\u0c07")
        buf.write("\13\u0081\3\u0081\3\u0081\7\u0081\u0c0b\n\u0081\f\u0081")
        buf.write("\16\u0081\u0c0e\13\u0081\3\u0081\3\u0081\5\u0081\u0c12")
        buf.write("\n\u0081\3\u0081\3\u0081\7\u0081\u0c16\n\u0081\f\u0081")
        buf.write("\16\u0081\u0c19\13\u0081\3\u0081\3\u0081\7\u0081\u0c1d")
        buf.write("\n\u0081\f\u0081\16\u0081\u0c20\13\u0081\3\u0081\3\u0081")
        buf.write("\5\u0081\u0c24\n\u0081\5\u0081\u0c26\n\u0081\3\u0082\3")
        buf.write("\u0082\3\u0082\5\u0082\u0c2b\n\u0082\3\u0083\3\u0083\7")
        buf.write("\u0083\u0c2f\n\u0083\f\u0083\16\u0083\u0c32\13\u0083\3")
        buf.write("\u0083\3\u0083\3\u0084\3\u0084\7\u0084\u0c38\n\u0084\f")
        buf.write("\u0084\16\u0084\u0c3b\13\u0084\3\u0084\3\u0084\3\u0085")
        buf.write("\3\u0085\7\u0085\u0c41\n\u0085\f\u0085\16\u0085\u0c44")
        buf.write("\13\u0085\3\u0085\3\u0085\7\u0085\u0c48\n\u0085\f\u0085")
        buf.write("\16\u0085\u0c4b\13\u0085\3\u0085\6\u0085\u0c4e\n\u0085")
        buf.write("\r\u0085\16\u0085\u0c4f\3\u0085\7\u0085\u0c53\n\u0085")
        buf.write("\f\u0085\16\u0085\u0c56\13\u0085\3\u0085\5\u0085\u0c59")
        buf.write("\n\u0085\3\u0085\7\u0085\u0c5c\n\u0085\f\u0085\16\u0085")
        buf.write("\u0c5f\13\u0085\3\u0085\5\u0085\u0c62\n\u0085\3\u0086")
        buf.write("\3\u0086\7\u0086\u0c66\n\u0086\f\u0086\16\u0086\u0c69")
        buf.write("\13\u0086\3\u0086\3\u0086\7\u0086\u0c6d\n\u0086\f\u0086")
        buf.write("\16\u0086\u0c70\13\u0086\3\u0086\3\u0086\3\u0086\3\u0086")
        buf.write("\7\u0086\u0c76\n\u0086\f\u0086\16\u0086\u0c79\13\u0086")
        buf.write("\3\u0086\5\u0086\u0c7c\n\u0086\3\u0086\3\u0086\7\u0086")
        buf.write("\u0c80\n\u0086\f\u0086\16\u0086\u0c83\13\u0086\3\u0086")
        buf.write("\3\u0086\3\u0087\3\u0087\7\u0087\u0c89\n\u0087\f\u0087")
        buf.write("\16\u0087\u0c8c\13\u0087\3\u0087\3\u0087\3\u0088\3\u0088")
        buf.write("\7\u0088\u0c92\n\u0088\f\u0088\16\u0088\u0c95\13\u0088")
        buf.write("\3\u0088\3\u0088\3\u0088\5\u0088\u0c9a\n\u0088\3\u0088")
        buf.write("\3\u0088\3\u0088\3\u0088\5\u0088\u0ca0\n\u0088\3\u0089")
        buf.write("\5\u0089\u0ca3\n\u0089\3\u0089\3\u0089\7\u0089\u0ca7\n")
        buf.write("\u0089\f\u0089\16\u0089\u0caa\13\u0089\3\u0089\3\u0089")
        buf.write("\5\u0089\u0cae\n\u0089\3\u008a\3\u008a\3\u008b\3\u008b")
        buf.write("\3\u008c\3\u008c\3\u008d\3\u008d\3\u008e\3\u008e\3\u008f")
        buf.write("\3\u008f\3\u0090\3\u0090\3\u0091\3\u0091\3\u0092\3\u0092")
        buf.write("\3\u0092\3\u0092\3\u0092\5\u0092\u0cc5\n\u0092\3\u0093")
        buf.write("\3\u0093\3\u0093\3\u0093\5\u0093\u0ccb\n\u0093\3\u0094")
        buf.write("\3\u0094\3\u0095\7\u0095\u0cd0\n\u0095\f\u0095\16\u0095")
        buf.write("\u0cd3\13\u0095\3\u0095\3\u0095\7\u0095\u0cd7\n\u0095")
        buf.write("\f\u0095\16\u0095\u0cda\13\u0095\3\u0095\3\u0095\5\u0095")
        buf.write("\u0cde\n\u0095\3\u0096\3\u0096\3\u0096\3\u0097\3\u0097")
        buf.write("\6\u0097\u0ce5\n\u0097\r\u0097\16\u0097\u0ce6\3\u0098")
        buf.write("\3\u0098\6\u0098\u0ceb\n\u0098\r\u0098\16\u0098\u0cec")
        buf.write("\3\u0099\3\u0099\3\u0099\3\u0099\3\u0099\3\u0099\3\u0099")
        buf.write("\3\u0099\5\u0099\u0cf7\n\u0099\3\u0099\7\u0099\u0cfa\n")
        buf.write("\u0099\f\u0099\16\u0099\u0cfd\13\u0099\3\u009a\6\u009a")
        buf.write("\u0d00\n\u009a\r\u009a\16\u009a\u0d01\3\u009b\3\u009b")
        buf.write("\3\u009b\7\u009b\u0d07\n\u009b\f\u009b\16\u009b\u0d0a")
        buf.write("\13\u009b\5\u009b\u0d0c\n\u009b\3\u009c\3\u009c\3\u009d")
        buf.write("\3\u009d\3\u009e\3\u009e\3\u009f\3\u009f\3\u00a0\6\u00a0")
        buf.write("\u0d17\n\u00a0\r\u00a0\16\u00a0\u0d18\3\u00a1\3\u00a1")
        buf.write("\7\u00a1\u0d1d\n\u00a1\f\u00a1\16\u00a1\u0d20\13\u00a1")
        buf.write("\3\u00a1\3\u00a1\7\u00a1\u0d24\n\u00a1\f\u00a1\16\u00a1")
        buf.write("\u0d27\13\u00a1\3\u00a1\5\u00a1\u0d2a\n\u00a1\3\u00a2")
        buf.write("\3\u00a2\3\u00a3\3\u00a3\3\u00a4\3\u00a4\3\u00a5\3\u00a5")
        buf.write("\3\u00a6\3\u00a6\3\u00a7\3\u00a7\3\u00a8\3\u00a8\5\u00a8")
        buf.write("\u0d3a\n\u00a8\3\u00a8\7\u00a8\u0d3d\n\u00a8\f\u00a8\16")
        buf.write("\u00a8\u0d40\13\u00a8\3\u00a9\3\u00a9\7\u00a9\u0d44\n")
        buf.write("\u00a9\f\u00a9\16\u00a9\u0d47\13\u00a9\3\u00a9\3\u00a9")
        buf.write("\5\u00a9\u0d4b\n\u00a9\3\u00a9\3\u00a9\3\u00aa\3\u00aa")
        buf.write("\7\u00aa\u0d51\n\u00aa\f\u00aa\16\u00aa\u0d54\13\u00aa")
        buf.write("\3\u00aa\3\u00aa\5\u00aa\u0d58\n\u00aa\3\u00aa\3\u00aa")
        buf.write("\6\u00aa\u0d5c\n\u00aa\r\u00aa\16\u00aa\u0d5d\3\u00aa")
        buf.write("\3\u00aa\3\u00ab\3\u00ab\3\u00ab\7\u00ab\u0d65\n\u00ab")
        buf.write("\f\u00ab\16\u00ab\u0d68\13\u00ab\3\u00ab\3\u00ab\3\u00ac")
        buf.write("\3\u00ac\5\u00ac\u0d6e\n\u00ac\3\u00ad\3\u00ad\3\u00ae")
        buf.write("\3\u00ae\7\u00ae\u0d74\n\u00ae\f\u00ae\16\u00ae\u0d77")
        buf.write("\13\u00ae\3\u00ae\3\u00ae\7\u00ae\u0d7b\n\u00ae\f\u00ae")
        buf.write("\16\u00ae\u0d7e\13\u00ae\3\u00ae\2\2\u00af\2\4\6\b\n\f")
        buf.write("\16\20\22\24\26\30\32\34\36 \"$&(*,.\60\62\64\668:<>@")
        buf.write("BDFHJLNPRTVXZ\\^`bdfhjlnprtvxz|~\u0080\u0082\u0084\u0086")
        buf.write("\u0088\u008a\u008c\u008e\u0090\u0092\u0094\u0096\u0098")
        buf.write("\u009a\u009c\u009e\u00a0\u00a2\u00a4\u00a6\u00a8\u00aa")
        buf.write("\u00ac\u00ae\u00b0\u00b2\u00b4\u00b6\u00b8\u00ba\u00bc")
        buf.write("\u00be\u00c0\u00c2\u00c4\u00c6\u00c8\u00ca\u00cc\u00ce")
        buf.write("\u00d0\u00d2\u00d4\u00d6\u00d8\u00da\u00dc\u00de\u00e0")
        buf.write("\u00e2\u00e4\u00e6\u00e8\u00ea\u00ec\u00ee\u00f0\u00f2")
        buf.write("\u00f4\u00f6\u00f8\u00fa\u00fc\u00fe\u0100\u0102\u0104")
        buf.write("\u0106\u0108\u010a\u010c\u010e\u0110\u0112\u0114\u0116")
        buf.write("\u0118\u011a\u011c\u011e\u0120\u0122\u0124\u0126\u0128")
        buf.write("\u012a\u012c\u012e\u0130\u0132\u0134\u0136\u0138\u013a")
        buf.write("\u013c\u013e\u0140\u0142\u0144\u0146\u0148\u014a\u014c")
        buf.write("\u014e\u0150\u0152\u0154\u0156\u0158\u015a\2 \4\2**,,")
        buf.write("\3\2NO\3\2UV\3\2./\3\2*+\4\2\7\7\35\35\4\2\u0089\u0089")
        buf.write("\u008c\u0093\3\2\u00a1\u00a3\3\2\u00a6\u00a8\4\2==UU\4")
        buf.write("\2::cc\3\2\37#\4\2\64\65\678\3\2\60\63\4\2hhjj\4\2ggi")
        buf.write("i\3\2\24\25\3\2\21\23\4\2\66\66ff\3\2\32\33\3\2qv\4\2")
        buf.write("}}\u0082\u0082\3\2mp\4\2hhkk\3\2w|\3\2~\u0080\3\2\u0083")
        buf.write("\u0085\3\2\u0087\u0088\3\2@G\t\2?GIIQTXX]^k\u0088\u0094")
        buf.write("\u0094\2\u0f1c\2\u015d\3\2\2\2\4\u0176\3\2\2\2\6\u0190")
        buf.write("\3\2\2\2\b\u0196\3\2\2\2\n\u01bb\3\2\2\2\f\u01c0\3\2\2")
        buf.write("\2\16\u01c3\3\2\2\2\20\u01cd\3\2\2\2\22\u01d0\3\2\2\2")
        buf.write("\24\u01d5\3\2\2\2\26\u01fc\3\2\2\2\30\u01ff\3\2\2\2\32")
        buf.write("\u025a\3\2\2\2\34\u025e\3\2\2\2\36\u026e\3\2\2\2 \u029d")
        buf.write("\3\2\2\2\"\u02c1\3\2\2\2$\u02d9\3\2\2\2&\u02db\3\2\2\2")
        buf.write("(\u02e1\3\2\2\2*\u02ee\3\2\2\2,\u02ff\3\2\2\2.\u032c\3")
        buf.write("\2\2\2\60\u0345\3\2\2\2\62\u0363\3\2\2\2\64\u037c\3\2")
        buf.write("\2\2\66\u0383\3\2\2\28\u0385\3\2\2\2:\u038f\3\2\2\2<\u03bb")
        buf.write("\3\2\2\2>\u03ea\3\2\2\2@\u03fe\3\2\2\2B\u0454\3\2\2\2")
        buf.write("D\u0459\3\2\2\2F\u0473\3\2\2\2H\u04a0\3\2\2\2J\u0511\3")
        buf.write("\2\2\2L\u051b\3\2\2\2N\u0546\3\2\2\2P\u0581\3\2\2\2R\u05b0")
        buf.write("\3\2\2\2T\u05c3\3\2\2\2V\u05d4\3\2\2\2X\u05e5\3\2\2\2")
        buf.write("Z\u0609\3\2\2\2\\\u062c\3\2\2\2^\u0635\3\2\2\2`\u0657")
        buf.write("\3\2\2\2b\u067b\3\2\2\2d\u0691\3\2\2\2f\u069b\3\2\2\2")
        buf.write("h\u069f\3\2\2\2j\u06ac\3\2\2\2l\u06ae\3\2\2\2n\u06c2\3")
        buf.write("\2\2\2p\u06d1\3\2\2\2r\u06d4\3\2\2\2t\u06e0\3\2\2\2v\u06f0")
        buf.write("\3\2\2\2x\u0702\3\2\2\2z\u0734\3\2\2\2|\u0745\3\2\2\2")
        buf.write("~\u074c\3\2\2\2\u0080\u0768\3\2\2\2\u0082\u0771\3\2\2")
        buf.write("\2\u0084\u077a\3\2\2\2\u0086\u0784\3\2\2\2\u0088\u0786")
        buf.write("\3\2\2\2\u008a\u0799\3\2\2\2\u008c\u079b\3\2\2\2\u008e")
        buf.write("\u07b9\3\2\2\2\u0090\u07cd\3\2\2\2\u0092\u07ee\3\2\2\2")
        buf.write("\u0094\u0800\3\2\2\2\u0096\u0808\3\2\2\2\u0098\u080a\3")
        buf.write("\2\2\2\u009a\u080c\3\2\2\2\u009c\u0820\3\2\2\2\u009e\u0834")
        buf.write("\3\2\2\2\u00a0\u0843\3\2\2\2\u00a2\u0852\3\2\2\2\u00a4")
        buf.write("\u0859\3\2\2\2\u00a6\u0871\3\2\2\2\u00a8\u0886\3\2\2\2")
        buf.write("\u00aa\u0889\3\2\2\2\u00ac\u0898\3\2\2\2\u00ae\u08a6\3")
        buf.write("\2\2\2\u00b0\u08b5\3\2\2\2\u00b2\u08c4\3\2\2\2\u00b4\u08dc")
        buf.write("\3\2\2\2\u00b6\u08ea\3\2\2\2\u00b8\u08ec\3\2\2\2\u00ba")
        buf.write("\u08f8\3\2\2\2\u00bc\u08ff\3\2\2\2\u00be\u0901\3\2\2\2")
        buf.write("\u00c0\u0913\3\2\2\2\u00c2\u0915\3\2\2\2\u00c4\u0928\3")
        buf.write("\2\2\2\u00c6\u092a\3\2\2\2\u00c8\u0956\3\2\2\2\u00ca\u0963")
        buf.write("\3\2\2\2\u00cc\u096f\3\2\2\2\u00ce\u097d\3\2\2\2\u00d0")
        buf.write("\u09a9\3\2\2\2\u00d2\u09d8\3\2\2\2\u00d4\u0a09\3\2\2\2")
        buf.write("\u00d6\u0a0b\3\2\2\2\u00d8\u0a1b\3\2\2\2\u00da\u0a49\3")
        buf.write("\2\2\2\u00dc\u0a4d\3\2\2\2\u00de\u0a4f\3\2\2\2\u00e0\u0a59")
        buf.write("\3\2\2\2\u00e2\u0a64\3\2\2\2\u00e4\u0a66\3\2\2\2\u00e6")
        buf.write("\u0a76\3\2\2\2\u00e8\u0a78\3\2\2\2\u00ea\u0a88\3\2\2\2")
        buf.write("\u00ec\u0aaa\3\2\2\2\u00ee\u0ad9\3\2\2\2\u00f0\u0adb\3")
        buf.write("\2\2\2\u00f2\u0b18\3\2\2\2\u00f4\u0b1a\3\2\2\2\u00f6\u0b3a")
        buf.write("\3\2\2\2\u00f8\u0b54\3\2\2\2\u00fa\u0b56\3\2\2\2\u00fc")
        buf.write("\u0b93\3\2\2\2\u00fe\u0bba\3\2\2\2\u0100\u0c25\3\2\2\2")
        buf.write("\u0102\u0c2a\3\2\2\2\u0104\u0c2c\3\2\2\2\u0106\u0c35\3")
        buf.write("\2\2\2\u0108\u0c3e\3\2\2\2\u010a\u0c63\3\2\2\2\u010c\u0c86")
        buf.write("\3\2\2\2\u010e\u0c9f\3\2\2\2\u0110\u0ca2\3\2\2\2\u0112")
        buf.write("\u0caf\3\2\2\2\u0114\u0cb1\3\2\2\2\u0116\u0cb3\3\2\2\2")
        buf.write("\u0118\u0cb5\3\2\2\2\u011a\u0cb7\3\2\2\2\u011c\u0cb9\3")
        buf.write("\2\2\2\u011e\u0cbb\3\2\2\2\u0120\u0cbd\3\2\2\2\u0122\u0cc4")
        buf.write("\3\2\2\2\u0124\u0cca\3\2\2\2\u0126\u0ccc\3\2\2\2\u0128")
        buf.write("\u0cdd\3\2\2\2\u012a\u0cdf\3\2\2\2\u012c\u0ce4\3\2\2\2")
        buf.write("\u012e\u0cea\3\2\2\2\u0130\u0cf6\3\2\2\2\u0132\u0cff\3")
        buf.write("\2\2\2\u0134\u0d0b\3\2\2\2\u0136\u0d0d\3\2\2\2\u0138\u0d0f")
        buf.write("\3\2\2\2\u013a\u0d11\3\2\2\2\u013c\u0d13\3\2\2\2\u013e")
        buf.write("\u0d16\3\2\2\2\u0140\u0d29\3\2\2\2\u0142\u0d2b\3\2\2\2")
        buf.write("\u0144\u0d2d\3\2\2\2\u0146\u0d2f\3\2\2\2\u0148\u0d31\3")
        buf.write("\2\2\2\u014a\u0d33\3\2\2\2\u014c\u0d35\3\2\2\2\u014e\u0d39")
        buf.write("\3\2\2\2\u0150\u0d4a\3\2\2\2\u0152\u0d57\3\2\2\2\u0154")
        buf.write("\u0d61\3\2\2\2\u0156\u0d6d\3\2\2\2\u0158\u0d6f\3\2\2\2")
        buf.write("\u015a\u0d71\3\2\2\2\u015c\u015e\5\6\4\2\u015d\u015c\3")
        buf.write("\2\2\2\u015d\u015e\3\2\2\2\u015e\u0162\3\2\2\2\u015f\u0161")
        buf.write("\7\7\2\2\u0160\u015f\3\2\2\2\u0161\u0164\3\2\2\2\u0162")
        buf.write("\u0160\3\2\2\2\u0162\u0163\3\2\2\2\u0163\u0168\3\2\2\2")
        buf.write("\u0164\u0162\3\2\2\2\u0165\u0167\5\b\5\2\u0166\u0165\3")
        buf.write("\2\2\2\u0167\u016a\3\2\2\2\u0168\u0166\3\2\2\2\u0168\u0169")
        buf.write("\3\2\2\2\u0169\u016b\3\2\2\2\u016a\u0168\3\2\2\2\u016b")
        buf.write("\u016c\5\n\6\2\u016c\u0170\5\f\7\2\u016d\u016f\5\22\n")
        buf.write("\2\u016e\u016d\3\2\2\2\u016f\u0172\3\2\2\2\u0170\u016e")
        buf.write("\3\2\2\2\u0170\u0171\3\2\2\2\u0171\u0173\3\2\2\2\u0172")
        buf.write("\u0170\3\2\2\2\u0173\u0174\7\2\2\3\u0174\3\3\2\2\2\u0175")
        buf.write("\u0177\5\6\4\2\u0176\u0175\3\2\2\2\u0176\u0177\3\2\2\2")
        buf.write("\u0177\u017b\3\2\2\2\u0178\u017a\7\7\2\2\u0179\u0178\3")
        buf.write("\2\2\2\u017a\u017d\3\2\2\2\u017b\u0179\3\2\2\2\u017b\u017c")
        buf.write("\3\2\2\2\u017c\u0181\3\2\2\2\u017d\u017b\3\2\2\2\u017e")
        buf.write("\u0180\5\b\5\2\u017f\u017e\3\2\2\2\u0180\u0183\3\2\2\2")
        buf.write("\u0181\u017f\3\2\2\2\u0181\u0182\3\2\2\2\u0182\u0184\3")
        buf.write("\2\2\2\u0183\u0181\3\2\2\2\u0184\u0185\5\n\6\2\u0185\u018b")
        buf.write("\5\f\7\2\u0186\u0187\5\u0082B\2\u0187\u0188\5\u0094K\2")
        buf.write("\u0188\u018a\3\2\2\2\u0189\u0186\3\2\2\2\u018a\u018d\3")
        buf.write("\2\2\2\u018b\u0189\3\2\2\2\u018b\u018c\3\2\2\2\u018c\u018e")
        buf.write("\3\2\2\2\u018d\u018b\3\2\2\2\u018e\u018f\7\2\2\3\u018f")
        buf.write("\5\3\2\2\2\u0190\u0192\7\3\2\2\u0191\u0193\7\7\2\2\u0192")
        buf.write("\u0191\3\2\2\2\u0193\u0194\3\2\2\2\u0194\u0192\3\2\2\2")
        buf.write("\u0194\u0195\3\2\2\2\u0195\7\3\2\2\2\u0196\u0197\t\2\2")
        buf.write("\2\u0197\u019b\7?\2\2\u0198\u019a\7\7\2\2\u0199\u0198")
        buf.write("\3\2\2\2\u019a\u019d\3\2\2\2\u019b\u0199\3\2\2\2\u019b")
        buf.write("\u019c\3\2\2\2\u019c\u019e\3\2\2\2\u019d\u019b\3\2\2\2")
        buf.write("\u019e\u01a2\7\34\2\2\u019f\u01a1\7\7\2\2\u01a0\u019f")
        buf.write("\3\2\2\2\u01a1\u01a4\3\2\2\2\u01a2\u01a0\3\2\2\2\u01a2")
        buf.write("\u01a3\3\2\2\2\u01a3\u01ae\3\2\2\2\u01a4\u01a2\3\2\2\2")
        buf.write("\u01a5\u01a7\7\r\2\2\u01a6\u01a8\5\u0156\u00ac\2\u01a7")
        buf.write("\u01a6\3\2\2\2\u01a8\u01a9\3\2\2\2\u01a9\u01a7\3\2\2\2")
        buf.write("\u01a9\u01aa\3\2\2\2\u01aa\u01ab\3\2\2\2\u01ab\u01ac\7")
        buf.write("\16\2\2\u01ac\u01af\3\2\2\2\u01ad\u01af\5\u0156\u00ac")
        buf.write("\2\u01ae\u01a5\3\2\2\2\u01ae\u01ad\3\2\2\2\u01af\u01b3")
        buf.write("\3\2\2\2\u01b0\u01b2\7\7\2\2\u01b1\u01b0\3\2\2\2\u01b2")
        buf.write("\u01b5\3\2\2\2\u01b3\u01b1\3\2\2\2\u01b3\u01b4\3\2\2\2")
        buf.write("\u01b4\t\3\2\2\2\u01b5\u01b3\3\2\2\2\u01b6\u01b7\7H\2")
        buf.write("\2\u01b7\u01b9\5\u015a\u00ae\2\u01b8\u01ba\5\u0094K\2")
        buf.write("\u01b9\u01b8\3\2\2\2\u01b9\u01ba\3\2\2\2\u01ba\u01bc\3")
        buf.write("\2\2\2\u01bb\u01b6\3\2\2\2\u01bb\u01bc\3\2\2\2\u01bc\13")
        buf.write("\3\2\2\2\u01bd\u01bf\5\16\b\2\u01be\u01bd\3\2\2\2\u01bf")
        buf.write("\u01c2\3\2\2\2\u01c0\u01be\3\2\2\2\u01c0\u01c1\3\2\2\2")
        buf.write("\u01c1\r\3\2\2\2\u01c2\u01c0\3\2\2\2\u01c3\u01c4\7I\2")
        buf.write("\2\u01c4\u01c8\5\u015a\u00ae\2\u01c5\u01c6\7\t\2\2\u01c6")
        buf.write("\u01c9\7\21\2\2\u01c7\u01c9\5\20\t\2\u01c8\u01c5\3\2\2")
        buf.write("\2\u01c8\u01c7\3\2\2\2\u01c8\u01c9\3\2\2\2\u01c9\u01cb")
        buf.write("\3\2\2\2\u01ca\u01cc\5\u0094K\2\u01cb\u01ca\3\2\2\2\u01cb")
        buf.write("\u01cc\3\2\2\2\u01cc\17\3\2\2\2\u01cd\u01ce\7f\2\2\u01ce")
        buf.write("\u01cf\5\u0158\u00ad\2\u01cf\21\3\2\2\2\u01d0\u01d2\5")
        buf.write("\26\f\2\u01d1\u01d3\5\u0096L\2\u01d2\u01d1\3\2\2\2\u01d2")
        buf.write("\u01d3\3\2\2\2\u01d3\23\3\2\2\2\u01d4\u01d6\5\u012c\u0097")
        buf.write("\2\u01d5\u01d4\3\2\2\2\u01d5\u01d6\3\2\2\2\u01d6\u01d7")
        buf.write("\3\2\2\2\u01d7\u01db\7P\2\2\u01d8\u01da\7\7\2\2\u01d9")
        buf.write("\u01d8\3\2\2\2\u01da\u01dd\3\2\2\2\u01db\u01d9\3\2\2\2")
        buf.write("\u01db\u01dc\3\2\2\2\u01dc\u01de\3\2\2\2\u01dd\u01db\3")
        buf.write("\2\2\2\u01de\u01e6\5\u0158\u00ad\2\u01df\u01e1\7\7\2\2")
        buf.write("\u01e0\u01df\3\2\2\2\u01e1\u01e4\3\2\2\2\u01e2\u01e0\3")
        buf.write("\2\2\2\u01e2\u01e3\3\2\2\2\u01e3\u01e5\3\2\2\2\u01e4\u01e2")
        buf.write("\3\2\2\2\u01e5\u01e7\5,\27\2\u01e6\u01e2\3\2\2\2\u01e6")
        buf.write("\u01e7\3\2\2\2\u01e7\u01eb\3\2\2\2\u01e8\u01ea\7\7\2\2")
        buf.write("\u01e9\u01e8\3\2\2\2\u01ea\u01ed\3\2\2\2\u01eb\u01e9\3")
        buf.write("\2\2\2\u01eb\u01ec\3\2\2\2\u01ec\u01ee\3\2\2\2\u01ed\u01eb")
        buf.write("\3\2\2\2\u01ee\u01f2\7\36\2\2\u01ef\u01f1\7\7\2\2\u01f0")
        buf.write("\u01ef\3\2\2\2\u01f1\u01f4\3\2\2\2\u01f2\u01f0\3\2\2\2")
        buf.write("\u01f2\u01f3\3\2\2\2\u01f3\u01f5\3\2\2\2\u01f4\u01f2\3")
        buf.write("\2\2\2\u01f5\u01f6\5d\63\2\u01f6\25\3\2\2\2\u01f7\u01fd")
        buf.write("\5\30\r\2\u01f8\u01fd\5X-\2\u01f9\u01fd\5@!\2\u01fa\u01fd")
        buf.write("\5H%\2\u01fb\u01fd\5\24\13\2\u01fc\u01f7\3\2\2\2\u01fc")
        buf.write("\u01f8\3\2\2\2\u01fc\u01f9\3\2\2\2\u01fc\u01fa\3\2\2\2")
        buf.write("\u01fc\u01fb\3\2\2\2\u01fd\27\3\2\2\2\u01fe\u0200\5\u012c")
        buf.write("\u0097\2\u01ff\u01fe\3\2\2\2\u01ff\u0200\3\2\2\2\u0200")
        buf.write("\u020c\3\2\2\2\u0201\u020d\7J\2\2\u0202\u0206\7L\2\2\u0203")
        buf.write("\u0205\7\7\2\2\u0204\u0203\3\2\2\2\u0205\u0208\3\2\2\2")
        buf.write("\u0206\u0204\3\2\2\2\u0206\u0207\3\2\2\2\u0207\u020a\3")
        buf.write("\2\2\2\u0208\u0206\3\2\2\2\u0209\u0202\3\2\2\2\u0209\u020a")
        buf.write("\3\2\2\2\u020a\u020b\3\2\2\2\u020b\u020d\7K\2\2\u020c")
        buf.write("\u0201\3\2\2\2\u020c\u0209\3\2\2\2\u020d\u0211\3\2\2\2")
        buf.write("\u020e\u0210\7\7\2\2\u020f\u020e\3\2\2\2\u0210\u0213\3")
        buf.write("\2\2\2\u0211\u020f\3\2\2\2\u0211\u0212\3\2\2\2\u0212\u0214")
        buf.write("\3\2\2\2\u0213\u0211\3\2\2\2\u0214\u021c\5\u0158\u00ad")
        buf.write("\2\u0215\u0217\7\7\2\2\u0216\u0215\3\2\2\2\u0217\u021a")
        buf.write("\3\2\2\2\u0218\u0216\3\2\2\2\u0218\u0219\3\2\2\2\u0219")
        buf.write("\u021b\3\2\2\2\u021a\u0218\3\2\2\2\u021b\u021d\5,\27\2")
        buf.write("\u021c\u0218\3\2\2\2\u021c\u021d\3\2\2\2\u021d\u0225\3")
        buf.write("\2\2\2\u021e\u0220\7\7\2\2\u021f\u021e\3\2\2\2\u0220\u0223")
        buf.write("\3\2\2\2\u0221\u021f\3\2\2\2\u0221\u0222\3\2\2\2\u0222")
        buf.write("\u0224\3\2\2\2\u0223\u0221\3\2\2\2\u0224\u0226\5\32\16")
        buf.write("\2\u0225\u0221\3\2\2\2\u0225\u0226\3\2\2\2\u0226\u0235")
        buf.write("\3\2\2\2\u0227\u0229\7\7\2\2\u0228\u0227\3\2\2\2\u0229")
        buf.write("\u022c\3\2\2\2\u022a\u0228\3\2\2\2\u022a\u022b\3\2\2\2")
        buf.write("\u022b\u022d\3\2\2\2\u022c\u022a\3\2\2\2\u022d\u0231\7")
        buf.write("\34\2\2\u022e\u0230\7\7\2\2\u022f\u022e\3\2\2\2\u0230")
        buf.write("\u0233\3\2\2\2\u0231\u022f\3\2\2\2\u0231\u0232\3\2\2\2")
        buf.write("\u0232\u0234\3\2\2\2\u0233\u0231\3\2\2\2\u0234\u0236\5")
        buf.write("\"\22\2\u0235\u022a\3\2\2\2\u0235\u0236\3\2\2\2\u0236")
        buf.write("\u023e\3\2\2\2\u0237\u0239\7\7\2\2\u0238\u0237\3\2\2\2")
        buf.write("\u0239\u023c\3\2\2\2\u023a\u0238\3\2\2\2\u023a\u023b\3")
        buf.write("\2\2\2\u023b\u023d\3\2\2\2\u023c\u023a\3\2\2\2\u023d\u023f")
        buf.write("\5\60\31\2\u023e\u023a\3\2\2\2\u023e\u023f\3\2\2\2\u023f")
        buf.write("\u024e\3\2\2\2\u0240\u0242\7\7\2\2\u0241\u0240\3\2\2\2")
        buf.write("\u0242\u0245\3\2\2\2\u0243\u0241\3\2\2\2\u0243\u0244\3")
        buf.write("\2\2\2\u0244\u0246\3\2\2\2\u0245\u0243\3\2\2\2\u0246\u024f")
        buf.write("\5\34\17\2\u0247\u0249\7\7\2\2\u0248\u0247\3\2\2\2\u0249")
        buf.write("\u024c\3\2\2\2\u024a\u0248\3\2\2\2\u024a\u024b\3\2\2\2")
        buf.write("\u024b\u024d\3\2\2\2\u024c\u024a\3\2\2\2\u024d\u024f\5")
        buf.write("^\60\2\u024e\u0243\3\2\2\2\u024e\u024a\3\2\2\2\u024e\u024f")
        buf.write("\3\2\2\2\u024f\31\3\2\2\2\u0250\u0252\5\u012c\u0097\2")
        buf.write("\u0251\u0250\3\2\2\2\u0251\u0252\3\2\2\2\u0252\u0253\3")
        buf.write("\2\2\2\u0253\u0257\7Q\2\2\u0254\u0256\7\7\2\2\u0255\u0254")
        buf.write("\3\2\2\2\u0256\u0259\3\2\2\2\u0257\u0255\3\2\2\2\u0257")
        buf.write("\u0258\3\2\2\2\u0258\u025b\3\2\2\2\u0259\u0257\3\2\2\2")
        buf.write("\u025a\u0251\3\2\2\2\u025a\u025b\3\2\2\2\u025b\u025c\3")
        buf.write("\2\2\2\u025c\u025d\5\36\20\2\u025d\33\3\2\2\2\u025e\u0262")
        buf.write("\7\17\2\2\u025f\u0261\7\7\2\2\u0260\u025f\3\2\2\2\u0261")
        buf.write("\u0264\3\2\2\2\u0262\u0260\3\2\2\2\u0262\u0263\3\2\2\2")
        buf.write("\u0263\u0265\3\2\2\2\u0264\u0262\3\2\2\2\u0265\u0269\5")
        buf.write("\64\33\2\u0266\u0268\7\7\2\2\u0267\u0266\3\2\2\2\u0268")
        buf.write("\u026b\3\2\2\2\u0269\u0267\3\2\2\2\u0269\u026a\3\2\2\2")
        buf.write("\u026a\u026c\3\2\2\2\u026b\u0269\3\2\2\2\u026c\u026d\7")
        buf.write("\20\2\2\u026d\35\3\2\2\2\u026e\u0272\7\13\2\2\u026f\u0271")
        buf.write("\7\7\2\2\u0270\u026f\3\2\2\2\u0271\u0274\3\2\2\2\u0272")
        buf.write("\u0270\3\2\2\2\u0272\u0273\3\2\2\2\u0273\u0292\3\2\2\2")
        buf.write("\u0274\u0272\3\2\2\2\u0275\u0286\5 \21\2\u0276\u0278\7")
        buf.write("\7\2\2\u0277\u0276\3\2\2\2\u0278\u027b\3\2\2\2\u0279\u0277")
        buf.write("\3\2\2\2\u0279\u027a\3\2\2\2\u027a\u027c\3\2\2\2\u027b")
        buf.write("\u0279\3\2\2\2\u027c\u0280\7\n\2\2\u027d\u027f\7\7\2\2")
        buf.write("\u027e\u027d\3\2\2\2\u027f\u0282\3\2\2\2\u0280\u027e\3")
        buf.write("\2\2\2\u0280\u0281\3\2\2\2\u0281\u0283\3\2\2\2\u0282\u0280")
        buf.write("\3\2\2\2\u0283\u0285\5 \21\2\u0284\u0279\3\2\2\2\u0285")
        buf.write("\u0288\3\2\2\2\u0286\u0284\3\2\2\2\u0286\u0287\3\2\2\2")
        buf.write("\u0287\u0290\3\2\2\2\u0288\u0286\3\2\2\2\u0289\u028b\7")
        buf.write("\7\2\2\u028a\u0289\3\2\2\2\u028b\u028e\3\2\2\2\u028c\u028a")
        buf.write("\3\2\2\2\u028c\u028d\3\2\2\2\u028d\u028f\3\2\2\2\u028e")
        buf.write("\u028c\3\2\2\2\u028f\u0291\7\n\2\2\u0290\u028c\3\2\2\2")
        buf.write("\u0290\u0291\3\2\2\2\u0291\u0293\3\2\2\2\u0292\u0275\3")
        buf.write("\2\2\2\u0292\u0293\3\2\2\2\u0293\u0297\3\2\2\2\u0294\u0296")
        buf.write("\7\7\2\2\u0295\u0294\3\2\2\2\u0296\u0299\3\2\2\2\u0297")
        buf.write("\u0295\3\2\2\2\u0297\u0298\3\2\2\2\u0298\u029a\3\2\2\2")
        buf.write("\u0299\u0297\3\2\2\2\u029a\u029b\7\f\2\2\u029b\37\3\2")
        buf.write("\2\2\u029c\u029e\5\u012c\u0097\2\u029d\u029c\3\2\2\2\u029d")
        buf.write("\u029e\3\2\2\2\u029e\u02a0\3\2\2\2\u029f\u02a1\t\3\2\2")
        buf.write("\u02a0\u029f\3\2\2\2\u02a0\u02a1\3\2\2\2\u02a1\u02a5\3")
        buf.write("\2\2\2\u02a2\u02a4\7\7\2\2\u02a3\u02a2\3\2\2\2\u02a4\u02a7")
        buf.write("\3\2\2\2\u02a5\u02a3\3\2\2\2\u02a5\u02a6\3\2\2\2\u02a6")
        buf.write("\u02a8\3\2\2\2\u02a7\u02a5\3\2\2\2\u02a8\u02a9\5\u0158")
        buf.write("\u00ad\2\u02a9\u02ad\7\34\2\2\u02aa\u02ac\7\7\2\2\u02ab")
        buf.write("\u02aa\3\2\2\2\u02ac\u02af\3\2\2\2\u02ad\u02ab\3\2\2\2")
        buf.write("\u02ad\u02ae\3\2\2\2\u02ae\u02b0\3\2\2\2\u02af\u02ad\3")
        buf.write("\2\2\2\u02b0\u02bf\5d\63\2\u02b1\u02b3\7\7\2\2\u02b2\u02b1")
        buf.write("\3\2\2\2\u02b3\u02b6\3\2\2\2\u02b4\u02b2\3\2\2\2\u02b4")
        buf.write("\u02b5\3\2\2\2\u02b5\u02b7\3\2\2\2\u02b6\u02b4\3\2\2\2")
        buf.write("\u02b7\u02bb\7\36\2\2\u02b8\u02ba\7\7\2\2\u02b9\u02b8")
        buf.write("\3\2\2\2\u02ba\u02bd\3\2\2\2\u02bb\u02b9\3\2\2\2\u02bb")
        buf.write("\u02bc\3\2\2\2\u02bc\u02be\3\2\2\2\u02bd\u02bb\3\2\2\2")
        buf.write("\u02be\u02c0\5\u0098M\2\u02bf\u02b4\3\2\2\2\u02bf\u02c0")
        buf.write("\3\2\2\2\u02c0!\3\2\2\2\u02c1\u02d2\5(\25\2\u02c2\u02c4")
        buf.write("\7\7\2\2\u02c3\u02c2\3\2\2\2\u02c4\u02c7\3\2\2\2\u02c5")
        buf.write("\u02c3\3\2\2\2\u02c5\u02c6\3\2\2\2\u02c6\u02c8\3\2\2\2")
        buf.write("\u02c7\u02c5\3\2\2\2\u02c8\u02cc\7\n\2\2\u02c9\u02cb\7")
        buf.write("\7\2\2\u02ca\u02c9\3\2\2\2\u02cb\u02ce\3\2\2\2\u02cc\u02ca")
        buf.write("\3\2\2\2\u02cc\u02cd\3\2\2\2\u02cd\u02cf\3\2\2\2\u02ce")
        buf.write("\u02cc\3\2\2\2\u02cf\u02d1\5(\25\2\u02d0\u02c5\3\2\2\2")
        buf.write("\u02d1\u02d4\3\2\2\2\u02d2\u02d0\3\2\2\2\u02d2\u02d3\3")
        buf.write("\2\2\2\u02d3#\3\2\2\2\u02d4\u02d2\3\2\2\2\u02d5\u02da")
        buf.write("\5&\24\2\u02d6\u02da\5*\26\2\u02d7\u02da\5l\67\2\u02d8")
        buf.write("\u02da\5v<\2\u02d9\u02d5\3\2\2\2\u02d9\u02d6\3\2\2\2\u02d9")
        buf.write("\u02d7\3\2\2\2\u02d9\u02d8\3\2\2\2\u02da%\3\2\2\2\u02db")
        buf.write("\u02dc\5l\67\2\u02dc\u02dd\5\u00d0i\2\u02dd\'\3\2\2\2")
        buf.write("\u02de\u02e0\5\u014e\u00a8\2\u02df\u02de\3\2\2\2\u02e0")
        buf.write("\u02e3\3\2\2\2\u02e1\u02df\3\2\2\2\u02e1\u02e2\3\2\2\2")
        buf.write("\u02e2\u02e7\3\2\2\2\u02e3\u02e1\3\2\2\2\u02e4\u02e6\7")
        buf.write("\7\2\2\u02e5\u02e4\3\2\2\2\u02e6\u02e9\3\2\2\2\u02e7\u02e5")
        buf.write("\3\2\2\2\u02e7\u02e8\3\2\2\2\u02e8\u02ea\3\2\2\2\u02e9")
        buf.write("\u02e7\3\2\2\2\u02ea\u02eb\5$\23\2\u02eb)\3\2\2\2\u02ec")
        buf.write("\u02ef\5l\67\2\u02ed\u02ef\5v<\2\u02ee\u02ec\3\2\2\2\u02ee")
        buf.write("\u02ed\3\2\2\2\u02ef\u02f3\3\2\2\2\u02f0\u02f2\7\7\2\2")
        buf.write("\u02f1\u02f0\3\2\2\2\u02f2\u02f5\3\2\2\2\u02f3\u02f1\3")
        buf.write("\2\2\2\u02f3\u02f4\3\2\2\2\u02f4\u02f6\3\2\2\2\u02f5\u02f3")
        buf.write("\3\2\2\2\u02f6\u02fa\7R\2\2\u02f7\u02f9\7\7\2\2\u02f8")
        buf.write("\u02f7\3\2\2\2\u02f9\u02fc\3\2\2\2\u02fa\u02f8\3\2\2\2")
        buf.write("\u02fa\u02fb\3\2\2\2\u02fb\u02fd\3\2\2\2\u02fc\u02fa\3")
        buf.write("\2\2\2\u02fd\u02fe\5\u0098M\2\u02fe+\3\2\2\2\u02ff\u0303")
        buf.write("\7\60\2\2\u0300\u0302\7\7\2\2\u0301\u0300\3\2\2\2\u0302")
        buf.write("\u0305\3\2\2\2\u0303\u0301\3\2\2\2\u0303\u0304\3\2\2\2")
        buf.write("\u0304\u0306\3\2\2\2\u0305\u0303\3\2\2\2\u0306\u0317\5")
        buf.write(".\30\2\u0307\u0309\7\7\2\2\u0308\u0307\3\2\2\2\u0309\u030c")
        buf.write("\3\2\2\2\u030a\u0308\3\2\2\2\u030a\u030b\3\2\2\2\u030b")
        buf.write("\u030d\3\2\2\2\u030c\u030a\3\2\2\2\u030d\u0311\7\n\2\2")
        buf.write("\u030e\u0310\7\7\2\2\u030f\u030e\3\2\2\2\u0310\u0313\3")
        buf.write("\2\2\2\u0311\u030f\3\2\2\2\u0311\u0312\3\2\2\2\u0312\u0314")
        buf.write("\3\2\2\2\u0313\u0311\3\2\2\2\u0314\u0316\5.\30\2\u0315")
        buf.write("\u030a\3\2\2\2\u0316\u0319\3\2\2\2\u0317\u0315\3\2\2\2")
        buf.write("\u0317\u0318\3\2\2\2\u0318\u0321\3\2\2\2\u0319\u0317\3")
        buf.write("\2\2\2\u031a\u031c\7\7\2\2\u031b\u031a\3\2\2\2\u031c\u031f")
        buf.write("\3\2\2\2\u031d\u031b\3\2\2\2\u031d\u031e\3\2\2\2\u031e")
        buf.write("\u0320\3\2\2\2\u031f\u031d\3\2\2\2\u0320\u0322\7\n\2\2")
        buf.write("\u0321\u031d\3\2\2\2\u0321\u0322\3\2\2\2\u0322\u0326\3")
        buf.write("\2\2\2\u0323\u0325\7\7\2\2\u0324\u0323\3\2\2\2\u0325\u0328")
        buf.write("\3\2\2\2\u0326\u0324\3\2\2\2\u0326\u0327\3\2\2\2\u0327")
        buf.write("\u0329\3\2\2\2\u0328\u0326\3\2\2\2\u0329\u032a\7\61\2")
        buf.write("\2\u032a-\3\2\2\2\u032b\u032d\5\u013e\u00a0\2\u032c\u032b")
        buf.write("\3\2\2\2\u032c\u032d\3\2\2\2\u032d\u0331\3\2\2\2\u032e")
        buf.write("\u0330\7\7\2\2\u032f\u032e\3\2\2\2\u0330\u0333\3\2\2\2")
        buf.write("\u0331\u032f\3\2\2\2\u0331\u0332\3\2\2\2\u0332\u0334\3")
        buf.write("\2\2\2\u0333\u0331\3\2\2\2\u0334\u0343\5\u0158\u00ad\2")
        buf.write("\u0335\u0337\7\7\2\2\u0336\u0335\3\2\2\2\u0337\u033a\3")
        buf.write("\2\2\2\u0338\u0336\3\2\2\2\u0338\u0339\3\2\2\2\u0339\u033b")
        buf.write("\3\2\2\2\u033a\u0338\3\2\2\2\u033b\u033f\7\34\2\2\u033c")
        buf.write("\u033e\7\7\2\2\u033d\u033c\3\2\2\2\u033e\u0341\3\2\2\2")
        buf.write("\u033f\u033d\3\2\2\2\u033f\u0340\3\2\2\2\u0340\u0342\3")
        buf.write("\2\2\2\u0341\u033f\3\2\2\2\u0342\u0344\5d\63\2\u0343\u0338")
        buf.write("\3\2\2\2\u0343\u0344\3\2\2\2\u0344/\3\2\2\2\u0345\u0349")
        buf.write("\7X\2\2\u0346\u0348\7\7\2\2\u0347\u0346\3\2\2\2\u0348")
        buf.write("\u034b\3\2\2\2\u0349\u0347\3\2\2\2\u0349\u034a\3\2\2\2")
        buf.write("\u034a\u034c\3\2\2\2\u034b\u0349\3\2\2\2\u034c\u035d\5")
        buf.write("\62\32\2\u034d\u034f\7\7\2\2\u034e\u034d\3\2\2\2\u034f")
        buf.write("\u0352\3\2\2\2\u0350\u034e\3\2\2\2\u0350\u0351\3\2\2\2")
        buf.write("\u0351\u0353\3\2\2\2\u0352\u0350\3\2\2\2\u0353\u0357\7")
        buf.write("\n\2\2\u0354\u0356\7\7\2\2\u0355\u0354\3\2\2\2\u0356\u0359")
        buf.write("\3\2\2\2\u0357\u0355\3\2\2\2\u0357\u0358\3\2\2\2\u0358")
        buf.write("\u035a\3\2\2\2\u0359\u0357\3\2\2\2\u035a\u035c\5\62\32")
        buf.write("\2\u035b\u0350\3\2\2\2\u035c\u035f\3\2\2\2\u035d\u035b")
        buf.write("\3\2\2\2\u035d\u035e\3\2\2\2\u035e\61\3\2\2\2\u035f\u035d")
        buf.write("\3\2\2\2\u0360\u0362\5\u014e\u00a8\2\u0361\u0360\3\2\2")
        buf.write("\2\u0362\u0365\3\2\2\2\u0363\u0361\3\2\2\2\u0363\u0364")
        buf.write("\3\2\2\2\u0364\u0366\3\2\2\2\u0365\u0363\3\2\2\2\u0366")
        buf.write("\u036a\5\u0158\u00ad\2\u0367\u0369\7\7\2\2\u0368\u0367")
        buf.write("\3\2\2\2\u0369\u036c\3\2\2\2\u036a\u0368\3\2\2\2\u036a")
        buf.write("\u036b\3\2\2\2\u036b\u036d\3\2\2\2\u036c\u036a\3\2\2\2")
        buf.write("\u036d\u0371\7\34\2\2\u036e\u0370\7\7\2\2\u036f\u036e")
        buf.write("\3\2\2\2\u0370\u0373\3\2\2\2\u0371\u036f\3\2\2\2\u0371")
        buf.write("\u0372\3\2\2\2\u0372\u0374\3\2\2\2\u0373\u0371\3\2\2\2")
        buf.write("\u0374\u0375\5d\63\2\u0375\63\3\2\2\2\u0376\u0378\5\66")
        buf.write("\34\2\u0377\u0379\5\u0096L\2\u0378\u0377\3\2\2\2\u0378")
        buf.write("\u0379\3\2\2\2\u0379\u037b\3\2\2\2\u037a\u0376\3\2\2\2")
        buf.write("\u037b\u037e\3\2\2\2\u037c\u037a\3\2\2\2\u037c\u037d\3")
        buf.write("\2\2\2\u037d\65\3\2\2\2\u037e\u037c\3\2\2\2\u037f\u0384")
        buf.write("\5\26\f\2\u0380\u0384\5:\36\2\u0381\u0384\58\35\2\u0382")
        buf.write("\u0384\5Z.\2\u0383\u037f\3\2\2\2\u0383\u0380\3\2\2\2\u0383")
        buf.write("\u0381\3\2\2\2\u0383\u0382\3\2\2\2\u0384\67\3\2\2\2\u0385")
        buf.write("\u0389\7T\2\2\u0386\u0388\7\7\2\2\u0387\u0386\3\2\2\2")
        buf.write("\u0388\u038b\3\2\2\2\u0389\u0387\3\2\2\2\u0389\u038a\3")
        buf.write("\2\2\2\u038a\u038c\3\2\2\2\u038b\u0389\3\2\2\2\u038c\u038d")
        buf.write("\5\u0088E\2\u038d9\3\2\2\2\u038e\u0390\5\u012c\u0097\2")
        buf.write("\u038f\u038e\3\2\2\2\u038f\u0390\3\2\2\2\u0390\u0391\3")
        buf.write("\2\2\2\u0391\u0395\7S\2\2\u0392\u0394\7\7\2\2\u0393\u0392")
        buf.write("\3\2\2\2\u0394\u0397\3\2\2\2\u0395\u0393\3\2\2\2\u0395")
        buf.write("\u0396\3\2\2\2\u0396\u0398\3\2\2\2\u0397\u0395\3\2\2\2")
        buf.write("\u0398\u03a0\7M\2\2\u0399\u039b\7\7\2\2\u039a\u0399\3")
        buf.write("\2\2\2\u039b\u039e\3\2\2\2\u039c\u039a\3\2\2\2\u039c\u039d")
        buf.write("\3\2\2\2\u039d\u039f\3\2\2\2\u039e\u039c\3\2\2\2\u039f")
        buf.write("\u03a1\5\u0158\u00ad\2\u03a0\u039c\3\2\2\2\u03a0\u03a1")
        buf.write("\3\2\2\2\u03a1\u03b0\3\2\2\2\u03a2\u03a4\7\7\2\2\u03a3")
        buf.write("\u03a2\3\2\2\2\u03a4\u03a7\3\2\2\2\u03a5\u03a3\3\2\2\2")
        buf.write("\u03a5\u03a6\3\2\2\2\u03a6\u03a8\3\2\2\2\u03a7\u03a5\3")
        buf.write("\2\2\2\u03a8\u03ac\7\34\2\2\u03a9\u03ab\7\7\2\2\u03aa")
        buf.write("\u03a9\3\2\2\2\u03ab\u03ae\3\2\2\2\u03ac\u03aa\3\2\2\2")
        buf.write("\u03ac\u03ad\3\2\2\2\u03ad\u03af\3\2\2\2\u03ae\u03ac\3")
        buf.write("\2\2\2\u03af\u03b1\5\"\22\2\u03b0\u03a5\3\2\2\2\u03b0")
        buf.write("\u03b1\3\2\2\2\u03b1\u03b9\3\2\2\2\u03b2\u03b4\7\7\2\2")
        buf.write("\u03b3\u03b2\3\2\2\2\u03b4\u03b7\3\2\2\2\u03b5\u03b3\3")
        buf.write("\2\2\2\u03b5\u03b6\3\2\2\2\u03b6\u03b8\3\2\2\2\u03b7\u03b5")
        buf.write("\3\2\2\2\u03b8\u03ba\5\34\17\2\u03b9\u03b5\3\2\2\2\u03b9")
        buf.write("\u03ba\3\2\2\2\u03ba;\3\2\2\2\u03bb\u03bf\7\13\2\2\u03bc")
        buf.write("\u03be\7\7\2\2\u03bd\u03bc\3\2\2\2\u03be\u03c1\3\2\2\2")
        buf.write("\u03bf\u03bd\3\2\2\2\u03bf\u03c0\3\2\2\2\u03c0\u03df\3")
        buf.write("\2\2\2\u03c1\u03bf\3\2\2\2\u03c2\u03d3\5> \2\u03c3\u03c5")
        buf.write("\7\7\2\2\u03c4\u03c3\3\2\2\2\u03c5\u03c8\3\2\2\2\u03c6")
        buf.write("\u03c4\3\2\2\2\u03c6\u03c7\3\2\2\2\u03c7\u03c9\3\2\2\2")
        buf.write("\u03c8\u03c6\3\2\2\2\u03c9\u03cd\7\n\2\2\u03ca\u03cc\7")
        buf.write("\7\2\2\u03cb\u03ca\3\2\2\2\u03cc\u03cf\3\2\2\2\u03cd\u03cb")
        buf.write("\3\2\2\2\u03cd\u03ce\3\2\2\2\u03ce\u03d0\3\2\2\2\u03cf")
        buf.write("\u03cd\3\2\2\2\u03d0\u03d2\5> \2\u03d1\u03c6\3\2\2\2\u03d2")
        buf.write("\u03d5\3\2\2\2\u03d3\u03d1\3\2\2\2\u03d3\u03d4\3\2\2\2")
        buf.write("\u03d4\u03dd\3\2\2\2\u03d5\u03d3\3\2\2\2\u03d6\u03d8\7")
        buf.write("\7\2\2\u03d7\u03d6\3\2\2\2\u03d8\u03db\3\2\2\2\u03d9\u03d7")
        buf.write("\3\2\2\2\u03d9\u03da\3\2\2\2\u03da\u03dc\3\2\2\2\u03db")
        buf.write("\u03d9\3\2\2\2\u03dc\u03de\7\n\2\2\u03dd\u03d9\3\2\2\2")
        buf.write("\u03dd\u03de\3\2\2\2\u03de\u03e0\3\2\2\2\u03df\u03c2\3")
        buf.write("\2\2\2\u03df\u03e0\3\2\2\2\u03e0\u03e4\3\2\2\2\u03e1\u03e3")
        buf.write("\7\7\2\2\u03e2\u03e1\3\2\2\2\u03e3\u03e6\3\2\2\2\u03e4")
        buf.write("\u03e2\3\2\2\2\u03e4\u03e5\3\2\2\2\u03e5\u03e7\3\2\2\2")
        buf.write("\u03e6\u03e4\3\2\2\2\u03e7\u03e8\7\f\2\2\u03e8=\3\2\2")
        buf.write("\2\u03e9\u03eb\5\u012e\u0098\2\u03ea\u03e9\3\2\2\2\u03ea")
        buf.write("\u03eb\3\2\2\2\u03eb\u03ec\3\2\2\2\u03ec\u03fb\5V,\2\u03ed")
        buf.write("\u03ef\7\7\2\2\u03ee\u03ed\3\2\2\2\u03ef\u03f2\3\2\2\2")
        buf.write("\u03f0\u03ee\3\2\2\2\u03f0\u03f1\3\2\2\2\u03f1\u03f3\3")
        buf.write("\2\2\2\u03f2\u03f0\3\2\2\2\u03f3\u03f7\7\36\2\2\u03f4")
        buf.write("\u03f6\7\7\2\2\u03f5\u03f4\3\2\2\2\u03f6\u03f9\3\2\2\2")
        buf.write("\u03f7\u03f5\3\2\2\2\u03f7\u03f8\3\2\2\2\u03f8\u03fa\3")
        buf.write("\2\2\2\u03f9\u03f7\3\2\2\2\u03fa\u03fc\5\u0098M\2\u03fb")
        buf.write("\u03f0\3\2\2\2\u03fb\u03fc\3\2\2\2\u03fc?\3\2\2\2\u03fd")
        buf.write("\u03ff\5\u012c\u0097\2\u03fe\u03fd\3\2\2\2\u03fe\u03ff")
        buf.write("\3\2\2\2\u03ff\u0400\3\2\2\2\u0400\u0408\7L\2\2\u0401")
        buf.write("\u0403\7\7\2\2\u0402\u0401\3\2\2\2\u0403\u0406\3\2\2\2")
        buf.write("\u0404\u0402\3\2\2\2\u0404\u0405\3\2\2\2\u0405\u0407\3")
        buf.write("\2\2\2\u0406\u0404\3\2\2\2\u0407\u0409\5,\27\2\u0408\u0404")
        buf.write("\3\2\2\2\u0408\u0409\3\2\2\2\u0409\u0419\3\2\2\2\u040a")
        buf.write("\u040c\7\7\2\2\u040b\u040a\3\2\2\2\u040c\u040f\3\2\2\2")
        buf.write("\u040d\u040b\3\2\2\2\u040d\u040e\3\2\2\2\u040e\u0410\3")
        buf.write("\2\2\2\u040f\u040d\3\2\2\2\u0410\u0414\5|?\2\u0411\u0413")
        buf.write("\7\7\2\2\u0412\u0411\3\2\2\2\u0413\u0416\3\2\2\2\u0414")
        buf.write("\u0412\3\2\2\2\u0414\u0415\3\2\2\2\u0415\u0417\3\2\2\2")
        buf.write("\u0416\u0414\3\2\2\2\u0417\u0418\7\t\2\2\u0418\u041a\3")
        buf.write("\2\2\2\u0419\u040d\3\2\2\2\u0419\u041a\3\2\2\2\u041a\u041e")
        buf.write("\3\2\2\2\u041b\u041d\7\7\2\2\u041c\u041b\3\2\2\2\u041d")
        buf.write("\u0420\3\2\2\2\u041e\u041c\3\2\2\2\u041e\u041f\3\2\2\2")
        buf.write("\u041f\u0421\3\2\2\2\u0420\u041e\3\2\2\2\u0421\u0425\5")
        buf.write("\u0158\u00ad\2\u0422\u0424\7\7\2\2\u0423\u0422\3\2\2\2")
        buf.write("\u0424\u0427\3\2\2\2\u0425\u0423\3\2\2\2\u0425\u0426\3")
        buf.write("\2\2\2\u0426\u0428\3\2\2\2\u0427\u0425\3\2\2\2\u0428\u0437")
        buf.write("\5<\37\2\u0429\u042b\7\7\2\2\u042a\u0429\3\2\2\2\u042b")
        buf.write("\u042e\3\2\2\2\u042c\u042a\3\2\2\2\u042c\u042d\3\2\2\2")
        buf.write("\u042d\u042f\3\2\2\2\u042e\u042c\3\2\2\2\u042f\u0433\7")
        buf.write("\34\2\2\u0430\u0432\7\7\2\2\u0431\u0430\3\2\2\2\u0432")
        buf.write("\u0435\3\2\2\2\u0433\u0431\3\2\2\2\u0433\u0434\3\2\2\2")
        buf.write("\u0434\u0436\3\2\2\2\u0435\u0433\3\2\2\2\u0436\u0438\5")
        buf.write("d\63\2\u0437\u042c\3\2\2\2\u0437\u0438\3\2\2\2\u0438\u0440")
        buf.write("\3\2\2\2\u0439\u043b\7\7\2\2\u043a\u0439\3\2\2\2\u043b")
        buf.write("\u043e\3\2\2\2\u043c\u043a\3\2\2\2\u043c\u043d\3\2\2\2")
        buf.write("\u043d\u043f\3\2\2\2\u043e\u043c\3\2\2\2\u043f\u0441\5")
        buf.write("\60\31\2\u0440\u043c\3\2\2\2\u0440\u0441\3\2\2\2\u0441")
        buf.write("\u0449\3\2\2\2\u0442\u0444\7\7\2\2\u0443\u0442\3\2\2\2")
        buf.write("\u0444\u0447\3\2\2\2\u0445\u0443\3\2\2\2\u0445\u0446\3")
        buf.write("\2\2\2\u0446\u0448\3\2\2\2\u0447\u0445\3\2\2\2\u0448\u044a")
        buf.write("\5B\"\2\u0449\u0445\3\2\2\2\u0449\u044a\3\2\2\2\u044a")
        buf.write("A\3\2\2\2\u044b\u0455\5\u0088E\2\u044c\u0450\7\36\2\2")
        buf.write("\u044d\u044f\7\7\2\2\u044e\u044d\3\2\2\2\u044f\u0452\3")
        buf.write("\2\2\2\u0450\u044e\3\2\2\2\u0450\u0451\3\2\2\2\u0451\u0453")
        buf.write("\3\2\2\2\u0452\u0450\3\2\2\2\u0453\u0455\5\u0098M\2\u0454")
        buf.write("\u044b\3\2\2\2\u0454\u044c\3\2\2\2\u0455C\3\2\2\2\u0456")
        buf.write("\u0458\5\u014e\u00a8\2\u0457\u0456\3\2\2\2\u0458\u045b")
        buf.write("\3\2\2\2\u0459\u0457\3\2\2\2\u0459\u045a\3\2\2\2\u045a")
        buf.write("\u045f\3\2\2\2\u045b\u0459\3\2\2\2\u045c\u045e\7\7\2\2")
        buf.write("\u045d\u045c\3\2\2\2\u045e\u0461\3\2\2\2\u045f\u045d\3")
        buf.write("\2\2\2\u045f\u0460\3\2\2\2\u0460\u0462\3\2\2\2\u0461\u045f")
        buf.write("\3\2\2\2\u0462\u0471\5\u0158\u00ad\2\u0463\u0465\7\7\2")
        buf.write("\2\u0464\u0463\3\2\2\2\u0465\u0468\3\2\2\2\u0466\u0464")
        buf.write("\3\2\2\2\u0466\u0467\3\2\2\2\u0467\u0469\3\2\2\2\u0468")
        buf.write("\u0466\3\2\2\2\u0469\u046d\7\34\2\2\u046a\u046c\7\7\2")
        buf.write("\2\u046b\u046a\3\2\2\2\u046c\u046f\3\2\2\2\u046d\u046b")
        buf.write("\3\2\2\2\u046d\u046e\3\2\2\2\u046e\u0470\3\2\2\2\u046f")
        buf.write("\u046d\3\2\2\2\u0470\u0472\5d\63\2\u0471\u0466\3\2\2\2")
        buf.write("\u0471\u0472\3\2\2\2\u0472E\3\2\2\2\u0473\u0477\7\13\2")
        buf.write("\2\u0474\u0476\7\7\2\2\u0475\u0474\3\2\2\2\u0476\u0479")
        buf.write("\3\2\2\2\u0477\u0475\3\2\2\2\u0477\u0478\3\2\2\2\u0478")
        buf.write("\u047a\3\2\2\2\u0479\u0477\3\2\2\2\u047a\u048b\5D#\2\u047b")
        buf.write("\u047d\7\7\2\2\u047c\u047b\3\2\2\2\u047d\u0480\3\2\2\2")
        buf.write("\u047e\u047c\3\2\2\2\u047e\u047f\3\2\2\2\u047f\u0481\3")
        buf.write("\2\2\2\u0480\u047e\3\2\2\2\u0481\u0485\7\n\2\2\u0482\u0484")
        buf.write("\7\7\2\2\u0483\u0482\3\2\2\2\u0484\u0487\3\2\2\2\u0485")
        buf.write("\u0483\3\2\2\2\u0485\u0486\3\2\2\2\u0486\u0488\3\2\2\2")
        buf.write("\u0487\u0485\3\2\2\2\u0488\u048a\5D#\2\u0489\u047e\3\2")
        buf.write("\2\2\u048a\u048d\3\2\2\2\u048b\u0489\3\2\2\2\u048b\u048c")
        buf.write("\3\2\2\2\u048c\u0495\3\2\2\2\u048d\u048b\3\2\2\2\u048e")
        buf.write("\u0490\7\7\2\2\u048f\u048e\3\2\2\2\u0490\u0493\3\2\2\2")
        buf.write("\u0491\u048f\3\2\2\2\u0491\u0492\3\2\2\2\u0492\u0494\3")
        buf.write("\2\2\2\u0493\u0491\3\2\2\2\u0494\u0496\7\n\2\2\u0495\u0491")
        buf.write("\3\2\2\2\u0495\u0496\3\2\2\2\u0496\u049a\3\2\2\2\u0497")
        buf.write("\u0499\7\7\2\2\u0498\u0497\3\2\2\2\u0499\u049c\3\2\2\2")
        buf.write("\u049a\u0498\3\2\2\2\u049a\u049b\3\2\2\2\u049b\u049d\3")
        buf.write("\2\2\2\u049c\u049a\3\2\2\2\u049d\u049e\7\f\2\2\u049eG")
        buf.write("\3\2\2\2\u049f\u04a1\5\u012c\u0097\2\u04a0\u049f\3\2\2")
        buf.write("\2\u04a0\u04a1\3\2\2\2\u04a1\u04a2\3\2\2\2\u04a2\u04aa")
        buf.write("\t\3\2\2\u04a3\u04a5\7\7\2\2\u04a4\u04a3\3\2\2\2\u04a5")
        buf.write("\u04a8\3\2\2\2\u04a6\u04a4\3\2\2\2\u04a6\u04a7\3\2\2\2")
        buf.write("\u04a7\u04a9\3\2\2\2\u04a8\u04a6\3\2\2\2\u04a9\u04ab\5")
        buf.write(",\27\2\u04aa\u04a6\3\2\2\2\u04aa\u04ab\3\2\2\2\u04ab\u04bb")
        buf.write("\3\2\2\2\u04ac\u04ae\7\7\2\2\u04ad\u04ac\3\2\2\2\u04ae")
        buf.write("\u04b1\3\2\2\2\u04af\u04ad\3\2\2\2\u04af\u04b0\3\2\2\2")
        buf.write("\u04b0\u04b2\3\2\2\2\u04b1\u04af\3\2\2\2\u04b2\u04b6\5")
        buf.write("|?\2\u04b3\u04b5\7\7\2\2\u04b4\u04b3\3\2\2\2\u04b5\u04b8")
        buf.write("\3\2\2\2\u04b6\u04b4\3\2\2\2\u04b6\u04b7\3\2\2\2\u04b7")
        buf.write("\u04b9\3\2\2\2\u04b8\u04b6\3\2\2\2\u04b9\u04ba\7\t\2\2")
        buf.write("\u04ba\u04bc\3\2\2\2\u04bb\u04af\3\2\2\2\u04bb\u04bc\3")
        buf.write("\2\2\2\u04bc\u04c0\3\2\2\2\u04bd\u04bf\7\7\2\2\u04be\u04bd")
        buf.write("\3\2\2\2\u04bf\u04c2\3\2\2\2\u04c0\u04be\3\2\2\2\u04c0")
        buf.write("\u04c1\3\2\2\2\u04c1\u04c5\3\2\2\2\u04c2\u04c0\3\2\2\2")
        buf.write("\u04c3\u04c6\5F$\2\u04c4\u04c6\5D#\2\u04c5\u04c3\3\2\2")
        buf.write("\2\u04c5\u04c4\3\2\2\2\u04c6\u04ce\3\2\2\2\u04c7\u04c9")
        buf.write("\7\7\2\2\u04c8\u04c7\3\2\2\2\u04c9\u04cc\3\2\2\2\u04ca")
        buf.write("\u04c8\3\2\2\2\u04ca\u04cb\3\2\2\2\u04cb\u04cd\3\2\2\2")
        buf.write("\u04cc\u04ca\3\2\2\2\u04cd\u04cf\5\60\31\2\u04ce\u04ca")
        buf.write("\3\2\2\2\u04ce\u04cf\3\2\2\2\u04cf\u04e1\3\2\2\2\u04d0")
        buf.write("\u04d2\7\7\2\2\u04d1\u04d0\3\2\2\2\u04d2\u04d5\3\2\2\2")
        buf.write("\u04d3\u04d1\3\2\2\2\u04d3\u04d4\3\2\2\2\u04d4\u04df\3")
        buf.write("\2\2\2\u04d5\u04d3\3\2\2\2\u04d6\u04da\7\36\2\2\u04d7")
        buf.write("\u04d9\7\7\2\2\u04d8\u04d7\3\2\2\2\u04d9\u04dc\3\2\2\2")
        buf.write("\u04da\u04d8\3\2\2\2\u04da\u04db\3\2\2\2\u04db\u04dd\3")
        buf.write("\2\2\2\u04dc\u04da\3\2\2\2\u04dd\u04e0\5\u0098M\2\u04de")
        buf.write("\u04e0\5J&\2\u04df\u04d6\3\2\2\2\u04df\u04de\3\2\2\2\u04e0")
        buf.write("\u04e2\3\2\2\2\u04e1\u04d3\3\2\2\2\u04e1\u04e2\3\2\2\2")
        buf.write("\u04e2\u04e9\3\2\2\2\u04e3\u04e5\7\7\2\2\u04e4\u04e3\3")
        buf.write("\2\2\2\u04e5\u04e6\3\2\2\2\u04e6\u04e4\3\2\2\2\u04e6\u04e7")
        buf.write("\3\2\2\2\u04e7\u04e8\3\2\2\2\u04e8\u04ea\7\35\2\2\u04e9")
        buf.write("\u04e4\3\2\2\2\u04e9\u04ea\3\2\2\2\u04ea\u04ee\3\2\2\2")
        buf.write("\u04eb\u04ed\7\7\2\2\u04ec\u04eb\3\2\2\2\u04ed\u04f0\3")
        buf.write("\2\2\2\u04ee\u04ec\3\2\2\2\u04ee\u04ef\3\2\2\2\u04ef\u050f")
        buf.write("\3\2\2\2\u04f0\u04ee\3\2\2\2\u04f1\u04f3\5L\'\2\u04f2")
        buf.write("\u04f1\3\2\2\2\u04f2\u04f3\3\2\2\2\u04f3\u04fe\3\2\2\2")
        buf.write("\u04f4\u04f6\7\7\2\2\u04f5\u04f4\3\2\2\2\u04f6\u04f9\3")
        buf.write("\2\2\2\u04f7\u04f5\3\2\2\2\u04f7\u04f8\3\2\2\2\u04f8\u04fb")
        buf.write("\3\2\2\2\u04f9\u04f7\3\2\2\2\u04fa\u04fc\5\u0094K\2\u04fb")
        buf.write("\u04fa\3\2\2\2\u04fb\u04fc\3\2\2\2\u04fc\u04fd\3\2\2\2")
        buf.write("\u04fd\u04ff\5N(\2\u04fe\u04f7\3\2\2\2\u04fe\u04ff\3\2")
        buf.write("\2\2\u04ff\u0510\3\2\2\2\u0500\u0502\5N(\2\u0501\u0500")
        buf.write("\3\2\2\2\u0501\u0502\3\2\2\2\u0502\u050d\3\2\2\2\u0503")
        buf.write("\u0505\7\7\2\2\u0504\u0503\3\2\2\2\u0505\u0508\3\2\2\2")
        buf.write("\u0506\u0504\3\2\2\2\u0506\u0507\3\2\2\2\u0507\u050a\3")
        buf.write("\2\2\2\u0508\u0506\3\2\2\2\u0509\u050b\5\u0094K\2\u050a")
        buf.write("\u0509\3\2\2\2\u050a\u050b\3\2\2\2\u050b\u050c\3\2\2\2")
        buf.write("\u050c\u050e\5L\'\2\u050d\u0506\3\2\2\2\u050d\u050e\3")
        buf.write("\2\2\2\u050e\u0510\3\2\2\2\u050f\u04f2\3\2\2\2\u050f\u0501")
        buf.write("\3\2\2\2\u0510I\3\2\2\2\u0511\u0515\7R\2\2\u0512\u0514")
        buf.write("\7\7\2\2\u0513\u0512\3\2\2\2\u0514\u0517\3\2\2\2\u0515")
        buf.write("\u0513\3\2\2\2\u0515\u0516\3\2\2\2\u0516\u0518\3\2\2\2")
        buf.write("\u0517\u0515\3\2\2\2\u0518\u0519\5\u0098M\2\u0519K\3\2")
        buf.write("\2\2\u051a\u051c\5\u012c\u0097\2\u051b\u051a\3\2\2\2\u051b")
        buf.write("\u051c\3\2\2\2\u051c\u051d\3\2\2\2\u051d\u0543\7B\2\2")
        buf.write("\u051e\u0520\7\7\2\2\u051f\u051e\3\2\2\2\u0520\u0523\3")
        buf.write("\2\2\2\u0521\u051f\3\2\2\2\u0521\u0522\3\2\2\2\u0522\u0524")
        buf.write("\3\2\2\2\u0523\u0521\3\2\2\2\u0524\u0528\7\13\2\2\u0525")
        buf.write("\u0527\7\7\2\2\u0526\u0525\3\2\2\2\u0527\u052a\3\2\2\2")
        buf.write("\u0528\u0526\3\2\2\2\u0528\u0529\3\2\2\2\u0529\u052b\3")
        buf.write("\2\2\2\u052a\u0528\3\2\2\2\u052b\u053a\7\f\2\2\u052c\u052e")
        buf.write("\7\7\2\2\u052d\u052c\3\2\2\2\u052e\u0531\3\2\2\2\u052f")
        buf.write("\u052d\3\2\2\2\u052f\u0530\3\2\2\2\u0530\u0532\3\2\2\2")
        buf.write("\u0531\u052f\3\2\2\2\u0532\u0536\7\34\2\2\u0533\u0535")
        buf.write("\7\7\2\2\u0534\u0533\3\2\2\2\u0535\u0538\3\2\2\2\u0536")
        buf.write("\u0534\3\2\2\2\u0536\u0537\3\2\2\2\u0537\u0539\3\2\2\2")
        buf.write("\u0538\u0536\3\2\2\2\u0539\u053b\5d\63\2\u053a\u052f\3")
        buf.write("\2\2\2\u053a\u053b\3\2\2\2\u053b\u053f\3\2\2\2\u053c\u053e")
        buf.write("\7\7\2\2\u053d\u053c\3\2\2\2\u053e\u0541\3\2\2\2\u053f")
        buf.write("\u053d\3\2\2\2\u053f\u0540\3\2\2\2\u0540\u0542\3\2\2\2")
        buf.write("\u0541\u053f\3\2\2\2\u0542\u0544\5B\"\2\u0543\u0521\3")
        buf.write("\2\2\2\u0543\u0544\3\2\2\2\u0544M\3\2\2\2\u0545\u0547")
        buf.write("\5\u012c\u0097\2\u0546\u0545\3\2\2\2\u0546\u0547\3\2\2")
        buf.write("\2\u0547\u0548\3\2\2\2\u0548\u057f\7C\2\2\u0549\u054b")
        buf.write("\7\7\2\2\u054a\u0549\3\2\2\2\u054b\u054e\3\2\2\2\u054c")
        buf.write("\u054a\3\2\2\2\u054c\u054d\3\2\2\2\u054d\u054f\3\2\2\2")
        buf.write("\u054e\u054c\3\2\2\2\u054f\u0553\7\13\2\2\u0550\u0552")
        buf.write("\7\7\2\2\u0551\u0550\3\2\2\2\u0552\u0555\3\2\2\2\u0553")
        buf.write("\u0551\3\2\2\2\u0553\u0554\3\2\2\2\u0554\u0556\3\2\2\2")
        buf.write("\u0555\u0553\3\2\2\2\u0556\u055e\5R*\2\u0557\u0559\7\7")
        buf.write("\2\2\u0558\u0557\3\2\2\2\u0559\u055c\3\2\2\2\u055a\u0558")
        buf.write("\3\2\2\2\u055a\u055b\3\2\2\2\u055b\u055d\3\2\2\2\u055c")
        buf.write("\u055a\3\2\2\2\u055d\u055f\7\n\2\2\u055e\u055a\3\2\2\2")
        buf.write("\u055e\u055f\3\2\2\2\u055f\u0563\3\2\2\2\u0560\u0562\7")
        buf.write("\7\2\2\u0561\u0560\3\2\2\2\u0562\u0565\3\2\2\2\u0563\u0561")
        buf.write("\3\2\2\2\u0563\u0564\3\2\2\2\u0564\u0566\3\2\2\2\u0565")
        buf.write("\u0563\3\2\2\2\u0566\u0575\7\f\2\2\u0567\u0569\7\7\2\2")
        buf.write("\u0568\u0567\3\2\2\2\u0569\u056c\3\2\2\2\u056a\u0568\3")
        buf.write("\2\2\2\u056a\u056b\3\2\2\2\u056b\u056d\3\2\2\2\u056c\u056a")
        buf.write("\3\2\2\2\u056d\u0571\7\34\2\2\u056e\u0570\7\7\2\2\u056f")
        buf.write("\u056e\3\2\2\2\u0570\u0573\3\2\2\2\u0571\u056f\3\2\2\2")
        buf.write("\u0571\u0572\3\2\2\2\u0572\u0574\3\2\2\2\u0573\u0571\3")
        buf.write("\2\2\2\u0574\u0576\5d\63\2\u0575\u056a\3\2\2\2\u0575\u0576")
        buf.write("\3\2\2\2\u0576\u057a\3\2\2\2\u0577\u0579\7\7\2\2\u0578")
        buf.write("\u0577\3\2\2\2\u0579\u057c\3\2\2\2\u057a\u0578\3\2\2\2")
        buf.write("\u057a\u057b\3\2\2\2\u057b\u057d\3\2\2\2\u057c\u057a\3")
        buf.write("\2\2\2\u057d\u057e\5B\"\2\u057e\u0580\3\2\2\2\u057f\u054c")
        buf.write("\3\2\2\2\u057f\u0580\3\2\2\2\u0580O\3\2\2\2\u0581\u0585")
        buf.write("\7\13\2\2\u0582\u0584\7\7\2\2\u0583\u0582\3\2\2\2\u0584")
        buf.write("\u0587\3\2\2\2\u0585\u0583\3\2\2\2\u0585\u0586\3\2\2\2")
        buf.write("\u0586\u05a5\3\2\2\2\u0587\u0585\3\2\2\2\u0588\u0599\5")
        buf.write("R*\2\u0589\u058b\7\7\2\2\u058a\u0589\3\2\2\2\u058b\u058e")
        buf.write("\3\2\2\2\u058c\u058a\3\2\2\2\u058c\u058d\3\2\2\2\u058d")
        buf.write("\u058f\3\2\2\2\u058e\u058c\3\2\2\2\u058f\u0593\7\n\2\2")
        buf.write("\u0590\u0592\7\7\2\2\u0591\u0590\3\2\2\2\u0592\u0595\3")
        buf.write("\2\2\2\u0593\u0591\3\2\2\2\u0593\u0594\3\2\2\2\u0594\u0596")
        buf.write("\3\2\2\2\u0595\u0593\3\2\2\2\u0596\u0598\5R*\2\u0597\u058c")
        buf.write("\3\2\2\2\u0598\u059b\3\2\2\2\u0599\u0597\3\2\2\2\u0599")
        buf.write("\u059a\3\2\2\2\u059a\u05a3\3\2\2\2\u059b\u0599\3\2\2\2")
        buf.write("\u059c\u059e\7\7\2\2\u059d\u059c\3\2\2\2\u059e\u05a1\3")
        buf.write("\2\2\2\u059f\u059d\3\2\2\2\u059f\u05a0\3\2\2\2\u05a0\u05a2")
        buf.write("\3\2\2\2\u05a1\u059f\3\2\2\2\u05a2\u05a4\7\n\2\2\u05a3")
        buf.write("\u059f\3\2\2\2\u05a3\u05a4\3\2\2\2\u05a4\u05a6\3\2\2\2")
        buf.write("\u05a5\u0588\3\2\2\2\u05a5\u05a6\3\2\2\2\u05a6\u05aa\3")
        buf.write("\2\2\2\u05a7\u05a9\7\7\2\2\u05a8\u05a7\3\2\2\2\u05a9\u05ac")
        buf.write("\3\2\2\2\u05aa\u05a8\3\2\2\2\u05aa\u05ab\3\2\2\2\u05ab")
        buf.write("\u05ad\3\2\2\2\u05ac\u05aa\3\2\2\2\u05ad\u05ae\7\f\2\2")
        buf.write("\u05aeQ\3\2\2\2\u05af\u05b1\5\u012e\u0098\2\u05b0\u05af")
        buf.write("\3\2\2\2\u05b0\u05b1\3\2\2\2\u05b1\u05b2\3\2\2\2\u05b2")
        buf.write("\u05c1\5T+\2\u05b3\u05b5\7\7\2\2\u05b4\u05b3\3\2\2\2\u05b5")
        buf.write("\u05b8\3\2\2\2\u05b6\u05b4\3\2\2\2\u05b6\u05b7\3\2\2\2")
        buf.write("\u05b7\u05b9\3\2\2\2\u05b8\u05b6\3\2\2\2\u05b9\u05bd\7")
        buf.write("\36\2\2\u05ba\u05bc\7\7\2\2\u05bb\u05ba\3\2\2\2\u05bc")
        buf.write("\u05bf\3\2\2\2\u05bd\u05bb\3\2\2\2\u05bd\u05be\3\2\2\2")
        buf.write("\u05be\u05c0\3\2\2\2\u05bf\u05bd\3\2\2\2\u05c0\u05c2\5")
        buf.write("\u0098M\2\u05c1\u05b6\3\2\2\2\u05c1\u05c2\3\2\2\2\u05c2")
        buf.write("S\3\2\2\2\u05c3\u05c7\5\u0158\u00ad\2\u05c4\u05c6\7\7")
        buf.write("\2\2\u05c5\u05c4\3\2\2\2\u05c6\u05c9\3\2\2\2\u05c7\u05c5")
        buf.write("\3\2\2\2\u05c7\u05c8\3\2\2\2\u05c8\u05d2\3\2\2\2\u05c9")
        buf.write("\u05c7\3\2\2\2\u05ca\u05ce\7\34\2\2\u05cb\u05cd\7\7\2")
        buf.write("\2\u05cc\u05cb\3\2\2\2\u05cd\u05d0\3\2\2\2\u05ce\u05cc")
        buf.write("\3\2\2\2\u05ce\u05cf\3\2\2\2\u05cf\u05d1\3\2\2\2\u05d0")
        buf.write("\u05ce\3\2\2\2\u05d1\u05d3\5d\63\2\u05d2\u05ca\3\2\2\2")
        buf.write("\u05d2\u05d3\3\2\2\2\u05d3U\3\2\2\2\u05d4\u05d8\5\u0158")
        buf.write("\u00ad\2\u05d5\u05d7\7\7\2\2\u05d6\u05d5\3\2\2\2\u05d7")
        buf.write("\u05da\3\2\2\2\u05d8\u05d6\3\2\2\2\u05d8\u05d9\3\2\2\2")
        buf.write("\u05d9\u05db\3\2\2\2\u05da\u05d8\3\2\2\2\u05db\u05df\7")
        buf.write("\34\2\2\u05dc\u05de\7\7\2\2\u05dd\u05dc\3\2\2\2\u05de")
        buf.write("\u05e1\3\2\2\2\u05df\u05dd\3\2\2\2\u05df\u05e0\3\2\2\2")
        buf.write("\u05e0\u05e2\3\2\2\2\u05e1\u05df\3\2\2\2\u05e2\u05e3\5")
        buf.write("d\63\2\u05e3W\3\2\2\2\u05e4\u05e6\5\u012c\u0097\2\u05e5")
        buf.write("\u05e4\3\2\2\2\u05e5\u05e6\3\2\2\2\u05e6\u05e7\3\2\2\2")
        buf.write("\u05e7\u05eb\7M\2\2\u05e8\u05ea\7\7\2\2\u05e9\u05e8\3")
        buf.write("\2\2\2\u05ea\u05ed\3\2\2\2\u05eb\u05e9\3\2\2\2\u05eb\u05ec")
        buf.write("\3\2\2\2\u05ec\u05ee\3\2\2\2\u05ed\u05eb\3\2\2\2\u05ee")
        buf.write("\u05fd\5\u0158\u00ad\2\u05ef\u05f1\7\7\2\2\u05f0\u05ef")
        buf.write("\3\2\2\2\u05f1\u05f4\3\2\2\2\u05f2\u05f0\3\2\2\2\u05f2")
        buf.write("\u05f3\3\2\2\2\u05f3\u05f5\3\2\2\2\u05f4\u05f2\3\2\2\2")
        buf.write("\u05f5\u05f9\7\34\2\2\u05f6\u05f8\7\7\2\2\u05f7\u05f6")
        buf.write("\3\2\2\2\u05f8\u05fb\3\2\2\2\u05f9\u05f7\3\2\2\2\u05f9")
        buf.write("\u05fa\3\2\2\2\u05fa\u05fc\3\2\2\2\u05fb\u05f9\3\2\2\2")
        buf.write("\u05fc\u05fe\5\"\22\2\u05fd\u05f2\3\2\2\2\u05fd\u05fe")
        buf.write("\3\2\2\2\u05fe\u0606\3\2\2\2\u05ff\u0601\7\7\2\2\u0600")
        buf.write("\u05ff\3\2\2\2\u0601\u0604\3\2\2\2\u0602\u0600\3\2\2\2")
        buf.write("\u0602\u0603\3\2\2\2\u0603\u0605\3\2\2\2\u0604\u0602\3")
        buf.write("\2\2\2\u0605\u0607\5\34\17\2\u0606\u0602\3\2\2\2\u0606")
        buf.write("\u0607\3\2\2\2\u0607Y\3\2\2\2\u0608\u060a\5\u012c\u0097")
        buf.write("\2\u0609\u0608\3\2\2\2\u0609\u060a\3\2\2\2\u060a\u060b")
        buf.write("\3\2\2\2\u060b\u060f\7Q\2\2\u060c\u060e\7\7\2\2\u060d")
        buf.write("\u060c\3\2\2\2\u060e\u0611\3\2\2\2\u060f\u060d\3\2\2\2")
        buf.write("\u060f\u0610\3\2\2\2\u0610\u0612\3\2\2\2\u0611\u060f\3")
        buf.write("\2\2\2\u0612\u0621\5<\37\2\u0613\u0615\7\7\2\2\u0614\u0613")
        buf.write("\3\2\2\2\u0615\u0618\3\2\2\2\u0616\u0614\3\2\2\2\u0616")
        buf.write("\u0617\3\2\2\2\u0617\u0619\3\2\2\2\u0618\u0616\3\2\2\2")
        buf.write("\u0619\u061d\7\34\2\2\u061a\u061c\7\7\2\2\u061b\u061a")
        buf.write("\3\2\2\2\u061c\u061f\3\2\2\2\u061d\u061b\3\2\2\2\u061d")
        buf.write("\u061e\3\2\2\2\u061e\u0620\3\2\2\2\u061f\u061d\3\2\2\2")
        buf.write("\u0620\u0622\5\\/\2\u0621\u0616\3\2\2\2\u0621\u0622\3")
        buf.write("\2\2\2\u0622\u0626\3\2\2\2\u0623\u0625\7\7\2\2\u0624\u0623")
        buf.write("\3\2\2\2\u0625\u0628\3\2\2\2\u0626\u0624\3\2\2\2\u0626")
        buf.write("\u0627\3\2\2\2\u0627\u062a\3\2\2\2\u0628\u0626\3\2\2\2")
        buf.write("\u0629\u062b\5\u0088E\2\u062a\u0629\3\2\2\2\u062a\u062b")
        buf.write("\3\2\2\2\u062b[\3\2\2\2\u062c\u0630\t\4\2\2\u062d\u062f")
        buf.write("\7\7\2\2\u062e\u062d\3\2\2\2\u062f\u0632\3\2\2\2\u0630")
        buf.write("\u062e\3\2\2\2\u0630\u0631\3\2\2\2\u0631\u0633\3\2\2\2")
        buf.write("\u0632\u0630\3\2\2\2\u0633\u0634\5\u00d0i\2\u0634]\3\2")
        buf.write("\2\2\u0635\u0639\7\17\2\2\u0636\u0638\7\7\2\2\u0637\u0636")
        buf.write("\3\2\2\2\u0638\u063b\3\2\2\2\u0639\u0637\3\2\2\2\u0639")
        buf.write("\u063a\3\2\2\2\u063a\u063d\3\2\2\2\u063b\u0639\3\2\2\2")
        buf.write("\u063c\u063e\5`\61\2\u063d\u063c\3\2\2\2\u063d\u063e\3")
        buf.write("\2\2\2\u063e\u064d\3\2\2\2\u063f\u0641\7\7\2\2\u0640\u063f")
        buf.write("\3\2\2\2\u0641\u0644\3\2\2\2\u0642\u0640\3\2\2\2\u0642")
        buf.write("\u0643\3\2\2\2\u0643\u0645\3\2\2\2\u0644\u0642\3\2\2\2")
        buf.write("\u0645\u0649\7\35\2\2\u0646\u0648\7\7\2\2\u0647\u0646")
        buf.write("\3\2\2\2\u0648\u064b\3\2\2\2\u0649\u0647\3\2\2\2\u0649")
        buf.write("\u064a\3\2\2\2\u064a\u064c\3\2\2\2\u064b\u0649\3\2\2\2")
        buf.write("\u064c\u064e\5\64\33\2\u064d\u0642\3\2\2\2\u064d\u064e")
        buf.write("\3\2\2\2\u064e\u0652\3\2\2\2\u064f\u0651\7\7\2\2\u0650")
        buf.write("\u064f\3\2\2\2\u0651\u0654\3\2\2\2\u0652\u0650\3\2\2\2")
        buf.write("\u0652\u0653\3\2\2\2\u0653\u0655\3\2\2\2\u0654\u0652\3")
        buf.write("\2\2\2\u0655\u0656\7\20\2\2\u0656_\3\2\2\2\u0657\u0668")
        buf.write("\5b\62\2\u0658\u065a\7\7\2\2\u0659\u0658\3\2\2\2\u065a")
        buf.write("\u065d\3\2\2\2\u065b\u0659\3\2\2\2\u065b\u065c\3\2\2\2")
        buf.write("\u065c\u065e\3\2\2\2\u065d\u065b\3\2\2\2\u065e\u0662\7")
        buf.write("\n\2\2\u065f\u0661\7\7\2\2\u0660\u065f\3\2\2\2\u0661\u0664")
        buf.write("\3\2\2\2\u0662\u0660\3\2\2\2\u0662\u0663\3\2\2\2\u0663")
        buf.write("\u0665\3\2\2\2\u0664\u0662\3\2\2\2\u0665\u0667\5b\62\2")
        buf.write("\u0666\u065b\3\2\2\2\u0667\u066a\3\2\2\2\u0668\u0666\3")
        buf.write("\2\2\2\u0668\u0669\3\2\2\2\u0669\u066e\3\2\2\2\u066a\u0668")
        buf.write("\3\2\2\2\u066b\u066d\7\7\2\2\u066c\u066b\3\2\2\2\u066d")
        buf.write("\u0670\3\2\2\2\u066e\u066c\3\2\2\2\u066e\u066f\3\2\2\2")
        buf.write("\u066f\u0672\3\2\2\2\u0670\u066e\3\2\2\2\u0671\u0673\7")
        buf.write("\n\2\2\u0672\u0671\3\2\2\2\u0672\u0673\3\2\2\2\u0673a")
        buf.write("\3\2\2\2\u0674\u0678\5\u012c\u0097\2\u0675\u0677\7\7\2")
        buf.write("\2\u0676\u0675\3\2\2\2\u0677\u067a\3\2\2\2\u0678\u0676")
        buf.write("\3\2\2\2\u0678\u0679\3\2\2\2\u0679\u067c\3\2\2\2\u067a")
        buf.write("\u0678\3\2\2\2\u067b\u0674\3\2\2\2\u067b\u067c\3\2\2\2")
        buf.write("\u067c\u067d\3\2\2\2\u067d\u0685\5\u0158\u00ad\2\u067e")
        buf.write("\u0680\7\7\2\2\u067f\u067e\3\2\2\2\u0680\u0683\3\2\2\2")
        buf.write("\u0681\u067f\3\2\2\2\u0681\u0682\3\2\2\2\u0682\u0684\3")
        buf.write("\2\2\2\u0683\u0681\3\2\2\2\u0684\u0686\5\u00d0i\2\u0685")
        buf.write("\u0681\3\2\2\2\u0685\u0686\3\2\2\2\u0686\u068e\3\2\2\2")
        buf.write("\u0687\u0689\7\7\2\2\u0688\u0687\3\2\2\2\u0689\u068c\3")
        buf.write("\2\2\2\u068a\u0688\3\2\2\2\u068a\u068b\3\2\2\2\u068b\u068d")
        buf.write("\3\2\2\2\u068c\u068a\3\2\2\2\u068d\u068f\5\34\17\2\u068e")
        buf.write("\u068a\3\2\2\2\u068e\u068f\3\2\2\2\u068fc\3\2\2\2\u0690")
        buf.write("\u0692\5\u0132\u009a\2\u0691\u0690\3\2\2\2\u0691\u0692")
        buf.write("\3\2\2\2\u0692\u0697\3\2\2\2\u0693\u0698\5z>\2\u0694\u0698")
        buf.write("\5h\65\2\u0695\u0698\5f\64\2\u0696\u0698\5v<\2\u0697\u0693")
        buf.write("\3\2\2\2\u0697\u0694\3\2\2\2\u0697\u0695\3\2\2\2\u0697")
        buf.write("\u0696\3\2\2\2\u0698e\3\2\2\2\u0699\u069c\5l\67\2\u069a")
        buf.write("\u069c\7l\2\2\u069b\u0699\3\2\2\2\u069b\u069a\3\2\2\2")
        buf.write("\u069cg\3\2\2\2\u069d\u06a0\5f\64\2\u069e\u06a0\5z>\2")
        buf.write("\u069f\u069d\3\2\2\2\u069f\u069e\3\2\2\2\u06a0\u06a4\3")
        buf.write("\2\2\2\u06a1\u06a3\7\7\2\2\u06a2\u06a1\3\2\2\2\u06a3\u06a6")
        buf.write("\3\2\2\2\u06a4\u06a2\3\2\2\2\u06a4\u06a5\3\2\2\2\u06a5")
        buf.write("\u06a8\3\2\2\2\u06a6\u06a4\3\2\2\2\u06a7\u06a9\5j\66\2")
        buf.write("\u06a8\u06a7\3\2\2\2\u06a9\u06aa\3\2\2\2\u06aa\u06a8\3")
        buf.write("\2\2\2\u06aa\u06ab\3\2\2\2\u06abi\3\2\2\2\u06ac\u06ad")
        buf.write("\t\5\2\2\u06adk\3\2\2\2\u06ae\u06bf\5n8\2\u06af\u06b1")
        buf.write("\7\7\2\2\u06b0\u06af\3\2\2\2\u06b1\u06b4\3\2\2\2\u06b2")
        buf.write("\u06b0\3\2\2\2\u06b2\u06b3\3\2\2\2\u06b3\u06b5\3\2\2\2")
        buf.write("\u06b4\u06b2\3\2\2\2\u06b5\u06b9\7\t\2\2\u06b6\u06b8\7")
        buf.write("\7\2\2\u06b7\u06b6\3\2\2\2\u06b8\u06bb\3\2\2\2\u06b9\u06b7")
        buf.write("\3\2\2\2\u06b9\u06ba\3\2\2\2\u06ba\u06bc\3\2\2\2\u06bb")
        buf.write("\u06b9\3\2\2\2\u06bc\u06be\5n8\2\u06bd\u06b2\3\2\2\2\u06be")
        buf.write("\u06c1\3\2\2\2\u06bf\u06bd\3\2\2\2\u06bf\u06c0\3\2\2\2")
        buf.write("\u06c0m\3\2\2\2\u06c1\u06bf\3\2\2\2\u06c2\u06ca\5\u0158")
        buf.write("\u00ad\2\u06c3\u06c5\7\7\2\2\u06c4\u06c3\3\2\2\2\u06c5")
        buf.write("\u06c8\3\2\2\2\u06c6\u06c4\3\2\2\2\u06c6\u06c7\3\2\2\2")
        buf.write("\u06c7\u06c9\3\2\2\2\u06c8\u06c6\3\2\2\2\u06c9\u06cb\5")
        buf.write("\u00ceh\2\u06ca\u06c6\3\2\2\2\u06ca\u06cb\3\2\2\2\u06cb")
        buf.write("o\3\2\2\2\u06cc\u06ce\5r:\2\u06cd\u06cc\3\2\2\2\u06cd")
        buf.write("\u06ce\3\2\2\2\u06ce\u06cf\3\2\2\2\u06cf\u06d2\5d\63\2")
        buf.write("\u06d0\u06d2\7\21\2\2\u06d1\u06cd\3\2\2\2\u06d1\u06d0")
        buf.write("\3\2\2\2\u06d2q\3\2\2\2\u06d3\u06d5\5t;\2\u06d4\u06d3")
        buf.write("\3\2\2\2\u06d5\u06d6\3\2\2\2\u06d6\u06d4\3\2\2\2\u06d6")
        buf.write("\u06d7\3\2\2\2\u06d7s\3\2\2\2\u06d8\u06dc\5\u013c\u009f")
        buf.write("\2\u06d9\u06db\7\7\2\2\u06da\u06d9\3\2\2\2\u06db\u06de")
        buf.write("\3\2\2\2\u06dc\u06da\3\2\2\2\u06dc\u06dd\3\2\2\2\u06dd")
        buf.write("\u06e1\3\2\2\2\u06de\u06dc\3\2\2\2\u06df\u06e1\5\u014e")
        buf.write("\u00a8\2\u06e0\u06d8\3\2\2\2\u06e0\u06df\3\2\2\2\u06e1")
        buf.write("u\3\2\2\2\u06e2\u06e6\5|?\2\u06e3\u06e5\7\7\2\2\u06e4")
        buf.write("\u06e3\3\2\2\2\u06e5\u06e8\3\2\2\2\u06e6\u06e4\3\2\2\2")
        buf.write("\u06e6\u06e7\3\2\2\2\u06e7\u06e9\3\2\2\2\u06e8\u06e6\3")
        buf.write("\2\2\2\u06e9\u06ed\7\t\2\2\u06ea\u06ec\7\7\2\2\u06eb\u06ea")
        buf.write("\3\2\2\2\u06ec\u06ef\3\2\2\2\u06ed\u06eb\3\2\2\2\u06ed")
        buf.write("\u06ee\3\2\2\2\u06ee\u06f1\3\2\2\2\u06ef\u06ed\3\2\2\2")
        buf.write("\u06f0\u06e2\3\2\2\2\u06f0\u06f1\3\2\2\2\u06f1\u06f2\3")
        buf.write("\2\2\2\u06f2\u06f6\5x=\2\u06f3\u06f5\7\7\2\2\u06f4\u06f3")
        buf.write("\3\2\2\2\u06f5\u06f8\3\2\2\2\u06f6\u06f4\3\2\2\2\u06f6")
        buf.write("\u06f7\3\2\2\2\u06f7\u06f9\3\2\2\2\u06f8\u06f6\3\2\2\2")
        buf.write("\u06f9\u06fd\7$\2\2\u06fa\u06fc\7\7\2\2\u06fb\u06fa\3")
        buf.write("\2\2\2\u06fc\u06ff\3\2\2\2\u06fd\u06fb\3\2\2\2\u06fd\u06fe")
        buf.write("\3\2\2\2\u06fe\u0700\3\2\2\2\u06ff\u06fd\3\2\2\2\u0700")
        buf.write("\u0701\5d\63\2\u0701w\3\2\2\2\u0702\u0706\7\13\2\2\u0703")
        buf.write("\u0705\7\7\2\2\u0704\u0703\3\2\2\2\u0705\u0708\3\2\2\2")
        buf.write("\u0706\u0704\3\2\2\2\u0706\u0707\3\2\2\2\u0707\u070b\3")
        buf.write("\2\2\2\u0708\u0706\3\2\2\2\u0709\u070c\5V,\2\u070a\u070c")
        buf.write("\5d\63\2\u070b\u0709\3\2\2\2\u070b\u070a\3\2\2\2\u070b")
        buf.write("\u070c\3\2\2\2\u070c\u0720\3\2\2\2\u070d\u070f\7\7\2\2")
        buf.write("\u070e\u070d\3\2\2\2\u070f\u0712\3\2\2\2\u0710\u070e\3")
        buf.write("\2\2\2\u0710\u0711\3\2\2\2\u0711\u0713\3\2\2\2\u0712\u0710")
        buf.write("\3\2\2\2\u0713\u0717\7\n\2\2\u0714\u0716\7\7\2\2\u0715")
        buf.write("\u0714\3\2\2\2\u0716\u0719\3\2\2\2\u0717\u0715\3\2\2\2")
        buf.write("\u0717\u0718\3\2\2\2\u0718\u071c\3\2\2\2\u0719\u0717\3")
        buf.write("\2\2\2\u071a\u071d\5V,\2\u071b\u071d\5d\63\2\u071c\u071a")
        buf.write("\3\2\2\2\u071c\u071b\3\2\2\2\u071d\u071f\3\2\2\2\u071e")
        buf.write("\u0710\3\2\2\2\u071f\u0722\3\2\2\2\u0720\u071e\3\2\2\2")
        buf.write("\u0720\u0721\3\2\2\2\u0721\u072a\3\2\2\2\u0722\u0720\3")
        buf.write("\2\2\2\u0723\u0725\7\7\2\2\u0724\u0723\3\2\2\2\u0725\u0728")
        buf.write("\3\2\2\2\u0726\u0724\3\2\2\2\u0726\u0727\3\2\2\2\u0727")
        buf.write("\u0729\3\2\2\2\u0728\u0726\3\2\2\2\u0729\u072b\7\n\2\2")
        buf.write("\u072a\u0726\3\2\2\2\u072a\u072b\3\2\2\2\u072b\u072f\3")
        buf.write("\2\2\2\u072c\u072e\7\7\2\2\u072d\u072c\3\2\2\2\u072e\u0731")
        buf.write("\3\2\2\2\u072f\u072d\3\2\2\2\u072f\u0730\3\2\2\2\u0730")
        buf.write("\u0732\3\2\2\2\u0731\u072f\3\2\2\2\u0732\u0733\7\f\2\2")
        buf.write("\u0733y\3\2\2\2\u0734\u0738\7\13\2\2\u0735\u0737\7\7\2")
        buf.write("\2\u0736\u0735\3\2\2\2\u0737\u073a\3\2\2\2\u0738\u0736")
        buf.write("\3\2\2\2\u0738\u0739\3\2\2\2\u0739\u073b\3\2\2\2\u073a")
        buf.write("\u0738\3\2\2\2\u073b\u073f\5d\63\2\u073c\u073e\7\7\2\2")
        buf.write("\u073d\u073c\3\2\2\2\u073e\u0741\3\2\2\2\u073f\u073d\3")
        buf.write("\2\2\2\u073f\u0740\3\2\2\2\u0740\u0742\3\2\2\2\u0741\u073f")
        buf.write("\3\2\2\2\u0742\u0743\7\f\2\2\u0743{\3\2\2\2\u0744\u0746")
        buf.write("\5\u0132\u009a\2\u0745\u0744\3\2\2\2\u0745\u0746\3\2\2")
        buf.write("\2\u0746\u074a\3\2\2\2\u0747\u074b\5z>\2\u0748\u074b\5")
        buf.write("h\65\2\u0749\u074b\5f\64\2\u074a\u0747\3\2\2\2\u074a\u0748")
        buf.write("\3\2\2\2\u074a\u0749\3\2\2\2\u074b}\3\2\2\2\u074c\u0750")
        buf.write("\7\13\2\2\u074d\u074f\7\7\2\2\u074e\u074d\3\2\2\2\u074f")
        buf.write("\u0752\3\2\2\2\u0750\u074e\3\2\2\2\u0750\u0751\3\2\2\2")
        buf.write("\u0751\u0755\3\2\2\2\u0752\u0750\3\2\2\2\u0753\u0756\5")
        buf.write("l\67\2\u0754\u0756\5~@\2\u0755\u0753\3\2\2\2\u0755\u0754")
        buf.write("\3\2\2\2\u0756\u075a\3\2\2\2\u0757\u0759\7\7\2\2\u0758")
        buf.write("\u0757\3\2\2\2\u0759\u075c\3\2\2\2\u075a\u0758\3\2\2\2")
        buf.write("\u075a\u075b\3\2\2\2\u075b\u075d\3\2\2\2\u075c\u075a\3")
        buf.write("\2\2\2\u075d\u075e\7\f\2\2\u075e\177\3\2\2\2\u075f\u0765")
        buf.write("\5\u0082B\2\u0760\u0761\5\u0096L\2\u0761\u0762\5\u0082")
        buf.write("B\2\u0762\u0764\3\2\2\2\u0763\u0760\3\2\2\2\u0764\u0767")
        buf.write("\3\2\2\2\u0765\u0763\3\2\2\2\u0765\u0766\3\2\2\2\u0766")
        buf.write("\u0769\3\2\2\2\u0767\u0765\3\2\2\2\u0768\u075f\3\2\2\2")
        buf.write("\u0768\u0769\3\2\2\2\u0769\u076b\3\2\2\2\u076a\u076c\5")
        buf.write("\u0096L\2\u076b\u076a\3\2\2\2\u076b\u076c\3\2\2\2\u076c")
        buf.write("\u0081\3\2\2\2\u076d\u0770\5\u0084C\2\u076e\u0770\5\u014e")
        buf.write("\u00a8\2\u076f\u076d\3\2\2\2\u076f\u076e\3\2\2\2\u0770")
        buf.write("\u0773\3\2\2\2\u0771\u076f\3\2\2\2\u0771\u0772\3\2\2\2")
        buf.write("\u0772\u0778\3\2\2\2\u0773\u0771\3\2\2\2\u0774\u0779\5")
        buf.write("\26\f\2\u0775\u0779\5\u0092J\2\u0776\u0779\5\u008aF\2")
        buf.write("\u0777\u0779\5\u0098M\2\u0778\u0774\3\2\2\2\u0778\u0775")
        buf.write("\3\2\2\2\u0778\u0776\3\2\2\2\u0778\u0777\3\2\2\2\u0779")
        buf.write("\u0083\3\2\2\2\u077a\u077b\5\u0158\u00ad\2\u077b\u077f")
        buf.write("\t\6\2\2\u077c\u077e\7\7\2\2\u077d\u077c\3\2\2\2\u077e")
        buf.write("\u0781\3\2\2\2\u077f\u077d\3\2\2\2\u077f\u0780\3\2\2\2")
        buf.write("\u0780\u0085\3\2\2\2\u0781\u077f\3\2\2\2\u0782\u0785\5")
        buf.write("\u0088E\2\u0783\u0785\5\u0082B\2\u0784\u0782\3\2\2\2\u0784")
        buf.write("\u0783\3\2\2\2\u0785\u0087\3\2\2\2\u0786\u078a\7\17\2")
        buf.write("\2\u0787\u0789\7\7\2\2\u0788\u0787\3\2\2\2\u0789\u078c")
        buf.write("\3\2\2\2\u078a\u0788\3\2\2\2\u078a\u078b\3\2\2\2\u078b")
        buf.write("\u078d\3\2\2\2\u078c\u078a\3\2\2\2\u078d\u0791\5\u0080")
        buf.write("A\2\u078e\u0790\7\7\2\2\u078f\u078e\3\2\2\2\u0790\u0793")
        buf.write("\3\2\2\2\u0791\u078f\3\2\2\2\u0791\u0792\3\2\2\2\u0792")
        buf.write("\u0794\3\2\2\2\u0793\u0791\3\2\2\2\u0794\u0795\7\20\2")
        buf.write("\2\u0795\u0089\3\2\2\2\u0796\u079a\5\u008cG\2\u0797\u079a")
        buf.write("\5\u008eH\2\u0798\u079a\5\u0090I\2\u0799\u0796\3\2\2\2")
        buf.write("\u0799\u0797\3\2\2\2\u0799\u0798\3\2\2\2\u079a\u008b\3")
        buf.write("\2\2\2\u079b\u079f\7_\2\2\u079c\u079e\7\7\2\2\u079d\u079c")
        buf.write("\3\2\2\2\u079e\u07a1\3\2\2\2\u079f\u079d\3\2\2\2\u079f")
        buf.write("\u07a0\3\2\2\2\u07a0\u07a2\3\2\2\2\u07a1\u079f\3\2\2\2")
        buf.write("\u07a2\u07a6\7\13\2\2\u07a3\u07a5\5\u014e\u00a8\2\u07a4")
        buf.write("\u07a3\3\2\2\2\u07a5\u07a8\3\2\2\2\u07a6\u07a4\3\2\2\2")
        buf.write("\u07a6\u07a7\3\2\2\2\u07a7\u07ab\3\2\2\2\u07a8\u07a6\3")
        buf.write("\2\2\2\u07a9\u07ac\5D#\2\u07aa\u07ac\5F$\2\u07ab\u07a9")
        buf.write("\3\2\2\2\u07ab\u07aa\3\2\2\2\u07ac\u07ad\3\2\2\2\u07ad")
        buf.write("\u07ae\7h\2\2\u07ae\u07af\5\u0098M\2\u07af\u07b3\7\f\2")
        buf.write("\2\u07b0\u07b2\7\7\2\2\u07b1\u07b0\3\2\2\2\u07b2\u07b5")
        buf.write("\3\2\2\2\u07b3\u07b1\3\2\2\2\u07b3\u07b4\3\2\2\2\u07b4")
        buf.write("\u07b7\3\2\2\2\u07b5\u07b3\3\2\2\2\u07b6\u07b8\5\u0086")
        buf.write("D\2\u07b7\u07b6\3\2\2\2\u07b7\u07b8\3\2\2\2\u07b8\u008d")
        buf.write("\3\2\2\2\u07b9\u07bd\7a\2\2\u07ba\u07bc\7\7\2\2\u07bb")
        buf.write("\u07ba\3\2\2\2\u07bc\u07bf\3\2\2\2\u07bd\u07bb\3\2\2\2")
        buf.write("\u07bd\u07be\3\2\2\2\u07be\u07c0\3\2\2\2\u07bf\u07bd\3")
        buf.write("\2\2\2\u07c0\u07c1\7\13\2\2\u07c1\u07c2\5\u0098M\2\u07c2")
        buf.write("\u07c6\7\f\2\2\u07c3\u07c5\7\7\2\2\u07c4\u07c3\3\2\2\2")
        buf.write("\u07c5\u07c8\3\2\2\2\u07c6\u07c4\3\2\2\2\u07c6\u07c7\3")
        buf.write("\2\2\2\u07c7\u07cb\3\2\2\2\u07c8\u07c6\3\2\2\2\u07c9\u07cc")
        buf.write("\5\u0086D\2\u07ca\u07cc\7\35\2\2\u07cb\u07c9\3\2\2\2\u07cb")
        buf.write("\u07ca\3\2\2\2\u07cc\u008f\3\2\2\2\u07cd\u07d1\7`\2\2")
        buf.write("\u07ce\u07d0\7\7\2\2\u07cf\u07ce\3\2\2\2\u07d0\u07d3\3")
        buf.write("\2\2\2\u07d1\u07cf\3\2\2\2\u07d1\u07d2\3\2\2\2\u07d2\u07d5")
        buf.write("\3\2\2\2\u07d3\u07d1\3\2\2\2\u07d4\u07d6\5\u0086D\2\u07d5")
        buf.write("\u07d4\3\2\2\2\u07d5\u07d6\3\2\2\2\u07d6\u07da\3\2\2\2")
        buf.write("\u07d7\u07d9\7\7\2\2\u07d8\u07d7\3\2\2\2\u07d9\u07dc\3")
        buf.write("\2\2\2\u07da\u07d8\3\2\2\2\u07da\u07db\3\2\2\2\u07db\u07dd")
        buf.write("\3\2\2\2\u07dc\u07da\3\2\2\2\u07dd\u07e1\7a\2\2\u07de")
        buf.write("\u07e0\7\7\2\2\u07df\u07de\3\2\2\2\u07e0\u07e3\3\2\2\2")
        buf.write("\u07e1\u07df\3\2\2\2\u07e1\u07e2\3\2\2\2\u07e2\u07e4\3")
        buf.write("\2\2\2\u07e3\u07e1\3\2\2\2\u07e4\u07e5\7\13\2\2\u07e5")
        buf.write("\u07e6\5\u0098M\2\u07e6\u07e7\7\f\2\2\u07e7\u0091\3\2")
        buf.write("\2\2\u07e8\u07e9\5\u00bc_\2\u07e9\u07ea\7\36\2\2\u07ea")
        buf.write("\u07ef\3\2\2\2\u07eb\u07ec\5\u00c0a\2\u07ec\u07ed\5\u0112")
        buf.write("\u008a\2\u07ed\u07ef\3\2\2\2\u07ee\u07e8\3\2\2\2\u07ee")
        buf.write("\u07eb\3\2\2\2\u07ef\u07f3\3\2\2\2\u07f0\u07f2\7\7\2\2")
        buf.write("\u07f1\u07f0\3\2\2\2\u07f2\u07f5\3\2\2\2\u07f3\u07f1\3")
        buf.write("\2\2\2\u07f3\u07f4\3\2\2\2\u07f4\u07f6\3\2\2\2\u07f5\u07f3")
        buf.write("\3\2\2\2\u07f6\u07f7\5\u0098M\2\u07f7\u0093\3\2\2\2\u07f8")
        buf.write("\u07fc\t\7\2\2\u07f9\u07fb\7\7\2\2\u07fa\u07f9\3\2\2\2")
        buf.write("\u07fb\u07fe\3\2\2\2\u07fc\u07fa\3\2\2\2\u07fc\u07fd\3")
        buf.write("\2\2\2\u07fd\u0801\3\2\2\2\u07fe\u07fc\3\2\2\2\u07ff\u0801")
        buf.write("\7\2\2\3\u0800\u07f8\3\2\2\2\u0800\u07ff\3\2\2\2\u0801")
        buf.write("\u0095\3\2\2\2\u0802\u0804\t\7\2\2\u0803\u0802\3\2\2\2")
        buf.write("\u0804\u0805\3\2\2\2\u0805\u0803\3\2\2\2\u0805\u0806\3")
        buf.write("\2\2\2\u0806\u0809\3\2\2\2\u0807\u0809\7\2\2\3\u0808\u0803")
        buf.write("\3\2\2\2\u0808\u0807\3\2\2\2\u0809\u0097\3\2\2\2\u080a")
        buf.write("\u080b\5\u009aN\2\u080b\u0099\3\2\2\2\u080c\u081d\5\u009c")
        buf.write("O\2\u080d\u080f\7\7\2\2\u080e\u080d\3\2\2\2\u080f\u0812")
        buf.write("\3\2\2\2\u0810\u080e\3\2\2\2\u0810\u0811\3\2\2\2\u0811")
        buf.write("\u0813\3\2\2\2\u0812\u0810\3\2\2\2\u0813\u0817\7\31\2")
        buf.write("\2\u0814\u0816\7\7\2\2\u0815\u0814\3\2\2\2\u0816\u0819")
        buf.write("\3\2\2\2\u0817\u0815\3\2\2\2\u0817\u0818\3\2\2\2\u0818")
        buf.write("\u081a\3\2\2\2\u0819\u0817\3\2\2\2\u081a\u081c\5\u009c")
        buf.write("O\2\u081b\u0810\3\2\2\2\u081c\u081f\3\2\2\2\u081d\u081b")
        buf.write("\3\2\2\2\u081d\u081e\3\2\2\2\u081e\u009b\3\2\2\2\u081f")
        buf.write("\u081d\3\2\2\2\u0820\u0831\5\u009eP\2\u0821\u0823\7\7")
        buf.write("\2\2\u0822\u0821\3\2\2\2\u0823\u0826\3\2\2\2\u0824\u0822")
        buf.write("\3\2\2\2\u0824\u0825\3\2\2\2\u0825\u0827\3\2\2\2\u0826")
        buf.write("\u0824\3\2\2\2\u0827\u082b\7\30\2\2\u0828\u082a\7\7\2")
        buf.write("\2\u0829\u0828\3\2\2\2\u082a\u082d\3\2\2\2\u082b\u0829")
        buf.write("\3\2\2\2\u082b\u082c\3\2\2\2\u082c\u082e\3\2\2\2\u082d")
        buf.write("\u082b\3\2\2\2\u082e\u0830\5\u009eP\2\u082f\u0824\3\2")
        buf.write("\2\2\u0830\u0833\3\2\2\2\u0831\u082f\3\2\2\2\u0831\u0832")
        buf.write("\3\2\2\2\u0832\u009d\3\2\2\2\u0833\u0831\3\2\2\2\u0834")
        buf.write("\u0840\5\u00a0Q\2\u0835\u0839\5\u0114\u008b\2\u0836\u0838")
        buf.write("\7\7\2\2\u0837\u0836\3\2\2\2\u0838\u083b\3\2\2\2\u0839")
        buf.write("\u0837\3\2\2\2\u0839\u083a\3\2\2\2\u083a\u083c\3\2\2\2")
        buf.write("\u083b\u0839\3\2\2\2\u083c\u083d\5\u00a0Q\2\u083d\u083f")
        buf.write("\3\2\2\2\u083e\u0835\3\2\2\2\u083f\u0842\3\2\2\2\u0840")
        buf.write("\u083e\3\2\2\2\u0840\u0841\3\2\2\2\u0841\u009f\3\2\2\2")
        buf.write("\u0842\u0840\3\2\2\2\u0843\u084f\5\u00a2R\2\u0844\u0848")
        buf.write("\5\u0116\u008c\2\u0845\u0847\7\7\2\2\u0846\u0845\3\2\2")
        buf.write("\2\u0847\u084a\3\2\2\2\u0848\u0846\3\2\2\2\u0848\u0849")
        buf.write("\3\2\2\2\u0849\u084b\3\2\2\2\u084a\u0848\3\2\2\2\u084b")
        buf.write("\u084c\5\u00a2R\2\u084c\u084e\3\2\2\2\u084d\u0844\3\2")
        buf.write("\2\2\u084e\u0851\3\2\2\2\u084f\u084d\3\2\2\2\u084f\u0850")
        buf.write("\3\2\2\2\u0850\u00a1\3\2\2\2\u0851\u084f\3\2\2\2\u0852")
        buf.write("\u0856\5\u00a4S\2\u0853\u0855\5\u00caf\2\u0854\u0853\3")
        buf.write("\2\2\2\u0855\u0858\3\2\2\2\u0856\u0854\3\2\2\2\u0856\u0857")
        buf.write("\3\2\2\2\u0857\u00a3\3\2\2\2\u0858\u0856\3\2\2\2\u0859")
        buf.write("\u086e\5\u00a6T\2\u085a\u085e\5\u0118\u008d\2\u085b\u085d")
        buf.write("\7\7\2\2\u085c\u085b\3\2\2\2\u085d\u0860\3\2\2\2\u085e")
        buf.write("\u085c\3\2\2\2\u085e\u085f\3\2\2\2\u085f\u0861\3\2\2\2")
        buf.write("\u0860\u085e\3\2\2\2\u0861\u0862\5\u00a6T\2\u0862\u086d")
        buf.write("\3\2\2\2\u0863\u0867\5\u011a\u008e\2\u0864\u0866\7\7\2")
        buf.write("\2\u0865\u0864\3\2\2\2\u0866\u0869\3\2\2\2\u0867\u0865")
        buf.write("\3\2\2\2\u0867\u0868\3\2\2\2\u0868\u086a\3\2\2\2\u0869")
        buf.write("\u0867\3\2\2\2\u086a\u086b\5d\63\2\u086b\u086d\3\2\2\2")
        buf.write("\u086c\u085a\3\2\2\2\u086c\u0863\3\2\2\2\u086d\u0870\3")
        buf.write("\2\2\2\u086e\u086c\3\2\2\2\u086e\u086f\3\2\2\2\u086f\u00a5")
        buf.write("\3\2\2\2\u0870\u086e\3\2\2\2\u0871\u0883\5\u00aaV\2\u0872")
        buf.write("\u0874\7\7\2\2\u0873\u0872\3\2\2\2\u0874\u0877\3\2\2\2")
        buf.write("\u0875\u0873\3\2\2\2\u0875\u0876\3\2\2\2\u0876\u0878\3")
        buf.write("\2\2\2\u0877\u0875\3\2\2\2\u0878\u087c\5\u00a8U\2\u0879")
        buf.write("\u087b\7\7\2\2\u087a\u0879\3\2\2\2\u087b\u087e\3\2\2\2")
        buf.write("\u087c\u087a\3\2\2\2\u087c\u087d\3\2\2\2\u087d\u087f\3")
        buf.write("\2\2\2\u087e\u087c\3\2\2\2\u087f\u0880\5\u00aaV\2\u0880")
        buf.write("\u0882\3\2\2\2\u0881\u0875\3\2\2\2\u0882\u0885\3\2\2\2")
        buf.write("\u0883\u0881\3\2\2\2\u0883\u0884\3\2\2\2\u0884\u00a7\3")
        buf.write("\2\2\2\u0885\u0883\3\2\2\2\u0886\u0887\7/\2\2\u0887\u0888")
        buf.write("\7\34\2\2\u0888\u00a9\3\2\2\2\u0889\u0895\5\u00acW\2\u088a")
        buf.write("\u088e\5\u0158\u00ad\2\u088b\u088d\7\7\2\2\u088c\u088b")
        buf.write("\3\2\2\2\u088d\u0890\3\2\2\2\u088e\u088c\3\2\2\2\u088e")
        buf.write("\u088f\3\2\2\2\u088f\u0891\3\2\2\2\u0890\u088e\3\2\2\2")
        buf.write("\u0891\u0892\5\u00acW\2\u0892\u0894\3\2\2\2\u0893\u088a")
        buf.write("\3\2\2\2\u0894\u0897\3\2\2\2\u0895\u0893\3\2\2\2\u0895")
        buf.write("\u0896\3\2\2\2\u0896\u00ab\3\2\2\2\u0897\u0895\3\2\2\2")
        buf.write("\u0898\u08a3\5\u00aeX\2\u0899\u089d\7&\2\2\u089a\u089c")
        buf.write("\7\7\2\2\u089b\u089a\3\2\2\2\u089c\u089f\3\2\2\2\u089d")
        buf.write("\u089b\3\2\2\2\u089d\u089e\3\2\2\2\u089e\u08a0\3\2\2\2")
        buf.write("\u089f\u089d\3\2\2\2\u08a0\u08a2\5\u00aeX\2\u08a1\u0899")
        buf.write("\3\2\2\2\u08a2\u08a5\3\2\2\2\u08a3\u08a1\3\2\2\2\u08a3")
        buf.write("\u08a4\3\2\2\2\u08a4\u00ad\3\2\2\2\u08a5\u08a3\3\2\2\2")
        buf.write("\u08a6\u08b2\5\u00b0Y\2\u08a7\u08ab\5\u011c\u008f\2\u08a8")
        buf.write("\u08aa\7\7\2\2\u08a9\u08a8\3\2\2\2\u08aa\u08ad\3\2\2\2")
        buf.write("\u08ab\u08a9\3\2\2\2\u08ab\u08ac\3\2\2\2\u08ac\u08ae\3")
        buf.write("\2\2\2\u08ad\u08ab\3\2\2\2\u08ae\u08af\5\u00b0Y\2\u08af")
        buf.write("\u08b1\3\2\2\2\u08b0\u08a7\3\2\2\2\u08b1\u08b4\3\2\2\2")
        buf.write("\u08b2\u08b0\3\2\2\2\u08b2\u08b3\3\2\2\2\u08b3\u00af\3")
        buf.write("\2\2\2\u08b4\u08b2\3\2\2\2\u08b5\u08c1\5\u00b2Z\2\u08b6")
        buf.write("\u08ba\5\u011e\u0090\2\u08b7\u08b9\7\7\2\2\u08b8\u08b7")
        buf.write("\3\2\2\2\u08b9\u08bc\3\2\2\2\u08ba\u08b8\3\2\2\2\u08ba")
        buf.write("\u08bb\3\2\2\2\u08bb\u08bd\3\2\2\2\u08bc\u08ba\3\2\2\2")
        buf.write("\u08bd\u08be\5\u00b2Z\2\u08be\u08c0\3\2\2\2\u08bf\u08b6")
        buf.write("\3\2\2\2\u08c0\u08c3\3\2\2\2\u08c1\u08bf\3\2\2\2\u08c1")
        buf.write("\u08c2\3\2\2\2\u08c2\u00b1\3\2\2\2\u08c3\u08c1\3\2\2\2")
        buf.write("\u08c4\u08d6\5\u00b4[\2\u08c5\u08c7\7\7\2\2\u08c6\u08c5")
        buf.write("\3\2\2\2\u08c7\u08ca\3\2\2\2\u08c8\u08c6\3\2\2\2\u08c8")
        buf.write("\u08c9\3\2\2\2\u08c9\u08cb\3\2\2\2\u08ca\u08c8\3\2\2\2")
        buf.write("\u08cb\u08cf\5\u0120\u0091\2\u08cc\u08ce\7\7\2\2\u08cd")
        buf.write("\u08cc\3\2\2\2\u08ce\u08d1\3\2\2\2\u08cf\u08cd\3\2\2\2")
        buf.write("\u08cf\u08d0\3\2\2\2\u08d0\u08d2\3\2\2\2\u08d1\u08cf\3")
        buf.write("\2\2\2\u08d2\u08d3\5d\63\2\u08d3\u08d5\3\2\2\2\u08d4\u08c8")
        buf.write("\3\2\2\2\u08d5\u08d8\3\2\2\2\u08d6\u08d4\3\2\2\2\u08d6")
        buf.write("\u08d7\3\2\2\2\u08d7\u00b3\3\2\2\2\u08d8\u08d6\3\2\2\2")
        buf.write("\u08d9\u08db\5\u00b6\\\2\u08da\u08d9\3\2\2\2\u08db\u08de")
        buf.write("\3\2\2\2\u08dc\u08da\3\2\2\2\u08dc\u08dd\3\2\2\2\u08dd")
        buf.write("\u08df\3\2\2\2\u08de\u08dc\3\2\2\2\u08df\u08e0\5\u00b8")
        buf.write("]\2\u08e0\u00b5\3\2\2\2\u08e1\u08eb\5\u014e\u00a8\2\u08e2")
        buf.write("\u08eb\5\u0084C\2\u08e3\u08e7\5\u0122\u0092\2\u08e4\u08e6")
        buf.write("\7\7\2\2\u08e5\u08e4\3\2\2\2\u08e6\u08e9\3\2\2\2\u08e7")
        buf.write("\u08e5\3\2\2\2\u08e7\u08e8\3\2\2\2\u08e8\u08eb\3\2\2\2")
        buf.write("\u08e9\u08e7\3\2\2\2\u08ea\u08e1\3\2\2\2\u08ea\u08e2\3")
        buf.write("\2\2\2\u08ea\u08e3\3\2\2\2\u08eb\u00b7\3\2\2\2\u08ec\u08f0")
        buf.write("\5\u00d4k\2\u08ed\u08ef\5\u00ba^\2\u08ee\u08ed\3\2\2\2")
        buf.write("\u08ef\u08f2\3\2\2\2\u08f0\u08ee\3\2\2\2\u08f0\u08f1\3")
        buf.write("\2\2\2\u08f1\u00b9\3\2\2\2\u08f2\u08f0\3\2\2\2\u08f3\u08f9")
        buf.write("\5\u0124\u0093\2\u08f4\u08f9\5\u00ceh\2\u08f5\u08f9\5")
        buf.write("\u00caf\2\u08f6\u08f9\5\u00c6d\2\u08f7\u08f9\5\u00c8e")
        buf.write("\2\u08f8\u08f3\3\2\2\2\u08f8\u08f4\3\2\2\2\u08f8\u08f5")
        buf.write("\3\2\2\2\u08f8\u08f6\3\2\2\2\u08f8\u08f7\3\2\2\2\u08f9")
        buf.write("\u00bb\3\2\2\2\u08fa\u08fb\5\u00b8]\2\u08fb\u08fc\5\u00c4")
        buf.write("c\2\u08fc\u0900\3\2\2\2\u08fd\u0900\5\u0158\u00ad\2\u08fe")
        buf.write("\u0900\5\u00be`\2\u08ff\u08fa\3\2\2\2\u08ff\u08fd\3\2")
        buf.write("\2\2\u08ff\u08fe\3\2\2\2\u0900\u00bd\3\2\2\2\u0901\u0905")
        buf.write("\7\13\2\2\u0902\u0904\7\7\2\2\u0903\u0902\3\2\2\2\u0904")
        buf.write("\u0907\3\2\2\2\u0905\u0903\3\2\2\2\u0905\u0906\3\2\2\2")
        buf.write("\u0906\u0908\3\2\2\2\u0907\u0905\3\2\2\2\u0908\u090c\5")
        buf.write("\u00bc_\2\u0909\u090b\7\7\2\2\u090a\u0909\3\2\2\2\u090b")
        buf.write("\u090e\3\2\2\2\u090c\u090a\3\2\2\2\u090c\u090d\3\2\2\2")
        buf.write("\u090d\u090f\3\2\2\2\u090e\u090c\3\2\2\2\u090f\u0910\7")
        buf.write("\f\2\2\u0910\u00bf\3\2\2\2\u0911\u0914\5\u00b4[\2\u0912")
        buf.write("\u0914\5\u00c2b\2\u0913\u0911\3\2\2\2\u0913\u0912\3\2")
        buf.write("\2\2\u0914\u00c1\3\2\2\2\u0915\u0919\7\13\2\2\u0916\u0918")
        buf.write("\7\7\2\2\u0917\u0916\3\2\2\2\u0918\u091b\3\2\2\2\u0919")
        buf.write("\u0917\3\2\2\2\u0919\u091a\3\2\2\2\u091a\u091c\3\2\2\2")
        buf.write("\u091b\u0919\3\2\2\2\u091c\u0920\5\u00c0a\2\u091d\u091f")
        buf.write("\7\7\2\2\u091e\u091d\3\2\2\2\u091f\u0922\3\2\2\2\u0920")
        buf.write("\u091e\3\2\2\2\u0920\u0921\3\2\2\2\u0921\u0923\3\2\2\2")
        buf.write("\u0922\u0920\3\2\2\2\u0923\u0924\7\f\2\2\u0924\u00c3\3")
        buf.write("\2\2\2\u0925\u0929\5\u00ceh\2\u0926\u0929\5\u00c6d\2\u0927")
        buf.write("\u0929\5\u00c8e\2\u0928\u0925\3\2\2\2\u0928\u0926\3\2")
        buf.write("\2\2\u0928\u0927\3\2\2\2\u0929\u00c5\3\2\2\2\u092a\u092e")
        buf.write("\7\r\2\2\u092b\u092d\7\7\2\2\u092c\u092b\3\2\2\2\u092d")
        buf.write("\u0930\3\2\2\2\u092e\u092c\3\2\2\2\u092e\u092f\3\2\2\2")
        buf.write("\u092f\u0931\3\2\2\2\u0930\u092e\3\2\2\2\u0931\u0942\5")
        buf.write("\u0098M\2\u0932\u0934\7\7\2\2\u0933\u0932\3\2\2\2\u0934")
        buf.write("\u0937\3\2\2\2\u0935\u0933\3\2\2\2\u0935\u0936\3\2\2\2")
        buf.write("\u0936\u0938\3\2\2\2\u0937\u0935\3\2\2\2\u0938\u093c\7")
        buf.write("\n\2\2\u0939\u093b\7\7\2\2\u093a\u0939\3\2\2\2\u093b\u093e")
        buf.write("\3\2\2\2\u093c\u093a\3\2\2\2\u093c\u093d\3\2\2\2\u093d")
        buf.write("\u093f\3\2\2\2\u093e\u093c\3\2\2\2\u093f\u0941\5\u0098")
        buf.write("M\2\u0940\u0935\3\2\2\2\u0941\u0944\3\2\2\2\u0942\u0940")
        buf.write("\3\2\2\2\u0942\u0943\3\2\2\2\u0943\u094c\3\2\2\2\u0944")
        buf.write("\u0942\3\2\2\2\u0945\u0947\7\7\2\2\u0946\u0945\3\2\2\2")
        buf.write("\u0947\u094a\3\2\2\2\u0948\u0946\3\2\2\2\u0948\u0949\3")
        buf.write("\2\2\2\u0949\u094b\3\2\2\2\u094a\u0948\3\2\2\2\u094b\u094d")
        buf.write("\7\n\2\2\u094c\u0948\3\2\2\2\u094c\u094d\3\2\2\2\u094d")
        buf.write("\u0951\3\2\2\2\u094e\u0950\7\7\2\2\u094f\u094e\3\2\2\2")
        buf.write("\u0950\u0953\3\2\2\2\u0951\u094f\3\2\2\2\u0951\u0952\3")
        buf.write("\2\2\2\u0952\u0954\3\2\2\2\u0953\u0951\3\2\2\2\u0954\u0955")
        buf.write("\7\16\2\2\u0955\u00c7\3\2\2\2\u0956\u095a\5\u0128\u0095")
        buf.write("\2\u0957\u0959\7\7\2\2\u0958\u0957\3\2\2\2\u0959\u095c")
        buf.write("\3\2\2\2\u095a\u0958\3\2\2\2\u095a\u095b\3\2\2\2\u095b")
        buf.write("\u0960\3\2\2\2\u095c\u095a\3\2\2\2\u095d\u0961\5\u0158")
        buf.write("\u00ad\2\u095e\u0961\5\u00d6l\2\u095f\u0961\7J\2\2\u0960")
        buf.write("\u095d\3\2\2\2\u0960\u095e\3\2\2\2\u0960\u095f\3\2\2\2")
        buf.write("\u0961\u00c9\3\2\2\2\u0962\u0964\5\u00ceh\2\u0963\u0962")
        buf.write("\3\2\2\2\u0963\u0964\3\2\2\2\u0964\u096a\3\2\2\2\u0965")
        buf.write("\u0967\5\u00d0i\2\u0966\u0965\3\2\2\2\u0966\u0967\3\2")
        buf.write("\2\2\u0967\u0968\3\2\2\2\u0968\u096b\5\u00ccg\2\u0969")
        buf.write("\u096b\5\u00d0i\2\u096a\u0966\3\2\2\2\u096a\u0969\3\2")
        buf.write("\2\2\u096b\u00cb\3\2\2\2\u096c\u096e\5\u014e\u00a8\2\u096d")
        buf.write("\u096c\3\2\2\2\u096e\u0971\3\2\2\2\u096f\u096d\3\2\2\2")
        buf.write("\u096f\u0970\3\2\2\2\u0970\u0973\3\2\2\2\u0971\u096f\3")
        buf.write("\2\2\2\u0972\u0974\5\u0084C\2\u0973\u0972\3\2\2\2\u0973")
        buf.write("\u0974\3\2\2\2\u0974\u0978\3\2\2\2\u0975\u0977\7\7\2\2")
        buf.write("\u0976\u0975\3\2\2\2\u0977\u097a\3\2\2\2\u0978\u0976\3")
        buf.write("\2\2\2\u0978\u0979\3\2\2\2\u0979\u097b\3\2\2\2\u097a\u0978")
        buf.write("\3\2\2\2\u097b\u097c\5\u00eav\2\u097c\u00cd\3\2\2\2\u097d")
        buf.write("\u0981\7\60\2\2\u097e\u0980\7\7\2\2\u097f\u097e\3\2\2")
        buf.write("\2\u0980\u0983\3\2\2\2\u0981\u097f\3\2\2\2\u0981\u0982")
        buf.write("\3\2\2\2\u0982\u0984\3\2\2\2\u0983\u0981\3\2\2\2\u0984")
        buf.write("\u0995\5p9\2\u0985\u0987\7\7\2\2\u0986\u0985\3\2\2\2\u0987")
        buf.write("\u098a\3\2\2\2\u0988\u0986\3\2\2\2\u0988\u0989\3\2\2\2")
        buf.write("\u0989\u098b\3\2\2\2\u098a\u0988\3\2\2\2\u098b\u098f\7")
        buf.write("\n\2\2\u098c\u098e\7\7\2\2\u098d\u098c\3\2\2\2\u098e\u0991")
        buf.write("\3\2\2\2\u098f\u098d\3\2\2\2\u098f\u0990\3\2\2\2\u0990")
        buf.write("\u0992\3\2\2\2\u0991\u098f\3\2\2\2\u0992\u0994\5p9\2\u0993")
        buf.write("\u0988\3\2\2\2\u0994\u0997\3\2\2\2\u0995\u0993\3\2\2\2")
        buf.write("\u0995\u0996\3\2\2\2\u0996\u099f\3\2\2\2\u0997\u0995\3")
        buf.write("\2\2\2\u0998\u099a\7\7\2\2\u0999\u0998\3\2\2\2\u099a\u099d")
        buf.write("\3\2\2\2\u099b\u0999\3\2\2\2\u099b\u099c\3\2\2\2\u099c")
        buf.write("\u099e\3\2\2\2\u099d\u099b\3\2\2\2\u099e\u09a0\7\n\2\2")
        buf.write("\u099f\u099b\3\2\2\2\u099f\u09a0\3\2\2\2\u09a0\u09a4\3")
        buf.write("\2\2\2\u09a1\u09a3\7\7\2\2\u09a2\u09a1\3\2\2\2\u09a3\u09a6")
        buf.write("\3\2\2\2\u09a4\u09a2\3\2\2\2\u09a4\u09a5\3\2\2\2\u09a5")
        buf.write("\u09a7\3\2\2\2\u09a6\u09a4\3\2\2\2\u09a7\u09a8\7\61\2")
        buf.write("\2\u09a8\u00cf\3\2\2\2\u09a9\u09ad\7\13\2\2\u09aa\u09ac")
        buf.write("\7\7\2\2\u09ab\u09aa\3\2\2\2\u09ac\u09af\3\2\2\2\u09ad")
        buf.write("\u09ab\3\2\2\2\u09ad\u09ae\3\2\2\2\u09ae\u09d3\3\2\2\2")
        buf.write("\u09af\u09ad\3\2\2\2\u09b0\u09c1\5\u00d2j\2\u09b1\u09b3")
        buf.write("\7\7\2\2\u09b2\u09b1\3\2\2\2\u09b3\u09b6\3\2\2\2\u09b4")
        buf.write("\u09b2\3\2\2\2\u09b4\u09b5\3\2\2\2\u09b5\u09b7\3\2\2\2")
        buf.write("\u09b6\u09b4\3\2\2\2\u09b7\u09bb\7\n\2\2\u09b8\u09ba\7")
        buf.write("\7\2\2\u09b9\u09b8\3\2\2\2\u09ba\u09bd\3\2\2\2\u09bb\u09b9")
        buf.write("\3\2\2\2\u09bb\u09bc\3\2\2\2\u09bc\u09be\3\2\2\2\u09bd")
        buf.write("\u09bb\3\2\2\2\u09be\u09c0\5\u00d2j\2\u09bf\u09b4\3\2")
        buf.write("\2\2\u09c0\u09c3\3\2\2\2\u09c1\u09bf\3\2\2\2\u09c1\u09c2")
        buf.write("\3\2\2\2\u09c2\u09cb\3\2\2\2\u09c3\u09c1\3\2\2\2\u09c4")
        buf.write("\u09c6\7\7\2\2\u09c5\u09c4\3\2\2\2\u09c6\u09c9\3\2\2\2")
        buf.write("\u09c7\u09c5\3\2\2\2\u09c7\u09c8\3\2\2\2\u09c8\u09ca\3")
        buf.write("\2\2\2\u09c9\u09c7\3\2\2\2\u09ca\u09cc\7\n\2\2\u09cb\u09c7")
        buf.write("\3\2\2\2\u09cb\u09cc\3\2\2\2\u09cc\u09d0\3\2\2\2\u09cd")
        buf.write("\u09cf\7\7\2\2\u09ce\u09cd\3\2\2\2\u09cf\u09d2\3\2\2\2")
        buf.write("\u09d0\u09ce\3\2\2\2\u09d0\u09d1\3\2\2\2\u09d1\u09d4\3")
        buf.write("\2\2\2\u09d2\u09d0\3\2\2\2\u09d3\u09b0\3\2\2\2\u09d3\u09d4")
        buf.write("\3\2\2\2\u09d4\u09d5\3\2\2\2\u09d5\u09d6\7\f\2\2\u09d6")
        buf.write("\u00d1\3\2\2\2\u09d7\u09d9\5\u014e\u00a8\2\u09d8\u09d7")
        buf.write("\3\2\2\2\u09d8\u09d9\3\2\2\2\u09d9\u09dd\3\2\2\2\u09da")
        buf.write("\u09dc\7\7\2\2\u09db\u09da\3\2\2\2\u09dc\u09df\3\2\2\2")
        buf.write("\u09dd\u09db\3\2\2\2\u09dd\u09de\3\2\2\2\u09de\u09ee\3")
        buf.write("\2\2\2\u09df\u09dd\3\2\2\2\u09e0\u09e4\5\u0158\u00ad\2")
        buf.write("\u09e1\u09e3\7\7\2\2\u09e2\u09e1\3\2\2\2\u09e3\u09e6\3")
        buf.write("\2\2\2\u09e4\u09e2\3\2\2\2\u09e4\u09e5\3\2\2\2\u09e5\u09e7")
        buf.write("\3\2\2\2\u09e6\u09e4\3\2\2\2\u09e7\u09eb\7\36\2\2\u09e8")
        buf.write("\u09ea\7\7\2\2\u09e9\u09e8\3\2\2\2\u09ea\u09ed\3\2\2\2")
        buf.write("\u09eb\u09e9\3\2\2\2\u09eb\u09ec\3\2\2\2\u09ec\u09ef\3")
        buf.write("\2\2\2\u09ed\u09eb\3\2\2\2\u09ee\u09e0\3\2\2\2\u09ee\u09ef")
        buf.write("\3\2\2\2\u09ef\u09f1\3\2\2\2\u09f0\u09f2\7\21\2\2\u09f1")
        buf.write("\u09f0\3\2\2\2\u09f1\u09f2\3\2\2\2\u09f2\u09f6\3\2\2\2")
        buf.write("\u09f3\u09f5\7\7\2\2\u09f4\u09f3\3\2\2\2\u09f5\u09f8\3")
        buf.write("\2\2\2\u09f6\u09f4\3\2\2\2\u09f6\u09f7\3\2\2\2\u09f7\u09f9")
        buf.write("\3\2\2\2\u09f8\u09f6\3\2\2\2\u09f9\u09fa\5\u0098M\2\u09fa")
        buf.write("\u00d3\3\2\2\2\u09fb\u0a0a\5\u00d6l\2\u09fc\u0a0a\5\u0158")
        buf.write("\u00ad\2\u09fd\u0a0a\5\u00dan\2\u09fe\u0a0a\5\u00dco\2")
        buf.write("\u09ff\u0a0a\5\u0110\u0089\2\u0a00\u0a0a\5\u00f2z\2\u0a01")
        buf.write("\u0a0a\5\u00f4{\2\u0a02\u0a0a\5\u00d8m\2\u0a03\u0a0a\5")
        buf.write("\u00f6|\2\u0a04\u0a0a\5\u00f8}\2\u0a05\u0a0a\5\u00fa~")
        buf.write("\2\u0a06\u0a0a\5\u00fe\u0080\2\u0a07\u0a0a\5\u0108\u0085")
        buf.write("\2\u0a08\u0a0a\5\u010e\u0088\2\u0a09\u09fb\3\2\2\2\u0a09")
        buf.write("\u09fc\3\2\2\2\u0a09\u09fd\3\2\2\2\u0a09\u09fe\3\2\2\2")
        buf.write("\u0a09\u09ff\3\2\2\2\u0a09\u0a00\3\2\2\2\u0a09\u0a01\3")
        buf.write("\2\2\2\u0a09\u0a02\3\2\2\2\u0a09\u0a03\3\2\2\2\u0a09\u0a04")
        buf.write("\3\2\2\2\u0a09\u0a05\3\2\2\2\u0a09\u0a06\3\2\2\2\u0a09")
        buf.write("\u0a07\3\2\2\2\u0a09\u0a08\3\2\2\2\u0a0a\u00d5\3\2\2\2")
        buf.write("\u0a0b\u0a0f\7\13\2\2\u0a0c\u0a0e\7\7\2\2\u0a0d\u0a0c")
        buf.write("\3\2\2\2\u0a0e\u0a11\3\2\2\2\u0a0f\u0a0d\3\2\2\2\u0a0f")
        buf.write("\u0a10\3\2\2\2\u0a10\u0a12\3\2\2\2\u0a11\u0a0f\3\2\2\2")
        buf.write("\u0a12\u0a16\5\u0098M\2\u0a13\u0a15\7\7\2\2\u0a14\u0a13")
        buf.write("\3\2\2\2\u0a15\u0a18\3\2\2\2\u0a16\u0a14\3\2\2\2\u0a16")
        buf.write("\u0a17\3\2\2\2\u0a17\u0a19\3\2\2\2\u0a18\u0a16\3\2\2\2")
        buf.write("\u0a19\u0a1a\7\f\2\2\u0a1a\u00d7\3\2\2\2\u0a1b\u0a1f\7")
        buf.write("\r\2\2\u0a1c\u0a1e\7\7\2\2\u0a1d\u0a1c\3\2\2\2\u0a1e\u0a21")
        buf.write("\3\2\2\2\u0a1f\u0a1d\3\2\2\2\u0a1f\u0a20\3\2\2\2\u0a20")
        buf.write("\u0a45\3\2\2\2\u0a21\u0a1f\3\2\2\2\u0a22\u0a33\5\u0098")
        buf.write("M\2\u0a23\u0a25\7\7\2\2\u0a24\u0a23\3\2\2\2\u0a25\u0a28")
        buf.write("\3\2\2\2\u0a26\u0a24\3\2\2\2\u0a26\u0a27\3\2\2\2\u0a27")
        buf.write("\u0a29\3\2\2\2\u0a28\u0a26\3\2\2\2\u0a29\u0a2d\7\n\2\2")
        buf.write("\u0a2a\u0a2c\7\7\2\2\u0a2b\u0a2a\3\2\2\2\u0a2c\u0a2f\3")
        buf.write("\2\2\2\u0a2d\u0a2b\3\2\2\2\u0a2d\u0a2e\3\2\2\2\u0a2e\u0a30")
        buf.write("\3\2\2\2\u0a2f\u0a2d\3\2\2\2\u0a30\u0a32\5\u0098M\2\u0a31")
        buf.write("\u0a26\3\2\2\2\u0a32\u0a35\3\2\2\2\u0a33\u0a31\3\2\2\2")
        buf.write("\u0a33\u0a34\3\2\2\2\u0a34\u0a3d\3\2\2\2\u0a35\u0a33\3")
        buf.write("\2\2\2\u0a36\u0a38\7\7\2\2\u0a37\u0a36\3\2\2\2\u0a38\u0a3b")
        buf.write("\3\2\2\2\u0a39\u0a37\3\2\2\2\u0a39\u0a3a\3\2\2\2\u0a3a")
        buf.write("\u0a3c\3\2\2\2\u0a3b\u0a39\3\2\2\2\u0a3c\u0a3e\7\n\2\2")
        buf.write("\u0a3d\u0a39\3\2\2\2\u0a3d\u0a3e\3\2\2\2\u0a3e\u0a42\3")
        buf.write("\2\2\2\u0a3f\u0a41\7\7\2\2\u0a40\u0a3f\3\2\2\2\u0a41\u0a44")
        buf.write("\3\2\2\2\u0a42\u0a40\3\2\2\2\u0a42\u0a43\3\2\2\2\u0a43")
        buf.write("\u0a46\3\2\2\2\u0a44\u0a42\3\2\2\2\u0a45\u0a22\3\2\2\2")
        buf.write("\u0a45\u0a46\3\2\2\2\u0a46\u0a47\3\2\2\2\u0a47\u0a48\7")
        buf.write("\16\2\2\u0a48\u00d9\3\2\2\2\u0a49\u0a4a\t\b\2\2\u0a4a")
        buf.write("\u00db\3\2\2\2\u0a4b\u0a4e\5\u00dep\2\u0a4c\u0a4e\5\u00e0")
        buf.write("q\2\u0a4d\u0a4b\3\2\2\2\u0a4d\u0a4c\3\2\2\2\u0a4e\u00dd")
        buf.write("\3\2\2\2\u0a4f\u0a54\7\u0097\2\2\u0a50\u0a53\5\u00e2r")
        buf.write("\2\u0a51\u0a53\5\u00e4s\2\u0a52\u0a50\3\2\2\2\u0a52\u0a51")
        buf.write("\3\2\2\2\u0a53\u0a56\3\2\2\2\u0a54\u0a52\3\2\2\2\u0a54")
        buf.write("\u0a55\3\2\2\2\u0a55\u0a57\3\2\2\2\u0a56\u0a54\3\2\2\2")
        buf.write("\u0a57\u0a58\7\u00a0\2\2\u0a58\u00df\3\2\2\2\u0a59\u0a5f")
        buf.write("\7\u0098\2\2\u0a5a\u0a5e\5\u00e6t\2\u0a5b\u0a5e\5\u00e8")
        buf.write("u\2\u0a5c\u0a5e\7\u00a6\2\2\u0a5d\u0a5a\3\2\2\2\u0a5d")
        buf.write("\u0a5b\3\2\2\2\u0a5d\u0a5c\3\2\2\2\u0a5e\u0a61\3\2\2\2")
        buf.write("\u0a5f\u0a5d\3\2\2\2\u0a5f\u0a60\3\2\2\2\u0a60\u0a62\3")
        buf.write("\2\2\2\u0a61\u0a5f\3\2\2\2\u0a62\u0a63\7\u00a5\2\2\u0a63")
        buf.write("\u00e1\3\2\2\2\u0a64\u0a65\t\t\2\2\u0a65\u00e3\3\2\2\2")
        buf.write("\u0a66\u0a6a\7\u00a4\2\2\u0a67\u0a69\7\7\2\2\u0a68\u0a67")
        buf.write("\3\2\2\2\u0a69\u0a6c\3\2\2\2\u0a6a\u0a68\3\2\2\2\u0a6a")
        buf.write("\u0a6b\3\2\2\2\u0a6b\u0a6d\3\2\2\2\u0a6c\u0a6a\3\2\2\2")
        buf.write("\u0a6d\u0a71\5\u0098M\2\u0a6e\u0a70\7\7\2\2\u0a6f\u0a6e")
        buf.write("\3\2\2\2\u0a70\u0a73\3\2\2\2\u0a71\u0a6f\3\2\2\2\u0a71")
        buf.write("\u0a72\3\2\2\2\u0a72\u0a74\3\2\2\2\u0a73\u0a71\3\2\2\2")
        buf.write("\u0a74\u0a75\7\20\2\2\u0a75\u00e5\3\2\2\2\u0a76\u0a77")
        buf.write("\t\n\2\2\u0a77\u00e7\3\2\2\2\u0a78\u0a7c\7\u00a9\2\2\u0a79")
        buf.write("\u0a7b\7\7\2\2\u0a7a\u0a79\3\2\2\2\u0a7b\u0a7e\3\2\2\2")
        buf.write("\u0a7c\u0a7a\3\2\2\2\u0a7c\u0a7d\3\2\2\2\u0a7d\u0a7f\3")
        buf.write("\2\2\2\u0a7e\u0a7c\3\2\2\2\u0a7f\u0a83\5\u0098M\2\u0a80")
        buf.write("\u0a82\7\7\2\2\u0a81\u0a80\3\2\2\2\u0a82\u0a85\3\2\2\2")
        buf.write("\u0a83\u0a81\3\2\2\2\u0a83\u0a84\3\2\2\2\u0a84\u0a86\3")
        buf.write("\2\2\2\u0a85\u0a83\3\2\2\2\u0a86\u0a87\7\20\2\2\u0a87")
        buf.write("\u00e9\3\2\2\2\u0a88\u0a8c\7\17\2\2\u0a89\u0a8b\7\7\2")
        buf.write("\2\u0a8a\u0a89\3\2\2\2\u0a8b\u0a8e\3\2\2\2\u0a8c\u0a8a")
        buf.write("\3\2\2\2\u0a8c\u0a8d\3\2\2\2\u0a8d\u0a9f\3\2\2\2\u0a8e")
        buf.write("\u0a8c\3\2\2\2\u0a8f\u0a91\5\u00ecw\2\u0a90\u0a8f\3\2")
        buf.write("\2\2\u0a90\u0a91\3\2\2\2\u0a91\u0a95\3\2\2\2\u0a92\u0a94")
        buf.write("\7\7\2\2\u0a93\u0a92\3\2\2\2\u0a94\u0a97\3\2\2\2\u0a95")
        buf.write("\u0a93\3\2\2\2\u0a95\u0a96\3\2\2\2\u0a96\u0a98\3\2\2\2")
        buf.write("\u0a97\u0a95\3\2\2\2\u0a98\u0a9c\7$\2\2\u0a99\u0a9b\7")
        buf.write("\7\2\2\u0a9a\u0a99\3\2\2\2\u0a9b\u0a9e\3\2\2\2\u0a9c\u0a9a")
        buf.write("\3\2\2\2\u0a9c\u0a9d\3\2\2\2\u0a9d\u0aa0\3\2\2\2\u0a9e")
        buf.write("\u0a9c\3\2\2\2\u0a9f\u0a90\3\2\2\2\u0a9f\u0aa0\3\2\2\2")
        buf.write("\u0aa0\u0aa1\3\2\2\2\u0aa1\u0aa5\5\u0080A\2\u0aa2\u0aa4")
        buf.write("\7\7\2\2\u0aa3\u0aa2\3\2\2\2\u0aa4\u0aa7\3\2\2\2\u0aa5")
        buf.write("\u0aa3\3\2\2\2\u0aa5\u0aa6\3\2\2\2\u0aa6\u0aa8\3\2\2\2")
        buf.write("\u0aa7\u0aa5\3\2\2\2\u0aa8\u0aa9\7\20\2\2\u0aa9\u00eb")
        buf.write("\3\2\2\2\u0aaa\u0abb\5\u00eex\2\u0aab\u0aad\7\7\2\2\u0aac")
        buf.write("\u0aab\3\2\2\2\u0aad\u0ab0\3\2\2\2\u0aae\u0aac\3\2\2\2")
        buf.write("\u0aae\u0aaf\3\2\2\2\u0aaf\u0ab1\3\2\2\2\u0ab0\u0aae\3")
        buf.write("\2\2\2\u0ab1\u0ab5\7\n\2\2\u0ab2\u0ab4\7\7\2\2\u0ab3\u0ab2")
        buf.write("\3\2\2\2\u0ab4\u0ab7\3\2\2\2\u0ab5\u0ab3\3\2\2\2\u0ab5")
        buf.write("\u0ab6\3\2\2\2\u0ab6\u0ab8\3\2\2\2\u0ab7\u0ab5\3\2\2\2")
        buf.write("\u0ab8\u0aba\5\u00eex\2\u0ab9\u0aae\3\2\2\2\u0aba\u0abd")
        buf.write("\3\2\2\2\u0abb\u0ab9\3\2\2\2\u0abb\u0abc\3\2\2\2\u0abc")
        buf.write("\u0ac5\3\2\2\2\u0abd\u0abb\3\2\2\2\u0abe\u0ac0\7\7\2\2")
        buf.write("\u0abf\u0abe\3\2\2\2\u0ac0\u0ac3\3\2\2\2\u0ac1\u0abf\3")
        buf.write("\2\2\2\u0ac1\u0ac2\3\2\2\2\u0ac2\u0ac4\3\2\2\2\u0ac3\u0ac1")
        buf.write("\3\2\2\2\u0ac4\u0ac6\7\n\2\2\u0ac5\u0ac1\3\2\2\2\u0ac5")
        buf.write("\u0ac6\3\2\2\2\u0ac6\u00ed\3\2\2\2\u0ac7\u0ada\5D#\2\u0ac8")
        buf.write("\u0ad7\5F$\2\u0ac9\u0acb\7\7\2\2\u0aca\u0ac9\3\2\2\2\u0acb")
        buf.write("\u0ace\3\2\2\2\u0acc\u0aca\3\2\2\2\u0acc\u0acd\3\2\2\2")
        buf.write("\u0acd\u0acf\3\2\2\2\u0ace\u0acc\3\2\2\2\u0acf\u0ad3\7")
        buf.write("\34\2\2\u0ad0\u0ad2\7\7\2\2\u0ad1\u0ad0\3\2\2\2\u0ad2")
        buf.write("\u0ad5\3\2\2\2\u0ad3\u0ad1\3\2\2\2\u0ad3\u0ad4\3\2\2\2")
        buf.write("\u0ad4\u0ad6\3\2\2\2\u0ad5\u0ad3\3\2\2\2\u0ad6\u0ad8\5")
        buf.write("d\63\2\u0ad7\u0acc\3\2\2\2\u0ad7\u0ad8\3\2\2\2\u0ad8\u0ada")
        buf.write("\3\2\2\2\u0ad9\u0ac7\3\2\2\2\u0ad9\u0ac8\3\2\2\2\u0ada")
        buf.write("\u00ef\3\2\2\2\u0adb\u0aeb\7L\2\2\u0adc\u0ade\7\7\2\2")
        buf.write("\u0add\u0adc\3\2\2\2\u0ade\u0ae1\3\2\2\2\u0adf\u0add\3")
        buf.write("\2\2\2\u0adf\u0ae0\3\2\2\2\u0ae0\u0ae2\3\2\2\2\u0ae1\u0adf")
        buf.write("\3\2\2\2\u0ae2\u0ae6\5d\63\2\u0ae3\u0ae5\7\7\2\2\u0ae4")
        buf.write("\u0ae3\3\2\2\2\u0ae5\u0ae8\3\2\2\2\u0ae6\u0ae4\3\2\2\2")
        buf.write("\u0ae6\u0ae7\3\2\2\2\u0ae7\u0ae9\3\2\2\2\u0ae8\u0ae6\3")
        buf.write("\2\2\2\u0ae9\u0aea\7\t\2\2\u0aea\u0aec\3\2\2\2\u0aeb\u0adf")
        buf.write("\3\2\2\2\u0aeb\u0aec\3\2\2\2\u0aec\u0af0\3\2\2\2\u0aed")
        buf.write("\u0aef\7\7\2\2\u0aee\u0aed\3\2\2\2\u0aef\u0af2\3\2\2\2")
        buf.write("\u0af0\u0aee\3\2\2\2\u0af0\u0af1\3\2\2\2\u0af1\u0af3\3")
        buf.write("\2\2\2\u0af2\u0af0\3\2\2\2\u0af3\u0b02\5P)\2\u0af4\u0af6")
        buf.write("\7\7\2\2\u0af5\u0af4\3\2\2\2\u0af6\u0af9\3\2\2\2\u0af7")
        buf.write("\u0af5\3\2\2\2\u0af7\u0af8\3\2\2\2\u0af8\u0afa\3\2\2\2")
        buf.write("\u0af9\u0af7\3\2\2\2\u0afa\u0afe\7\34\2\2\u0afb\u0afd")
        buf.write("\7\7\2\2\u0afc\u0afb\3\2\2\2\u0afd\u0b00\3\2\2\2\u0afe")
        buf.write("\u0afc\3\2\2\2\u0afe\u0aff\3\2\2\2\u0aff\u0b01\3\2\2\2")
        buf.write("\u0b00\u0afe\3\2\2\2\u0b01\u0b03\5d\63\2\u0b02\u0af7\3")
        buf.write("\2\2\2\u0b02\u0b03\3\2\2\2\u0b03\u0b0b\3\2\2\2\u0b04\u0b06")
        buf.write("\7\7\2\2\u0b05\u0b04\3\2\2\2\u0b06\u0b09\3\2\2\2\u0b07")
        buf.write("\u0b05\3\2\2\2\u0b07\u0b08\3\2\2\2\u0b08\u0b0a\3\2\2\2")
        buf.write("\u0b09\u0b07\3\2\2\2\u0b0a\u0b0c\5\60\31\2\u0b0b\u0b07")
        buf.write("\3\2\2\2\u0b0b\u0b0c\3\2\2\2\u0b0c\u0b14\3\2\2\2\u0b0d")
        buf.write("\u0b0f\7\7\2\2\u0b0e\u0b0d\3\2\2\2\u0b0f\u0b12\3\2\2\2")
        buf.write("\u0b10\u0b0e\3\2\2\2\u0b10\u0b11\3\2\2\2\u0b11\u0b13\3")
        buf.write("\2\2\2\u0b12\u0b10\3\2\2\2\u0b13\u0b15\5B\"\2\u0b14\u0b10")
        buf.write("\3\2\2\2\u0b14\u0b15\3\2\2\2\u0b15\u00f1\3\2\2\2\u0b16")
        buf.write("\u0b19\5\u00eav\2\u0b17\u0b19\5\u00f0y\2\u0b18\u0b16\3")
        buf.write("\2\2\2\u0b18\u0b17\3\2\2\2\u0b19\u00f3\3\2\2\2\u0b1a\u0b2f")
        buf.write("\7M\2\2\u0b1b\u0b1d\7\7\2\2\u0b1c\u0b1b\3\2\2\2\u0b1d")
        buf.write("\u0b20\3\2\2\2\u0b1e\u0b1c\3\2\2\2\u0b1e\u0b1f\3\2\2\2")
        buf.write("\u0b1f\u0b21\3\2\2\2\u0b20\u0b1e\3\2\2\2\u0b21\u0b25\7")
        buf.write("\34\2\2\u0b22\u0b24\7\7\2\2\u0b23\u0b22\3\2\2\2\u0b24")
        buf.write("\u0b27\3\2\2\2\u0b25\u0b23\3\2\2\2\u0b25\u0b26\3\2\2\2")
        buf.write("\u0b26\u0b28\3\2\2\2\u0b27\u0b25\3\2\2\2\u0b28\u0b2c\5")
        buf.write("\"\22\2\u0b29\u0b2b\7\7\2\2\u0b2a\u0b29\3\2\2\2\u0b2b")
        buf.write("\u0b2e\3\2\2\2\u0b2c\u0b2a\3\2\2\2\u0b2c\u0b2d\3\2\2\2")
        buf.write("\u0b2d\u0b30\3\2\2\2\u0b2e\u0b2c\3\2\2\2\u0b2f\u0b1e\3")
        buf.write("\2\2\2\u0b2f\u0b30\3\2\2\2\u0b30\u0b38\3\2\2\2\u0b31\u0b33")
        buf.write("\7\7\2\2\u0b32\u0b31\3\2\2\2\u0b33\u0b36\3\2\2\2\u0b34")
        buf.write("\u0b32\3\2\2\2\u0b34\u0b35\3\2\2\2\u0b35\u0b37\3\2\2\2")
        buf.write("\u0b36\u0b34\3\2\2\2\u0b37\u0b39\5\34\17\2\u0b38\u0b34")
        buf.write("\3\2\2\2\u0b38\u0b39\3\2\2\2\u0b39\u00f5\3\2\2\2\u0b3a")
        buf.write("\u0b3b\t\13\2\2\u0b3b\u00f7\3\2\2\2\u0b3c\u0b4d\7V\2\2")
        buf.write("\u0b3d\u0b41\7\60\2\2\u0b3e\u0b40\7\7\2\2\u0b3f\u0b3e")
        buf.write("\3\2\2\2\u0b40\u0b43\3\2\2\2\u0b41\u0b3f\3\2\2\2\u0b41")
        buf.write("\u0b42\3\2\2\2\u0b42\u0b44\3\2\2\2\u0b43\u0b41\3\2\2\2")
        buf.write("\u0b44\u0b48\5d\63\2\u0b45\u0b47\7\7\2\2\u0b46\u0b45\3")
        buf.write("\2\2\2\u0b47\u0b4a\3\2\2\2\u0b48\u0b46\3\2\2\2\u0b48\u0b49")
        buf.write("\3\2\2\2\u0b49\u0b4b\3\2\2\2\u0b4a\u0b48\3\2\2\2\u0b4b")
        buf.write("\u0b4c\7\61\2\2\u0b4c\u0b4e\3\2\2\2\u0b4d\u0b3d\3\2\2")
        buf.write("\2\u0b4d\u0b4e\3\2\2\2\u0b4e\u0b51\3\2\2\2\u0b4f\u0b50")
        buf.write("\7*\2\2\u0b50\u0b52\5\u0158\u00ad\2\u0b51\u0b4f\3\2\2")
        buf.write("\2\u0b51\u0b52\3\2\2\2\u0b52\u0b55\3\2\2\2\u0b53\u0b55")
        buf.write("\7>\2\2\u0b54\u0b3c\3\2\2\2\u0b54\u0b53\3\2\2\2\u0b55")
        buf.write("\u00f9\3\2\2\2\u0b56\u0b5a\7Y\2\2\u0b57\u0b59\7\7\2\2")
        buf.write("\u0b58\u0b57\3\2\2\2\u0b59\u0b5c\3\2\2\2\u0b5a\u0b58\3")
        buf.write("\2\2\2\u0b5a\u0b5b\3\2\2\2\u0b5b\u0b5d\3\2\2\2\u0b5c\u0b5a")
        buf.write("\3\2\2\2\u0b5d\u0b61\7\13\2\2\u0b5e\u0b60\7\7\2\2\u0b5f")
        buf.write("\u0b5e\3\2\2\2\u0b60\u0b63\3\2\2\2\u0b61\u0b5f\3\2\2\2")
        buf.write("\u0b61\u0b62\3\2\2\2\u0b62\u0b64\3\2\2\2\u0b63\u0b61\3")
        buf.write("\2\2\2\u0b64\u0b68\5\u0098M\2\u0b65\u0b67\7\7\2\2\u0b66")
        buf.write("\u0b65\3\2\2\2\u0b67\u0b6a\3\2\2\2\u0b68\u0b66\3\2\2\2")
        buf.write("\u0b68\u0b69\3\2\2\2\u0b69\u0b6b\3\2\2\2\u0b6a\u0b68\3")
        buf.write("\2\2\2\u0b6b\u0b6f\7\f\2\2\u0b6c\u0b6e\7\7\2\2\u0b6d\u0b6c")
        buf.write("\3\2\2\2\u0b6e\u0b71\3\2\2\2\u0b6f\u0b6d\3\2\2\2\u0b6f")
        buf.write("\u0b70\3\2\2\2\u0b70\u0b91\3\2\2\2\u0b71\u0b6f\3\2\2\2")
        buf.write("\u0b72\u0b92\5\u0086D\2\u0b73\u0b75\5\u0086D\2\u0b74\u0b73")
        buf.write("\3\2\2\2\u0b74\u0b75\3\2\2\2\u0b75\u0b79\3\2\2\2\u0b76")
        buf.write("\u0b78\7\7\2\2\u0b77\u0b76\3\2\2\2\u0b78\u0b7b\3\2\2\2")
        buf.write("\u0b79\u0b77\3\2\2\2\u0b79\u0b7a\3\2\2\2\u0b7a\u0b7d\3")
        buf.write("\2\2\2\u0b7b\u0b79\3\2\2\2\u0b7c\u0b7e\7\35\2\2\u0b7d")
        buf.write("\u0b7c\3\2\2\2\u0b7d\u0b7e\3\2\2\2\u0b7e\u0b82\3\2\2\2")
        buf.write("\u0b7f\u0b81\7\7\2\2\u0b80\u0b7f\3\2\2\2\u0b81\u0b84\3")
        buf.write("\2\2\2\u0b82\u0b80\3\2\2\2\u0b82\u0b83\3\2\2\2\u0b83\u0b85")
        buf.write("\3\2\2\2\u0b84\u0b82\3\2\2\2\u0b85\u0b89\7Z\2\2\u0b86")
        buf.write("\u0b88\7\7\2\2\u0b87\u0b86\3\2\2\2\u0b88\u0b8b\3\2\2\2")
        buf.write("\u0b89\u0b87\3\2\2\2\u0b89\u0b8a\3\2\2\2\u0b8a\u0b8e\3")
        buf.write("\2\2\2\u0b8b\u0b89\3\2\2\2\u0b8c\u0b8f\5\u0086D\2\u0b8d")
        buf.write("\u0b8f\7\35\2\2\u0b8e\u0b8c\3\2\2\2\u0b8e\u0b8d\3\2\2")
        buf.write("\2\u0b8f\u0b92\3\2\2\2\u0b90\u0b92\7\35\2\2\u0b91\u0b72")
        buf.write("\3\2\2\2\u0b91\u0b74\3\2\2\2\u0b91\u0b90\3\2\2\2\u0b92")
        buf.write("\u00fb\3\2\2\2\u0b93\u0bb5\7\13\2\2\u0b94\u0b96\5\u014e")
        buf.write("\u00a8\2\u0b95\u0b94\3\2\2\2\u0b96\u0b99\3\2\2\2\u0b97")
        buf.write("\u0b95\3\2\2\2\u0b97\u0b98\3\2\2\2\u0b98\u0b9d\3\2\2\2")
        buf.write("\u0b99\u0b97\3\2\2\2\u0b9a\u0b9c\7\7\2\2\u0b9b\u0b9a\3")
        buf.write("\2\2\2\u0b9c\u0b9f\3\2\2\2\u0b9d\u0b9b\3\2\2\2\u0b9d\u0b9e")
        buf.write("\3\2\2\2\u0b9e\u0ba0\3\2\2\2\u0b9f\u0b9d\3\2\2\2\u0ba0")
        buf.write("\u0ba4\7N\2\2\u0ba1\u0ba3\7\7\2\2\u0ba2\u0ba1\3\2\2\2")
        buf.write("\u0ba3\u0ba6\3\2\2\2\u0ba4\u0ba2\3\2\2\2\u0ba4\u0ba5\3")
        buf.write("\2\2\2\u0ba5\u0ba7\3\2\2\2\u0ba6\u0ba4\3\2\2\2\u0ba7\u0bab")
        buf.write("\5D#\2\u0ba8\u0baa\7\7\2\2\u0ba9\u0ba8\3\2\2\2\u0baa\u0bad")
        buf.write("\3\2\2\2\u0bab\u0ba9\3\2\2\2\u0bab\u0bac\3\2\2\2\u0bac")
        buf.write("\u0bae\3\2\2\2\u0bad\u0bab\3\2\2\2\u0bae\u0bb2\7\36\2")
        buf.write("\2\u0baf\u0bb1\7\7\2\2\u0bb0\u0baf\3\2\2\2\u0bb1\u0bb4")
        buf.write("\3\2\2\2\u0bb2\u0bb0\3\2\2\2\u0bb2\u0bb3\3\2\2\2\u0bb3")
        buf.write("\u0bb6\3\2\2\2\u0bb4\u0bb2\3\2\2\2\u0bb5\u0b97\3\2\2\2")
        buf.write("\u0bb5\u0bb6\3\2\2\2\u0bb6\u0bb7\3\2\2\2\u0bb7\u0bb8\5")
        buf.write("\u0098M\2\u0bb8\u0bb9\7\f\2\2\u0bb9\u00fd\3\2\2\2\u0bba")
        buf.write("\u0bbe\7[\2\2\u0bbb\u0bbd\7\7\2\2\u0bbc\u0bbb\3\2\2\2")
        buf.write("\u0bbd\u0bc0\3\2\2\2\u0bbe\u0bbc\3\2\2\2\u0bbe\u0bbf\3")
        buf.write("\2\2\2\u0bbf\u0bc2\3\2\2\2\u0bc0\u0bbe\3\2\2\2\u0bc1\u0bc3")
        buf.write("\5\u00fc\177\2\u0bc2\u0bc1\3\2\2\2\u0bc2\u0bc3\3\2\2\2")
        buf.write("\u0bc3\u0bc7\3\2\2\2\u0bc4\u0bc6\7\7\2\2\u0bc5\u0bc4\3")
        buf.write("\2\2\2\u0bc6\u0bc9\3\2\2\2\u0bc7\u0bc5\3\2\2\2\u0bc7\u0bc8")
        buf.write("\3\2\2\2\u0bc8\u0bca\3\2\2\2\u0bc9\u0bc7\3\2\2\2\u0bca")
        buf.write("\u0bce\7\17\2\2\u0bcb\u0bcd\7\7\2\2\u0bcc\u0bcb\3\2\2")
        buf.write("\2\u0bcd\u0bd0\3\2\2\2\u0bce\u0bcc\3\2\2\2\u0bce\u0bcf")
        buf.write("\3\2\2\2\u0bcf\u0bda\3\2\2\2\u0bd0\u0bce\3\2\2\2\u0bd1")
        buf.write("\u0bd5\5\u0100\u0081\2\u0bd2\u0bd4\7\7\2\2\u0bd3\u0bd2")
        buf.write("\3\2\2\2\u0bd4\u0bd7\3\2\2\2\u0bd5\u0bd3\3\2\2\2\u0bd5")
        buf.write("\u0bd6\3\2\2\2\u0bd6\u0bd9\3\2\2\2\u0bd7\u0bd5\3\2\2\2")
        buf.write("\u0bd8\u0bd1\3\2\2\2\u0bd9\u0bdc\3\2\2\2\u0bda\u0bd8\3")
        buf.write("\2\2\2\u0bda\u0bdb\3\2\2\2\u0bdb\u0be0\3\2\2\2\u0bdc\u0bda")
        buf.write("\3\2\2\2\u0bdd\u0bdf\7\7\2\2\u0bde\u0bdd\3\2\2\2\u0bdf")
        buf.write("\u0be2\3\2\2\2\u0be0\u0bde\3\2\2\2\u0be0\u0be1\3\2\2\2")
        buf.write("\u0be1\u0be3\3\2\2\2\u0be2\u0be0\3\2\2\2\u0be3\u0be4\7")
        buf.write("\20\2\2\u0be4\u00ff\3\2\2\2\u0be5\u0bf6\5\u0102\u0082")
        buf.write("\2\u0be6\u0be8\7\7\2\2\u0be7\u0be6\3\2\2\2\u0be8\u0beb")
        buf.write("\3\2\2\2\u0be9\u0be7\3\2\2\2\u0be9\u0bea\3\2\2\2\u0bea")
        buf.write("\u0bec\3\2\2\2\u0beb\u0be9\3\2\2\2\u0bec\u0bf0\7\n\2\2")
        buf.write("\u0bed\u0bef\7\7\2\2\u0bee\u0bed\3\2\2\2\u0bef\u0bf2\3")
        buf.write("\2\2\2\u0bf0\u0bee\3\2\2\2\u0bf0\u0bf1\3\2\2\2\u0bf1\u0bf3")
        buf.write("\3\2\2\2\u0bf2\u0bf0\3\2\2\2\u0bf3\u0bf5\5\u0102\u0082")
        buf.write("\2\u0bf4\u0be9\3\2\2\2\u0bf5\u0bf8\3\2\2\2\u0bf6\u0bf4")
        buf.write("\3\2\2\2\u0bf6\u0bf7\3\2\2\2\u0bf7\u0c00\3\2\2\2\u0bf8")
        buf.write("\u0bf6\3\2\2\2\u0bf9\u0bfb\7\7\2\2\u0bfa\u0bf9\3\2\2\2")
        buf.write("\u0bfb\u0bfe\3\2\2\2\u0bfc\u0bfa\3\2\2\2\u0bfc\u0bfd\3")
        buf.write("\2\2\2\u0bfd\u0bff\3\2\2\2\u0bfe\u0bfc\3\2\2\2\u0bff\u0c01")
        buf.write("\7\n\2\2\u0c00\u0bfc\3\2\2\2\u0c00\u0c01\3\2\2\2\u0c01")
        buf.write("\u0c05\3\2\2\2\u0c02\u0c04\7\7\2\2\u0c03\u0c02\3\2\2\2")
        buf.write("\u0c04\u0c07\3\2\2\2\u0c05\u0c03\3\2\2\2\u0c05\u0c06\3")
        buf.write("\2\2\2\u0c06\u0c08\3\2\2\2\u0c07\u0c05\3\2\2\2\u0c08\u0c0c")
        buf.write("\7$\2\2\u0c09\u0c0b\7\7\2\2\u0c0a\u0c09\3\2\2\2\u0c0b")
        buf.write("\u0c0e\3\2\2\2\u0c0c\u0c0a\3\2\2\2\u0c0c\u0c0d\3\2\2\2")
        buf.write("\u0c0d\u0c0f\3\2\2\2\u0c0e\u0c0c\3\2\2\2\u0c0f\u0c11\5")
        buf.write("\u0086D\2\u0c10\u0c12\5\u0094K\2\u0c11\u0c10\3\2\2\2\u0c11")
        buf.write("\u0c12\3\2\2\2\u0c12\u0c26\3\2\2\2\u0c13\u0c17\7Z\2\2")
        buf.write("\u0c14\u0c16\7\7\2\2\u0c15\u0c14\3\2\2\2\u0c16\u0c19\3")
        buf.write("\2\2\2\u0c17\u0c15\3\2\2\2\u0c17\u0c18\3\2\2\2\u0c18\u0c1a")
        buf.write("\3\2\2\2\u0c19\u0c17\3\2\2\2\u0c1a\u0c1e\7$\2\2\u0c1b")
        buf.write("\u0c1d\7\7\2\2\u0c1c\u0c1b\3\2\2\2\u0c1d\u0c20\3\2\2\2")
        buf.write("\u0c1e\u0c1c\3\2\2\2\u0c1e\u0c1f\3\2\2\2\u0c1f\u0c21\3")
        buf.write("\2\2\2\u0c20\u0c1e\3\2\2\2\u0c21\u0c23\5\u0086D\2\u0c22")
        buf.write("\u0c24\5\u0094K\2\u0c23\u0c22\3\2\2\2\u0c23\u0c24\3\2")
        buf.write("\2\2\u0c24\u0c26\3\2\2\2\u0c25\u0be5\3\2\2\2\u0c25\u0c13")
        buf.write("\3\2\2\2\u0c26\u0101\3\2\2\2\u0c27\u0c2b\5\u0098M\2\u0c28")
        buf.write("\u0c2b\5\u0104\u0083\2\u0c29\u0c2b\5\u0106\u0084\2\u0c2a")
        buf.write("\u0c27\3\2\2\2\u0c2a\u0c28\3\2\2\2\u0c2a\u0c29\3\2\2\2")
        buf.write("\u0c2b\u0103\3\2\2\2\u0c2c\u0c30\5\u0118\u008d\2\u0c2d")
        buf.write("\u0c2f\7\7\2\2\u0c2e\u0c2d\3\2\2\2\u0c2f\u0c32\3\2\2\2")
        buf.write("\u0c30\u0c2e\3\2\2\2\u0c30\u0c31\3\2\2\2\u0c31\u0c33\3")
        buf.write("\2\2\2\u0c32\u0c30\3\2\2\2\u0c33\u0c34\5\u0098M\2\u0c34")
        buf.write("\u0105\3\2\2\2\u0c35\u0c39\5\u011a\u008e\2\u0c36\u0c38")
        buf.write("\7\7\2\2\u0c37\u0c36\3\2\2\2\u0c38\u0c3b\3\2\2\2\u0c39")
        buf.write("\u0c37\3\2\2\2\u0c39\u0c3a\3\2\2\2\u0c3a\u0c3c\3\2\2\2")
        buf.write("\u0c3b\u0c39\3\2\2\2\u0c3c\u0c3d\5d\63\2\u0c3d\u0107\3")
        buf.write("\2\2\2\u0c3e\u0c42\7\\\2\2\u0c3f\u0c41\7\7\2\2\u0c40\u0c3f")
        buf.write("\3\2\2\2\u0c41\u0c44\3\2\2\2\u0c42\u0c40\3\2\2\2\u0c42")
        buf.write("\u0c43\3\2\2\2\u0c43\u0c45\3\2\2\2\u0c44\u0c42\3\2\2\2")
        buf.write("\u0c45\u0c61\5\u0088E\2\u0c46\u0c48\7\7\2\2\u0c47\u0c46")
        buf.write("\3\2\2\2\u0c48\u0c4b\3\2\2\2\u0c49\u0c47\3\2\2\2\u0c49")
        buf.write("\u0c4a\3\2\2\2\u0c4a\u0c4c\3\2\2\2\u0c4b\u0c49\3\2\2\2")
        buf.write("\u0c4c\u0c4e\5\u010a\u0086\2\u0c4d\u0c49\3\2\2\2\u0c4e")
        buf.write("\u0c4f\3\2\2\2\u0c4f\u0c4d\3\2\2\2\u0c4f\u0c50\3\2\2\2")
        buf.write("\u0c50\u0c58\3\2\2\2\u0c51\u0c53\7\7\2\2\u0c52\u0c51\3")
        buf.write("\2\2\2\u0c53\u0c56\3\2\2\2\u0c54\u0c52\3\2\2\2\u0c54\u0c55")
        buf.write("\3\2\2\2\u0c55\u0c57\3\2\2\2\u0c56\u0c54\3\2\2\2\u0c57")
        buf.write("\u0c59\5\u010c\u0087\2\u0c58\u0c54\3\2\2\2\u0c58\u0c59")
        buf.write("\3\2\2\2\u0c59\u0c62\3\2\2\2\u0c5a\u0c5c\7\7\2\2\u0c5b")
        buf.write("\u0c5a\3\2\2\2\u0c5c\u0c5f\3\2\2\2\u0c5d\u0c5b\3\2\2\2")
        buf.write("\u0c5d\u0c5e\3\2\2\2\u0c5e\u0c60\3\2\2\2\u0c5f\u0c5d\3")
        buf.write("\2\2\2\u0c60\u0c62\5\u010c\u0087\2\u0c61\u0c4d\3\2\2\2")
        buf.write("\u0c61\u0c5d\3\2\2\2\u0c62\u0109\3\2\2\2\u0c63\u0c67\7")
        buf.write("]\2\2\u0c64\u0c66\7\7\2\2\u0c65\u0c64\3\2\2\2\u0c66\u0c69")
        buf.write("\3\2\2\2\u0c67\u0c65\3\2\2\2\u0c67\u0c68\3\2\2\2\u0c68")
        buf.write("\u0c6a\3\2\2\2\u0c69\u0c67\3\2\2\2\u0c6a\u0c6e\7\13\2")
        buf.write("\2\u0c6b\u0c6d\5\u014e\u00a8\2\u0c6c\u0c6b\3\2\2\2\u0c6d")
        buf.write("\u0c70\3\2\2\2\u0c6e\u0c6c\3\2\2\2\u0c6e\u0c6f\3\2\2\2")
        buf.write("\u0c6f\u0c71\3\2\2\2\u0c70\u0c6e\3\2\2\2\u0c71\u0c72\5")
        buf.write("\u0158\u00ad\2\u0c72\u0c73\7\34\2\2\u0c73\u0c7b\5d\63")
        buf.write("\2\u0c74\u0c76\7\7\2\2\u0c75\u0c74\3\2\2\2\u0c76\u0c79")
        buf.write("\3\2\2\2\u0c77\u0c75\3\2\2\2\u0c77\u0c78\3\2\2\2\u0c78")
        buf.write("\u0c7a\3\2\2\2\u0c79\u0c77\3\2\2\2\u0c7a\u0c7c\7\n\2\2")
        buf.write("\u0c7b\u0c77\3\2\2\2\u0c7b\u0c7c\3\2\2\2\u0c7c\u0c7d\3")
        buf.write("\2\2\2\u0c7d\u0c81\7\f\2\2\u0c7e\u0c80\7\7\2\2\u0c7f\u0c7e")
        buf.write("\3\2\2\2\u0c80\u0c83\3\2\2\2\u0c81\u0c7f\3\2\2\2\u0c81")
        buf.write("\u0c82\3\2\2\2\u0c82\u0c84\3\2\2\2\u0c83\u0c81\3\2\2\2")
        buf.write("\u0c84\u0c85\5\u0088E\2\u0c85\u010b\3\2\2\2\u0c86\u0c8a")
        buf.write("\7^\2\2\u0c87\u0c89\7\7\2\2\u0c88\u0c87\3\2\2\2\u0c89")
        buf.write("\u0c8c\3\2\2\2\u0c8a\u0c88\3\2\2\2\u0c8a\u0c8b\3\2\2\2")
        buf.write("\u0c8b\u0c8d\3\2\2\2\u0c8c\u0c8a\3\2\2\2\u0c8d\u0c8e\5")
        buf.write("\u0088E\2\u0c8e\u010d\3\2\2\2\u0c8f\u0c93\7b\2\2\u0c90")
        buf.write("\u0c92\7\7\2\2\u0c91\u0c90\3\2\2\2\u0c92\u0c95\3\2\2\2")
        buf.write("\u0c93\u0c91\3\2\2\2\u0c93\u0c94\3\2\2\2\u0c94\u0c96\3")
        buf.write("\2\2\2\u0c95\u0c93\3\2\2\2\u0c96\u0ca0\5\u0098M\2\u0c97")
        buf.write("\u0c99\t\f\2\2\u0c98\u0c9a\5\u0098M\2\u0c99\u0c98\3\2")
        buf.write("\2\2\u0c99\u0c9a\3\2\2\2\u0c9a\u0ca0\3\2\2\2\u0c9b\u0ca0")
        buf.write("\7d\2\2\u0c9c\u0ca0\7;\2\2\u0c9d\u0ca0\7e\2\2\u0c9e\u0ca0")
        buf.write("\7<\2\2\u0c9f\u0c8f\3\2\2\2\u0c9f\u0c97\3\2\2\2\u0c9f")
        buf.write("\u0c9b\3\2\2\2\u0c9f\u0c9c\3\2\2\2\u0c9f\u0c9d\3\2\2\2")
        buf.write("\u0c9f\u0c9e\3\2\2\2\u0ca0\u010f\3\2\2\2\u0ca1\u0ca3\5")
        buf.write("|?\2\u0ca2\u0ca1\3\2\2\2\u0ca2\u0ca3\3\2\2\2\u0ca3\u0ca4")
        buf.write("\3\2\2\2\u0ca4\u0ca8\7\'\2\2\u0ca5\u0ca7\7\7\2\2\u0ca6")
        buf.write("\u0ca5\3\2\2\2\u0ca7\u0caa\3\2\2\2\u0ca8\u0ca6\3\2\2\2")
        buf.write("\u0ca8\u0ca9\3\2\2\2\u0ca9\u0cad\3\2\2\2\u0caa\u0ca8\3")
        buf.write("\2\2\2\u0cab\u0cae\5\u0158\u00ad\2\u0cac\u0cae\7J\2\2")
        buf.write("\u0cad\u0cab\3\2\2\2\u0cad\u0cac\3\2\2\2\u0cae\u0111\3")
        buf.write("\2\2\2\u0caf\u0cb0\t\r\2\2\u0cb0\u0113\3\2\2\2\u0cb1\u0cb2")
        buf.write("\t\16\2\2\u0cb2\u0115\3\2\2\2\u0cb3\u0cb4\t\17\2\2\u0cb4")
        buf.write("\u0117\3\2\2\2\u0cb5\u0cb6\t\20\2\2\u0cb6\u0119\3\2\2")
        buf.write("\2\u0cb7\u0cb8\t\21\2\2\u0cb8\u011b\3\2\2\2\u0cb9\u0cba")
        buf.write("\t\22\2\2\u0cba\u011d\3\2\2\2\u0cbb\u0cbc\t\23\2\2\u0cbc")
        buf.write("\u011f\3\2\2\2\u0cbd\u0cbe\t\24\2\2\u0cbe\u0121\3\2\2")
        buf.write("\2\u0cbf\u0cc5\7\26\2\2\u0cc0\u0cc5\7\27\2\2\u0cc1\u0cc5")
        buf.write("\7\25\2\2\u0cc2\u0cc5\7\24\2\2\u0cc3\u0cc5\5\u0126\u0094")
        buf.write("\2\u0cc4\u0cbf\3\2\2\2\u0cc4\u0cc0\3\2\2\2\u0cc4\u0cc1")
        buf.write("\3\2\2\2\u0cc4\u0cc2\3\2\2\2\u0cc4\u0cc3\3\2\2\2\u0cc5")
        buf.write("\u0123\3\2\2\2\u0cc6\u0ccb\7\26\2\2\u0cc7\u0ccb\7\27\2")
        buf.write("\2\u0cc8\u0cc9\7\33\2\2\u0cc9\u0ccb\5\u0126\u0094\2\u0cca")
        buf.write("\u0cc6\3\2\2\2\u0cca\u0cc7\3\2\2\2\u0cca\u0cc8\3\2\2\2")
        buf.write("\u0ccb\u0125\3\2\2\2\u0ccc\u0ccd\t\25\2\2\u0ccd\u0127")
        buf.write("\3\2\2\2\u0cce\u0cd0\7\7\2\2\u0ccf\u0cce\3\2\2\2\u0cd0")
        buf.write("\u0cd3\3\2\2\2\u0cd1\u0ccf\3\2\2\2\u0cd1\u0cd2\3\2\2\2")
        buf.write("\u0cd2\u0cd4\3\2\2\2\u0cd3\u0cd1\3\2\2\2\u0cd4\u0cde\7")
        buf.write("\t\2\2\u0cd5\u0cd7\7\7\2\2\u0cd6\u0cd5\3\2\2\2\u0cd7\u0cda")
        buf.write("\3\2\2\2\u0cd8\u0cd6\3\2\2\2\u0cd8\u0cd9\3\2\2\2\u0cd9")
        buf.write("\u0cdb\3\2\2\2\u0cda\u0cd8\3\2\2\2\u0cdb\u0cde\5\u012a")
        buf.write("\u0096\2\u0cdc\u0cde\7\'\2\2\u0cdd\u0cd1\3\2\2\2\u0cdd")
        buf.write("\u0cd8\3\2\2\2\u0cdd\u0cdc\3\2\2\2\u0cde\u0129\3\2\2\2")
        buf.write("\u0cdf\u0ce0\7/\2\2\u0ce0\u0ce1\7\t\2\2\u0ce1\u012b\3")
        buf.write("\2\2\2\u0ce2\u0ce5\5\u014e\u00a8\2\u0ce3\u0ce5\5\u0130")
        buf.write("\u0099\2\u0ce4\u0ce2\3\2\2\2\u0ce4\u0ce3\3\2\2\2\u0ce5")
        buf.write("\u0ce6\3\2\2\2\u0ce6\u0ce4\3\2\2\2\u0ce6\u0ce7\3\2\2\2")
        buf.write("\u0ce7\u012d\3\2\2\2\u0ce8\u0ceb\5\u014e\u00a8\2\u0ce9")
        buf.write("\u0ceb\5\u0148\u00a5\2\u0cea\u0ce8\3\2\2\2\u0cea\u0ce9")
        buf.write("\3\2\2\2\u0ceb\u0cec\3\2\2\2\u0cec\u0cea\3\2\2\2\u0cec")
        buf.write("\u0ced\3\2\2\2\u0ced\u012f\3\2\2\2\u0cee\u0cf7\5\u0136")
        buf.write("\u009c\2\u0cef\u0cf7\5\u0138\u009d\2\u0cf0\u0cf7\5\u013a")
        buf.write("\u009e\2\u0cf1\u0cf7\5\u0142\u00a2\2\u0cf2\u0cf7\5\u0144")
        buf.write("\u00a3\2\u0cf3\u0cf7\5\u0146\u00a4\2\u0cf4\u0cf7\5\u0148")
        buf.write("\u00a5\2\u0cf5\u0cf7\5\u014c\u00a7\2\u0cf6\u0cee\3\2\2")
        buf.write("\2\u0cf6\u0cef\3\2\2\2\u0cf6\u0cf0\3\2\2\2\u0cf6\u0cf1")
        buf.write("\3\2\2\2\u0cf6\u0cf2\3\2\2\2\u0cf6\u0cf3\3\2\2\2\u0cf6")
        buf.write("\u0cf4\3\2\2\2\u0cf6\u0cf5\3\2\2\2\u0cf7\u0cfb\3\2\2\2")
        buf.write("\u0cf8\u0cfa\7\7\2\2\u0cf9\u0cf8\3\2\2\2\u0cfa\u0cfd\3")
        buf.write("\2\2\2\u0cfb\u0cf9\3\2\2\2\u0cfb\u0cfc\3\2\2\2\u0cfc\u0131")
        buf.write("\3\2\2\2\u0cfd\u0cfb\3\2\2\2\u0cfe\u0d00\5\u0134\u009b")
        buf.write("\2\u0cff\u0cfe\3\2\2\2\u0d00\u0d01\3\2\2\2\u0d01\u0cff")
        buf.write("\3\2\2\2\u0d01\u0d02\3\2\2\2\u0d02\u0133\3\2\2\2\u0d03")
        buf.write("\u0d0c\5\u014e\u00a8\2\u0d04\u0d08\7|\2\2\u0d05\u0d07")
        buf.write("\7\7\2\2\u0d06\u0d05\3\2\2\2\u0d07\u0d0a\3\2\2\2\u0d08")
        buf.write("\u0d06\3\2\2\2\u0d08\u0d09\3\2\2\2\u0d09\u0d0c\3\2\2\2")
        buf.write("\u0d0a\u0d08\3\2\2\2\u0d0b\u0d03\3\2\2\2\u0d0b\u0d04\3")
        buf.write("\2\2\2\u0d0c\u0135\3\2\2\2\u0d0d\u0d0e\t\26\2\2\u0d0e")
        buf.write("\u0137\3\2\2\2\u0d0f\u0d10\t\27\2\2\u0d10\u0139\3\2\2")
        buf.write("\2\u0d11\u0d12\t\30\2\2\u0d12\u013b\3\2\2\2\u0d13\u0d14")
        buf.write("\t\31\2\2\u0d14\u013d\3\2\2\2\u0d15\u0d17\5\u0140\u00a1")
        buf.write("\2\u0d16\u0d15\3\2\2\2\u0d17\u0d18\3\2\2\2\u0d18\u0d16")
        buf.write("\3\2\2\2\u0d18\u0d19\3\2\2\2\u0d19\u013f\3\2\2\2\u0d1a")
        buf.write("\u0d1e\5\u014a\u00a6\2\u0d1b\u0d1d\7\7\2\2\u0d1c\u0d1b")
        buf.write("\3\2\2\2\u0d1d\u0d20\3\2\2\2\u0d1e\u0d1c\3\2\2\2\u0d1e")
        buf.write("\u0d1f\3\2\2\2\u0d1f\u0d2a\3\2\2\2\u0d20\u0d1e\3\2\2\2")
        buf.write("\u0d21\u0d25\5\u013c\u009f\2\u0d22\u0d24\7\7\2\2\u0d23")
        buf.write("\u0d22\3\2\2\2\u0d24\u0d27\3\2\2\2\u0d25\u0d23\3\2\2\2")
        buf.write("\u0d25\u0d26\3\2\2\2\u0d26\u0d2a\3\2\2\2\u0d27\u0d25\3")
        buf.write("\2\2\2\u0d28\u0d2a\5\u014e\u00a8\2\u0d29\u0d1a\3\2\2\2")
        buf.write("\u0d29\u0d21\3\2\2\2\u0d29\u0d28\3\2\2\2\u0d2a\u0141\3")
        buf.write("\2\2\2\u0d2b\u0d2c\t\32\2\2\u0d2c\u0143\3\2\2\2\u0d2d")
        buf.write("\u0d2e\7\u0081\2\2\u0d2e\u0145\3\2\2\2\u0d2f\u0d30\t\33")
        buf.write("\2\2\u0d30\u0147\3\2\2\2\u0d31\u0d32\t\34\2\2\u0d32\u0149")
        buf.write("\3\2\2\2\u0d33\u0d34\7\u0086\2\2\u0d34\u014b\3\2\2\2\u0d35")
        buf.write("\u0d36\t\35\2\2\u0d36\u014d\3\2\2\2\u0d37\u0d3a\5\u0150")
        buf.write("\u00a9\2\u0d38\u0d3a\5\u0152\u00aa\2\u0d39\u0d37\3\2\2")
        buf.write("\2\u0d39\u0d38\3\2\2\2\u0d3a\u0d3e\3\2\2\2\u0d3b\u0d3d")
        buf.write("\7\7\2\2\u0d3c\u0d3b\3\2\2\2\u0d3d\u0d40\3\2\2\2\u0d3e")
        buf.write("\u0d3c\3\2\2\2\u0d3e\u0d3f\3\2\2\2\u0d3f\u014f\3\2\2\2")
        buf.write("\u0d40\u0d3e\3\2\2\2\u0d41\u0d45\5\u0154\u00ab\2\u0d42")
        buf.write("\u0d44\7\7\2\2\u0d43\u0d42\3\2\2\2\u0d44\u0d47\3\2\2\2")
        buf.write("\u0d45\u0d43\3\2\2\2\u0d45\u0d46\3\2\2\2\u0d46\u0d4b\3")
        buf.write("\2\2\2\u0d47\u0d45\3\2\2\2\u0d48\u0d4b\7*\2\2\u0d49\u0d4b")
        buf.write("\7,\2\2\u0d4a\u0d41\3\2\2\2\u0d4a\u0d48\3\2\2\2\u0d4a")
        buf.write("\u0d49\3\2\2\2\u0d4b\u0d4c\3\2\2\2\u0d4c\u0d4d\5\u0156")
        buf.write("\u00ac\2\u0d4d\u0151\3\2\2\2\u0d4e\u0d52\5\u0154\u00ab")
        buf.write("\2\u0d4f\u0d51\7\7\2\2\u0d50\u0d4f\3\2\2\2\u0d51\u0d54")
        buf.write("\3\2\2\2\u0d52\u0d50\3\2\2\2\u0d52\u0d53\3\2\2\2\u0d53")
        buf.write("\u0d58\3\2\2\2\u0d54\u0d52\3\2\2\2\u0d55\u0d58\7*\2\2")
        buf.write("\u0d56\u0d58\7,\2\2\u0d57\u0d4e\3\2\2\2\u0d57\u0d55\3")
        buf.write("\2\2\2\u0d57\u0d56\3\2\2\2\u0d58\u0d59\3\2\2\2\u0d59\u0d5b")
        buf.write("\7\r\2\2\u0d5a\u0d5c\5\u0156\u00ac\2\u0d5b\u0d5a\3\2\2")
        buf.write("\2\u0d5c\u0d5d\3\2\2\2\u0d5d\u0d5b\3\2\2\2\u0d5d\u0d5e")
        buf.write("\3\2\2\2\u0d5e\u0d5f\3\2\2\2\u0d5f\u0d60\7\16\2\2\u0d60")
        buf.write("\u0153\3\2\2\2\u0d61\u0d62\t\2\2\2\u0d62\u0d66\t\36\2")
        buf.write("\2\u0d63\u0d65\7\7\2\2\u0d64\u0d63\3\2\2\2\u0d65\u0d68")
        buf.write("\3\2\2\2\u0d66\u0d64\3\2\2\2\u0d66\u0d67\3\2\2\2\u0d67")
        buf.write("\u0d69\3\2\2\2\u0d68\u0d66\3\2\2\2\u0d69\u0d6a\7\34\2")
        buf.write("\2\u0d6a\u0155\3\2\2\2\u0d6b\u0d6e\5&\24\2\u0d6c\u0d6e")
        buf.write("\5l\67\2\u0d6d\u0d6b\3\2\2\2\u0d6d\u0d6c\3\2\2\2\u0d6e")
        buf.write("\u0157\3\2\2\2\u0d6f\u0d70\t\37\2\2\u0d70\u0159\3\2\2")
        buf.write("\2\u0d71\u0d7c\5\u0158\u00ad\2\u0d72\u0d74\7\7\2\2\u0d73")
        buf.write("\u0d72\3\2\2\2\u0d74\u0d77\3\2\2\2\u0d75\u0d73\3\2\2\2")
        buf.write("\u0d75\u0d76\3\2\2\2\u0d76\u0d78\3\2\2\2\u0d77\u0d75\3")
        buf.write("\2\2\2\u0d78\u0d79\7\t\2\2\u0d79\u0d7b\5\u0158\u00ad\2")
        buf.write("\u0d7a\u0d75\3\2\2\2\u0d7b\u0d7e\3\2\2\2\u0d7c\u0d7a\3")
        buf.write("\2\2\2\u0d7c\u0d7d\3\2\2\2\u0d7d\u015b\3\2\2\2\u0d7e\u0d7c")
        buf.write("\3\2\2\2\u0214\u015d\u0162\u0168\u0170\u0176\u017b\u0181")
        buf.write("\u018b\u0194\u019b\u01a2\u01a9\u01ae\u01b3\u01b9\u01bb")
        buf.write("\u01c0\u01c8\u01cb\u01d2\u01d5\u01db\u01e2\u01e6\u01eb")
        buf.write("\u01f2\u01fc\u01ff\u0206\u0209\u020c\u0211\u0218\u021c")
        buf.write("\u0221\u0225\u022a\u0231\u0235\u023a\u023e\u0243\u024a")
        buf.write("\u024e\u0251\u0257\u025a\u0262\u0269\u0272\u0279\u0280")
        buf.write("\u0286\u028c\u0290\u0292\u0297\u029d\u02a0\u02a5\u02ad")
        buf.write("\u02b4\u02bb\u02bf\u02c5\u02cc\u02d2\u02d9\u02e1\u02e7")
        buf.write("\u02ee\u02f3\u02fa\u0303\u030a\u0311\u0317\u031d\u0321")
        buf.write("\u0326\u032c\u0331\u0338\u033f\u0343\u0349\u0350\u0357")
        buf.write("\u035d\u0363\u036a\u0371\u0378\u037c\u0383\u0389\u038f")
        buf.write("\u0395\u039c\u03a0\u03a5\u03ac\u03b0\u03b5\u03b9\u03bf")
        buf.write("\u03c6\u03cd\u03d3\u03d9\u03dd\u03df\u03e4\u03ea\u03f0")
        buf.write("\u03f7\u03fb\u03fe\u0404\u0408\u040d\u0414\u0419\u041e")
        buf.write("\u0425\u042c\u0433\u0437\u043c\u0440\u0445\u0449\u0450")
        buf.write("\u0454\u0459\u045f\u0466\u046d\u0471\u0477\u047e\u0485")
        buf.write("\u048b\u0491\u0495\u049a\u04a0\u04a6\u04aa\u04af\u04b6")
        buf.write("\u04bb\u04c0\u04c5\u04ca\u04ce\u04d3\u04da\u04df\u04e1")
        buf.write("\u04e6\u04e9\u04ee\u04f2\u04f7\u04fb\u04fe\u0501\u0506")
        buf.write("\u050a\u050d\u050f\u0515\u051b\u0521\u0528\u052f\u0536")
        buf.write("\u053a\u053f\u0543\u0546\u054c\u0553\u055a\u055e\u0563")
        buf.write("\u056a\u0571\u0575\u057a\u057f\u0585\u058c\u0593\u0599")
        buf.write("\u059f\u05a3\u05a5\u05aa\u05b0\u05b6\u05bd\u05c1\u05c7")
        buf.write("\u05ce\u05d2\u05d8\u05df\u05e5\u05eb\u05f2\u05f9\u05fd")
        buf.write("\u0602\u0606\u0609\u060f\u0616\u061d\u0621\u0626\u062a")
        buf.write("\u0630\u0639\u063d\u0642\u0649\u064d\u0652\u065b\u0662")
        buf.write("\u0668\u066e\u0672\u0678\u067b\u0681\u0685\u068a\u068e")
        buf.write("\u0691\u0697\u069b\u069f\u06a4\u06aa\u06b2\u06b9\u06bf")
        buf.write("\u06c6\u06ca\u06cd\u06d1\u06d6\u06dc\u06e0\u06e6\u06ed")
        buf.write("\u06f0\u06f6\u06fd\u0706\u070b\u0710\u0717\u071c\u0720")
        buf.write("\u0726\u072a\u072f\u0738\u073f\u0745\u074a\u0750\u0755")
        buf.write("\u075a\u0765\u0768\u076b\u076f\u0771\u0778\u077f\u0784")
        buf.write("\u078a\u0791\u0799\u079f\u07a6\u07ab\u07b3\u07b7\u07bd")
        buf.write("\u07c6\u07cb\u07d1\u07d5\u07da\u07e1\u07ee\u07f3\u07fc")
        buf.write("\u0800\u0805\u0808\u0810\u0817\u081d\u0824\u082b\u0831")
        buf.write("\u0839\u0840\u0848\u084f\u0856\u085e\u0867\u086c\u086e")
        buf.write("\u0875\u087c\u0883\u088e\u0895\u089d\u08a3\u08ab\u08b2")
        buf.write("\u08ba\u08c1\u08c8\u08cf\u08d6\u08dc\u08e7\u08ea\u08f0")
        buf.write("\u08f8\u08ff\u0905\u090c\u0913\u0919\u0920\u0928\u092e")
        buf.write("\u0935\u093c\u0942\u0948\u094c\u0951\u095a\u0960\u0963")
        buf.write("\u0966\u096a\u096f\u0973\u0978\u0981\u0988\u098f\u0995")
        buf.write("\u099b\u099f\u09a4\u09ad\u09b4\u09bb\u09c1\u09c7\u09cb")
        buf.write("\u09d0\u09d3\u09d8\u09dd\u09e4\u09eb\u09ee\u09f1\u09f6")
        buf.write("\u0a09\u0a0f\u0a16\u0a1f\u0a26\u0a2d\u0a33\u0a39\u0a3d")
        buf.write("\u0a42\u0a45\u0a4d\u0a52\u0a54\u0a5d\u0a5f\u0a6a\u0a71")
        buf.write("\u0a7c\u0a83\u0a8c\u0a90\u0a95\u0a9c\u0a9f\u0aa5\u0aae")
        buf.write("\u0ab5\u0abb\u0ac1\u0ac5\u0acc\u0ad3\u0ad7\u0ad9\u0adf")
        buf.write("\u0ae6\u0aeb\u0af0\u0af7\u0afe\u0b02\u0b07\u0b0b\u0b10")
        buf.write("\u0b14\u0b18\u0b1e\u0b25\u0b2c\u0b2f\u0b34\u0b38\u0b41")
        buf.write("\u0b48\u0b4d\u0b51\u0b54\u0b5a\u0b61\u0b68\u0b6f\u0b74")
        buf.write("\u0b79\u0b7d\u0b82\u0b89\u0b8e\u0b91\u0b97\u0b9d\u0ba4")
        buf.write("\u0bab\u0bb2\u0bb5\u0bbe\u0bc2\u0bc7\u0bce\u0bd5\u0bda")
        buf.write("\u0be0\u0be9\u0bf0\u0bf6\u0bfc\u0c00\u0c05\u0c0c\u0c11")
        buf.write("\u0c17\u0c1e\u0c23\u0c25\u0c2a\u0c30\u0c39\u0c42\u0c49")
        buf.write("\u0c4f\u0c54\u0c58\u0c5d\u0c61\u0c67\u0c6e\u0c77\u0c7b")
        buf.write("\u0c81\u0c8a\u0c93\u0c99\u0c9f\u0ca2\u0ca8\u0cad\u0cc4")
        buf.write("\u0cca\u0cd1\u0cd8\u0cdd\u0ce4\u0ce6\u0cea\u0cec\u0cf6")
        buf.write("\u0cfb\u0d01\u0d08\u0d0b\u0d18\u0d1e\u0d25\u0d29\u0d39")
        buf.write("\u0d3e\u0d45\u0d4a\u0d52\u0d57\u0d5d\u0d66\u0d6d\u0d75")
        buf.write("\u0d7c")
        return buf.getvalue()


class KotlinParser ( Parser ):

    grammarFileName = "KotlinParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "'...'", "'.'", "','", "'('", 
                     "')'", "'['", "']'", "'{'", "'}'", "'*'", "'%'", "'/'", 
                     "'+'", "'-'", "'++'", "'--'", "'&&'", "'||'", "<INVALID>", 
                     "'!'", "':'", "';'", "'='", "'+='", "'-='", "'*='", 
                     "'/='", "'%='", "'->'", "'=>'", "'..'", "'::'", "';;'", 
                     "'#'", "'@'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "'?'", "'<'", "'>'", "'<='", "'>='", "'!='", 
                     "'!=='", "'as?'", "'=='", "'==='", "'''", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "'file'", "'field'", "'property'", "'get'", "'set'", 
                     "'receiver'", "'param'", "'setparam'", "'delegate'", 
                     "'package'", "'import'", "'class'", "'interface'", 
                     "'fun'", "'object'", "'val'", "'var'", "'typealias'", 
                     "'constructor'", "'by'", "'companion'", "'init'", "'this'", 
                     "'super'", "'typeof'", "'where'", "'if'", "'else'", 
                     "'when'", "'try'", "'catch'", "'finally'", "'for'", 
                     "'do'", "'while'", "'throw'", "'return'", "'continue'", 
                     "'break'", "'as'", "'is'", "'in'", "<INVALID>", "<INVALID>", 
                     "'out'", "'dynamic'", "'public'", "'private'", "'protected'", 
                     "'internal'", "'enum'", "'sealed'", "'annotation'", 
                     "'data'", "'inner'", "'value'", "'tailrec'", "'operator'", 
                     "'inline'", "'infix'", "'external'", "'suspend'", "'override'", 
                     "'abstract'", "'final'", "'open'", "'const'", "'lateinit'", 
                     "'vararg'", "'noinline'", "'crossinline'", "'reified'", 
                     "'expect'", "'actual'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "'null'", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "'\"\"\"'" ]

    symbolicNames = [ "<INVALID>", "ShebangLine", "DelimitedComment", "LineComment", 
                      "WS", "NL", "RESERVED", "DOT", "COMMA", "LPAREN", 
                      "RPAREN", "LSQUARE", "RSQUARE", "LCURL", "RCURL", 
                      "MULT", "MOD", "DIV", "ADD", "SUB", "INCR", "DECR", 
                      "CONJ", "DISJ", "EXCL_WS", "EXCL_NO_WS", "COLON", 
                      "SEMICOLON", "ASSIGNMENT", "ADD_ASSIGNMENT", "SUB_ASSIGNMENT", 
                      "MULT_ASSIGNMENT", "DIV_ASSIGNMENT", "MOD_ASSIGNMENT", 
                      "ARROW", "DOUBLE_ARROW", "RANGE", "COLONCOLON", "DOUBLE_SEMICOLON", 
                      "HASH", "AT_NO_WS", "AT_POST_WS", "AT_PRE_WS", "AT_BOTH_WS", 
                      "QUEST_WS", "QUEST_NO_WS", "LANGLE", "RANGLE", "LE", 
                      "GE", "EXCL_EQ", "EXCL_EQEQ", "AS_SAFE", "EQEQ", "EQEQEQ", 
                      "SINGLE_QUOTE", "RETURN_AT", "CONTINUE_AT", "BREAK_AT", 
                      "THIS_AT", "SUPER_AT", "FILE", "FIELD", "PROPERTY", 
                      "GET", "SET", "RECEIVER", "PARAM", "SETPARAM", "DELEGATE", 
                      "PACKAGE", "IMPORT", "CLASS", "INTERFACE", "FUN", 
                      "OBJECT", "VAL", "VAR", "TYPE_ALIAS", "CONSTRUCTOR", 
                      "BY", "COMPANION", "INIT", "THIS", "SUPER", "TYPEOF", 
                      "WHERE", "IF", "ELSE", "WHEN", "TRY", "CATCH", "FINALLY", 
                      "FOR", "DO", "WHILE", "THROW", "RETURN", "CONTINUE", 
                      "BREAK", "AS", "IS", "IN", "NOT_IS", "NOT_IN", "OUT", 
                      "DYNAMIC", "PUBLIC", "PRIVATE", "PROTECTED", "INTERNAL", 
                      "ENUM", "SEALED", "ANNOTATION", "DATA", "INNER", "VALUE", 
                      "TAILREC", "OPERATOR", "INLINE", "INFIX", "EXTERNAL", 
                      "SUSPEND", "OVERRIDE", "ABSTRACT", "FINAL", "OPEN", 
                      "CONST", "LATEINIT", "VARARG", "NOINLINE", "CROSSINLINE", 
                      "REIFIED", "EXPECT", "ACTUAL", "RealLiteral", "FloatLiteral", 
                      "DoubleLiteral", "IntegerLiteral", "HexLiteral", "BinLiteral", 
                      "UnsignedLiteral", "LongLiteral", "BooleanLiteral", 
                      "NullLiteral", "CharacterLiteral", "Identifier", "IdentifierOrSoftKey", 
                      "FieldIdentifier", "QUOTE_OPEN", "TRIPLE_QUOTE_OPEN", 
                      "UNICODE_CLASS_LL", "UNICODE_CLASS_LM", "UNICODE_CLASS_LO", 
                      "UNICODE_CLASS_LT", "UNICODE_CLASS_LU", "UNICODE_CLASS_ND", 
                      "UNICODE_CLASS_NL", "QUOTE_CLOSE", "LineStrRef", "LineStrText", 
                      "LineStrEscapedChar", "LineStrExprStart", "TRIPLE_QUOTE_CLOSE", 
                      "MultiLineStringQuote", "MultiLineStrRef", "MultiLineStrText", 
                      "MultiLineStrExprStart", "Inside_Comment", "Inside_WS", 
                      "Inside_NL", "ErrorCharacter" ]

    RULE_kotlinFile = 0
    RULE_script = 1
    RULE_shebangLine = 2
    RULE_fileAnnotation = 3
    RULE_packageHeader = 4
    RULE_importList = 5
    RULE_importHeader = 6
    RULE_importAlias = 7
    RULE_topLevelObject = 8
    RULE_typeAlias = 9
    RULE_declaration = 10
    RULE_classDeclaration = 11
    RULE_primaryConstructor = 12
    RULE_classBody = 13
    RULE_classParameters = 14
    RULE_classParameter = 15
    RULE_delegationSpecifiers = 16
    RULE_delegationSpecifier = 17
    RULE_constructorInvocation = 18
    RULE_annotatedDelegationSpecifier = 19
    RULE_explicitDelegation = 20
    RULE_typeParameters = 21
    RULE_typeParameter = 22
    RULE_typeConstraints = 23
    RULE_typeConstraint = 24
    RULE_classMemberDeclarations = 25
    RULE_classMemberDeclaration = 26
    RULE_anonymousInitializer = 27
    RULE_companionObject = 28
    RULE_functionValueParameters = 29
    RULE_functionValueParameter = 30
    RULE_functionDeclaration = 31
    RULE_functionBody = 32
    RULE_variableDeclaration = 33
    RULE_multiVariableDeclaration = 34
    RULE_propertyDeclaration = 35
    RULE_propertyDelegate = 36
    RULE_getter = 37
    RULE_setter = 38
    RULE_parametersWithOptionalType = 39
    RULE_functionValueParameterWithOptionalType = 40
    RULE_parameterWithOptionalType = 41
    RULE_parameter = 42
    RULE_objectDeclaration = 43
    RULE_secondaryConstructor = 44
    RULE_constructorDelegationCall = 45
    RULE_enumClassBody = 46
    RULE_enumEntries = 47
    RULE_enumEntry = 48
    RULE_type = 49
    RULE_typeReference = 50
    RULE_nullableType = 51
    RULE_quest = 52
    RULE_userType = 53
    RULE_simpleUserType = 54
    RULE_typeProjection = 55
    RULE_typeProjectionModifiers = 56
    RULE_typeProjectionModifier = 57
    RULE_functionType = 58
    RULE_functionTypeParameters = 59
    RULE_parenthesizedType = 60
    RULE_receiverType = 61
    RULE_parenthesizedUserType = 62
    RULE_statements = 63
    RULE_statement = 64
    RULE_label = 65
    RULE_controlStructureBody = 66
    RULE_block = 67
    RULE_loopStatement = 68
    RULE_forStatement = 69
    RULE_whileStatement = 70
    RULE_doWhileStatement = 71
    RULE_assignment = 72
    RULE_semi = 73
    RULE_semis = 74
    RULE_expression = 75
    RULE_disjunction = 76
    RULE_conjunction = 77
    RULE_equality = 78
    RULE_comparison = 79
    RULE_genericCallLikeComparison = 80
    RULE_infixOperation = 81
    RULE_elvisExpression = 82
    RULE_elvis = 83
    RULE_infixFunctionCall = 84
    RULE_rangeExpression = 85
    RULE_additiveExpression = 86
    RULE_multiplicativeExpression = 87
    RULE_asExpression = 88
    RULE_prefixUnaryExpression = 89
    RULE_unaryPrefix = 90
    RULE_postfixUnaryExpression = 91
    RULE_postfixUnarySuffix = 92
    RULE_directlyAssignableExpression = 93
    RULE_parenthesizedDirectlyAssignableExpression = 94
    RULE_assignableExpression = 95
    RULE_parenthesizedAssignableExpression = 96
    RULE_assignableSuffix = 97
    RULE_indexingSuffix = 98
    RULE_navigationSuffix = 99
    RULE_callSuffix = 100
    RULE_annotatedLambda = 101
    RULE_typeArguments = 102
    RULE_valueArguments = 103
    RULE_valueArgument = 104
    RULE_primaryExpression = 105
    RULE_parenthesizedExpression = 106
    RULE_collectionLiteral = 107
    RULE_literalConstant = 108
    RULE_stringLiteral = 109
    RULE_lineStringLiteral = 110
    RULE_multiLineStringLiteral = 111
    RULE_lineStringContent = 112
    RULE_lineStringExpression = 113
    RULE_multiLineStringContent = 114
    RULE_multiLineStringExpression = 115
    RULE_lambdaLiteral = 116
    RULE_lambdaParameters = 117
    RULE_lambdaParameter = 118
    RULE_anonymousFunction = 119
    RULE_functionLiteral = 120
    RULE_objectLiteral = 121
    RULE_thisExpression = 122
    RULE_superExpression = 123
    RULE_ifExpression = 124
    RULE_whenSubject = 125
    RULE_whenExpression = 126
    RULE_whenEntry = 127
    RULE_whenCondition = 128
    RULE_rangeTest = 129
    RULE_typeTest = 130
    RULE_tryExpression = 131
    RULE_catchBlock = 132
    RULE_finallyBlock = 133
    RULE_jumpExpression = 134
    RULE_callableReference = 135
    RULE_assignmentAndOperator = 136
    RULE_equalityOperator = 137
    RULE_comparisonOperator = 138
    RULE_inOperator = 139
    RULE_isOperator = 140
    RULE_additiveOperator = 141
    RULE_multiplicativeOperator = 142
    RULE_asOperator = 143
    RULE_prefixUnaryOperator = 144
    RULE_postfixUnaryOperator = 145
    RULE_excl = 146
    RULE_memberAccessOperator = 147
    RULE_safeNav = 148
    RULE_modifiers = 149
    RULE_parameterModifiers = 150
    RULE_modifier = 151
    RULE_typeModifiers = 152
    RULE_typeModifier = 153
    RULE_classModifier = 154
    RULE_memberModifier = 155
    RULE_visibilityModifier = 156
    RULE_varianceModifier = 157
    RULE_typeParameterModifiers = 158
    RULE_typeParameterModifier = 159
    RULE_functionModifier = 160
    RULE_propertyModifier = 161
    RULE_inheritanceModifier = 162
    RULE_parameterModifier = 163
    RULE_reificationModifier = 164
    RULE_platformModifier = 165
    RULE_annotation = 166
    RULE_singleAnnotation = 167
    RULE_multiAnnotation = 168
    RULE_annotationUseSiteTarget = 169
    RULE_unescapedAnnotation = 170
    RULE_simpleIdentifier = 171
    RULE_identifier = 172

    ruleNames =  [ "kotlinFile", "script", "shebangLine", "fileAnnotation", 
                   "packageHeader", "importList", "importHeader", "importAlias", 
                   "topLevelObject", "typeAlias", "declaration", "classDeclaration", 
                   "primaryConstructor", "classBody", "classParameters", 
                   "classParameter", "delegationSpecifiers", "delegationSpecifier", 
                   "constructorInvocation", "annotatedDelegationSpecifier", 
                   "explicitDelegation", "typeParameters", "typeParameter", 
                   "typeConstraints", "typeConstraint", "classMemberDeclarations", 
                   "classMemberDeclaration", "anonymousInitializer", "companionObject", 
                   "functionValueParameters", "functionValueParameter", 
                   "functionDeclaration", "functionBody", "variableDeclaration", 
                   "multiVariableDeclaration", "propertyDeclaration", "propertyDelegate", 
                   "getter", "setter", "parametersWithOptionalType", "functionValueParameterWithOptionalType", 
                   "parameterWithOptionalType", "parameter", "objectDeclaration", 
                   "secondaryConstructor", "constructorDelegationCall", 
                   "enumClassBody", "enumEntries", "enumEntry", "type", 
                   "typeReference", "nullableType", "quest", "userType", 
                   "simpleUserType", "typeProjection", "typeProjectionModifiers", 
                   "typeProjectionModifier", "functionType", "functionTypeParameters", 
                   "parenthesizedType", "receiverType", "parenthesizedUserType", 
                   "statements", "statement", "label", "controlStructureBody", 
                   "block", "loopStatement", "forStatement", "whileStatement", 
                   "doWhileStatement", "assignment", "semi", "semis", "expression", 
                   "disjunction", "conjunction", "equality", "comparison", 
                   "genericCallLikeComparison", "infixOperation", "elvisExpression", 
                   "elvis", "infixFunctionCall", "rangeExpression", "additiveExpression", 
                   "multiplicativeExpression", "asExpression", "prefixUnaryExpression", 
                   "unaryPrefix", "postfixUnaryExpression", "postfixUnarySuffix", 
                   "directlyAssignableExpression", "parenthesizedDirectlyAssignableExpression", 
                   "assignableExpression", "parenthesizedAssignableExpression", 
                   "assignableSuffix", "indexingSuffix", "navigationSuffix", 
                   "callSuffix", "annotatedLambda", "typeArguments", "valueArguments", 
                   "valueArgument", "primaryExpression", "parenthesizedExpression", 
                   "collectionLiteral", "literalConstant", "stringLiteral", 
                   "lineStringLiteral", "multiLineStringLiteral", "lineStringContent", 
                   "lineStringExpression", "multiLineStringContent", "multiLineStringExpression", 
                   "lambdaLiteral", "lambdaParameters", "lambdaParameter", 
                   "anonymousFunction", "functionLiteral", "objectLiteral", 
                   "thisExpression", "superExpression", "ifExpression", 
                   "whenSubject", "whenExpression", "whenEntry", "whenCondition", 
                   "rangeTest", "typeTest", "tryExpression", "catchBlock", 
                   "finallyBlock", "jumpExpression", "callableReference", 
                   "assignmentAndOperator", "equalityOperator", "comparisonOperator", 
                   "inOperator", "isOperator", "additiveOperator", "multiplicativeOperator", 
                   "asOperator", "prefixUnaryOperator", "postfixUnaryOperator", 
                   "excl", "memberAccessOperator", "safeNav", "modifiers", 
                   "parameterModifiers", "modifier", "typeModifiers", "typeModifier", 
                   "classModifier", "memberModifier", "visibilityModifier", 
                   "varianceModifier", "typeParameterModifiers", "typeParameterModifier", 
                   "functionModifier", "propertyModifier", "inheritanceModifier", 
                   "parameterModifier", "reificationModifier", "platformModifier", 
                   "annotation", "singleAnnotation", "multiAnnotation", 
                   "annotationUseSiteTarget", "unescapedAnnotation", "simpleIdentifier", 
                   "identifier" ]

    EOF = Token.EOF
    ShebangLine=1
    DelimitedComment=2
    LineComment=3
    WS=4
    NL=5
    RESERVED=6
    DOT=7
    COMMA=8
    LPAREN=9
    RPAREN=10
    LSQUARE=11
    RSQUARE=12
    LCURL=13
    RCURL=14
    MULT=15
    MOD=16
    DIV=17
    ADD=18
    SUB=19
    INCR=20
    DECR=21
    CONJ=22
    DISJ=23
    EXCL_WS=24
    EXCL_NO_WS=25
    COLON=26
    SEMICOLON=27
    ASSIGNMENT=28
    ADD_ASSIGNMENT=29
    SUB_ASSIGNMENT=30
    MULT_ASSIGNMENT=31
    DIV_ASSIGNMENT=32
    MOD_ASSIGNMENT=33
    ARROW=34
    DOUBLE_ARROW=35
    RANGE=36
    COLONCOLON=37
    DOUBLE_SEMICOLON=38
    HASH=39
    AT_NO_WS=40
    AT_POST_WS=41
    AT_PRE_WS=42
    AT_BOTH_WS=43
    QUEST_WS=44
    QUEST_NO_WS=45
    LANGLE=46
    RANGLE=47
    LE=48
    GE=49
    EXCL_EQ=50
    EXCL_EQEQ=51
    AS_SAFE=52
    EQEQ=53
    EQEQEQ=54
    SINGLE_QUOTE=55
    RETURN_AT=56
    CONTINUE_AT=57
    BREAK_AT=58
    THIS_AT=59
    SUPER_AT=60
    FILE=61
    FIELD=62
    PROPERTY=63
    GET=64
    SET=65
    RECEIVER=66
    PARAM=67
    SETPARAM=68
    DELEGATE=69
    PACKAGE=70
    IMPORT=71
    CLASS=72
    INTERFACE=73
    FUN=74
    OBJECT=75
    VAL=76
    VAR=77
    TYPE_ALIAS=78
    CONSTRUCTOR=79
    BY=80
    COMPANION=81
    INIT=82
    THIS=83
    SUPER=84
    TYPEOF=85
    WHERE=86
    IF=87
    ELSE=88
    WHEN=89
    TRY=90
    CATCH=91
    FINALLY=92
    FOR=93
    DO=94
    WHILE=95
    THROW=96
    RETURN=97
    CONTINUE=98
    BREAK=99
    AS=100
    IS=101
    IN=102
    NOT_IS=103
    NOT_IN=104
    OUT=105
    DYNAMIC=106
    PUBLIC=107
    PRIVATE=108
    PROTECTED=109
    INTERNAL=110
    ENUM=111
    SEALED=112
    ANNOTATION=113
    DATA=114
    INNER=115
    VALUE=116
    TAILREC=117
    OPERATOR=118
    INLINE=119
    INFIX=120
    EXTERNAL=121
    SUSPEND=122
    OVERRIDE=123
    ABSTRACT=124
    FINAL=125
    OPEN=126
    CONST=127
    LATEINIT=128
    VARARG=129
    NOINLINE=130
    CROSSINLINE=131
    REIFIED=132
    EXPECT=133
    ACTUAL=134
    RealLiteral=135
    FloatLiteral=136
    DoubleLiteral=137
    IntegerLiteral=138
    HexLiteral=139
    BinLiteral=140
    UnsignedLiteral=141
    LongLiteral=142
    BooleanLiteral=143
    NullLiteral=144
    CharacterLiteral=145
    Identifier=146
    IdentifierOrSoftKey=147
    FieldIdentifier=148
    QUOTE_OPEN=149
    TRIPLE_QUOTE_OPEN=150
    UNICODE_CLASS_LL=151
    UNICODE_CLASS_LM=152
    UNICODE_CLASS_LO=153
    UNICODE_CLASS_LT=154
    UNICODE_CLASS_LU=155
    UNICODE_CLASS_ND=156
    UNICODE_CLASS_NL=157
    QUOTE_CLOSE=158
    LineStrRef=159
    LineStrText=160
    LineStrEscapedChar=161
    LineStrExprStart=162
    TRIPLE_QUOTE_CLOSE=163
    MultiLineStringQuote=164
    MultiLineStrRef=165
    MultiLineStrText=166
    MultiLineStrExprStart=167
    Inside_Comment=168
    Inside_WS=169
    Inside_NL=170
    ErrorCharacter=171

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.9.2")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class KotlinFileContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def packageHeader(self):
            return self.getTypedRuleContext(KotlinParser.PackageHeaderContext,0)


        def importList(self):
            return self.getTypedRuleContext(KotlinParser.ImportListContext,0)


        def EOF(self):
            return self.getToken(KotlinParser.EOF, 0)

        def shebangLine(self):
            return self.getTypedRuleContext(KotlinParser.ShebangLineContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def fileAnnotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.FileAnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.FileAnnotationContext,i)


        def topLevelObject(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TopLevelObjectContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TopLevelObjectContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_kotlinFile




    def kotlinFile(self):

        localctx = KotlinParser.KotlinFileContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_kotlinFile)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 347
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.ShebangLine:
                self.state = 346
                self.shebangLine()


            self.state = 352
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 349
                self.match(KotlinParser.NL)
                self.state = 354
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 358
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,2,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 355
                    self.fileAnnotation() 
                self.state = 360
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,2,self._ctx)

            self.state = 361
            self.packageHeader()
            self.state = 362
            self.importList()
            self.state = 366
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 72)) & ~0x3f) == 0 and ((1 << (_la - 72)) & ((1 << (KotlinParser.CLASS - 72)) | (1 << (KotlinParser.INTERFACE - 72)) | (1 << (KotlinParser.FUN - 72)) | (1 << (KotlinParser.OBJECT - 72)) | (1 << (KotlinParser.VAL - 72)) | (1 << (KotlinParser.VAR - 72)) | (1 << (KotlinParser.TYPE_ALIAS - 72)) | (1 << (KotlinParser.PUBLIC - 72)) | (1 << (KotlinParser.PRIVATE - 72)) | (1 << (KotlinParser.PROTECTED - 72)) | (1 << (KotlinParser.INTERNAL - 72)) | (1 << (KotlinParser.ENUM - 72)) | (1 << (KotlinParser.SEALED - 72)) | (1 << (KotlinParser.ANNOTATION - 72)) | (1 << (KotlinParser.DATA - 72)) | (1 << (KotlinParser.INNER - 72)) | (1 << (KotlinParser.VALUE - 72)) | (1 << (KotlinParser.TAILREC - 72)) | (1 << (KotlinParser.OPERATOR - 72)) | (1 << (KotlinParser.INLINE - 72)) | (1 << (KotlinParser.INFIX - 72)) | (1 << (KotlinParser.EXTERNAL - 72)) | (1 << (KotlinParser.SUSPEND - 72)) | (1 << (KotlinParser.OVERRIDE - 72)) | (1 << (KotlinParser.ABSTRACT - 72)) | (1 << (KotlinParser.FINAL - 72)) | (1 << (KotlinParser.OPEN - 72)) | (1 << (KotlinParser.CONST - 72)) | (1 << (KotlinParser.LATEINIT - 72)) | (1 << (KotlinParser.VARARG - 72)) | (1 << (KotlinParser.NOINLINE - 72)) | (1 << (KotlinParser.CROSSINLINE - 72)) | (1 << (KotlinParser.EXPECT - 72)) | (1 << (KotlinParser.ACTUAL - 72)))) != 0):
                self.state = 363
                self.topLevelObject()
                self.state = 368
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 369
            self.match(KotlinParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ScriptContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def packageHeader(self):
            return self.getTypedRuleContext(KotlinParser.PackageHeaderContext,0)


        def importList(self):
            return self.getTypedRuleContext(KotlinParser.ImportListContext,0)


        def EOF(self):
            return self.getToken(KotlinParser.EOF, 0)

        def shebangLine(self):
            return self.getTypedRuleContext(KotlinParser.ShebangLineContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def fileAnnotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.FileAnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.FileAnnotationContext,i)


        def statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.StatementContext)
            else:
                return self.getTypedRuleContext(KotlinParser.StatementContext,i)


        def semi(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.SemiContext)
            else:
                return self.getTypedRuleContext(KotlinParser.SemiContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_script




    def script(self):

        localctx = KotlinParser.ScriptContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_script)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 372
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.ShebangLine:
                self.state = 371
                self.shebangLine()


            self.state = 377
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 374
                self.match(KotlinParser.NL)
                self.state = 379
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 383
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,6,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 380
                    self.fileAnnotation() 
                self.state = 385
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,6,self._ctx)

            self.state = 386
            self.packageHeader()
            self.state = 387
            self.importList()
            self.state = 393
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LPAREN) | (1 << KotlinParser.LSQUARE) | (1 << KotlinParser.LCURL) | (1 << KotlinParser.ADD) | (1 << KotlinParser.SUB) | (1 << KotlinParser.INCR) | (1 << KotlinParser.DECR) | (1 << KotlinParser.EXCL_WS) | (1 << KotlinParser.EXCL_NO_WS) | (1 << KotlinParser.COLONCOLON) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.RETURN_AT) | (1 << KotlinParser.CONTINUE_AT) | (1 << KotlinParser.BREAK_AT) | (1 << KotlinParser.THIS_AT) | (1 << KotlinParser.SUPER_AT) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.CLASS - 64)) | (1 << (KotlinParser.INTERFACE - 64)) | (1 << (KotlinParser.FUN - 64)) | (1 << (KotlinParser.OBJECT - 64)) | (1 << (KotlinParser.VAL - 64)) | (1 << (KotlinParser.VAR - 64)) | (1 << (KotlinParser.TYPE_ALIAS - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.THIS - 64)) | (1 << (KotlinParser.SUPER - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.IF - 64)) | (1 << (KotlinParser.WHEN - 64)) | (1 << (KotlinParser.TRY - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.FOR - 64)) | (1 << (KotlinParser.DO - 64)) | (1 << (KotlinParser.WHILE - 64)) | (1 << (KotlinParser.THROW - 64)) | (1 << (KotlinParser.RETURN - 64)) | (1 << (KotlinParser.CONTINUE - 64)) | (1 << (KotlinParser.BREAK - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.RealLiteral - 128)) | (1 << (KotlinParser.IntegerLiteral - 128)) | (1 << (KotlinParser.HexLiteral - 128)) | (1 << (KotlinParser.BinLiteral - 128)) | (1 << (KotlinParser.UnsignedLiteral - 128)) | (1 << (KotlinParser.LongLiteral - 128)) | (1 << (KotlinParser.BooleanLiteral - 128)) | (1 << (KotlinParser.NullLiteral - 128)) | (1 << (KotlinParser.CharacterLiteral - 128)) | (1 << (KotlinParser.Identifier - 128)) | (1 << (KotlinParser.QUOTE_OPEN - 128)) | (1 << (KotlinParser.TRIPLE_QUOTE_OPEN - 128)))) != 0):
                self.state = 388
                self.statement()
                self.state = 389
                self.semi()
                self.state = 395
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 396
            self.match(KotlinParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ShebangLineContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ShebangLine(self):
            return self.getToken(KotlinParser.ShebangLine, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_shebangLine




    def shebangLine(self):

        localctx = KotlinParser.ShebangLineContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_shebangLine)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 398
            self.match(KotlinParser.ShebangLine)
            self.state = 400 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 399
                    self.match(KotlinParser.NL)

                else:
                    raise NoViableAltException(self)
                self.state = 402 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,8,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FileAnnotationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def FILE(self):
            return self.getToken(KotlinParser.FILE, 0)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def AT_NO_WS(self):
            return self.getToken(KotlinParser.AT_NO_WS, 0)

        def AT_PRE_WS(self):
            return self.getToken(KotlinParser.AT_PRE_WS, 0)

        def LSQUARE(self):
            return self.getToken(KotlinParser.LSQUARE, 0)

        def RSQUARE(self):
            return self.getToken(KotlinParser.RSQUARE, 0)

        def unescapedAnnotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.UnescapedAnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.UnescapedAnnotationContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_fileAnnotation




    def fileAnnotation(self):

        localctx = KotlinParser.FileAnnotationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_fileAnnotation)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 404
            _la = self._input.LA(1)
            if not(_la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 405
            self.match(KotlinParser.FILE)
            self.state = 409
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 406
                self.match(KotlinParser.NL)
                self.state = 411
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 412
            self.match(KotlinParser.COLON)
            self.state = 416
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 413
                self.match(KotlinParser.NL)
                self.state = 418
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 428
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LSQUARE]:
                self.state = 419
                self.match(KotlinParser.LSQUARE)
                self.state = 421 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while True:
                    self.state = 420
                    self.unescapedAnnotation()
                    self.state = 423 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if not (((((_la - 61)) & ~0x3f) == 0 and ((1 << (_la - 61)) & ((1 << (KotlinParser.FILE - 61)) | (1 << (KotlinParser.FIELD - 61)) | (1 << (KotlinParser.PROPERTY - 61)) | (1 << (KotlinParser.GET - 61)) | (1 << (KotlinParser.SET - 61)) | (1 << (KotlinParser.RECEIVER - 61)) | (1 << (KotlinParser.PARAM - 61)) | (1 << (KotlinParser.SETPARAM - 61)) | (1 << (KotlinParser.DELEGATE - 61)) | (1 << (KotlinParser.IMPORT - 61)) | (1 << (KotlinParser.CONSTRUCTOR - 61)) | (1 << (KotlinParser.BY - 61)) | (1 << (KotlinParser.COMPANION - 61)) | (1 << (KotlinParser.INIT - 61)) | (1 << (KotlinParser.WHERE - 61)) | (1 << (KotlinParser.CATCH - 61)) | (1 << (KotlinParser.FINALLY - 61)) | (1 << (KotlinParser.OUT - 61)) | (1 << (KotlinParser.DYNAMIC - 61)) | (1 << (KotlinParser.PUBLIC - 61)) | (1 << (KotlinParser.PRIVATE - 61)) | (1 << (KotlinParser.PROTECTED - 61)) | (1 << (KotlinParser.INTERNAL - 61)) | (1 << (KotlinParser.ENUM - 61)) | (1 << (KotlinParser.SEALED - 61)) | (1 << (KotlinParser.ANNOTATION - 61)) | (1 << (KotlinParser.DATA - 61)) | (1 << (KotlinParser.INNER - 61)) | (1 << (KotlinParser.VALUE - 61)) | (1 << (KotlinParser.TAILREC - 61)) | (1 << (KotlinParser.OPERATOR - 61)) | (1 << (KotlinParser.INLINE - 61)) | (1 << (KotlinParser.INFIX - 61)) | (1 << (KotlinParser.EXTERNAL - 61)) | (1 << (KotlinParser.SUSPEND - 61)) | (1 << (KotlinParser.OVERRIDE - 61)) | (1 << (KotlinParser.ABSTRACT - 61)))) != 0) or ((((_la - 125)) & ~0x3f) == 0 and ((1 << (_la - 125)) & ((1 << (KotlinParser.FINAL - 125)) | (1 << (KotlinParser.OPEN - 125)) | (1 << (KotlinParser.CONST - 125)) | (1 << (KotlinParser.LATEINIT - 125)) | (1 << (KotlinParser.VARARG - 125)) | (1 << (KotlinParser.NOINLINE - 125)) | (1 << (KotlinParser.CROSSINLINE - 125)) | (1 << (KotlinParser.REIFIED - 125)) | (1 << (KotlinParser.EXPECT - 125)) | (1 << (KotlinParser.ACTUAL - 125)) | (1 << (KotlinParser.Identifier - 125)))) != 0)):
                        break

                self.state = 425
                self.match(KotlinParser.RSQUARE)
                pass
            elif token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 427
                self.unescapedAnnotation()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 433
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 430
                self.match(KotlinParser.NL)
                self.state = 435
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PackageHeaderContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PACKAGE(self):
            return self.getToken(KotlinParser.PACKAGE, 0)

        def identifier(self):
            return self.getTypedRuleContext(KotlinParser.IdentifierContext,0)


        def semi(self):
            return self.getTypedRuleContext(KotlinParser.SemiContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_packageHeader




    def packageHeader(self):

        localctx = KotlinParser.PackageHeaderContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_packageHeader)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 441
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.PACKAGE:
                self.state = 436
                self.match(KotlinParser.PACKAGE)
                self.state = 437
                self.identifier()
                self.state = 439
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,14,self._ctx)
                if la_ == 1:
                    self.state = 438
                    self.semi()




        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ImportListContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def importHeader(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ImportHeaderContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ImportHeaderContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_importList




    def importList(self):

        localctx = KotlinParser.ImportListContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_importList)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 446
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,16,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 443
                    self.importHeader() 
                self.state = 448
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,16,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ImportHeaderContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IMPORT(self):
            return self.getToken(KotlinParser.IMPORT, 0)

        def identifier(self):
            return self.getTypedRuleContext(KotlinParser.IdentifierContext,0)


        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def MULT(self):
            return self.getToken(KotlinParser.MULT, 0)

        def importAlias(self):
            return self.getTypedRuleContext(KotlinParser.ImportAliasContext,0)


        def semi(self):
            return self.getTypedRuleContext(KotlinParser.SemiContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_importHeader




    def importHeader(self):

        localctx = KotlinParser.ImportHeaderContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_importHeader)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 449
            self.match(KotlinParser.IMPORT)
            self.state = 450
            self.identifier()
            self.state = 454
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.DOT]:
                self.state = 451
                self.match(KotlinParser.DOT)
                self.state = 452
                self.match(KotlinParser.MULT)
                pass
            elif token in [KotlinParser.AS]:
                self.state = 453
                self.importAlias()
                pass
            elif token in [KotlinParser.EOF, KotlinParser.NL, KotlinParser.LPAREN, KotlinParser.LSQUARE, KotlinParser.LCURL, KotlinParser.ADD, KotlinParser.SUB, KotlinParser.INCR, KotlinParser.DECR, KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS, KotlinParser.SEMICOLON, KotlinParser.COLONCOLON, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.RETURN_AT, KotlinParser.CONTINUE_AT, KotlinParser.BREAK_AT, KotlinParser.THIS_AT, KotlinParser.SUPER_AT, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CLASS, KotlinParser.INTERFACE, KotlinParser.FUN, KotlinParser.OBJECT, KotlinParser.VAL, KotlinParser.VAR, KotlinParser.TYPE_ALIAS, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.THIS, KotlinParser.SUPER, KotlinParser.WHERE, KotlinParser.IF, KotlinParser.WHEN, KotlinParser.TRY, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.FOR, KotlinParser.DO, KotlinParser.WHILE, KotlinParser.THROW, KotlinParser.RETURN, KotlinParser.CONTINUE, KotlinParser.BREAK, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.RealLiteral, KotlinParser.IntegerLiteral, KotlinParser.HexLiteral, KotlinParser.BinLiteral, KotlinParser.UnsignedLiteral, KotlinParser.LongLiteral, KotlinParser.BooleanLiteral, KotlinParser.NullLiteral, KotlinParser.CharacterLiteral, KotlinParser.Identifier, KotlinParser.QUOTE_OPEN, KotlinParser.TRIPLE_QUOTE_OPEN]:
                pass
            else:
                pass
            self.state = 457
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,18,self._ctx)
            if la_ == 1:
                self.state = 456
                self.semi()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ImportAliasContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AS(self):
            return self.getToken(KotlinParser.AS, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_importAlias




    def importAlias(self):

        localctx = KotlinParser.ImportAliasContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_importAlias)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 459
            self.match(KotlinParser.AS)
            self.state = 460
            self.simpleIdentifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TopLevelObjectContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def declaration(self):
            return self.getTypedRuleContext(KotlinParser.DeclarationContext,0)


        def semis(self):
            return self.getTypedRuleContext(KotlinParser.SemisContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_topLevelObject




    def topLevelObject(self):

        localctx = KotlinParser.TopLevelObjectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_topLevelObject)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 462
            self.declaration()
            self.state = 464
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,19,self._ctx)
            if la_ == 1:
                self.state = 463
                self.semis()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeAliasContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TYPE_ALIAS(self):
            return self.getToken(KotlinParser.TYPE_ALIAS, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def typeParameters(self):
            return self.getTypedRuleContext(KotlinParser.TypeParametersContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeAlias




    def typeAlias(self):

        localctx = KotlinParser.TypeAliasContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_typeAlias)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 467
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 466
                self.modifiers()


            self.state = 469
            self.match(KotlinParser.TYPE_ALIAS)
            self.state = 473
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 470
                self.match(KotlinParser.NL)
                self.state = 475
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 476
            self.simpleIdentifier()
            self.state = 484
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,23,self._ctx)
            if la_ == 1:
                self.state = 480
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 477
                    self.match(KotlinParser.NL)
                    self.state = 482
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 483
                self.typeParameters()


            self.state = 489
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 486
                self.match(KotlinParser.NL)
                self.state = 491
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 492
            self.match(KotlinParser.ASSIGNMENT)
            self.state = 496
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 493
                self.match(KotlinParser.NL)
                self.state = 498
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 499
            self.type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def classDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.ClassDeclarationContext,0)


        def objectDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.ObjectDeclarationContext,0)


        def functionDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.FunctionDeclarationContext,0)


        def propertyDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.PropertyDeclarationContext,0)


        def typeAlias(self):
            return self.getTypedRuleContext(KotlinParser.TypeAliasContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_declaration




    def declaration(self):

        localctx = KotlinParser.DeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_declaration)
        try:
            self.state = 506
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,26,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 501
                self.classDeclaration()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 502
                self.objectDeclaration()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 503
                self.functionDeclaration()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 504
                self.propertyDeclaration()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 505
                self.typeAlias()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def CLASS(self):
            return self.getToken(KotlinParser.CLASS, 0)

        def INTERFACE(self):
            return self.getToken(KotlinParser.INTERFACE, 0)

        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def typeParameters(self):
            return self.getTypedRuleContext(KotlinParser.TypeParametersContext,0)


        def primaryConstructor(self):
            return self.getTypedRuleContext(KotlinParser.PrimaryConstructorContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def delegationSpecifiers(self):
            return self.getTypedRuleContext(KotlinParser.DelegationSpecifiersContext,0)


        def typeConstraints(self):
            return self.getTypedRuleContext(KotlinParser.TypeConstraintsContext,0)


        def classBody(self):
            return self.getTypedRuleContext(KotlinParser.ClassBodyContext,0)


        def enumClassBody(self):
            return self.getTypedRuleContext(KotlinParser.EnumClassBodyContext,0)


        def FUN(self):
            return self.getToken(KotlinParser.FUN, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_classDeclaration




    def classDeclaration(self):

        localctx = KotlinParser.ClassDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_classDeclaration)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 509
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 508
                self.modifiers()


            self.state = 522
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.CLASS]:
                self.state = 511
                self.match(KotlinParser.CLASS)
                pass
            elif token in [KotlinParser.INTERFACE, KotlinParser.FUN]:
                self.state = 519
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==KotlinParser.FUN:
                    self.state = 512
                    self.match(KotlinParser.FUN)
                    self.state = 516
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 513
                        self.match(KotlinParser.NL)
                        self.state = 518
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)



                self.state = 521
                self.match(KotlinParser.INTERFACE)
                pass
            else:
                raise NoViableAltException(self)

            self.state = 527
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 524
                self.match(KotlinParser.NL)
                self.state = 529
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 530
            self.simpleIdentifier()
            self.state = 538
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,33,self._ctx)
            if la_ == 1:
                self.state = 534
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 531
                    self.match(KotlinParser.NL)
                    self.state = 536
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 537
                self.typeParameters()


            self.state = 547
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,35,self._ctx)
            if la_ == 1:
                self.state = 543
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 540
                    self.match(KotlinParser.NL)
                    self.state = 545
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 546
                self.primaryConstructor()


            self.state = 563
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,38,self._ctx)
            if la_ == 1:
                self.state = 552
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 549
                    self.match(KotlinParser.NL)
                    self.state = 554
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 555
                self.match(KotlinParser.COLON)
                self.state = 559
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,37,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 556
                        self.match(KotlinParser.NL) 
                    self.state = 561
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,37,self._ctx)

                self.state = 562
                self.delegationSpecifiers()


            self.state = 572
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,40,self._ctx)
            if la_ == 1:
                self.state = 568
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 565
                    self.match(KotlinParser.NL)
                    self.state = 570
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 571
                self.typeConstraints()


            self.state = 588
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,43,self._ctx)
            if la_ == 1:
                self.state = 577
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 574
                    self.match(KotlinParser.NL)
                    self.state = 579
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 580
                self.classBody()

            elif la_ == 2:
                self.state = 584
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 581
                    self.match(KotlinParser.NL)
                    self.state = 586
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 587
                self.enumClassBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PrimaryConstructorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def classParameters(self):
            return self.getTypedRuleContext(KotlinParser.ClassParametersContext,0)


        def CONSTRUCTOR(self):
            return self.getToken(KotlinParser.CONSTRUCTOR, 0)

        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_primaryConstructor




    def primaryConstructor(self):

        localctx = KotlinParser.PrimaryConstructorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_primaryConstructor)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 600
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 79)) & ~0x3f) == 0 and ((1 << (_la - 79)) & ((1 << (KotlinParser.CONSTRUCTOR - 79)) | (1 << (KotlinParser.PUBLIC - 79)) | (1 << (KotlinParser.PRIVATE - 79)) | (1 << (KotlinParser.PROTECTED - 79)) | (1 << (KotlinParser.INTERNAL - 79)) | (1 << (KotlinParser.ENUM - 79)) | (1 << (KotlinParser.SEALED - 79)) | (1 << (KotlinParser.ANNOTATION - 79)) | (1 << (KotlinParser.DATA - 79)) | (1 << (KotlinParser.INNER - 79)) | (1 << (KotlinParser.VALUE - 79)) | (1 << (KotlinParser.TAILREC - 79)) | (1 << (KotlinParser.OPERATOR - 79)) | (1 << (KotlinParser.INLINE - 79)) | (1 << (KotlinParser.INFIX - 79)) | (1 << (KotlinParser.EXTERNAL - 79)) | (1 << (KotlinParser.SUSPEND - 79)) | (1 << (KotlinParser.OVERRIDE - 79)) | (1 << (KotlinParser.ABSTRACT - 79)) | (1 << (KotlinParser.FINAL - 79)) | (1 << (KotlinParser.OPEN - 79)) | (1 << (KotlinParser.CONST - 79)) | (1 << (KotlinParser.LATEINIT - 79)) | (1 << (KotlinParser.VARARG - 79)) | (1 << (KotlinParser.NOINLINE - 79)) | (1 << (KotlinParser.CROSSINLINE - 79)) | (1 << (KotlinParser.EXPECT - 79)) | (1 << (KotlinParser.ACTUAL - 79)))) != 0):
                self.state = 591
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                    self.state = 590
                    self.modifiers()


                self.state = 593
                self.match(KotlinParser.CONSTRUCTOR)
                self.state = 597
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 594
                    self.match(KotlinParser.NL)
                    self.state = 599
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 602
            self.classParameters()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassBodyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LCURL(self):
            return self.getToken(KotlinParser.LCURL, 0)

        def classMemberDeclarations(self):
            return self.getTypedRuleContext(KotlinParser.ClassMemberDeclarationsContext,0)


        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_classBody




    def classBody(self):

        localctx = KotlinParser.ClassBodyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_classBody)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 604
            self.match(KotlinParser.LCURL)
            self.state = 608
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,47,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 605
                    self.match(KotlinParser.NL) 
                self.state = 610
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,47,self._ctx)

            self.state = 611
            self.classMemberDeclarations()
            self.state = 615
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 612
                self.match(KotlinParser.NL)
                self.state = 617
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 618
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassParametersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def classParameter(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ClassParameterContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ClassParameterContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_classParameters




    def classParameters(self):

        localctx = KotlinParser.ClassParametersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_classParameters)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 620
            self.match(KotlinParser.LPAREN)
            self.state = 624
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,49,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 621
                    self.match(KotlinParser.NL) 
                self.state = 626
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,49,self._ctx)

            self.state = 656
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,55,self._ctx)
            if la_ == 1:
                self.state = 627
                self.classParameter()
                self.state = 644
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,52,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 631
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 628
                            self.match(KotlinParser.NL)
                            self.state = 633
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 634
                        self.match(KotlinParser.COMMA)
                        self.state = 638
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,51,self._ctx)
                        while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                            if _alt==1:
                                self.state = 635
                                self.match(KotlinParser.NL) 
                            self.state = 640
                            self._errHandler.sync(self)
                            _alt = self._interp.adaptivePredict(self._input,51,self._ctx)

                        self.state = 641
                        self.classParameter() 
                    self.state = 646
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,52,self._ctx)

                self.state = 654
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,54,self._ctx)
                if la_ == 1:
                    self.state = 650
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 647
                        self.match(KotlinParser.NL)
                        self.state = 652
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 653
                    self.match(KotlinParser.COMMA)




            self.state = 661
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 658
                self.match(KotlinParser.NL)
                self.state = 663
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 664
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassParameterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def VAL(self):
            return self.getToken(KotlinParser.VAL, 0)

        def VAR(self):
            return self.getToken(KotlinParser.VAR, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_classParameter




    def classParameter(self):

        localctx = KotlinParser.ClassParameterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_classParameter)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 667
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,57,self._ctx)
            if la_ == 1:
                self.state = 666
                self.modifiers()


            self.state = 670
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.VAL or _la==KotlinParser.VAR:
                self.state = 669
                _la = self._input.LA(1)
                if not(_la==KotlinParser.VAL or _la==KotlinParser.VAR):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 675
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 672
                self.match(KotlinParser.NL)
                self.state = 677
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 678
            self.simpleIdentifier()
            self.state = 679
            self.match(KotlinParser.COLON)
            self.state = 683
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 680
                self.match(KotlinParser.NL)
                self.state = 685
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 686
            self.type()
            self.state = 701
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,63,self._ctx)
            if la_ == 1:
                self.state = 690
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 687
                    self.match(KotlinParser.NL)
                    self.state = 692
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 693
                self.match(KotlinParser.ASSIGNMENT)
                self.state = 697
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 694
                    self.match(KotlinParser.NL)
                    self.state = 699
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 700
                self.expression()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DelegationSpecifiersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def annotatedDelegationSpecifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotatedDelegationSpecifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotatedDelegationSpecifierContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_delegationSpecifiers




    def delegationSpecifiers(self):

        localctx = KotlinParser.DelegationSpecifiersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_delegationSpecifiers)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 703
            self.annotatedDelegationSpecifier()
            self.state = 720
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,66,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 707
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 704
                        self.match(KotlinParser.NL)
                        self.state = 709
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 710
                    self.match(KotlinParser.COMMA)
                    self.state = 714
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,65,self._ctx)
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt==1:
                            self.state = 711
                            self.match(KotlinParser.NL) 
                        self.state = 716
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,65,self._ctx)

                    self.state = 717
                    self.annotatedDelegationSpecifier() 
                self.state = 722
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,66,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DelegationSpecifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def constructorInvocation(self):
            return self.getTypedRuleContext(KotlinParser.ConstructorInvocationContext,0)


        def explicitDelegation(self):
            return self.getTypedRuleContext(KotlinParser.ExplicitDelegationContext,0)


        def userType(self):
            return self.getTypedRuleContext(KotlinParser.UserTypeContext,0)


        def functionType(self):
            return self.getTypedRuleContext(KotlinParser.FunctionTypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_delegationSpecifier




    def delegationSpecifier(self):

        localctx = KotlinParser.DelegationSpecifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_delegationSpecifier)
        try:
            self.state = 727
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,67,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 723
                self.constructorInvocation()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 724
                self.explicitDelegation()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 725
                self.userType()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 726
                self.functionType()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ConstructorInvocationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def userType(self):
            return self.getTypedRuleContext(KotlinParser.UserTypeContext,0)


        def valueArguments(self):
            return self.getTypedRuleContext(KotlinParser.ValueArgumentsContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_constructorInvocation




    def constructorInvocation(self):

        localctx = KotlinParser.ConstructorInvocationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_constructorInvocation)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 729
            self.userType()
            self.state = 730
            self.valueArguments()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AnnotatedDelegationSpecifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def delegationSpecifier(self):
            return self.getTypedRuleContext(KotlinParser.DelegationSpecifierContext,0)


        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_annotatedDelegationSpecifier




    def annotatedDelegationSpecifier(self):

        localctx = KotlinParser.AnnotatedDelegationSpecifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_annotatedDelegationSpecifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 735
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,68,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 732
                    self.annotation() 
                self.state = 737
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,68,self._ctx)

            self.state = 741
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 738
                self.match(KotlinParser.NL)
                self.state = 743
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 744
            self.delegationSpecifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ExplicitDelegationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def BY(self):
            return self.getToken(KotlinParser.BY, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def userType(self):
            return self.getTypedRuleContext(KotlinParser.UserTypeContext,0)


        def functionType(self):
            return self.getTypedRuleContext(KotlinParser.FunctionTypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_explicitDelegation




    def explicitDelegation(self):

        localctx = KotlinParser.ExplicitDelegationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_explicitDelegation)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 748
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,70,self._ctx)
            if la_ == 1:
                self.state = 746
                self.userType()
                pass

            elif la_ == 2:
                self.state = 747
                self.functionType()
                pass


            self.state = 753
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 750
                self.match(KotlinParser.NL)
                self.state = 755
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 756
            self.match(KotlinParser.BY)
            self.state = 760
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 757
                self.match(KotlinParser.NL)
                self.state = 762
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 763
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeParametersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LANGLE(self):
            return self.getToken(KotlinParser.LANGLE, 0)

        def typeParameter(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeParameterContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeParameterContext,i)


        def RANGLE(self):
            return self.getToken(KotlinParser.RANGLE, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeParameters




    def typeParameters(self):

        localctx = KotlinParser.TypeParametersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_typeParameters)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 765
            self.match(KotlinParser.LANGLE)
            self.state = 769
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,73,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 766
                    self.match(KotlinParser.NL) 
                self.state = 771
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,73,self._ctx)

            self.state = 772
            self.typeParameter()
            self.state = 789
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,76,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 776
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 773
                        self.match(KotlinParser.NL)
                        self.state = 778
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 779
                    self.match(KotlinParser.COMMA)
                    self.state = 783
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,75,self._ctx)
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt==1:
                            self.state = 780
                            self.match(KotlinParser.NL) 
                        self.state = 785
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,75,self._ctx)

                    self.state = 786
                    self.typeParameter() 
                self.state = 791
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,76,self._ctx)

            self.state = 799
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,78,self._ctx)
            if la_ == 1:
                self.state = 795
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 792
                    self.match(KotlinParser.NL)
                    self.state = 797
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 798
                self.match(KotlinParser.COMMA)


            self.state = 804
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 801
                self.match(KotlinParser.NL)
                self.state = 806
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 807
            self.match(KotlinParser.RANGLE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeParameterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def typeParameterModifiers(self):
            return self.getTypedRuleContext(KotlinParser.TypeParameterModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeParameter




    def typeParameter(self):

        localctx = KotlinParser.TypeParameterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_typeParameter)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 810
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,80,self._ctx)
            if la_ == 1:
                self.state = 809
                self.typeParameterModifiers()


            self.state = 815
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 812
                self.match(KotlinParser.NL)
                self.state = 817
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 818
            self.simpleIdentifier()
            self.state = 833
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,84,self._ctx)
            if la_ == 1:
                self.state = 822
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 819
                    self.match(KotlinParser.NL)
                    self.state = 824
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 825
                self.match(KotlinParser.COLON)
                self.state = 829
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 826
                    self.match(KotlinParser.NL)
                    self.state = 831
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 832
                self.type()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeConstraintsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def WHERE(self):
            return self.getToken(KotlinParser.WHERE, 0)

        def typeConstraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeConstraintContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeConstraintContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeConstraints




    def typeConstraints(self):

        localctx = KotlinParser.TypeConstraintsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_typeConstraints)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 835
            self.match(KotlinParser.WHERE)
            self.state = 839
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 836
                self.match(KotlinParser.NL)
                self.state = 841
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 842
            self.typeConstraint()
            self.state = 859
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,88,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 846
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 843
                        self.match(KotlinParser.NL)
                        self.state = 848
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 849
                    self.match(KotlinParser.COMMA)
                    self.state = 853
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 850
                        self.match(KotlinParser.NL)
                        self.state = 855
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 856
                    self.typeConstraint() 
                self.state = 861
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,88,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeConstraintContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeConstraint




    def typeConstraint(self):

        localctx = KotlinParser.TypeConstraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_typeConstraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 865
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS:
                self.state = 862
                self.annotation()
                self.state = 867
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 868
            self.simpleIdentifier()
            self.state = 872
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 869
                self.match(KotlinParser.NL)
                self.state = 874
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 875
            self.match(KotlinParser.COLON)
            self.state = 879
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 876
                self.match(KotlinParser.NL)
                self.state = 881
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 882
            self.type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassMemberDeclarationsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def classMemberDeclaration(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ClassMemberDeclarationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ClassMemberDeclarationContext,i)


        def semis(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.SemisContext)
            else:
                return self.getTypedRuleContext(KotlinParser.SemisContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_classMemberDeclarations




    def classMemberDeclarations(self):

        localctx = KotlinParser.ClassMemberDeclarationsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_classMemberDeclarations)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 890
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 72)) & ~0x3f) == 0 and ((1 << (_la - 72)) & ((1 << (KotlinParser.CLASS - 72)) | (1 << (KotlinParser.INTERFACE - 72)) | (1 << (KotlinParser.FUN - 72)) | (1 << (KotlinParser.OBJECT - 72)) | (1 << (KotlinParser.VAL - 72)) | (1 << (KotlinParser.VAR - 72)) | (1 << (KotlinParser.TYPE_ALIAS - 72)) | (1 << (KotlinParser.CONSTRUCTOR - 72)) | (1 << (KotlinParser.COMPANION - 72)) | (1 << (KotlinParser.INIT - 72)) | (1 << (KotlinParser.PUBLIC - 72)) | (1 << (KotlinParser.PRIVATE - 72)) | (1 << (KotlinParser.PROTECTED - 72)) | (1 << (KotlinParser.INTERNAL - 72)) | (1 << (KotlinParser.ENUM - 72)) | (1 << (KotlinParser.SEALED - 72)) | (1 << (KotlinParser.ANNOTATION - 72)) | (1 << (KotlinParser.DATA - 72)) | (1 << (KotlinParser.INNER - 72)) | (1 << (KotlinParser.VALUE - 72)) | (1 << (KotlinParser.TAILREC - 72)) | (1 << (KotlinParser.OPERATOR - 72)) | (1 << (KotlinParser.INLINE - 72)) | (1 << (KotlinParser.INFIX - 72)) | (1 << (KotlinParser.EXTERNAL - 72)) | (1 << (KotlinParser.SUSPEND - 72)) | (1 << (KotlinParser.OVERRIDE - 72)) | (1 << (KotlinParser.ABSTRACT - 72)) | (1 << (KotlinParser.FINAL - 72)) | (1 << (KotlinParser.OPEN - 72)) | (1 << (KotlinParser.CONST - 72)) | (1 << (KotlinParser.LATEINIT - 72)) | (1 << (KotlinParser.VARARG - 72)) | (1 << (KotlinParser.NOINLINE - 72)) | (1 << (KotlinParser.CROSSINLINE - 72)) | (1 << (KotlinParser.EXPECT - 72)) | (1 << (KotlinParser.ACTUAL - 72)))) != 0):
                self.state = 884
                self.classMemberDeclaration()
                self.state = 886
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,92,self._ctx)
                if la_ == 1:
                    self.state = 885
                    self.semis()


                self.state = 892
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassMemberDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def declaration(self):
            return self.getTypedRuleContext(KotlinParser.DeclarationContext,0)


        def companionObject(self):
            return self.getTypedRuleContext(KotlinParser.CompanionObjectContext,0)


        def anonymousInitializer(self):
            return self.getTypedRuleContext(KotlinParser.AnonymousInitializerContext,0)


        def secondaryConstructor(self):
            return self.getTypedRuleContext(KotlinParser.SecondaryConstructorContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_classMemberDeclaration




    def classMemberDeclaration(self):

        localctx = KotlinParser.ClassMemberDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_classMemberDeclaration)
        try:
            self.state = 897
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,94,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 893
                self.declaration()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 894
                self.companionObject()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 895
                self.anonymousInitializer()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 896
                self.secondaryConstructor()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AnonymousInitializerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def INIT(self):
            return self.getToken(KotlinParser.INIT, 0)

        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_anonymousInitializer




    def anonymousInitializer(self):

        localctx = KotlinParser.AnonymousInitializerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_anonymousInitializer)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 899
            self.match(KotlinParser.INIT)
            self.state = 903
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 900
                self.match(KotlinParser.NL)
                self.state = 905
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 906
            self.block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CompanionObjectContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMPANION(self):
            return self.getToken(KotlinParser.COMPANION, 0)

        def OBJECT(self):
            return self.getToken(KotlinParser.OBJECT, 0)

        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def delegationSpecifiers(self):
            return self.getTypedRuleContext(KotlinParser.DelegationSpecifiersContext,0)


        def classBody(self):
            return self.getTypedRuleContext(KotlinParser.ClassBodyContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_companionObject




    def companionObject(self):

        localctx = KotlinParser.CompanionObjectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_companionObject)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 909
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 908
                self.modifiers()


            self.state = 911
            self.match(KotlinParser.COMPANION)
            self.state = 915
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 912
                self.match(KotlinParser.NL)
                self.state = 917
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 918
            self.match(KotlinParser.OBJECT)
            self.state = 926
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,99,self._ctx)
            if la_ == 1:
                self.state = 922
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 919
                    self.match(KotlinParser.NL)
                    self.state = 924
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 925
                self.simpleIdentifier()


            self.state = 942
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,102,self._ctx)
            if la_ == 1:
                self.state = 931
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 928
                    self.match(KotlinParser.NL)
                    self.state = 933
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 934
                self.match(KotlinParser.COLON)
                self.state = 938
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,101,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 935
                        self.match(KotlinParser.NL) 
                    self.state = 940
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,101,self._ctx)

                self.state = 941
                self.delegationSpecifiers()


            self.state = 951
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,104,self._ctx)
            if la_ == 1:
                self.state = 947
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 944
                    self.match(KotlinParser.NL)
                    self.state = 949
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 950
                self.classBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionValueParametersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def functionValueParameter(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.FunctionValueParameterContext)
            else:
                return self.getTypedRuleContext(KotlinParser.FunctionValueParameterContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionValueParameters




    def functionValueParameters(self):

        localctx = KotlinParser.FunctionValueParametersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_functionValueParameters)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 953
            self.match(KotlinParser.LPAREN)
            self.state = 957
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,105,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 954
                    self.match(KotlinParser.NL) 
                self.state = 959
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,105,self._ctx)

            self.state = 989
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if ((((_la - 40)) & ~0x3f) == 0 and ((1 << (_la - 40)) & ((1 << (KotlinParser.AT_NO_WS - 40)) | (1 << (KotlinParser.AT_PRE_WS - 40)) | (1 << (KotlinParser.FILE - 40)) | (1 << (KotlinParser.FIELD - 40)) | (1 << (KotlinParser.PROPERTY - 40)) | (1 << (KotlinParser.GET - 40)) | (1 << (KotlinParser.SET - 40)) | (1 << (KotlinParser.RECEIVER - 40)) | (1 << (KotlinParser.PARAM - 40)) | (1 << (KotlinParser.SETPARAM - 40)) | (1 << (KotlinParser.DELEGATE - 40)) | (1 << (KotlinParser.IMPORT - 40)) | (1 << (KotlinParser.CONSTRUCTOR - 40)) | (1 << (KotlinParser.BY - 40)) | (1 << (KotlinParser.COMPANION - 40)) | (1 << (KotlinParser.INIT - 40)) | (1 << (KotlinParser.WHERE - 40)) | (1 << (KotlinParser.CATCH - 40)) | (1 << (KotlinParser.FINALLY - 40)))) != 0) or ((((_la - 105)) & ~0x3f) == 0 and ((1 << (_la - 105)) & ((1 << (KotlinParser.OUT - 105)) | (1 << (KotlinParser.DYNAMIC - 105)) | (1 << (KotlinParser.PUBLIC - 105)) | (1 << (KotlinParser.PRIVATE - 105)) | (1 << (KotlinParser.PROTECTED - 105)) | (1 << (KotlinParser.INTERNAL - 105)) | (1 << (KotlinParser.ENUM - 105)) | (1 << (KotlinParser.SEALED - 105)) | (1 << (KotlinParser.ANNOTATION - 105)) | (1 << (KotlinParser.DATA - 105)) | (1 << (KotlinParser.INNER - 105)) | (1 << (KotlinParser.VALUE - 105)) | (1 << (KotlinParser.TAILREC - 105)) | (1 << (KotlinParser.OPERATOR - 105)) | (1 << (KotlinParser.INLINE - 105)) | (1 << (KotlinParser.INFIX - 105)) | (1 << (KotlinParser.EXTERNAL - 105)) | (1 << (KotlinParser.SUSPEND - 105)) | (1 << (KotlinParser.OVERRIDE - 105)) | (1 << (KotlinParser.ABSTRACT - 105)) | (1 << (KotlinParser.FINAL - 105)) | (1 << (KotlinParser.OPEN - 105)) | (1 << (KotlinParser.CONST - 105)) | (1 << (KotlinParser.LATEINIT - 105)) | (1 << (KotlinParser.VARARG - 105)) | (1 << (KotlinParser.NOINLINE - 105)) | (1 << (KotlinParser.CROSSINLINE - 105)) | (1 << (KotlinParser.REIFIED - 105)) | (1 << (KotlinParser.EXPECT - 105)) | (1 << (KotlinParser.ACTUAL - 105)) | (1 << (KotlinParser.Identifier - 105)))) != 0):
                self.state = 960
                self.functionValueParameter()
                self.state = 977
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,108,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 964
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 961
                            self.match(KotlinParser.NL)
                            self.state = 966
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 967
                        self.match(KotlinParser.COMMA)
                        self.state = 971
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 968
                            self.match(KotlinParser.NL)
                            self.state = 973
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 974
                        self.functionValueParameter() 
                    self.state = 979
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,108,self._ctx)

                self.state = 987
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,110,self._ctx)
                if la_ == 1:
                    self.state = 983
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 980
                        self.match(KotlinParser.NL)
                        self.state = 985
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 986
                    self.match(KotlinParser.COMMA)




            self.state = 994
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 991
                self.match(KotlinParser.NL)
                self.state = 996
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 997
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionValueParameterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def parameter(self):
            return self.getTypedRuleContext(KotlinParser.ParameterContext,0)


        def parameterModifiers(self):
            return self.getTypedRuleContext(KotlinParser.ParameterModifiersContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionValueParameter




    def functionValueParameter(self):

        localctx = KotlinParser.FunctionValueParameterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_functionValueParameter)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1000
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,113,self._ctx)
            if la_ == 1:
                self.state = 999
                self.parameterModifiers()


            self.state = 1002
            self.parameter()
            self.state = 1017
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,116,self._ctx)
            if la_ == 1:
                self.state = 1006
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1003
                    self.match(KotlinParser.NL)
                    self.state = 1008
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1009
                self.match(KotlinParser.ASSIGNMENT)
                self.state = 1013
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1010
                    self.match(KotlinParser.NL)
                    self.state = 1015
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1016
                self.expression()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def FUN(self):
            return self.getToken(KotlinParser.FUN, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def functionValueParameters(self):
            return self.getTypedRuleContext(KotlinParser.FunctionValueParametersContext,0)


        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def typeParameters(self):
            return self.getTypedRuleContext(KotlinParser.TypeParametersContext,0)


        def receiverType(self):
            return self.getTypedRuleContext(KotlinParser.ReceiverTypeContext,0)


        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def typeConstraints(self):
            return self.getTypedRuleContext(KotlinParser.TypeConstraintsContext,0)


        def functionBody(self):
            return self.getTypedRuleContext(KotlinParser.FunctionBodyContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_functionDeclaration




    def functionDeclaration(self):

        localctx = KotlinParser.FunctionDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_functionDeclaration)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1020
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 1019
                self.modifiers()


            self.state = 1022
            self.match(KotlinParser.FUN)
            self.state = 1030
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,119,self._ctx)
            if la_ == 1:
                self.state = 1026
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1023
                    self.match(KotlinParser.NL)
                    self.state = 1028
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1029
                self.typeParameters()


            self.state = 1047
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,122,self._ctx)
            if la_ == 1:
                self.state = 1035
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1032
                    self.match(KotlinParser.NL)
                    self.state = 1037
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1038
                self.receiverType()
                self.state = 1042
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1039
                    self.match(KotlinParser.NL)
                    self.state = 1044
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1045
                self.match(KotlinParser.DOT)


            self.state = 1052
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1049
                self.match(KotlinParser.NL)
                self.state = 1054
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1055
            self.simpleIdentifier()
            self.state = 1059
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1056
                self.match(KotlinParser.NL)
                self.state = 1061
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1062
            self.functionValueParameters()
            self.state = 1077
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,127,self._ctx)
            if la_ == 1:
                self.state = 1066
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1063
                    self.match(KotlinParser.NL)
                    self.state = 1068
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1069
                self.match(KotlinParser.COLON)
                self.state = 1073
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1070
                    self.match(KotlinParser.NL)
                    self.state = 1075
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1076
                self.type()


            self.state = 1086
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,129,self._ctx)
            if la_ == 1:
                self.state = 1082
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1079
                    self.match(KotlinParser.NL)
                    self.state = 1084
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1085
                self.typeConstraints()


            self.state = 1095
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,131,self._ctx)
            if la_ == 1:
                self.state = 1091
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1088
                    self.match(KotlinParser.NL)
                    self.state = 1093
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1094
                self.functionBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionBodyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionBody




    def functionBody(self):

        localctx = KotlinParser.FunctionBodyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_functionBody)
        self._la = 0 # Token type
        try:
            self.state = 1106
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LCURL]:
                self.enterOuterAlt(localctx, 1)
                self.state = 1097
                self.block()
                pass
            elif token in [KotlinParser.ASSIGNMENT]:
                self.enterOuterAlt(localctx, 2)
                self.state = 1098
                self.match(KotlinParser.ASSIGNMENT)
                self.state = 1102
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1099
                    self.match(KotlinParser.NL)
                    self.state = 1104
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1105
                self.expression()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class VariableDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_variableDeclaration




    def variableDeclaration(self):

        localctx = KotlinParser.VariableDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_variableDeclaration)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1111
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS:
                self.state = 1108
                self.annotation()
                self.state = 1113
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1117
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1114
                self.match(KotlinParser.NL)
                self.state = 1119
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1120
            self.simpleIdentifier()
            self.state = 1135
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,138,self._ctx)
            if la_ == 1:
                self.state = 1124
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1121
                    self.match(KotlinParser.NL)
                    self.state = 1126
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1127
                self.match(KotlinParser.COLON)
                self.state = 1131
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1128
                    self.match(KotlinParser.NL)
                    self.state = 1133
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1134
                self.type()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiVariableDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def variableDeclaration(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.VariableDeclarationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.VariableDeclarationContext,i)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiVariableDeclaration




    def multiVariableDeclaration(self):

        localctx = KotlinParser.MultiVariableDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_multiVariableDeclaration)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1137
            self.match(KotlinParser.LPAREN)
            self.state = 1141
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,139,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1138
                    self.match(KotlinParser.NL) 
                self.state = 1143
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,139,self._ctx)

            self.state = 1144
            self.variableDeclaration()
            self.state = 1161
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,142,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1148
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1145
                        self.match(KotlinParser.NL)
                        self.state = 1150
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1151
                    self.match(KotlinParser.COMMA)
                    self.state = 1155
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,141,self._ctx)
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt==1:
                            self.state = 1152
                            self.match(KotlinParser.NL) 
                        self.state = 1157
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,141,self._ctx)

                    self.state = 1158
                    self.variableDeclaration() 
                self.state = 1163
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,142,self._ctx)

            self.state = 1171
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,144,self._ctx)
            if la_ == 1:
                self.state = 1167
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1164
                    self.match(KotlinParser.NL)
                    self.state = 1169
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1170
                self.match(KotlinParser.COMMA)


            self.state = 1176
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1173
                self.match(KotlinParser.NL)
                self.state = 1178
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1179
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropertyDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def VAL(self):
            return self.getToken(KotlinParser.VAL, 0)

        def VAR(self):
            return self.getToken(KotlinParser.VAR, 0)

        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def typeParameters(self):
            return self.getTypedRuleContext(KotlinParser.TypeParametersContext,0)


        def receiverType(self):
            return self.getTypedRuleContext(KotlinParser.ReceiverTypeContext,0)


        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def typeConstraints(self):
            return self.getTypedRuleContext(KotlinParser.TypeConstraintsContext,0)


        def SEMICOLON(self):
            return self.getToken(KotlinParser.SEMICOLON, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def multiVariableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.MultiVariableDeclarationContext,0)


        def variableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.VariableDeclarationContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def propertyDelegate(self):
            return self.getTypedRuleContext(KotlinParser.PropertyDelegateContext,0)


        def getter(self):
            return self.getTypedRuleContext(KotlinParser.GetterContext,0)


        def setter(self):
            return self.getTypedRuleContext(KotlinParser.SetterContext,0)


        def semi(self):
            return self.getTypedRuleContext(KotlinParser.SemiContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_propertyDeclaration




    def propertyDeclaration(self):

        localctx = KotlinParser.PropertyDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_propertyDeclaration)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1182
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 1181
                self.modifiers()


            self.state = 1184
            _la = self._input.LA(1)
            if not(_la==KotlinParser.VAL or _la==KotlinParser.VAR):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 1192
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,148,self._ctx)
            if la_ == 1:
                self.state = 1188
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1185
                    self.match(KotlinParser.NL)
                    self.state = 1190
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1191
                self.typeParameters()


            self.state = 1209
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,151,self._ctx)
            if la_ == 1:
                self.state = 1197
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1194
                    self.match(KotlinParser.NL)
                    self.state = 1199
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1200
                self.receiverType()
                self.state = 1204
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1201
                    self.match(KotlinParser.NL)
                    self.state = 1206
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1207
                self.match(KotlinParser.DOT)


            self.state = 1214
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,152,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1211
                    self.match(KotlinParser.NL) 
                self.state = 1216
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,152,self._ctx)

            self.state = 1219
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LPAREN]:
                self.state = 1217
                self.multiVariableDeclaration()
                pass
            elif token in [KotlinParser.NL, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 1218
                self.variableDeclaration()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 1228
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,155,self._ctx)
            if la_ == 1:
                self.state = 1224
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1221
                    self.match(KotlinParser.NL)
                    self.state = 1226
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1227
                self.typeConstraints()


            self.state = 1247
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,159,self._ctx)
            if la_ == 1:
                self.state = 1233
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1230
                    self.match(KotlinParser.NL)
                    self.state = 1235
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1245
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [KotlinParser.ASSIGNMENT]:
                    self.state = 1236
                    self.match(KotlinParser.ASSIGNMENT)
                    self.state = 1240
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1237
                        self.match(KotlinParser.NL)
                        self.state = 1242
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1243
                    self.expression()
                    pass
                elif token in [KotlinParser.BY]:
                    self.state = 1244
                    self.propertyDelegate()
                    pass
                else:
                    raise NoViableAltException(self)



            self.state = 1255
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,161,self._ctx)
            if la_ == 1:
                self.state = 1250 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while True:
                    self.state = 1249
                    self.match(KotlinParser.NL)
                    self.state = 1252 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if not (_la==KotlinParser.NL):
                        break

                self.state = 1254
                self.match(KotlinParser.SEMICOLON)


            self.state = 1260
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,162,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1257
                    self.match(KotlinParser.NL) 
                self.state = 1262
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,162,self._ctx)

            self.state = 1293
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,171,self._ctx)
            if la_ == 1:
                self.state = 1264
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,163,self._ctx)
                if la_ == 1:
                    self.state = 1263
                    self.getter()


                self.state = 1276
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,166,self._ctx)
                if la_ == 1:
                    self.state = 1269
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,164,self._ctx)
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt==1:
                            self.state = 1266
                            self.match(KotlinParser.NL) 
                        self.state = 1271
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,164,self._ctx)

                    self.state = 1273
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if ((((_la - -1)) & ~0x3f) == 0 and ((1 << (_la - -1)) & ((1 << (KotlinParser.EOF - -1)) | (1 << (KotlinParser.NL - -1)) | (1 << (KotlinParser.SEMICOLON - -1)))) != 0):
                        self.state = 1272
                        self.semi()


                    self.state = 1275
                    self.setter()


                pass

            elif la_ == 2:
                self.state = 1279
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,167,self._ctx)
                if la_ == 1:
                    self.state = 1278
                    self.setter()


                self.state = 1291
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,170,self._ctx)
                if la_ == 1:
                    self.state = 1284
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,168,self._ctx)
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt==1:
                            self.state = 1281
                            self.match(KotlinParser.NL) 
                        self.state = 1286
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,168,self._ctx)

                    self.state = 1288
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if ((((_la - -1)) & ~0x3f) == 0 and ((1 << (_la - -1)) & ((1 << (KotlinParser.EOF - -1)) | (1 << (KotlinParser.NL - -1)) | (1 << (KotlinParser.SEMICOLON - -1)))) != 0):
                        self.state = 1287
                        self.semi()


                    self.state = 1290
                    self.getter()


                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropertyDelegateContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def BY(self):
            return self.getToken(KotlinParser.BY, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_propertyDelegate




    def propertyDelegate(self):

        localctx = KotlinParser.PropertyDelegateContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_propertyDelegate)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1295
            self.match(KotlinParser.BY)
            self.state = 1299
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1296
                self.match(KotlinParser.NL)
                self.state = 1301
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1302
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class GetterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def GET(self):
            return self.getToken(KotlinParser.GET, 0)

        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def functionBody(self):
            return self.getTypedRuleContext(KotlinParser.FunctionBodyContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_getter




    def getter(self):

        localctx = KotlinParser.GetterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_getter)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1305
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 1304
                self.modifiers()


            self.state = 1307
            self.match(KotlinParser.GET)
            self.state = 1345
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,180,self._ctx)
            if la_ == 1:
                self.state = 1311
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1308
                    self.match(KotlinParser.NL)
                    self.state = 1313
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1314
                self.match(KotlinParser.LPAREN)
                self.state = 1318
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1315
                    self.match(KotlinParser.NL)
                    self.state = 1320
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1321
                self.match(KotlinParser.RPAREN)
                self.state = 1336
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,178,self._ctx)
                if la_ == 1:
                    self.state = 1325
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1322
                        self.match(KotlinParser.NL)
                        self.state = 1327
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1328
                    self.match(KotlinParser.COLON)
                    self.state = 1332
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1329
                        self.match(KotlinParser.NL)
                        self.state = 1334
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1335
                    self.type()


                self.state = 1341
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1338
                    self.match(KotlinParser.NL)
                    self.state = 1343
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1344
                self.functionBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SetterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def SET(self):
            return self.getToken(KotlinParser.SET, 0)

        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def functionValueParameterWithOptionalType(self):
            return self.getTypedRuleContext(KotlinParser.FunctionValueParameterWithOptionalTypeContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def functionBody(self):
            return self.getTypedRuleContext(KotlinParser.FunctionBodyContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COMMA(self):
            return self.getToken(KotlinParser.COMMA, 0)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_setter




    def setter(self):

        localctx = KotlinParser.SetterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_setter)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1348
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 1347
                self.modifiers()


            self.state = 1350
            self.match(KotlinParser.SET)
            self.state = 1405
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,191,self._ctx)
            if la_ == 1:
                self.state = 1354
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1351
                    self.match(KotlinParser.NL)
                    self.state = 1356
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1357
                self.match(KotlinParser.LPAREN)
                self.state = 1361
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1358
                    self.match(KotlinParser.NL)
                    self.state = 1363
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1364
                self.functionValueParameterWithOptionalType()
                self.state = 1372
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,185,self._ctx)
                if la_ == 1:
                    self.state = 1368
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1365
                        self.match(KotlinParser.NL)
                        self.state = 1370
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1371
                    self.match(KotlinParser.COMMA)


                self.state = 1377
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1374
                    self.match(KotlinParser.NL)
                    self.state = 1379
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1380
                self.match(KotlinParser.RPAREN)
                self.state = 1395
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,189,self._ctx)
                if la_ == 1:
                    self.state = 1384
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1381
                        self.match(KotlinParser.NL)
                        self.state = 1386
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1387
                    self.match(KotlinParser.COLON)
                    self.state = 1391
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1388
                        self.match(KotlinParser.NL)
                        self.state = 1393
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1394
                    self.type()


                self.state = 1400
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1397
                    self.match(KotlinParser.NL)
                    self.state = 1402
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1403
                self.functionBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParametersWithOptionalTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def functionValueParameterWithOptionalType(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.FunctionValueParameterWithOptionalTypeContext)
            else:
                return self.getTypedRuleContext(KotlinParser.FunctionValueParameterWithOptionalTypeContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parametersWithOptionalType




    def parametersWithOptionalType(self):

        localctx = KotlinParser.ParametersWithOptionalTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_parametersWithOptionalType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1407
            self.match(KotlinParser.LPAREN)
            self.state = 1411
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,192,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1408
                    self.match(KotlinParser.NL) 
                self.state = 1413
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,192,self._ctx)

            self.state = 1443
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if ((((_la - 40)) & ~0x3f) == 0 and ((1 << (_la - 40)) & ((1 << (KotlinParser.AT_NO_WS - 40)) | (1 << (KotlinParser.AT_PRE_WS - 40)) | (1 << (KotlinParser.FILE - 40)) | (1 << (KotlinParser.FIELD - 40)) | (1 << (KotlinParser.PROPERTY - 40)) | (1 << (KotlinParser.GET - 40)) | (1 << (KotlinParser.SET - 40)) | (1 << (KotlinParser.RECEIVER - 40)) | (1 << (KotlinParser.PARAM - 40)) | (1 << (KotlinParser.SETPARAM - 40)) | (1 << (KotlinParser.DELEGATE - 40)) | (1 << (KotlinParser.IMPORT - 40)) | (1 << (KotlinParser.CONSTRUCTOR - 40)) | (1 << (KotlinParser.BY - 40)) | (1 << (KotlinParser.COMPANION - 40)) | (1 << (KotlinParser.INIT - 40)) | (1 << (KotlinParser.WHERE - 40)) | (1 << (KotlinParser.CATCH - 40)) | (1 << (KotlinParser.FINALLY - 40)))) != 0) or ((((_la - 105)) & ~0x3f) == 0 and ((1 << (_la - 105)) & ((1 << (KotlinParser.OUT - 105)) | (1 << (KotlinParser.DYNAMIC - 105)) | (1 << (KotlinParser.PUBLIC - 105)) | (1 << (KotlinParser.PRIVATE - 105)) | (1 << (KotlinParser.PROTECTED - 105)) | (1 << (KotlinParser.INTERNAL - 105)) | (1 << (KotlinParser.ENUM - 105)) | (1 << (KotlinParser.SEALED - 105)) | (1 << (KotlinParser.ANNOTATION - 105)) | (1 << (KotlinParser.DATA - 105)) | (1 << (KotlinParser.INNER - 105)) | (1 << (KotlinParser.VALUE - 105)) | (1 << (KotlinParser.TAILREC - 105)) | (1 << (KotlinParser.OPERATOR - 105)) | (1 << (KotlinParser.INLINE - 105)) | (1 << (KotlinParser.INFIX - 105)) | (1 << (KotlinParser.EXTERNAL - 105)) | (1 << (KotlinParser.SUSPEND - 105)) | (1 << (KotlinParser.OVERRIDE - 105)) | (1 << (KotlinParser.ABSTRACT - 105)) | (1 << (KotlinParser.FINAL - 105)) | (1 << (KotlinParser.OPEN - 105)) | (1 << (KotlinParser.CONST - 105)) | (1 << (KotlinParser.LATEINIT - 105)) | (1 << (KotlinParser.VARARG - 105)) | (1 << (KotlinParser.NOINLINE - 105)) | (1 << (KotlinParser.CROSSINLINE - 105)) | (1 << (KotlinParser.REIFIED - 105)) | (1 << (KotlinParser.EXPECT - 105)) | (1 << (KotlinParser.ACTUAL - 105)) | (1 << (KotlinParser.Identifier - 105)))) != 0):
                self.state = 1414
                self.functionValueParameterWithOptionalType()
                self.state = 1431
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,195,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 1418
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 1415
                            self.match(KotlinParser.NL)
                            self.state = 1420
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 1421
                        self.match(KotlinParser.COMMA)
                        self.state = 1425
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 1422
                            self.match(KotlinParser.NL)
                            self.state = 1427
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 1428
                        self.functionValueParameterWithOptionalType() 
                    self.state = 1433
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,195,self._ctx)

                self.state = 1441
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,197,self._ctx)
                if la_ == 1:
                    self.state = 1437
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1434
                        self.match(KotlinParser.NL)
                        self.state = 1439
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1440
                    self.match(KotlinParser.COMMA)




            self.state = 1448
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1445
                self.match(KotlinParser.NL)
                self.state = 1450
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1451
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionValueParameterWithOptionalTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def parameterWithOptionalType(self):
            return self.getTypedRuleContext(KotlinParser.ParameterWithOptionalTypeContext,0)


        def parameterModifiers(self):
            return self.getTypedRuleContext(KotlinParser.ParameterModifiersContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionValueParameterWithOptionalType




    def functionValueParameterWithOptionalType(self):

        localctx = KotlinParser.FunctionValueParameterWithOptionalTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_functionValueParameterWithOptionalType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1454
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,200,self._ctx)
            if la_ == 1:
                self.state = 1453
                self.parameterModifiers()


            self.state = 1456
            self.parameterWithOptionalType()
            self.state = 1471
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,203,self._ctx)
            if la_ == 1:
                self.state = 1460
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1457
                    self.match(KotlinParser.NL)
                    self.state = 1462
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1463
                self.match(KotlinParser.ASSIGNMENT)
                self.state = 1467
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1464
                    self.match(KotlinParser.NL)
                    self.state = 1469
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1470
                self.expression()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParameterWithOptionalTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_parameterWithOptionalType




    def parameterWithOptionalType(self):

        localctx = KotlinParser.ParameterWithOptionalTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_parameterWithOptionalType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1473
            self.simpleIdentifier()
            self.state = 1477
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,204,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1474
                    self.match(KotlinParser.NL) 
                self.state = 1479
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,204,self._ctx)

            self.state = 1488
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.COLON:
                self.state = 1480
                self.match(KotlinParser.COLON)
                self.state = 1484
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1481
                    self.match(KotlinParser.NL)
                    self.state = 1486
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1487
                self.type()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParameterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parameter




    def parameter(self):

        localctx = KotlinParser.ParameterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_parameter)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1490
            self.simpleIdentifier()
            self.state = 1494
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1491
                self.match(KotlinParser.NL)
                self.state = 1496
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1497
            self.match(KotlinParser.COLON)
            self.state = 1501
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1498
                self.match(KotlinParser.NL)
                self.state = 1503
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1504
            self.type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ObjectDeclarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def OBJECT(self):
            return self.getToken(KotlinParser.OBJECT, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def delegationSpecifiers(self):
            return self.getTypedRuleContext(KotlinParser.DelegationSpecifiersContext,0)


        def classBody(self):
            return self.getTypedRuleContext(KotlinParser.ClassBodyContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_objectDeclaration




    def objectDeclaration(self):

        localctx = KotlinParser.ObjectDeclarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_objectDeclaration)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1507
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 1506
                self.modifiers()


            self.state = 1509
            self.match(KotlinParser.OBJECT)
            self.state = 1513
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1510
                self.match(KotlinParser.NL)
                self.state = 1515
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1516
            self.simpleIdentifier()
            self.state = 1531
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,213,self._ctx)
            if la_ == 1:
                self.state = 1520
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1517
                    self.match(KotlinParser.NL)
                    self.state = 1522
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1523
                self.match(KotlinParser.COLON)
                self.state = 1527
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,212,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 1524
                        self.match(KotlinParser.NL) 
                    self.state = 1529
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,212,self._ctx)

                self.state = 1530
                self.delegationSpecifiers()


            self.state = 1540
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,215,self._ctx)
            if la_ == 1:
                self.state = 1536
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1533
                    self.match(KotlinParser.NL)
                    self.state = 1538
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1539
                self.classBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SecondaryConstructorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def CONSTRUCTOR(self):
            return self.getToken(KotlinParser.CONSTRUCTOR, 0)

        def functionValueParameters(self):
            return self.getTypedRuleContext(KotlinParser.FunctionValueParametersContext,0)


        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def constructorDelegationCall(self):
            return self.getTypedRuleContext(KotlinParser.ConstructorDelegationCallContext,0)


        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_secondaryConstructor




    def secondaryConstructor(self):

        localctx = KotlinParser.SecondaryConstructorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_secondaryConstructor)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1543
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS or ((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)) | (1 << (KotlinParser.ENUM - 107)) | (1 << (KotlinParser.SEALED - 107)) | (1 << (KotlinParser.ANNOTATION - 107)) | (1 << (KotlinParser.DATA - 107)) | (1 << (KotlinParser.INNER - 107)) | (1 << (KotlinParser.VALUE - 107)) | (1 << (KotlinParser.TAILREC - 107)) | (1 << (KotlinParser.OPERATOR - 107)) | (1 << (KotlinParser.INLINE - 107)) | (1 << (KotlinParser.INFIX - 107)) | (1 << (KotlinParser.EXTERNAL - 107)) | (1 << (KotlinParser.SUSPEND - 107)) | (1 << (KotlinParser.OVERRIDE - 107)) | (1 << (KotlinParser.ABSTRACT - 107)) | (1 << (KotlinParser.FINAL - 107)) | (1 << (KotlinParser.OPEN - 107)) | (1 << (KotlinParser.CONST - 107)) | (1 << (KotlinParser.LATEINIT - 107)) | (1 << (KotlinParser.VARARG - 107)) | (1 << (KotlinParser.NOINLINE - 107)) | (1 << (KotlinParser.CROSSINLINE - 107)) | (1 << (KotlinParser.EXPECT - 107)) | (1 << (KotlinParser.ACTUAL - 107)))) != 0):
                self.state = 1542
                self.modifiers()


            self.state = 1545
            self.match(KotlinParser.CONSTRUCTOR)
            self.state = 1549
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1546
                self.match(KotlinParser.NL)
                self.state = 1551
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1552
            self.functionValueParameters()
            self.state = 1567
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,220,self._ctx)
            if la_ == 1:
                self.state = 1556
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1553
                    self.match(KotlinParser.NL)
                    self.state = 1558
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1559
                self.match(KotlinParser.COLON)
                self.state = 1563
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1560
                    self.match(KotlinParser.NL)
                    self.state = 1565
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1566
                self.constructorDelegationCall()


            self.state = 1572
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,221,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1569
                    self.match(KotlinParser.NL) 
                self.state = 1574
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,221,self._ctx)

            self.state = 1576
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.LCURL:
                self.state = 1575
                self.block()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ConstructorDelegationCallContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def valueArguments(self):
            return self.getTypedRuleContext(KotlinParser.ValueArgumentsContext,0)


        def THIS(self):
            return self.getToken(KotlinParser.THIS, 0)

        def SUPER(self):
            return self.getToken(KotlinParser.SUPER, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_constructorDelegationCall




    def constructorDelegationCall(self):

        localctx = KotlinParser.ConstructorDelegationCallContext(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_constructorDelegationCall)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1578
            _la = self._input.LA(1)
            if not(_la==KotlinParser.THIS or _la==KotlinParser.SUPER):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 1582
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1579
                self.match(KotlinParser.NL)
                self.state = 1584
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1585
            self.valueArguments()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EnumClassBodyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LCURL(self):
            return self.getToken(KotlinParser.LCURL, 0)

        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def enumEntries(self):
            return self.getTypedRuleContext(KotlinParser.EnumEntriesContext,0)


        def SEMICOLON(self):
            return self.getToken(KotlinParser.SEMICOLON, 0)

        def classMemberDeclarations(self):
            return self.getTypedRuleContext(KotlinParser.ClassMemberDeclarationsContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_enumClassBody




    def enumClassBody(self):

        localctx = KotlinParser.EnumClassBodyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_enumClassBody)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1587
            self.match(KotlinParser.LCURL)
            self.state = 1591
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,224,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1588
                    self.match(KotlinParser.NL) 
                self.state = 1593
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,224,self._ctx)

            self.state = 1595
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if ((((_la - 40)) & ~0x3f) == 0 and ((1 << (_la - 40)) & ((1 << (KotlinParser.AT_NO_WS - 40)) | (1 << (KotlinParser.AT_PRE_WS - 40)) | (1 << (KotlinParser.FILE - 40)) | (1 << (KotlinParser.FIELD - 40)) | (1 << (KotlinParser.PROPERTY - 40)) | (1 << (KotlinParser.GET - 40)) | (1 << (KotlinParser.SET - 40)) | (1 << (KotlinParser.RECEIVER - 40)) | (1 << (KotlinParser.PARAM - 40)) | (1 << (KotlinParser.SETPARAM - 40)) | (1 << (KotlinParser.DELEGATE - 40)) | (1 << (KotlinParser.IMPORT - 40)) | (1 << (KotlinParser.CONSTRUCTOR - 40)) | (1 << (KotlinParser.BY - 40)) | (1 << (KotlinParser.COMPANION - 40)) | (1 << (KotlinParser.INIT - 40)) | (1 << (KotlinParser.WHERE - 40)) | (1 << (KotlinParser.CATCH - 40)) | (1 << (KotlinParser.FINALLY - 40)))) != 0) or ((((_la - 105)) & ~0x3f) == 0 and ((1 << (_la - 105)) & ((1 << (KotlinParser.OUT - 105)) | (1 << (KotlinParser.DYNAMIC - 105)) | (1 << (KotlinParser.PUBLIC - 105)) | (1 << (KotlinParser.PRIVATE - 105)) | (1 << (KotlinParser.PROTECTED - 105)) | (1 << (KotlinParser.INTERNAL - 105)) | (1 << (KotlinParser.ENUM - 105)) | (1 << (KotlinParser.SEALED - 105)) | (1 << (KotlinParser.ANNOTATION - 105)) | (1 << (KotlinParser.DATA - 105)) | (1 << (KotlinParser.INNER - 105)) | (1 << (KotlinParser.VALUE - 105)) | (1 << (KotlinParser.TAILREC - 105)) | (1 << (KotlinParser.OPERATOR - 105)) | (1 << (KotlinParser.INLINE - 105)) | (1 << (KotlinParser.INFIX - 105)) | (1 << (KotlinParser.EXTERNAL - 105)) | (1 << (KotlinParser.SUSPEND - 105)) | (1 << (KotlinParser.OVERRIDE - 105)) | (1 << (KotlinParser.ABSTRACT - 105)) | (1 << (KotlinParser.FINAL - 105)) | (1 << (KotlinParser.OPEN - 105)) | (1 << (KotlinParser.CONST - 105)) | (1 << (KotlinParser.LATEINIT - 105)) | (1 << (KotlinParser.VARARG - 105)) | (1 << (KotlinParser.NOINLINE - 105)) | (1 << (KotlinParser.CROSSINLINE - 105)) | (1 << (KotlinParser.REIFIED - 105)) | (1 << (KotlinParser.EXPECT - 105)) | (1 << (KotlinParser.ACTUAL - 105)) | (1 << (KotlinParser.Identifier - 105)))) != 0):
                self.state = 1594
                self.enumEntries()


            self.state = 1611
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,228,self._ctx)
            if la_ == 1:
                self.state = 1600
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1597
                    self.match(KotlinParser.NL)
                    self.state = 1602
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1603
                self.match(KotlinParser.SEMICOLON)
                self.state = 1607
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,227,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 1604
                        self.match(KotlinParser.NL) 
                    self.state = 1609
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,227,self._ctx)

                self.state = 1610
                self.classMemberDeclarations()


            self.state = 1616
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1613
                self.match(KotlinParser.NL)
                self.state = 1618
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1619
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EnumEntriesContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def enumEntry(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.EnumEntryContext)
            else:
                return self.getTypedRuleContext(KotlinParser.EnumEntryContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_enumEntries




    def enumEntries(self):

        localctx = KotlinParser.EnumEntriesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_enumEntries)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1621
            self.enumEntry()
            self.state = 1638
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,232,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1625
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1622
                        self.match(KotlinParser.NL)
                        self.state = 1627
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1628
                    self.match(KotlinParser.COMMA)
                    self.state = 1632
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1629
                        self.match(KotlinParser.NL)
                        self.state = 1634
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1635
                    self.enumEntry() 
                self.state = 1640
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,232,self._ctx)

            self.state = 1644
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,233,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1641
                    self.match(KotlinParser.NL) 
                self.state = 1646
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,233,self._ctx)

            self.state = 1648
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.COMMA:
                self.state = 1647
                self.match(KotlinParser.COMMA)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EnumEntryContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def modifiers(self):
            return self.getTypedRuleContext(KotlinParser.ModifiersContext,0)


        def valueArguments(self):
            return self.getTypedRuleContext(KotlinParser.ValueArgumentsContext,0)


        def classBody(self):
            return self.getTypedRuleContext(KotlinParser.ClassBodyContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_enumEntry




    def enumEntry(self):

        localctx = KotlinParser.EnumEntryContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_enumEntry)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1657
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,236,self._ctx)
            if la_ == 1:
                self.state = 1650
                self.modifiers()
                self.state = 1654
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1651
                    self.match(KotlinParser.NL)
                    self.state = 1656
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 1659
            self.simpleIdentifier()
            self.state = 1667
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,238,self._ctx)
            if la_ == 1:
                self.state = 1663
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1660
                    self.match(KotlinParser.NL)
                    self.state = 1665
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1666
                self.valueArguments()


            self.state = 1676
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,240,self._ctx)
            if la_ == 1:
                self.state = 1672
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1669
                    self.match(KotlinParser.NL)
                    self.state = 1674
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1675
                self.classBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def parenthesizedType(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedTypeContext,0)


        def nullableType(self):
            return self.getTypedRuleContext(KotlinParser.NullableTypeContext,0)


        def typeReference(self):
            return self.getTypedRuleContext(KotlinParser.TypeReferenceContext,0)


        def functionType(self):
            return self.getTypedRuleContext(KotlinParser.FunctionTypeContext,0)


        def typeModifiers(self):
            return self.getTypedRuleContext(KotlinParser.TypeModifiersContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_type




    def type(self):

        localctx = KotlinParser.TypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 98, self.RULE_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1679
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,241,self._ctx)
            if la_ == 1:
                self.state = 1678
                self.typeModifiers()


            self.state = 1685
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,242,self._ctx)
            if la_ == 1:
                self.state = 1681
                self.parenthesizedType()
                pass

            elif la_ == 2:
                self.state = 1682
                self.nullableType()
                pass

            elif la_ == 3:
                self.state = 1683
                self.typeReference()
                pass

            elif la_ == 4:
                self.state = 1684
                self.functionType()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeReferenceContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def userType(self):
            return self.getTypedRuleContext(KotlinParser.UserTypeContext,0)


        def DYNAMIC(self):
            return self.getToken(KotlinParser.DYNAMIC, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeReference




    def typeReference(self):

        localctx = KotlinParser.TypeReferenceContext(self, self._ctx, self.state)
        self.enterRule(localctx, 100, self.RULE_typeReference)
        try:
            self.state = 1689
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,243,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 1687
                self.userType()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 1688
                self.match(KotlinParser.DYNAMIC)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NullableTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typeReference(self):
            return self.getTypedRuleContext(KotlinParser.TypeReferenceContext,0)


        def parenthesizedType(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedTypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def quest(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.QuestContext)
            else:
                return self.getTypedRuleContext(KotlinParser.QuestContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_nullableType




    def nullableType(self):

        localctx = KotlinParser.NullableTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 102, self.RULE_nullableType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1693
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 1691
                self.typeReference()
                pass
            elif token in [KotlinParser.LPAREN]:
                self.state = 1692
                self.parenthesizedType()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 1698
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1695
                self.match(KotlinParser.NL)
                self.state = 1700
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1702 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 1701
                    self.quest()

                else:
                    raise NoViableAltException(self)
                self.state = 1704 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,246,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class QuestContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def QUEST_NO_WS(self):
            return self.getToken(KotlinParser.QUEST_NO_WS, 0)

        def QUEST_WS(self):
            return self.getToken(KotlinParser.QUEST_WS, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_quest




    def quest(self):

        localctx = KotlinParser.QuestContext(self, self._ctx, self.state)
        self.enterRule(localctx, 104, self.RULE_quest)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1706
            _la = self._input.LA(1)
            if not(_la==KotlinParser.QUEST_WS or _la==KotlinParser.QUEST_NO_WS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class UserTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleUserType(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.SimpleUserTypeContext)
            else:
                return self.getTypedRuleContext(KotlinParser.SimpleUserTypeContext,i)


        def DOT(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.DOT)
            else:
                return self.getToken(KotlinParser.DOT, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_userType




    def userType(self):

        localctx = KotlinParser.UserTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 106, self.RULE_userType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1708
            self.simpleUserType()
            self.state = 1725
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,249,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1712
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1709
                        self.match(KotlinParser.NL)
                        self.state = 1714
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1715
                    self.match(KotlinParser.DOT)
                    self.state = 1719
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1716
                        self.match(KotlinParser.NL)
                        self.state = 1721
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1722
                    self.simpleUserType() 
                self.state = 1727
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,249,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SimpleUserTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def typeArguments(self):
            return self.getTypedRuleContext(KotlinParser.TypeArgumentsContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_simpleUserType




    def simpleUserType(self):

        localctx = KotlinParser.SimpleUserTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 108, self.RULE_simpleUserType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1728
            self.simpleIdentifier()
            self.state = 1736
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,251,self._ctx)
            if la_ == 1:
                self.state = 1732
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1729
                    self.match(KotlinParser.NL)
                    self.state = 1734
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1735
                self.typeArguments()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeProjectionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def typeProjectionModifiers(self):
            return self.getTypedRuleContext(KotlinParser.TypeProjectionModifiersContext,0)


        def MULT(self):
            return self.getToken(KotlinParser.MULT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeProjection




    def typeProjection(self):

        localctx = KotlinParser.TypeProjectionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 110, self.RULE_typeProjection)
        try:
            self.state = 1743
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LPAREN, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.IN, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.enterOuterAlt(localctx, 1)
                self.state = 1739
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,252,self._ctx)
                if la_ == 1:
                    self.state = 1738
                    self.typeProjectionModifiers()


                self.state = 1741
                self.type()
                pass
            elif token in [KotlinParser.MULT]:
                self.enterOuterAlt(localctx, 2)
                self.state = 1742
                self.match(KotlinParser.MULT)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeProjectionModifiersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typeProjectionModifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeProjectionModifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeProjectionModifierContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeProjectionModifiers




    def typeProjectionModifiers(self):

        localctx = KotlinParser.TypeProjectionModifiersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 112, self.RULE_typeProjectionModifiers)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1746 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 1745
                    self.typeProjectionModifier()

                else:
                    raise NoViableAltException(self)
                self.state = 1748 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,254,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeProjectionModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def varianceModifier(self):
            return self.getTypedRuleContext(KotlinParser.VarianceModifierContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def annotation(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeProjectionModifier




    def typeProjectionModifier(self):

        localctx = KotlinParser.TypeProjectionModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 114, self.RULE_typeProjectionModifier)
        self._la = 0 # Token type
        try:
            self.state = 1758
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.IN, KotlinParser.OUT]:
                self.enterOuterAlt(localctx, 1)
                self.state = 1750
                self.varianceModifier()
                self.state = 1754
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1751
                    self.match(KotlinParser.NL)
                    self.state = 1756
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                self.enterOuterAlt(localctx, 2)
                self.state = 1757
                self.annotation()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def functionTypeParameters(self):
            return self.getTypedRuleContext(KotlinParser.FunctionTypeParametersContext,0)


        def ARROW(self):
            return self.getToken(KotlinParser.ARROW, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def receiverType(self):
            return self.getTypedRuleContext(KotlinParser.ReceiverTypeContext,0)


        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionType




    def functionType(self):

        localctx = KotlinParser.FunctionTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 116, self.RULE_functionType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1774
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,259,self._ctx)
            if la_ == 1:
                self.state = 1760
                self.receiverType()
                self.state = 1764
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1761
                    self.match(KotlinParser.NL)
                    self.state = 1766
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1767
                self.match(KotlinParser.DOT)
                self.state = 1771
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1768
                    self.match(KotlinParser.NL)
                    self.state = 1773
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 1776
            self.functionTypeParameters()
            self.state = 1780
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1777
                self.match(KotlinParser.NL)
                self.state = 1782
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1783
            self.match(KotlinParser.ARROW)
            self.state = 1787
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1784
                self.match(KotlinParser.NL)
                self.state = 1789
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1790
            self.type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionTypeParametersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def parameter(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ParameterContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ParameterContext,i)


        def type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionTypeParameters




    def functionTypeParameters(self):

        localctx = KotlinParser.FunctionTypeParametersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 118, self.RULE_functionTypeParameters)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1792
            self.match(KotlinParser.LPAREN)
            self.state = 1796
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,262,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1793
                    self.match(KotlinParser.NL) 
                self.state = 1798
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,262,self._ctx)

            self.state = 1801
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,263,self._ctx)
            if la_ == 1:
                self.state = 1799
                self.parameter()

            elif la_ == 2:
                self.state = 1800
                self.type()


            self.state = 1822
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,267,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1806
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1803
                        self.match(KotlinParser.NL)
                        self.state = 1808
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1809
                    self.match(KotlinParser.COMMA)
                    self.state = 1813
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 1810
                        self.match(KotlinParser.NL)
                        self.state = 1815
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 1818
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,266,self._ctx)
                    if la_ == 1:
                        self.state = 1816
                        self.parameter()
                        pass

                    elif la_ == 2:
                        self.state = 1817
                        self.type()
                        pass

             
                self.state = 1824
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,267,self._ctx)

            self.state = 1832
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,269,self._ctx)
            if la_ == 1:
                self.state = 1828
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 1825
                    self.match(KotlinParser.NL)
                    self.state = 1830
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 1831
                self.match(KotlinParser.COMMA)


            self.state = 1837
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1834
                self.match(KotlinParser.NL)
                self.state = 1839
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1840
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParenthesizedTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parenthesizedType




    def parenthesizedType(self):

        localctx = KotlinParser.ParenthesizedTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 120, self.RULE_parenthesizedType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1842
            self.match(KotlinParser.LPAREN)
            self.state = 1846
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1843
                self.match(KotlinParser.NL)
                self.state = 1848
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1849
            self.type()
            self.state = 1853
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1850
                self.match(KotlinParser.NL)
                self.state = 1855
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1856
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ReceiverTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def parenthesizedType(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedTypeContext,0)


        def nullableType(self):
            return self.getTypedRuleContext(KotlinParser.NullableTypeContext,0)


        def typeReference(self):
            return self.getTypedRuleContext(KotlinParser.TypeReferenceContext,0)


        def typeModifiers(self):
            return self.getTypedRuleContext(KotlinParser.TypeModifiersContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_receiverType




    def receiverType(self):

        localctx = KotlinParser.ReceiverTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 122, self.RULE_receiverType)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1859
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,273,self._ctx)
            if la_ == 1:
                self.state = 1858
                self.typeModifiers()


            self.state = 1864
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,274,self._ctx)
            if la_ == 1:
                self.state = 1861
                self.parenthesizedType()
                pass

            elif la_ == 2:
                self.state = 1862
                self.nullableType()
                pass

            elif la_ == 3:
                self.state = 1863
                self.typeReference()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParenthesizedUserTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def userType(self):
            return self.getTypedRuleContext(KotlinParser.UserTypeContext,0)


        def parenthesizedUserType(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedUserTypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parenthesizedUserType




    def parenthesizedUserType(self):

        localctx = KotlinParser.ParenthesizedUserTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 124, self.RULE_parenthesizedUserType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1866
            self.match(KotlinParser.LPAREN)
            self.state = 1870
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1867
                self.match(KotlinParser.NL)
                self.state = 1872
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1875
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 1873
                self.userType()
                pass
            elif token in [KotlinParser.LPAREN]:
                self.state = 1874
                self.parenthesizedUserType()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 1880
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1877
                self.match(KotlinParser.NL)
                self.state = 1882
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1883
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class StatementsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.StatementContext)
            else:
                return self.getTypedRuleContext(KotlinParser.StatementContext,i)


        def semis(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.SemisContext)
            else:
                return self.getTypedRuleContext(KotlinParser.SemisContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_statements




    def statements(self):

        localctx = KotlinParser.StatementsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 126, self.RULE_statements)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1894
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LPAREN) | (1 << KotlinParser.LSQUARE) | (1 << KotlinParser.LCURL) | (1 << KotlinParser.ADD) | (1 << KotlinParser.SUB) | (1 << KotlinParser.INCR) | (1 << KotlinParser.DECR) | (1 << KotlinParser.EXCL_WS) | (1 << KotlinParser.EXCL_NO_WS) | (1 << KotlinParser.COLONCOLON) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.RETURN_AT) | (1 << KotlinParser.CONTINUE_AT) | (1 << KotlinParser.BREAK_AT) | (1 << KotlinParser.THIS_AT) | (1 << KotlinParser.SUPER_AT) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.CLASS - 64)) | (1 << (KotlinParser.INTERFACE - 64)) | (1 << (KotlinParser.FUN - 64)) | (1 << (KotlinParser.OBJECT - 64)) | (1 << (KotlinParser.VAL - 64)) | (1 << (KotlinParser.VAR - 64)) | (1 << (KotlinParser.TYPE_ALIAS - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.THIS - 64)) | (1 << (KotlinParser.SUPER - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.IF - 64)) | (1 << (KotlinParser.WHEN - 64)) | (1 << (KotlinParser.TRY - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.FOR - 64)) | (1 << (KotlinParser.DO - 64)) | (1 << (KotlinParser.WHILE - 64)) | (1 << (KotlinParser.THROW - 64)) | (1 << (KotlinParser.RETURN - 64)) | (1 << (KotlinParser.CONTINUE - 64)) | (1 << (KotlinParser.BREAK - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.RealLiteral - 128)) | (1 << (KotlinParser.IntegerLiteral - 128)) | (1 << (KotlinParser.HexLiteral - 128)) | (1 << (KotlinParser.BinLiteral - 128)) | (1 << (KotlinParser.UnsignedLiteral - 128)) | (1 << (KotlinParser.LongLiteral - 128)) | (1 << (KotlinParser.BooleanLiteral - 128)) | (1 << (KotlinParser.NullLiteral - 128)) | (1 << (KotlinParser.CharacterLiteral - 128)) | (1 << (KotlinParser.Identifier - 128)) | (1 << (KotlinParser.QUOTE_OPEN - 128)) | (1 << (KotlinParser.TRIPLE_QUOTE_OPEN - 128)))) != 0):
                self.state = 1885
                self.statement()
                self.state = 1891
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,278,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 1886
                        self.semis()
                        self.state = 1887
                        self.statement() 
                    self.state = 1893
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,278,self._ctx)



            self.state = 1897
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,280,self._ctx)
            if la_ == 1:
                self.state = 1896
                self.semis()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class StatementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def declaration(self):
            return self.getTypedRuleContext(KotlinParser.DeclarationContext,0)


        def assignment(self):
            return self.getTypedRuleContext(KotlinParser.AssignmentContext,0)


        def loopStatement(self):
            return self.getTypedRuleContext(KotlinParser.LoopStatementContext,0)


        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def label(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.LabelContext)
            else:
                return self.getTypedRuleContext(KotlinParser.LabelContext,i)


        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_statement




    def statement(self):

        localctx = KotlinParser.StatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 128, self.RULE_statement)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1903
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,282,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1901
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                        self.state = 1899
                        self.label()
                        pass
                    elif token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                        self.state = 1900
                        self.annotation()
                        pass
                    else:
                        raise NoViableAltException(self)
             
                self.state = 1905
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,282,self._ctx)

            self.state = 1910
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,283,self._ctx)
            if la_ == 1:
                self.state = 1906
                self.declaration()
                pass

            elif la_ == 2:
                self.state = 1907
                self.assignment()
                pass

            elif la_ == 3:
                self.state = 1908
                self.loopStatement()
                pass

            elif la_ == 4:
                self.state = 1909
                self.expression()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LabelContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def AT_NO_WS(self):
            return self.getToken(KotlinParser.AT_NO_WS, 0)

        def AT_POST_WS(self):
            return self.getToken(KotlinParser.AT_POST_WS, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_label




    def label(self):

        localctx = KotlinParser.LabelContext(self, self._ctx, self.state)
        self.enterRule(localctx, 130, self.RULE_label)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1912
            self.simpleIdentifier()
            self.state = 1913
            _la = self._input.LA(1)
            if not(_la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_POST_WS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 1917
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,284,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1914
                    self.match(KotlinParser.NL) 
                self.state = 1919
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,284,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ControlStructureBodyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def statement(self):
            return self.getTypedRuleContext(KotlinParser.StatementContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_controlStructureBody




    def controlStructureBody(self):

        localctx = KotlinParser.ControlStructureBodyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 132, self.RULE_controlStructureBody)
        try:
            self.state = 1922
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,285,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 1920
                self.block()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 1921
                self.statement()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BlockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LCURL(self):
            return self.getToken(KotlinParser.LCURL, 0)

        def statements(self):
            return self.getTypedRuleContext(KotlinParser.StatementsContext,0)


        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_block




    def block(self):

        localctx = KotlinParser.BlockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 134, self.RULE_block)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1924
            self.match(KotlinParser.LCURL)
            self.state = 1928
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,286,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1925
                    self.match(KotlinParser.NL) 
                self.state = 1930
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,286,self._ctx)

            self.state = 1931
            self.statements()
            self.state = 1935
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1932
                self.match(KotlinParser.NL)
                self.state = 1937
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1938
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LoopStatementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def forStatement(self):
            return self.getTypedRuleContext(KotlinParser.ForStatementContext,0)


        def whileStatement(self):
            return self.getTypedRuleContext(KotlinParser.WhileStatementContext,0)


        def doWhileStatement(self):
            return self.getTypedRuleContext(KotlinParser.DoWhileStatementContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_loopStatement




    def loopStatement(self):

        localctx = KotlinParser.LoopStatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 136, self.RULE_loopStatement)
        try:
            self.state = 1943
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.FOR]:
                self.enterOuterAlt(localctx, 1)
                self.state = 1940
                self.forStatement()
                pass
            elif token in [KotlinParser.WHILE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 1941
                self.whileStatement()
                pass
            elif token in [KotlinParser.DO]:
                self.enterOuterAlt(localctx, 3)
                self.state = 1942
                self.doWhileStatement()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ForStatementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def FOR(self):
            return self.getToken(KotlinParser.FOR, 0)

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def IN(self):
            return self.getToken(KotlinParser.IN, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def variableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.VariableDeclarationContext,0)


        def multiVariableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.MultiVariableDeclarationContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def controlStructureBody(self):
            return self.getTypedRuleContext(KotlinParser.ControlStructureBodyContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_forStatement




    def forStatement(self):

        localctx = KotlinParser.ForStatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 138, self.RULE_forStatement)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1945
            self.match(KotlinParser.FOR)
            self.state = 1949
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1946
                self.match(KotlinParser.NL)
                self.state = 1951
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1952
            self.match(KotlinParser.LPAREN)
            self.state = 1956
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,290,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1953
                    self.annotation() 
                self.state = 1958
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,290,self._ctx)

            self.state = 1961
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.NL, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 1959
                self.variableDeclaration()
                pass
            elif token in [KotlinParser.LPAREN]:
                self.state = 1960
                self.multiVariableDeclaration()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 1963
            self.match(KotlinParser.IN)
            self.state = 1964
            self.expression()
            self.state = 1965
            self.match(KotlinParser.RPAREN)
            self.state = 1969
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,292,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1966
                    self.match(KotlinParser.NL) 
                self.state = 1971
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,292,self._ctx)

            self.state = 1973
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,293,self._ctx)
            if la_ == 1:
                self.state = 1972
                self.controlStructureBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WhileStatementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def WHILE(self):
            return self.getToken(KotlinParser.WHILE, 0)

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def controlStructureBody(self):
            return self.getTypedRuleContext(KotlinParser.ControlStructureBodyContext,0)


        def SEMICOLON(self):
            return self.getToken(KotlinParser.SEMICOLON, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_whileStatement




    def whileStatement(self):

        localctx = KotlinParser.WhileStatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 140, self.RULE_whileStatement)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1975
            self.match(KotlinParser.WHILE)
            self.state = 1979
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1976
                self.match(KotlinParser.NL)
                self.state = 1981
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1982
            self.match(KotlinParser.LPAREN)
            self.state = 1983
            self.expression()
            self.state = 1984
            self.match(KotlinParser.RPAREN)
            self.state = 1988
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 1985
                self.match(KotlinParser.NL)
                self.state = 1990
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1993
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LPAREN, KotlinParser.LSQUARE, KotlinParser.LCURL, KotlinParser.ADD, KotlinParser.SUB, KotlinParser.INCR, KotlinParser.DECR, KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS, KotlinParser.COLONCOLON, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.RETURN_AT, KotlinParser.CONTINUE_AT, KotlinParser.BREAK_AT, KotlinParser.THIS_AT, KotlinParser.SUPER_AT, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CLASS, KotlinParser.INTERFACE, KotlinParser.FUN, KotlinParser.OBJECT, KotlinParser.VAL, KotlinParser.VAR, KotlinParser.TYPE_ALIAS, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.THIS, KotlinParser.SUPER, KotlinParser.WHERE, KotlinParser.IF, KotlinParser.WHEN, KotlinParser.TRY, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.FOR, KotlinParser.DO, KotlinParser.WHILE, KotlinParser.THROW, KotlinParser.RETURN, KotlinParser.CONTINUE, KotlinParser.BREAK, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.RealLiteral, KotlinParser.IntegerLiteral, KotlinParser.HexLiteral, KotlinParser.BinLiteral, KotlinParser.UnsignedLiteral, KotlinParser.LongLiteral, KotlinParser.BooleanLiteral, KotlinParser.NullLiteral, KotlinParser.CharacterLiteral, KotlinParser.Identifier, KotlinParser.QUOTE_OPEN, KotlinParser.TRIPLE_QUOTE_OPEN]:
                self.state = 1991
                self.controlStructureBody()
                pass
            elif token in [KotlinParser.SEMICOLON]:
                self.state = 1992
                self.match(KotlinParser.SEMICOLON)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DoWhileStatementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def DO(self):
            return self.getToken(KotlinParser.DO, 0)

        def WHILE(self):
            return self.getToken(KotlinParser.WHILE, 0)

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def controlStructureBody(self):
            return self.getTypedRuleContext(KotlinParser.ControlStructureBodyContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_doWhileStatement




    def doWhileStatement(self):

        localctx = KotlinParser.DoWhileStatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 142, self.RULE_doWhileStatement)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1995
            self.match(KotlinParser.DO)
            self.state = 1999
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,297,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 1996
                    self.match(KotlinParser.NL) 
                self.state = 2001
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,297,self._ctx)

            self.state = 2003
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,298,self._ctx)
            if la_ == 1:
                self.state = 2002
                self.controlStructureBody()


            self.state = 2008
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2005
                self.match(KotlinParser.NL)
                self.state = 2010
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2011
            self.match(KotlinParser.WHILE)
            self.state = 2015
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2012
                self.match(KotlinParser.NL)
                self.state = 2017
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2018
            self.match(KotlinParser.LPAREN)
            self.state = 2019
            self.expression()
            self.state = 2020
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignmentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def directlyAssignableExpression(self):
            return self.getTypedRuleContext(KotlinParser.DirectlyAssignableExpressionContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def assignableExpression(self):
            return self.getTypedRuleContext(KotlinParser.AssignableExpressionContext,0)


        def assignmentAndOperator(self):
            return self.getTypedRuleContext(KotlinParser.AssignmentAndOperatorContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_assignment




    def assignment(self):

        localctx = KotlinParser.AssignmentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 144, self.RULE_assignment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2028
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,301,self._ctx)
            if la_ == 1:
                self.state = 2022
                self.directlyAssignableExpression()
                self.state = 2023
                self.match(KotlinParser.ASSIGNMENT)
                pass

            elif la_ == 2:
                self.state = 2025
                self.assignableExpression()
                self.state = 2026
                self.assignmentAndOperator()
                pass


            self.state = 2033
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2030
                self.match(KotlinParser.NL)
                self.state = 2035
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2036
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SemiContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def SEMICOLON(self):
            return self.getToken(KotlinParser.SEMICOLON, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def EOF(self):
            return self.getToken(KotlinParser.EOF, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_semi




    def semi(self):

        localctx = KotlinParser.SemiContext(self, self._ctx, self.state)
        self.enterRule(localctx, 146, self.RULE_semi)
        self._la = 0 # Token type
        try:
            self.state = 2046
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.NL, KotlinParser.SEMICOLON]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2038
                _la = self._input.LA(1)
                if not(_la==KotlinParser.NL or _la==KotlinParser.SEMICOLON):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 2042
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,303,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2039
                        self.match(KotlinParser.NL) 
                    self.state = 2044
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,303,self._ctx)

                pass
            elif token in [KotlinParser.EOF]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2045
                self.match(KotlinParser.EOF)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SemisContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def SEMICOLON(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.SEMICOLON)
            else:
                return self.getToken(KotlinParser.SEMICOLON, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def EOF(self):
            return self.getToken(KotlinParser.EOF, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_semis




    def semis(self):

        localctx = KotlinParser.SemisContext(self, self._ctx, self.state)
        self.enterRule(localctx, 148, self.RULE_semis)
        self._la = 0 # Token type
        try:
            self.state = 2054
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.NL, KotlinParser.SEMICOLON]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2049 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 2048
                        _la = self._input.LA(1)
                        if not(_la==KotlinParser.NL or _la==KotlinParser.SEMICOLON):
                            self._errHandler.recoverInline(self)
                        else:
                            self._errHandler.reportMatch(self)
                            self.consume()

                    else:
                        raise NoViableAltException(self)
                    self.state = 2051 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,305,self._ctx)

                pass
            elif token in [KotlinParser.EOF]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2053
                self.match(KotlinParser.EOF)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def disjunction(self):
            return self.getTypedRuleContext(KotlinParser.DisjunctionContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_expression




    def expression(self):

        localctx = KotlinParser.ExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 150, self.RULE_expression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2056
            self.disjunction()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DisjunctionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def conjunction(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ConjunctionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ConjunctionContext,i)


        def DISJ(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.DISJ)
            else:
                return self.getToken(KotlinParser.DISJ, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_disjunction




    def disjunction(self):

        localctx = KotlinParser.DisjunctionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 152, self.RULE_disjunction)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2058
            self.conjunction()
            self.state = 2075
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,309,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2062
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2059
                        self.match(KotlinParser.NL)
                        self.state = 2064
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2065
                    self.match(KotlinParser.DISJ)
                    self.state = 2069
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2066
                        self.match(KotlinParser.NL)
                        self.state = 2071
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2072
                    self.conjunction() 
                self.state = 2077
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,309,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ConjunctionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def equality(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.EqualityContext)
            else:
                return self.getTypedRuleContext(KotlinParser.EqualityContext,i)


        def CONJ(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.CONJ)
            else:
                return self.getToken(KotlinParser.CONJ, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_conjunction




    def conjunction(self):

        localctx = KotlinParser.ConjunctionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 154, self.RULE_conjunction)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2078
            self.equality()
            self.state = 2095
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,312,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2082
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2079
                        self.match(KotlinParser.NL)
                        self.state = 2084
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2085
                    self.match(KotlinParser.CONJ)
                    self.state = 2089
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2086
                        self.match(KotlinParser.NL)
                        self.state = 2091
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2092
                    self.equality() 
                self.state = 2097
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,312,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EqualityContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def comparison(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ComparisonContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ComparisonContext,i)


        def equalityOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.EqualityOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.EqualityOperatorContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_equality




    def equality(self):

        localctx = KotlinParser.EqualityContext(self, self._ctx, self.state)
        self.enterRule(localctx, 156, self.RULE_equality)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2098
            self.comparison()
            self.state = 2110
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,314,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2099
                    self.equalityOperator()
                    self.state = 2103
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2100
                        self.match(KotlinParser.NL)
                        self.state = 2105
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2106
                    self.comparison() 
                self.state = 2112
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,314,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ComparisonContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def genericCallLikeComparison(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.GenericCallLikeComparisonContext)
            else:
                return self.getTypedRuleContext(KotlinParser.GenericCallLikeComparisonContext,i)


        def comparisonOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ComparisonOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ComparisonOperatorContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_comparison




    def comparison(self):

        localctx = KotlinParser.ComparisonContext(self, self._ctx, self.state)
        self.enterRule(localctx, 158, self.RULE_comparison)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2113
            self.genericCallLikeComparison()
            self.state = 2125
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,316,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2114
                    self.comparisonOperator()
                    self.state = 2118
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2115
                        self.match(KotlinParser.NL)
                        self.state = 2120
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2121
                    self.genericCallLikeComparison() 
                self.state = 2127
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,316,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class GenericCallLikeComparisonContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def infixOperation(self):
            return self.getTypedRuleContext(KotlinParser.InfixOperationContext,0)


        def callSuffix(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.CallSuffixContext)
            else:
                return self.getTypedRuleContext(KotlinParser.CallSuffixContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_genericCallLikeComparison




    def genericCallLikeComparison(self):

        localctx = KotlinParser.GenericCallLikeComparisonContext(self, self._ctx, self.state)
        self.enterRule(localctx, 160, self.RULE_genericCallLikeComparison)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2128
            self.infixOperation()
            self.state = 2132
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,317,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2129
                    self.callSuffix() 
                self.state = 2134
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,317,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class InfixOperationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def elvisExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ElvisExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ElvisExpressionContext,i)


        def inOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.InOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.InOperatorContext,i)


        def isOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.IsOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.IsOperatorContext,i)


        def type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_infixOperation




    def infixOperation(self):

        localctx = KotlinParser.InfixOperationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 162, self.RULE_infixOperation)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2135
            self.elvisExpression()
            self.state = 2156
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,321,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2154
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [KotlinParser.IN, KotlinParser.NOT_IN]:
                        self.state = 2136
                        self.inOperator()
                        self.state = 2140
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 2137
                            self.match(KotlinParser.NL)
                            self.state = 2142
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 2143
                        self.elvisExpression()
                        pass
                    elif token in [KotlinParser.IS, KotlinParser.NOT_IS]:
                        self.state = 2145
                        self.isOperator()
                        self.state = 2149
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 2146
                            self.match(KotlinParser.NL)
                            self.state = 2151
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 2152
                        self.type()
                        pass
                    else:
                        raise NoViableAltException(self)
             
                self.state = 2158
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,321,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ElvisExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def infixFunctionCall(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.InfixFunctionCallContext)
            else:
                return self.getTypedRuleContext(KotlinParser.InfixFunctionCallContext,i)


        def elvis(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ElvisContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ElvisContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_elvisExpression




    def elvisExpression(self):

        localctx = KotlinParser.ElvisExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 164, self.RULE_elvisExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2159
            self.infixFunctionCall()
            self.state = 2177
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,324,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2163
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2160
                        self.match(KotlinParser.NL)
                        self.state = 2165
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2166
                    self.elvis()
                    self.state = 2170
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2167
                        self.match(KotlinParser.NL)
                        self.state = 2172
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2173
                    self.infixFunctionCall() 
                self.state = 2179
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,324,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ElvisContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def QUEST_NO_WS(self):
            return self.getToken(KotlinParser.QUEST_NO_WS, 0)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_elvis




    def elvis(self):

        localctx = KotlinParser.ElvisContext(self, self._ctx, self.state)
        self.enterRule(localctx, 166, self.RULE_elvis)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2180
            self.match(KotlinParser.QUEST_NO_WS)
            self.state = 2181
            self.match(KotlinParser.COLON)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class InfixFunctionCallContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def rangeExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.RangeExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.RangeExpressionContext,i)


        def simpleIdentifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.SimpleIdentifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_infixFunctionCall




    def infixFunctionCall(self):

        localctx = KotlinParser.InfixFunctionCallContext(self, self._ctx, self.state)
        self.enterRule(localctx, 168, self.RULE_infixFunctionCall)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2183
            self.rangeExpression()
            self.state = 2195
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,326,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2184
                    self.simpleIdentifier()
                    self.state = 2188
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2185
                        self.match(KotlinParser.NL)
                        self.state = 2190
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2191
                    self.rangeExpression() 
                self.state = 2197
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,326,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RangeExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def additiveExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AdditiveExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AdditiveExpressionContext,i)


        def RANGE(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.RANGE)
            else:
                return self.getToken(KotlinParser.RANGE, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_rangeExpression




    def rangeExpression(self):

        localctx = KotlinParser.RangeExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 170, self.RULE_rangeExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2198
            self.additiveExpression()
            self.state = 2209
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,328,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2199
                    self.match(KotlinParser.RANGE)
                    self.state = 2203
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2200
                        self.match(KotlinParser.NL)
                        self.state = 2205
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2206
                    self.additiveExpression() 
                self.state = 2211
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,328,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AdditiveExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def multiplicativeExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.MultiplicativeExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.MultiplicativeExpressionContext,i)


        def additiveOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AdditiveOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AdditiveOperatorContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_additiveExpression




    def additiveExpression(self):

        localctx = KotlinParser.AdditiveExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 172, self.RULE_additiveExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2212
            self.multiplicativeExpression()
            self.state = 2224
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,330,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2213
                    self.additiveOperator()
                    self.state = 2217
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2214
                        self.match(KotlinParser.NL)
                        self.state = 2219
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2220
                    self.multiplicativeExpression() 
                self.state = 2226
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,330,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiplicativeExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def asExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AsExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AsExpressionContext,i)


        def multiplicativeOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.MultiplicativeOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.MultiplicativeOperatorContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiplicativeExpression




    def multiplicativeExpression(self):

        localctx = KotlinParser.MultiplicativeExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 174, self.RULE_multiplicativeExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2227
            self.asExpression()
            self.state = 2239
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,332,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2228
                    self.multiplicativeOperator()
                    self.state = 2232
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2229
                        self.match(KotlinParser.NL)
                        self.state = 2234
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2235
                    self.asExpression() 
                self.state = 2241
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,332,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AsExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def prefixUnaryExpression(self):
            return self.getTypedRuleContext(KotlinParser.PrefixUnaryExpressionContext,0)


        def asOperator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AsOperatorContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AsOperatorContext,i)


        def type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_asExpression




    def asExpression(self):

        localctx = KotlinParser.AsExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 176, self.RULE_asExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2242
            self.prefixUnaryExpression()
            self.state = 2260
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,335,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2246
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2243
                        self.match(KotlinParser.NL)
                        self.state = 2248
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2249
                    self.asOperator()
                    self.state = 2253
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2250
                        self.match(KotlinParser.NL)
                        self.state = 2255
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2256
                    self.type() 
                self.state = 2262
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,335,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PrefixUnaryExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def postfixUnaryExpression(self):
            return self.getTypedRuleContext(KotlinParser.PostfixUnaryExpressionContext,0)


        def unaryPrefix(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.UnaryPrefixContext)
            else:
                return self.getTypedRuleContext(KotlinParser.UnaryPrefixContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_prefixUnaryExpression




    def prefixUnaryExpression(self):

        localctx = KotlinParser.PrefixUnaryExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 178, self.RULE_prefixUnaryExpression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2266
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,336,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2263
                    self.unaryPrefix() 
                self.state = 2268
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,336,self._ctx)

            self.state = 2269
            self.postfixUnaryExpression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class UnaryPrefixContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def annotation(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationContext,0)


        def label(self):
            return self.getTypedRuleContext(KotlinParser.LabelContext,0)


        def prefixUnaryOperator(self):
            return self.getTypedRuleContext(KotlinParser.PrefixUnaryOperatorContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_unaryPrefix




    def unaryPrefix(self):

        localctx = KotlinParser.UnaryPrefixContext(self, self._ctx, self.state)
        self.enterRule(localctx, 180, self.RULE_unaryPrefix)
        self._la = 0 # Token type
        try:
            self.state = 2280
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2271
                self.annotation()
                pass
            elif token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2272
                self.label()
                pass
            elif token in [KotlinParser.ADD, KotlinParser.SUB, KotlinParser.INCR, KotlinParser.DECR, KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS]:
                self.enterOuterAlt(localctx, 3)
                self.state = 2273
                self.prefixUnaryOperator()
                self.state = 2277
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2274
                    self.match(KotlinParser.NL)
                    self.state = 2279
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PostfixUnaryExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def primaryExpression(self):
            return self.getTypedRuleContext(KotlinParser.PrimaryExpressionContext,0)


        def postfixUnarySuffix(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.PostfixUnarySuffixContext)
            else:
                return self.getTypedRuleContext(KotlinParser.PostfixUnarySuffixContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_postfixUnaryExpression




    def postfixUnaryExpression(self):

        localctx = KotlinParser.PostfixUnaryExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 182, self.RULE_postfixUnaryExpression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2282
            self.primaryExpression()
            self.state = 2286
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,339,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2283
                    self.postfixUnarySuffix() 
                self.state = 2288
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,339,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PostfixUnarySuffixContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def postfixUnaryOperator(self):
            return self.getTypedRuleContext(KotlinParser.PostfixUnaryOperatorContext,0)


        def typeArguments(self):
            return self.getTypedRuleContext(KotlinParser.TypeArgumentsContext,0)


        def callSuffix(self):
            return self.getTypedRuleContext(KotlinParser.CallSuffixContext,0)


        def indexingSuffix(self):
            return self.getTypedRuleContext(KotlinParser.IndexingSuffixContext,0)


        def navigationSuffix(self):
            return self.getTypedRuleContext(KotlinParser.NavigationSuffixContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_postfixUnarySuffix




    def postfixUnarySuffix(self):

        localctx = KotlinParser.PostfixUnarySuffixContext(self, self._ctx, self.state)
        self.enterRule(localctx, 184, self.RULE_postfixUnarySuffix)
        try:
            self.state = 2294
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,340,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 2289
                self.postfixUnaryOperator()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 2290
                self.typeArguments()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 2291
                self.callSuffix()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 2292
                self.indexingSuffix()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 2293
                self.navigationSuffix()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DirectlyAssignableExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def postfixUnaryExpression(self):
            return self.getTypedRuleContext(KotlinParser.PostfixUnaryExpressionContext,0)


        def assignableSuffix(self):
            return self.getTypedRuleContext(KotlinParser.AssignableSuffixContext,0)


        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def parenthesizedDirectlyAssignableExpression(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedDirectlyAssignableExpressionContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_directlyAssignableExpression




    def directlyAssignableExpression(self):

        localctx = KotlinParser.DirectlyAssignableExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 186, self.RULE_directlyAssignableExpression)
        try:
            self.state = 2301
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,341,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 2296
                self.postfixUnaryExpression()
                self.state = 2297
                self.assignableSuffix()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 2299
                self.simpleIdentifier()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 2300
                self.parenthesizedDirectlyAssignableExpression()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParenthesizedDirectlyAssignableExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def directlyAssignableExpression(self):
            return self.getTypedRuleContext(KotlinParser.DirectlyAssignableExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parenthesizedDirectlyAssignableExpression




    def parenthesizedDirectlyAssignableExpression(self):

        localctx = KotlinParser.ParenthesizedDirectlyAssignableExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 188, self.RULE_parenthesizedDirectlyAssignableExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2303
            self.match(KotlinParser.LPAREN)
            self.state = 2307
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2304
                self.match(KotlinParser.NL)
                self.state = 2309
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2310
            self.directlyAssignableExpression()
            self.state = 2314
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2311
                self.match(KotlinParser.NL)
                self.state = 2316
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2317
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignableExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def prefixUnaryExpression(self):
            return self.getTypedRuleContext(KotlinParser.PrefixUnaryExpressionContext,0)


        def parenthesizedAssignableExpression(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedAssignableExpressionContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_assignableExpression




    def assignableExpression(self):

        localctx = KotlinParser.AssignableExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 190, self.RULE_assignableExpression)
        try:
            self.state = 2321
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,344,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 2319
                self.prefixUnaryExpression()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 2320
                self.parenthesizedAssignableExpression()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParenthesizedAssignableExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def assignableExpression(self):
            return self.getTypedRuleContext(KotlinParser.AssignableExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parenthesizedAssignableExpression




    def parenthesizedAssignableExpression(self):

        localctx = KotlinParser.ParenthesizedAssignableExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 192, self.RULE_parenthesizedAssignableExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2323
            self.match(KotlinParser.LPAREN)
            self.state = 2327
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2324
                self.match(KotlinParser.NL)
                self.state = 2329
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2330
            self.assignableExpression()
            self.state = 2334
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2331
                self.match(KotlinParser.NL)
                self.state = 2336
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2337
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignableSuffixContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typeArguments(self):
            return self.getTypedRuleContext(KotlinParser.TypeArgumentsContext,0)


        def indexingSuffix(self):
            return self.getTypedRuleContext(KotlinParser.IndexingSuffixContext,0)


        def navigationSuffix(self):
            return self.getTypedRuleContext(KotlinParser.NavigationSuffixContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_assignableSuffix




    def assignableSuffix(self):

        localctx = KotlinParser.AssignableSuffixContext(self, self._ctx, self.state)
        self.enterRule(localctx, 194, self.RULE_assignableSuffix)
        try:
            self.state = 2342
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LANGLE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2339
                self.typeArguments()
                pass
            elif token in [KotlinParser.LSQUARE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2340
                self.indexingSuffix()
                pass
            elif token in [KotlinParser.NL, KotlinParser.DOT, KotlinParser.COLONCOLON, KotlinParser.QUEST_NO_WS]:
                self.enterOuterAlt(localctx, 3)
                self.state = 2341
                self.navigationSuffix()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IndexingSuffixContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self):
            return self.getToken(KotlinParser.LSQUARE, 0)

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ExpressionContext,i)


        def RSQUARE(self):
            return self.getToken(KotlinParser.RSQUARE, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_indexingSuffix




    def indexingSuffix(self):

        localctx = KotlinParser.IndexingSuffixContext(self, self._ctx, self.state)
        self.enterRule(localctx, 196, self.RULE_indexingSuffix)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2344
            self.match(KotlinParser.LSQUARE)
            self.state = 2348
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2345
                self.match(KotlinParser.NL)
                self.state = 2350
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2351
            self.expression()
            self.state = 2368
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,351,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2355
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2352
                        self.match(KotlinParser.NL)
                        self.state = 2357
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2358
                    self.match(KotlinParser.COMMA)
                    self.state = 2362
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2359
                        self.match(KotlinParser.NL)
                        self.state = 2364
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2365
                    self.expression() 
                self.state = 2370
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,351,self._ctx)

            self.state = 2378
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,353,self._ctx)
            if la_ == 1:
                self.state = 2374
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2371
                    self.match(KotlinParser.NL)
                    self.state = 2376
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2377
                self.match(KotlinParser.COMMA)


            self.state = 2383
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2380
                self.match(KotlinParser.NL)
                self.state = 2385
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2386
            self.match(KotlinParser.RSQUARE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NavigationSuffixContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def memberAccessOperator(self):
            return self.getTypedRuleContext(KotlinParser.MemberAccessOperatorContext,0)


        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def parenthesizedExpression(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedExpressionContext,0)


        def CLASS(self):
            return self.getToken(KotlinParser.CLASS, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_navigationSuffix




    def navigationSuffix(self):

        localctx = KotlinParser.NavigationSuffixContext(self, self._ctx, self.state)
        self.enterRule(localctx, 198, self.RULE_navigationSuffix)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2388
            self.memberAccessOperator()
            self.state = 2392
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2389
                self.match(KotlinParser.NL)
                self.state = 2394
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2398
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 2395
                self.simpleIdentifier()
                pass
            elif token in [KotlinParser.LPAREN]:
                self.state = 2396
                self.parenthesizedExpression()
                pass
            elif token in [KotlinParser.CLASS]:
                self.state = 2397
                self.match(KotlinParser.CLASS)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CallSuffixContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def annotatedLambda(self):
            return self.getTypedRuleContext(KotlinParser.AnnotatedLambdaContext,0)


        def valueArguments(self):
            return self.getTypedRuleContext(KotlinParser.ValueArgumentsContext,0)


        def typeArguments(self):
            return self.getTypedRuleContext(KotlinParser.TypeArgumentsContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_callSuffix




    def callSuffix(self):

        localctx = KotlinParser.CallSuffixContext(self, self._ctx, self.state)
        self.enterRule(localctx, 200, self.RULE_callSuffix)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2401
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.LANGLE:
                self.state = 2400
                self.typeArguments()


            self.state = 2408
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,359,self._ctx)
            if la_ == 1:
                self.state = 2404
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==KotlinParser.LPAREN:
                    self.state = 2403
                    self.valueArguments()


                self.state = 2406
                self.annotatedLambda()
                pass

            elif la_ == 2:
                self.state = 2407
                self.valueArguments()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AnnotatedLambdaContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def lambdaLiteral(self):
            return self.getTypedRuleContext(KotlinParser.LambdaLiteralContext,0)


        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def label(self):
            return self.getTypedRuleContext(KotlinParser.LabelContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_annotatedLambda




    def annotatedLambda(self):

        localctx = KotlinParser.AnnotatedLambdaContext(self, self._ctx, self.state)
        self.enterRule(localctx, 202, self.RULE_annotatedLambda)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2413
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS:
                self.state = 2410
                self.annotation()
                self.state = 2415
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2417
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if ((((_la - 61)) & ~0x3f) == 0 and ((1 << (_la - 61)) & ((1 << (KotlinParser.FILE - 61)) | (1 << (KotlinParser.FIELD - 61)) | (1 << (KotlinParser.PROPERTY - 61)) | (1 << (KotlinParser.GET - 61)) | (1 << (KotlinParser.SET - 61)) | (1 << (KotlinParser.RECEIVER - 61)) | (1 << (KotlinParser.PARAM - 61)) | (1 << (KotlinParser.SETPARAM - 61)) | (1 << (KotlinParser.DELEGATE - 61)) | (1 << (KotlinParser.IMPORT - 61)) | (1 << (KotlinParser.CONSTRUCTOR - 61)) | (1 << (KotlinParser.BY - 61)) | (1 << (KotlinParser.COMPANION - 61)) | (1 << (KotlinParser.INIT - 61)) | (1 << (KotlinParser.WHERE - 61)) | (1 << (KotlinParser.CATCH - 61)) | (1 << (KotlinParser.FINALLY - 61)) | (1 << (KotlinParser.OUT - 61)) | (1 << (KotlinParser.DYNAMIC - 61)) | (1 << (KotlinParser.PUBLIC - 61)) | (1 << (KotlinParser.PRIVATE - 61)) | (1 << (KotlinParser.PROTECTED - 61)) | (1 << (KotlinParser.INTERNAL - 61)) | (1 << (KotlinParser.ENUM - 61)) | (1 << (KotlinParser.SEALED - 61)) | (1 << (KotlinParser.ANNOTATION - 61)) | (1 << (KotlinParser.DATA - 61)) | (1 << (KotlinParser.INNER - 61)) | (1 << (KotlinParser.VALUE - 61)) | (1 << (KotlinParser.TAILREC - 61)) | (1 << (KotlinParser.OPERATOR - 61)) | (1 << (KotlinParser.INLINE - 61)) | (1 << (KotlinParser.INFIX - 61)) | (1 << (KotlinParser.EXTERNAL - 61)) | (1 << (KotlinParser.SUSPEND - 61)) | (1 << (KotlinParser.OVERRIDE - 61)) | (1 << (KotlinParser.ABSTRACT - 61)))) != 0) or ((((_la - 125)) & ~0x3f) == 0 and ((1 << (_la - 125)) & ((1 << (KotlinParser.FINAL - 125)) | (1 << (KotlinParser.OPEN - 125)) | (1 << (KotlinParser.CONST - 125)) | (1 << (KotlinParser.LATEINIT - 125)) | (1 << (KotlinParser.VARARG - 125)) | (1 << (KotlinParser.NOINLINE - 125)) | (1 << (KotlinParser.CROSSINLINE - 125)) | (1 << (KotlinParser.REIFIED - 125)) | (1 << (KotlinParser.EXPECT - 125)) | (1 << (KotlinParser.ACTUAL - 125)) | (1 << (KotlinParser.Identifier - 125)))) != 0):
                self.state = 2416
                self.label()


            self.state = 2422
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2419
                self.match(KotlinParser.NL)
                self.state = 2424
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2425
            self.lambdaLiteral()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeArgumentsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LANGLE(self):
            return self.getToken(KotlinParser.LANGLE, 0)

        def typeProjection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeProjectionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeProjectionContext,i)


        def RANGLE(self):
            return self.getToken(KotlinParser.RANGLE, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeArguments




    def typeArguments(self):

        localctx = KotlinParser.TypeArgumentsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 204, self.RULE_typeArguments)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2427
            self.match(KotlinParser.LANGLE)
            self.state = 2431
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2428
                self.match(KotlinParser.NL)
                self.state = 2433
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2434
            self.typeProjection()
            self.state = 2451
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,366,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2438
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2435
                        self.match(KotlinParser.NL)
                        self.state = 2440
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2441
                    self.match(KotlinParser.COMMA)
                    self.state = 2445
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2442
                        self.match(KotlinParser.NL)
                        self.state = 2447
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2448
                    self.typeProjection() 
                self.state = 2453
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,366,self._ctx)

            self.state = 2461
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,368,self._ctx)
            if la_ == 1:
                self.state = 2457
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2454
                    self.match(KotlinParser.NL)
                    self.state = 2459
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2460
                self.match(KotlinParser.COMMA)


            self.state = 2466
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2463
                self.match(KotlinParser.NL)
                self.state = 2468
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2469
            self.match(KotlinParser.RANGLE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ValueArgumentsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def valueArgument(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ValueArgumentContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ValueArgumentContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_valueArguments




    def valueArguments(self):

        localctx = KotlinParser.ValueArgumentsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 206, self.RULE_valueArguments)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2471
            self.match(KotlinParser.LPAREN)
            self.state = 2475
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,370,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2472
                    self.match(KotlinParser.NL) 
                self.state = 2477
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,370,self._ctx)

            self.state = 2513
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.NL) | (1 << KotlinParser.LPAREN) | (1 << KotlinParser.LSQUARE) | (1 << KotlinParser.LCURL) | (1 << KotlinParser.MULT) | (1 << KotlinParser.ADD) | (1 << KotlinParser.SUB) | (1 << KotlinParser.INCR) | (1 << KotlinParser.DECR) | (1 << KotlinParser.EXCL_WS) | (1 << KotlinParser.EXCL_NO_WS) | (1 << KotlinParser.COLONCOLON) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.RETURN_AT) | (1 << KotlinParser.CONTINUE_AT) | (1 << KotlinParser.BREAK_AT) | (1 << KotlinParser.THIS_AT) | (1 << KotlinParser.SUPER_AT) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.FUN - 64)) | (1 << (KotlinParser.OBJECT - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.THIS - 64)) | (1 << (KotlinParser.SUPER - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.IF - 64)) | (1 << (KotlinParser.WHEN - 64)) | (1 << (KotlinParser.TRY - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.THROW - 64)) | (1 << (KotlinParser.RETURN - 64)) | (1 << (KotlinParser.CONTINUE - 64)) | (1 << (KotlinParser.BREAK - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.RealLiteral - 128)) | (1 << (KotlinParser.IntegerLiteral - 128)) | (1 << (KotlinParser.HexLiteral - 128)) | (1 << (KotlinParser.BinLiteral - 128)) | (1 << (KotlinParser.UnsignedLiteral - 128)) | (1 << (KotlinParser.LongLiteral - 128)) | (1 << (KotlinParser.BooleanLiteral - 128)) | (1 << (KotlinParser.NullLiteral - 128)) | (1 << (KotlinParser.CharacterLiteral - 128)) | (1 << (KotlinParser.Identifier - 128)) | (1 << (KotlinParser.QUOTE_OPEN - 128)) | (1 << (KotlinParser.TRIPLE_QUOTE_OPEN - 128)))) != 0):
                self.state = 2478
                self.valueArgument()
                self.state = 2495
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,373,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2482
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 2479
                            self.match(KotlinParser.NL)
                            self.state = 2484
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 2485
                        self.match(KotlinParser.COMMA)
                        self.state = 2489
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,372,self._ctx)
                        while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                            if _alt==1:
                                self.state = 2486
                                self.match(KotlinParser.NL) 
                            self.state = 2491
                            self._errHandler.sync(self)
                            _alt = self._interp.adaptivePredict(self._input,372,self._ctx)

                        self.state = 2492
                        self.valueArgument() 
                    self.state = 2497
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,373,self._ctx)

                self.state = 2505
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,375,self._ctx)
                if la_ == 1:
                    self.state = 2501
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2498
                        self.match(KotlinParser.NL)
                        self.state = 2503
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2504
                    self.match(KotlinParser.COMMA)


                self.state = 2510
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2507
                    self.match(KotlinParser.NL)
                    self.state = 2512
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 2515
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ValueArgumentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def annotation(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def MULT(self):
            return self.getToken(KotlinParser.MULT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_valueArgument




    def valueArgument(self):

        localctx = KotlinParser.ValueArgumentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 208, self.RULE_valueArgument)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2518
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,378,self._ctx)
            if la_ == 1:
                self.state = 2517
                self.annotation()


            self.state = 2523
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,379,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2520
                    self.match(KotlinParser.NL) 
                self.state = 2525
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,379,self._ctx)

            self.state = 2540
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,382,self._ctx)
            if la_ == 1:
                self.state = 2526
                self.simpleIdentifier()
                self.state = 2530
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2527
                    self.match(KotlinParser.NL)
                    self.state = 2532
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2533
                self.match(KotlinParser.ASSIGNMENT)
                self.state = 2537
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,381,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2534
                        self.match(KotlinParser.NL) 
                    self.state = 2539
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,381,self._ctx)



            self.state = 2543
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.MULT:
                self.state = 2542
                self.match(KotlinParser.MULT)


            self.state = 2548
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2545
                self.match(KotlinParser.NL)
                self.state = 2550
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2551
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PrimaryExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def parenthesizedExpression(self):
            return self.getTypedRuleContext(KotlinParser.ParenthesizedExpressionContext,0)


        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def literalConstant(self):
            return self.getTypedRuleContext(KotlinParser.LiteralConstantContext,0)


        def stringLiteral(self):
            return self.getTypedRuleContext(KotlinParser.StringLiteralContext,0)


        def callableReference(self):
            return self.getTypedRuleContext(KotlinParser.CallableReferenceContext,0)


        def functionLiteral(self):
            return self.getTypedRuleContext(KotlinParser.FunctionLiteralContext,0)


        def objectLiteral(self):
            return self.getTypedRuleContext(KotlinParser.ObjectLiteralContext,0)


        def collectionLiteral(self):
            return self.getTypedRuleContext(KotlinParser.CollectionLiteralContext,0)


        def thisExpression(self):
            return self.getTypedRuleContext(KotlinParser.ThisExpressionContext,0)


        def superExpression(self):
            return self.getTypedRuleContext(KotlinParser.SuperExpressionContext,0)


        def ifExpression(self):
            return self.getTypedRuleContext(KotlinParser.IfExpressionContext,0)


        def whenExpression(self):
            return self.getTypedRuleContext(KotlinParser.WhenExpressionContext,0)


        def tryExpression(self):
            return self.getTypedRuleContext(KotlinParser.TryExpressionContext,0)


        def jumpExpression(self):
            return self.getTypedRuleContext(KotlinParser.JumpExpressionContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_primaryExpression




    def primaryExpression(self):

        localctx = KotlinParser.PrimaryExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 210, self.RULE_primaryExpression)
        try:
            self.state = 2567
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,385,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 2553
                self.parenthesizedExpression()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 2554
                self.simpleIdentifier()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 2555
                self.literalConstant()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 2556
                self.stringLiteral()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 2557
                self.callableReference()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 2558
                self.functionLiteral()
                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 2559
                self.objectLiteral()
                pass

            elif la_ == 8:
                self.enterOuterAlt(localctx, 8)
                self.state = 2560
                self.collectionLiteral()
                pass

            elif la_ == 9:
                self.enterOuterAlt(localctx, 9)
                self.state = 2561
                self.thisExpression()
                pass

            elif la_ == 10:
                self.enterOuterAlt(localctx, 10)
                self.state = 2562
                self.superExpression()
                pass

            elif la_ == 11:
                self.enterOuterAlt(localctx, 11)
                self.state = 2563
                self.ifExpression()
                pass

            elif la_ == 12:
                self.enterOuterAlt(localctx, 12)
                self.state = 2564
                self.whenExpression()
                pass

            elif la_ == 13:
                self.enterOuterAlt(localctx, 13)
                self.state = 2565
                self.tryExpression()
                pass

            elif la_ == 14:
                self.enterOuterAlt(localctx, 14)
                self.state = 2566
                self.jumpExpression()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParenthesizedExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_parenthesizedExpression




    def parenthesizedExpression(self):

        localctx = KotlinParser.ParenthesizedExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 212, self.RULE_parenthesizedExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2569
            self.match(KotlinParser.LPAREN)
            self.state = 2573
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2570
                self.match(KotlinParser.NL)
                self.state = 2575
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2576
            self.expression()
            self.state = 2580
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2577
                self.match(KotlinParser.NL)
                self.state = 2582
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2583
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CollectionLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self):
            return self.getToken(KotlinParser.LSQUARE, 0)

        def RSQUARE(self):
            return self.getToken(KotlinParser.RSQUARE, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ExpressionContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_collectionLiteral




    def collectionLiteral(self):

        localctx = KotlinParser.CollectionLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 214, self.RULE_collectionLiteral)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2585
            self.match(KotlinParser.LSQUARE)
            self.state = 2589
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2586
                self.match(KotlinParser.NL)
                self.state = 2591
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2627
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LPAREN) | (1 << KotlinParser.LSQUARE) | (1 << KotlinParser.LCURL) | (1 << KotlinParser.ADD) | (1 << KotlinParser.SUB) | (1 << KotlinParser.INCR) | (1 << KotlinParser.DECR) | (1 << KotlinParser.EXCL_WS) | (1 << KotlinParser.EXCL_NO_WS) | (1 << KotlinParser.COLONCOLON) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.RETURN_AT) | (1 << KotlinParser.CONTINUE_AT) | (1 << KotlinParser.BREAK_AT) | (1 << KotlinParser.THIS_AT) | (1 << KotlinParser.SUPER_AT) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.FUN - 64)) | (1 << (KotlinParser.OBJECT - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.THIS - 64)) | (1 << (KotlinParser.SUPER - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.IF - 64)) | (1 << (KotlinParser.WHEN - 64)) | (1 << (KotlinParser.TRY - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.THROW - 64)) | (1 << (KotlinParser.RETURN - 64)) | (1 << (KotlinParser.CONTINUE - 64)) | (1 << (KotlinParser.BREAK - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.RealLiteral - 128)) | (1 << (KotlinParser.IntegerLiteral - 128)) | (1 << (KotlinParser.HexLiteral - 128)) | (1 << (KotlinParser.BinLiteral - 128)) | (1 << (KotlinParser.UnsignedLiteral - 128)) | (1 << (KotlinParser.LongLiteral - 128)) | (1 << (KotlinParser.BooleanLiteral - 128)) | (1 << (KotlinParser.NullLiteral - 128)) | (1 << (KotlinParser.CharacterLiteral - 128)) | (1 << (KotlinParser.Identifier - 128)) | (1 << (KotlinParser.QUOTE_OPEN - 128)) | (1 << (KotlinParser.TRIPLE_QUOTE_OPEN - 128)))) != 0):
                self.state = 2592
                self.expression()
                self.state = 2609
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,391,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2596
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 2593
                            self.match(KotlinParser.NL)
                            self.state = 2598
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 2599
                        self.match(KotlinParser.COMMA)
                        self.state = 2603
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 2600
                            self.match(KotlinParser.NL)
                            self.state = 2605
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 2606
                        self.expression() 
                    self.state = 2611
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,391,self._ctx)

                self.state = 2619
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,393,self._ctx)
                if la_ == 1:
                    self.state = 2615
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2612
                        self.match(KotlinParser.NL)
                        self.state = 2617
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2618
                    self.match(KotlinParser.COMMA)


                self.state = 2624
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2621
                    self.match(KotlinParser.NL)
                    self.state = 2626
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 2629
            self.match(KotlinParser.RSQUARE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LiteralConstantContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def BooleanLiteral(self):
            return self.getToken(KotlinParser.BooleanLiteral, 0)

        def IntegerLiteral(self):
            return self.getToken(KotlinParser.IntegerLiteral, 0)

        def HexLiteral(self):
            return self.getToken(KotlinParser.HexLiteral, 0)

        def BinLiteral(self):
            return self.getToken(KotlinParser.BinLiteral, 0)

        def CharacterLiteral(self):
            return self.getToken(KotlinParser.CharacterLiteral, 0)

        def RealLiteral(self):
            return self.getToken(KotlinParser.RealLiteral, 0)

        def NullLiteral(self):
            return self.getToken(KotlinParser.NullLiteral, 0)

        def LongLiteral(self):
            return self.getToken(KotlinParser.LongLiteral, 0)

        def UnsignedLiteral(self):
            return self.getToken(KotlinParser.UnsignedLiteral, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_literalConstant




    def literalConstant(self):

        localctx = KotlinParser.LiteralConstantContext(self, self._ctx, self.state)
        self.enterRule(localctx, 216, self.RULE_literalConstant)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2631
            _la = self._input.LA(1)
            if not(((((_la - 135)) & ~0x3f) == 0 and ((1 << (_la - 135)) & ((1 << (KotlinParser.RealLiteral - 135)) | (1 << (KotlinParser.IntegerLiteral - 135)) | (1 << (KotlinParser.HexLiteral - 135)) | (1 << (KotlinParser.BinLiteral - 135)) | (1 << (KotlinParser.UnsignedLiteral - 135)) | (1 << (KotlinParser.LongLiteral - 135)) | (1 << (KotlinParser.BooleanLiteral - 135)) | (1 << (KotlinParser.NullLiteral - 135)) | (1 << (KotlinParser.CharacterLiteral - 135)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class StringLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def lineStringLiteral(self):
            return self.getTypedRuleContext(KotlinParser.LineStringLiteralContext,0)


        def multiLineStringLiteral(self):
            return self.getTypedRuleContext(KotlinParser.MultiLineStringLiteralContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_stringLiteral




    def stringLiteral(self):

        localctx = KotlinParser.StringLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 218, self.RULE_stringLiteral)
        try:
            self.state = 2635
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.QUOTE_OPEN]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2633
                self.lineStringLiteral()
                pass
            elif token in [KotlinParser.TRIPLE_QUOTE_OPEN]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2634
                self.multiLineStringLiteral()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineStringLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def QUOTE_OPEN(self):
            return self.getToken(KotlinParser.QUOTE_OPEN, 0)

        def QUOTE_CLOSE(self):
            return self.getToken(KotlinParser.QUOTE_CLOSE, 0)

        def lineStringContent(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.LineStringContentContext)
            else:
                return self.getTypedRuleContext(KotlinParser.LineStringContentContext,i)


        def lineStringExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.LineStringExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.LineStringExpressionContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_lineStringLiteral




    def lineStringLiteral(self):

        localctx = KotlinParser.LineStringLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 220, self.RULE_lineStringLiteral)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2637
            self.match(KotlinParser.QUOTE_OPEN)
            self.state = 2642
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while ((((_la - 159)) & ~0x3f) == 0 and ((1 << (_la - 159)) & ((1 << (KotlinParser.LineStrRef - 159)) | (1 << (KotlinParser.LineStrText - 159)) | (1 << (KotlinParser.LineStrEscapedChar - 159)) | (1 << (KotlinParser.LineStrExprStart - 159)))) != 0):
                self.state = 2640
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [KotlinParser.LineStrRef, KotlinParser.LineStrText, KotlinParser.LineStrEscapedChar]:
                    self.state = 2638
                    self.lineStringContent()
                    pass
                elif token in [KotlinParser.LineStrExprStart]:
                    self.state = 2639
                    self.lineStringExpression()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 2644
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2645
            self.match(KotlinParser.QUOTE_CLOSE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiLineStringLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TRIPLE_QUOTE_OPEN(self):
            return self.getToken(KotlinParser.TRIPLE_QUOTE_OPEN, 0)

        def TRIPLE_QUOTE_CLOSE(self):
            return self.getToken(KotlinParser.TRIPLE_QUOTE_CLOSE, 0)

        def multiLineStringContent(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.MultiLineStringContentContext)
            else:
                return self.getTypedRuleContext(KotlinParser.MultiLineStringContentContext,i)


        def multiLineStringExpression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.MultiLineStringExpressionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.MultiLineStringExpressionContext,i)


        def MultiLineStringQuote(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.MultiLineStringQuote)
            else:
                return self.getToken(KotlinParser.MultiLineStringQuote, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiLineStringLiteral




    def multiLineStringLiteral(self):

        localctx = KotlinParser.MultiLineStringLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 222, self.RULE_multiLineStringLiteral)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2647
            self.match(KotlinParser.TRIPLE_QUOTE_OPEN)
            self.state = 2653
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while ((((_la - 164)) & ~0x3f) == 0 and ((1 << (_la - 164)) & ((1 << (KotlinParser.MultiLineStringQuote - 164)) | (1 << (KotlinParser.MultiLineStrRef - 164)) | (1 << (KotlinParser.MultiLineStrText - 164)) | (1 << (KotlinParser.MultiLineStrExprStart - 164)))) != 0):
                self.state = 2651
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,399,self._ctx)
                if la_ == 1:
                    self.state = 2648
                    self.multiLineStringContent()
                    pass

                elif la_ == 2:
                    self.state = 2649
                    self.multiLineStringExpression()
                    pass

                elif la_ == 3:
                    self.state = 2650
                    self.match(KotlinParser.MultiLineStringQuote)
                    pass


                self.state = 2655
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2656
            self.match(KotlinParser.TRIPLE_QUOTE_CLOSE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineStringContentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LineStrText(self):
            return self.getToken(KotlinParser.LineStrText, 0)

        def LineStrEscapedChar(self):
            return self.getToken(KotlinParser.LineStrEscapedChar, 0)

        def LineStrRef(self):
            return self.getToken(KotlinParser.LineStrRef, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_lineStringContent




    def lineStringContent(self):

        localctx = KotlinParser.LineStringContentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 224, self.RULE_lineStringContent)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2658
            _la = self._input.LA(1)
            if not(((((_la - 159)) & ~0x3f) == 0 and ((1 << (_la - 159)) & ((1 << (KotlinParser.LineStrRef - 159)) | (1 << (KotlinParser.LineStrText - 159)) | (1 << (KotlinParser.LineStrEscapedChar - 159)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineStringExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LineStrExprStart(self):
            return self.getToken(KotlinParser.LineStrExprStart, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_lineStringExpression




    def lineStringExpression(self):

        localctx = KotlinParser.LineStringExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 226, self.RULE_lineStringExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2660
            self.match(KotlinParser.LineStrExprStart)
            self.state = 2664
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2661
                self.match(KotlinParser.NL)
                self.state = 2666
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2667
            self.expression()
            self.state = 2671
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2668
                self.match(KotlinParser.NL)
                self.state = 2673
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2674
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiLineStringContentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def MultiLineStrText(self):
            return self.getToken(KotlinParser.MultiLineStrText, 0)

        def MultiLineStringQuote(self):
            return self.getToken(KotlinParser.MultiLineStringQuote, 0)

        def MultiLineStrRef(self):
            return self.getToken(KotlinParser.MultiLineStrRef, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiLineStringContent




    def multiLineStringContent(self):

        localctx = KotlinParser.MultiLineStringContentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 228, self.RULE_multiLineStringContent)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2676
            _la = self._input.LA(1)
            if not(((((_la - 164)) & ~0x3f) == 0 and ((1 << (_la - 164)) & ((1 << (KotlinParser.MultiLineStringQuote - 164)) | (1 << (KotlinParser.MultiLineStrRef - 164)) | (1 << (KotlinParser.MultiLineStrText - 164)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiLineStringExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def MultiLineStrExprStart(self):
            return self.getToken(KotlinParser.MultiLineStrExprStart, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiLineStringExpression




    def multiLineStringExpression(self):

        localctx = KotlinParser.MultiLineStringExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 230, self.RULE_multiLineStringExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2678
            self.match(KotlinParser.MultiLineStrExprStart)
            self.state = 2682
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2679
                self.match(KotlinParser.NL)
                self.state = 2684
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2685
            self.expression()
            self.state = 2689
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2686
                self.match(KotlinParser.NL)
                self.state = 2691
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2692
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LambdaLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LCURL(self):
            return self.getToken(KotlinParser.LCURL, 0)

        def statements(self):
            return self.getTypedRuleContext(KotlinParser.StatementsContext,0)


        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def ARROW(self):
            return self.getToken(KotlinParser.ARROW, 0)

        def lambdaParameters(self):
            return self.getTypedRuleContext(KotlinParser.LambdaParametersContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_lambdaLiteral




    def lambdaLiteral(self):

        localctx = KotlinParser.LambdaLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 232, self.RULE_lambdaLiteral)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2694
            self.match(KotlinParser.LCURL)
            self.state = 2698
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,405,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2695
                    self.match(KotlinParser.NL) 
                self.state = 2700
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,405,self._ctx)

            self.state = 2717
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,409,self._ctx)
            if la_ == 1:
                self.state = 2702
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,406,self._ctx)
                if la_ == 1:
                    self.state = 2701
                    self.lambdaParameters()


                self.state = 2707
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2704
                    self.match(KotlinParser.NL)
                    self.state = 2709
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2710
                self.match(KotlinParser.ARROW)
                self.state = 2714
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,408,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2711
                        self.match(KotlinParser.NL) 
                    self.state = 2716
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,408,self._ctx)



            self.state = 2719
            self.statements()
            self.state = 2723
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2720
                self.match(KotlinParser.NL)
                self.state = 2725
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2726
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LambdaParametersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def lambdaParameter(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.LambdaParameterContext)
            else:
                return self.getTypedRuleContext(KotlinParser.LambdaParameterContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_lambdaParameters




    def lambdaParameters(self):

        localctx = KotlinParser.LambdaParametersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 234, self.RULE_lambdaParameters)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2728
            self.lambdaParameter()
            self.state = 2745
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,413,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2732
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2729
                        self.match(KotlinParser.NL)
                        self.state = 2734
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2735
                    self.match(KotlinParser.COMMA)
                    self.state = 2739
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,412,self._ctx)
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt==1:
                            self.state = 2736
                            self.match(KotlinParser.NL) 
                        self.state = 2741
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,412,self._ctx)

                    self.state = 2742
                    self.lambdaParameter() 
                self.state = 2747
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,413,self._ctx)

            self.state = 2755
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,415,self._ctx)
            if la_ == 1:
                self.state = 2751
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2748
                    self.match(KotlinParser.NL)
                    self.state = 2753
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2754
                self.match(KotlinParser.COMMA)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LambdaParameterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def variableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.VariableDeclarationContext,0)


        def multiVariableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.MultiVariableDeclarationContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_lambdaParameter




    def lambdaParameter(self):

        localctx = KotlinParser.LambdaParameterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 236, self.RULE_lambdaParameter)
        self._la = 0 # Token type
        try:
            self.state = 2775
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.NL, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2757
                self.variableDeclaration()
                pass
            elif token in [KotlinParser.LPAREN]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2758
                self.multiVariableDeclaration()
                self.state = 2773
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,418,self._ctx)
                if la_ == 1:
                    self.state = 2762
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2759
                        self.match(KotlinParser.NL)
                        self.state = 2764
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2765
                    self.match(KotlinParser.COLON)
                    self.state = 2769
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2766
                        self.match(KotlinParser.NL)
                        self.state = 2771
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2772
                    self.type()


                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AnonymousFunctionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def FUN(self):
            return self.getToken(KotlinParser.FUN, 0)

        def parametersWithOptionalType(self):
            return self.getTypedRuleContext(KotlinParser.ParametersWithOptionalTypeContext,0)


        def type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeContext,i)


        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def typeConstraints(self):
            return self.getTypedRuleContext(KotlinParser.TypeConstraintsContext,0)


        def functionBody(self):
            return self.getTypedRuleContext(KotlinParser.FunctionBodyContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_anonymousFunction




    def anonymousFunction(self):

        localctx = KotlinParser.AnonymousFunctionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 238, self.RULE_anonymousFunction)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2777
            self.match(KotlinParser.FUN)
            self.state = 2793
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,422,self._ctx)
            if la_ == 1:
                self.state = 2781
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2778
                    self.match(KotlinParser.NL)
                    self.state = 2783
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2784
                self.type()
                self.state = 2788
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2785
                    self.match(KotlinParser.NL)
                    self.state = 2790
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2791
                self.match(KotlinParser.DOT)


            self.state = 2798
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2795
                self.match(KotlinParser.NL)
                self.state = 2800
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2801
            self.parametersWithOptionalType()
            self.state = 2816
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,426,self._ctx)
            if la_ == 1:
                self.state = 2805
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2802
                    self.match(KotlinParser.NL)
                    self.state = 2807
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2808
                self.match(KotlinParser.COLON)
                self.state = 2812
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2809
                    self.match(KotlinParser.NL)
                    self.state = 2814
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2815
                self.type()


            self.state = 2825
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,428,self._ctx)
            if la_ == 1:
                self.state = 2821
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2818
                    self.match(KotlinParser.NL)
                    self.state = 2823
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2824
                self.typeConstraints()


            self.state = 2834
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,430,self._ctx)
            if la_ == 1:
                self.state = 2830
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2827
                    self.match(KotlinParser.NL)
                    self.state = 2832
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2833
                self.functionBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def lambdaLiteral(self):
            return self.getTypedRuleContext(KotlinParser.LambdaLiteralContext,0)


        def anonymousFunction(self):
            return self.getTypedRuleContext(KotlinParser.AnonymousFunctionContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_functionLiteral




    def functionLiteral(self):

        localctx = KotlinParser.FunctionLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 240, self.RULE_functionLiteral)
        try:
            self.state = 2838
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LCURL]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2836
                self.lambdaLiteral()
                pass
            elif token in [KotlinParser.FUN]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2837
                self.anonymousFunction()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ObjectLiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def OBJECT(self):
            return self.getToken(KotlinParser.OBJECT, 0)

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def delegationSpecifiers(self):
            return self.getTypedRuleContext(KotlinParser.DelegationSpecifiersContext,0)


        def classBody(self):
            return self.getTypedRuleContext(KotlinParser.ClassBodyContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_objectLiteral




    def objectLiteral(self):

        localctx = KotlinParser.ObjectLiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 242, self.RULE_objectLiteral)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2840
            self.match(KotlinParser.OBJECT)
            self.state = 2861
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,435,self._ctx)
            if la_ == 1:
                self.state = 2844
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2841
                    self.match(KotlinParser.NL)
                    self.state = 2846
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2847
                self.match(KotlinParser.COLON)
                self.state = 2851
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,433,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2848
                        self.match(KotlinParser.NL) 
                    self.state = 2853
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,433,self._ctx)

                self.state = 2854
                self.delegationSpecifiers()
                self.state = 2858
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,434,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2855
                        self.match(KotlinParser.NL) 
                    self.state = 2860
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,434,self._ctx)



            self.state = 2870
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,437,self._ctx)
            if la_ == 1:
                self.state = 2866
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2863
                    self.match(KotlinParser.NL)
                    self.state = 2868
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2869
                self.classBody()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ThisExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def THIS(self):
            return self.getToken(KotlinParser.THIS, 0)

        def THIS_AT(self):
            return self.getToken(KotlinParser.THIS_AT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_thisExpression




    def thisExpression(self):

        localctx = KotlinParser.ThisExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 244, self.RULE_thisExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2872
            _la = self._input.LA(1)
            if not(_la==KotlinParser.THIS_AT or _la==KotlinParser.THIS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SuperExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def SUPER(self):
            return self.getToken(KotlinParser.SUPER, 0)

        def LANGLE(self):
            return self.getToken(KotlinParser.LANGLE, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def RANGLE(self):
            return self.getToken(KotlinParser.RANGLE, 0)

        def AT_NO_WS(self):
            return self.getToken(KotlinParser.AT_NO_WS, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def SUPER_AT(self):
            return self.getToken(KotlinParser.SUPER_AT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_superExpression




    def superExpression(self):

        localctx = KotlinParser.SuperExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 246, self.RULE_superExpression)
        self._la = 0 # Token type
        try:
            self.state = 2898
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.SUPER]:
                self.enterOuterAlt(localctx, 1)
                self.state = 2874
                self.match(KotlinParser.SUPER)
                self.state = 2891
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,440,self._ctx)
                if la_ == 1:
                    self.state = 2875
                    self.match(KotlinParser.LANGLE)
                    self.state = 2879
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2876
                        self.match(KotlinParser.NL)
                        self.state = 2881
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2882
                    self.type()
                    self.state = 2886
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 2883
                        self.match(KotlinParser.NL)
                        self.state = 2888
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 2889
                    self.match(KotlinParser.RANGLE)


                self.state = 2895
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,441,self._ctx)
                if la_ == 1:
                    self.state = 2893
                    self.match(KotlinParser.AT_NO_WS)
                    self.state = 2894
                    self.simpleIdentifier()


                pass
            elif token in [KotlinParser.SUPER_AT]:
                self.enterOuterAlt(localctx, 2)
                self.state = 2897
                self.match(KotlinParser.SUPER_AT)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IfExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IF(self):
            return self.getToken(KotlinParser.IF, 0)

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def controlStructureBody(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ControlStructureBodyContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ControlStructureBodyContext,i)


        def ELSE(self):
            return self.getToken(KotlinParser.ELSE, 0)

        def SEMICOLON(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.SEMICOLON)
            else:
                return self.getToken(KotlinParser.SEMICOLON, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_ifExpression




    def ifExpression(self):

        localctx = KotlinParser.IfExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 248, self.RULE_ifExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2900
            self.match(KotlinParser.IF)
            self.state = 2904
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2901
                self.match(KotlinParser.NL)
                self.state = 2906
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2907
            self.match(KotlinParser.LPAREN)
            self.state = 2911
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2908
                self.match(KotlinParser.NL)
                self.state = 2913
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2914
            self.expression()
            self.state = 2918
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 2915
                self.match(KotlinParser.NL)
                self.state = 2920
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 2921
            self.match(KotlinParser.RPAREN)
            self.state = 2925
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,446,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 2922
                    self.match(KotlinParser.NL) 
                self.state = 2927
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,446,self._ctx)

            self.state = 2959
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,453,self._ctx)
            if la_ == 1:
                self.state = 2928
                self.controlStructureBody()
                pass

            elif la_ == 2:
                self.state = 2930
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LPAREN) | (1 << KotlinParser.LSQUARE) | (1 << KotlinParser.LCURL) | (1 << KotlinParser.ADD) | (1 << KotlinParser.SUB) | (1 << KotlinParser.INCR) | (1 << KotlinParser.DECR) | (1 << KotlinParser.EXCL_WS) | (1 << KotlinParser.EXCL_NO_WS) | (1 << KotlinParser.COLONCOLON) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.RETURN_AT) | (1 << KotlinParser.CONTINUE_AT) | (1 << KotlinParser.BREAK_AT) | (1 << KotlinParser.THIS_AT) | (1 << KotlinParser.SUPER_AT) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.CLASS - 64)) | (1 << (KotlinParser.INTERFACE - 64)) | (1 << (KotlinParser.FUN - 64)) | (1 << (KotlinParser.OBJECT - 64)) | (1 << (KotlinParser.VAL - 64)) | (1 << (KotlinParser.VAR - 64)) | (1 << (KotlinParser.TYPE_ALIAS - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.THIS - 64)) | (1 << (KotlinParser.SUPER - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.IF - 64)) | (1 << (KotlinParser.WHEN - 64)) | (1 << (KotlinParser.TRY - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.FOR - 64)) | (1 << (KotlinParser.DO - 64)) | (1 << (KotlinParser.WHILE - 64)) | (1 << (KotlinParser.THROW - 64)) | (1 << (KotlinParser.RETURN - 64)) | (1 << (KotlinParser.CONTINUE - 64)) | (1 << (KotlinParser.BREAK - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.RealLiteral - 128)) | (1 << (KotlinParser.IntegerLiteral - 128)) | (1 << (KotlinParser.HexLiteral - 128)) | (1 << (KotlinParser.BinLiteral - 128)) | (1 << (KotlinParser.UnsignedLiteral - 128)) | (1 << (KotlinParser.LongLiteral - 128)) | (1 << (KotlinParser.BooleanLiteral - 128)) | (1 << (KotlinParser.NullLiteral - 128)) | (1 << (KotlinParser.CharacterLiteral - 128)) | (1 << (KotlinParser.Identifier - 128)) | (1 << (KotlinParser.QUOTE_OPEN - 128)) | (1 << (KotlinParser.TRIPLE_QUOTE_OPEN - 128)))) != 0):
                    self.state = 2929
                    self.controlStructureBody()


                self.state = 2935
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,448,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2932
                        self.match(KotlinParser.NL) 
                    self.state = 2937
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,448,self._ctx)

                self.state = 2939
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==KotlinParser.SEMICOLON:
                    self.state = 2938
                    self.match(KotlinParser.SEMICOLON)


                self.state = 2944
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2941
                    self.match(KotlinParser.NL)
                    self.state = 2946
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2947
                self.match(KotlinParser.ELSE)
                self.state = 2951
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2948
                    self.match(KotlinParser.NL)
                    self.state = 2953
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2956
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [KotlinParser.LPAREN, KotlinParser.LSQUARE, KotlinParser.LCURL, KotlinParser.ADD, KotlinParser.SUB, KotlinParser.INCR, KotlinParser.DECR, KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS, KotlinParser.COLONCOLON, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.RETURN_AT, KotlinParser.CONTINUE_AT, KotlinParser.BREAK_AT, KotlinParser.THIS_AT, KotlinParser.SUPER_AT, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CLASS, KotlinParser.INTERFACE, KotlinParser.FUN, KotlinParser.OBJECT, KotlinParser.VAL, KotlinParser.VAR, KotlinParser.TYPE_ALIAS, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.THIS, KotlinParser.SUPER, KotlinParser.WHERE, KotlinParser.IF, KotlinParser.WHEN, KotlinParser.TRY, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.FOR, KotlinParser.DO, KotlinParser.WHILE, KotlinParser.THROW, KotlinParser.RETURN, KotlinParser.CONTINUE, KotlinParser.BREAK, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.RealLiteral, KotlinParser.IntegerLiteral, KotlinParser.HexLiteral, KotlinParser.BinLiteral, KotlinParser.UnsignedLiteral, KotlinParser.LongLiteral, KotlinParser.BooleanLiteral, KotlinParser.NullLiteral, KotlinParser.CharacterLiteral, KotlinParser.Identifier, KotlinParser.QUOTE_OPEN, KotlinParser.TRIPLE_QUOTE_OPEN]:
                    self.state = 2954
                    self.controlStructureBody()
                    pass
                elif token in [KotlinParser.SEMICOLON]:
                    self.state = 2955
                    self.match(KotlinParser.SEMICOLON)
                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 3:
                self.state = 2958
                self.match(KotlinParser.SEMICOLON)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WhenSubjectContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def VAL(self):
            return self.getToken(KotlinParser.VAL, 0)

        def variableDeclaration(self):
            return self.getTypedRuleContext(KotlinParser.VariableDeclarationContext,0)


        def ASSIGNMENT(self):
            return self.getToken(KotlinParser.ASSIGNMENT, 0)

        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_whenSubject




    def whenSubject(self):

        localctx = KotlinParser.WhenSubjectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 250, self.RULE_whenSubject)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 2961
            self.match(KotlinParser.LPAREN)
            self.state = 2995
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,459,self._ctx)
            if la_ == 1:
                self.state = 2965
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS:
                    self.state = 2962
                    self.annotation()
                    self.state = 2967
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2971
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2968
                    self.match(KotlinParser.NL)
                    self.state = 2973
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2974
                self.match(KotlinParser.VAL)
                self.state = 2978
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,456,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 2975
                        self.match(KotlinParser.NL) 
                    self.state = 2980
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,456,self._ctx)

                self.state = 2981
                self.variableDeclaration()
                self.state = 2985
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2982
                    self.match(KotlinParser.NL)
                    self.state = 2987
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 2988
                self.match(KotlinParser.ASSIGNMENT)
                self.state = 2992
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 2989
                    self.match(KotlinParser.NL)
                    self.state = 2994
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 2997
            self.expression()
            self.state = 2998
            self.match(KotlinParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WhenExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def WHEN(self):
            return self.getToken(KotlinParser.WHEN, 0)

        def LCURL(self):
            return self.getToken(KotlinParser.LCURL, 0)

        def RCURL(self):
            return self.getToken(KotlinParser.RCURL, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def whenSubject(self):
            return self.getTypedRuleContext(KotlinParser.WhenSubjectContext,0)


        def whenEntry(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.WhenEntryContext)
            else:
                return self.getTypedRuleContext(KotlinParser.WhenEntryContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_whenExpression




    def whenExpression(self):

        localctx = KotlinParser.WhenExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 252, self.RULE_whenExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3000
            self.match(KotlinParser.WHEN)
            self.state = 3004
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,460,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 3001
                    self.match(KotlinParser.NL) 
                self.state = 3006
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,460,self._ctx)

            self.state = 3008
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.LPAREN:
                self.state = 3007
                self.whenSubject()


            self.state = 3013
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3010
                self.match(KotlinParser.NL)
                self.state = 3015
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3016
            self.match(KotlinParser.LCURL)
            self.state = 3020
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,463,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 3017
                    self.match(KotlinParser.NL) 
                self.state = 3022
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,463,self._ctx)

            self.state = 3032
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LPAREN) | (1 << KotlinParser.LSQUARE) | (1 << KotlinParser.LCURL) | (1 << KotlinParser.ADD) | (1 << KotlinParser.SUB) | (1 << KotlinParser.INCR) | (1 << KotlinParser.DECR) | (1 << KotlinParser.EXCL_WS) | (1 << KotlinParser.EXCL_NO_WS) | (1 << KotlinParser.COLONCOLON) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.RETURN_AT) | (1 << KotlinParser.CONTINUE_AT) | (1 << KotlinParser.BREAK_AT) | (1 << KotlinParser.THIS_AT) | (1 << KotlinParser.SUPER_AT) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.FUN - 64)) | (1 << (KotlinParser.OBJECT - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.THIS - 64)) | (1 << (KotlinParser.SUPER - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.IF - 64)) | (1 << (KotlinParser.ELSE - 64)) | (1 << (KotlinParser.WHEN - 64)) | (1 << (KotlinParser.TRY - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.THROW - 64)) | (1 << (KotlinParser.RETURN - 64)) | (1 << (KotlinParser.CONTINUE - 64)) | (1 << (KotlinParser.BREAK - 64)) | (1 << (KotlinParser.IS - 64)) | (1 << (KotlinParser.IN - 64)) | (1 << (KotlinParser.NOT_IS - 64)) | (1 << (KotlinParser.NOT_IN - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.RealLiteral - 128)) | (1 << (KotlinParser.IntegerLiteral - 128)) | (1 << (KotlinParser.HexLiteral - 128)) | (1 << (KotlinParser.BinLiteral - 128)) | (1 << (KotlinParser.UnsignedLiteral - 128)) | (1 << (KotlinParser.LongLiteral - 128)) | (1 << (KotlinParser.BooleanLiteral - 128)) | (1 << (KotlinParser.NullLiteral - 128)) | (1 << (KotlinParser.CharacterLiteral - 128)) | (1 << (KotlinParser.Identifier - 128)) | (1 << (KotlinParser.QUOTE_OPEN - 128)) | (1 << (KotlinParser.TRIPLE_QUOTE_OPEN - 128)))) != 0):
                self.state = 3023
                self.whenEntry()
                self.state = 3027
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,464,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 3024
                        self.match(KotlinParser.NL) 
                    self.state = 3029
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,464,self._ctx)

                self.state = 3034
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3038
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3035
                self.match(KotlinParser.NL)
                self.state = 3040
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3041
            self.match(KotlinParser.RCURL)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WhenEntryContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def whenCondition(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.WhenConditionContext)
            else:
                return self.getTypedRuleContext(KotlinParser.WhenConditionContext,i)


        def ARROW(self):
            return self.getToken(KotlinParser.ARROW, 0)

        def controlStructureBody(self):
            return self.getTypedRuleContext(KotlinParser.ControlStructureBodyContext,0)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.COMMA)
            else:
                return self.getToken(KotlinParser.COMMA, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def semi(self):
            return self.getTypedRuleContext(KotlinParser.SemiContext,0)


        def ELSE(self):
            return self.getToken(KotlinParser.ELSE, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_whenEntry




    def whenEntry(self):

        localctx = KotlinParser.WhenEntryContext(self, self._ctx, self.state)
        self.enterRule(localctx, 254, self.RULE_whenEntry)
        self._la = 0 # Token type
        try:
            self.state = 3107
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LPAREN, KotlinParser.LSQUARE, KotlinParser.LCURL, KotlinParser.ADD, KotlinParser.SUB, KotlinParser.INCR, KotlinParser.DECR, KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS, KotlinParser.COLONCOLON, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.RETURN_AT, KotlinParser.CONTINUE_AT, KotlinParser.BREAK_AT, KotlinParser.THIS_AT, KotlinParser.SUPER_AT, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.FUN, KotlinParser.OBJECT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.THIS, KotlinParser.SUPER, KotlinParser.WHERE, KotlinParser.IF, KotlinParser.WHEN, KotlinParser.TRY, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.THROW, KotlinParser.RETURN, KotlinParser.CONTINUE, KotlinParser.BREAK, KotlinParser.IS, KotlinParser.IN, KotlinParser.NOT_IS, KotlinParser.NOT_IN, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.RealLiteral, KotlinParser.IntegerLiteral, KotlinParser.HexLiteral, KotlinParser.BinLiteral, KotlinParser.UnsignedLiteral, KotlinParser.LongLiteral, KotlinParser.BooleanLiteral, KotlinParser.NullLiteral, KotlinParser.CharacterLiteral, KotlinParser.Identifier, KotlinParser.QUOTE_OPEN, KotlinParser.TRIPLE_QUOTE_OPEN]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3043
                self.whenCondition()
                self.state = 3060
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,469,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 3047
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 3044
                            self.match(KotlinParser.NL)
                            self.state = 3049
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 3050
                        self.match(KotlinParser.COMMA)
                        self.state = 3054
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 3051
                            self.match(KotlinParser.NL)
                            self.state = 3056
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 3057
                        self.whenCondition() 
                    self.state = 3062
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,469,self._ctx)

                self.state = 3070
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,471,self._ctx)
                if la_ == 1:
                    self.state = 3066
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 3063
                        self.match(KotlinParser.NL)
                        self.state = 3068
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 3069
                    self.match(KotlinParser.COMMA)


                self.state = 3075
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3072
                    self.match(KotlinParser.NL)
                    self.state = 3077
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3078
                self.match(KotlinParser.ARROW)
                self.state = 3082
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3079
                    self.match(KotlinParser.NL)
                    self.state = 3084
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3085
                self.controlStructureBody()
                self.state = 3087
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,474,self._ctx)
                if la_ == 1:
                    self.state = 3086
                    self.semi()


                pass
            elif token in [KotlinParser.ELSE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3089
                self.match(KotlinParser.ELSE)
                self.state = 3093
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3090
                    self.match(KotlinParser.NL)
                    self.state = 3095
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3096
                self.match(KotlinParser.ARROW)
                self.state = 3100
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3097
                    self.match(KotlinParser.NL)
                    self.state = 3102
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3103
                self.controlStructureBody()
                self.state = 3105
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,477,self._ctx)
                if la_ == 1:
                    self.state = 3104
                    self.semi()


                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WhenConditionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def rangeTest(self):
            return self.getTypedRuleContext(KotlinParser.RangeTestContext,0)


        def typeTest(self):
            return self.getTypedRuleContext(KotlinParser.TypeTestContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_whenCondition




    def whenCondition(self):

        localctx = KotlinParser.WhenConditionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 256, self.RULE_whenCondition)
        try:
            self.state = 3112
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.LPAREN, KotlinParser.LSQUARE, KotlinParser.LCURL, KotlinParser.ADD, KotlinParser.SUB, KotlinParser.INCR, KotlinParser.DECR, KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS, KotlinParser.COLONCOLON, KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS, KotlinParser.RETURN_AT, KotlinParser.CONTINUE_AT, KotlinParser.BREAK_AT, KotlinParser.THIS_AT, KotlinParser.SUPER_AT, KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.FUN, KotlinParser.OBJECT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.THIS, KotlinParser.SUPER, KotlinParser.WHERE, KotlinParser.IF, KotlinParser.WHEN, KotlinParser.TRY, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.THROW, KotlinParser.RETURN, KotlinParser.CONTINUE, KotlinParser.BREAK, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.RealLiteral, KotlinParser.IntegerLiteral, KotlinParser.HexLiteral, KotlinParser.BinLiteral, KotlinParser.UnsignedLiteral, KotlinParser.LongLiteral, KotlinParser.BooleanLiteral, KotlinParser.NullLiteral, KotlinParser.CharacterLiteral, KotlinParser.Identifier, KotlinParser.QUOTE_OPEN, KotlinParser.TRIPLE_QUOTE_OPEN]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3109
                self.expression()
                pass
            elif token in [KotlinParser.IN, KotlinParser.NOT_IN]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3110
                self.rangeTest()
                pass
            elif token in [KotlinParser.IS, KotlinParser.NOT_IS]:
                self.enterOuterAlt(localctx, 3)
                self.state = 3111
                self.typeTest()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RangeTestContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def inOperator(self):
            return self.getTypedRuleContext(KotlinParser.InOperatorContext,0)


        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_rangeTest




    def rangeTest(self):

        localctx = KotlinParser.RangeTestContext(self, self._ctx, self.state)
        self.enterRule(localctx, 258, self.RULE_rangeTest)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3114
            self.inOperator()
            self.state = 3118
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3115
                self.match(KotlinParser.NL)
                self.state = 3120
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3121
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeTestContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def isOperator(self):
            return self.getTypedRuleContext(KotlinParser.IsOperatorContext,0)


        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeTest




    def typeTest(self):

        localctx = KotlinParser.TypeTestContext(self, self._ctx, self.state)
        self.enterRule(localctx, 260, self.RULE_typeTest)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3123
            self.isOperator()
            self.state = 3127
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3124
                self.match(KotlinParser.NL)
                self.state = 3129
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3130
            self.type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TryExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TRY(self):
            return self.getToken(KotlinParser.TRY, 0)

        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def finallyBlock(self):
            return self.getTypedRuleContext(KotlinParser.FinallyBlockContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def catchBlock(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.CatchBlockContext)
            else:
                return self.getTypedRuleContext(KotlinParser.CatchBlockContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_tryExpression




    def tryExpression(self):

        localctx = KotlinParser.TryExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 262, self.RULE_tryExpression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3132
            self.match(KotlinParser.TRY)
            self.state = 3136
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3133
                self.match(KotlinParser.NL)
                self.state = 3138
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3139
            self.block()
            self.state = 3167
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,488,self._ctx)
            if la_ == 1:
                self.state = 3147 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 3143
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        while _la==KotlinParser.NL:
                            self.state = 3140
                            self.match(KotlinParser.NL)
                            self.state = 3145
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)

                        self.state = 3146
                        self.catchBlock()

                    else:
                        raise NoViableAltException(self)
                    self.state = 3149 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,484,self._ctx)

                self.state = 3158
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,486,self._ctx)
                if la_ == 1:
                    self.state = 3154
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 3151
                        self.match(KotlinParser.NL)
                        self.state = 3156
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 3157
                    self.finallyBlock()


                pass

            elif la_ == 2:
                self.state = 3163
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3160
                    self.match(KotlinParser.NL)
                    self.state = 3165
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3166
                self.finallyBlock()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CatchBlockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def CATCH(self):
            return self.getToken(KotlinParser.CATCH, 0)

        def LPAREN(self):
            return self.getToken(KotlinParser.LPAREN, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def type(self):
            return self.getTypedRuleContext(KotlinParser.TypeContext,0)


        def RPAREN(self):
            return self.getToken(KotlinParser.RPAREN, 0)

        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def COMMA(self):
            return self.getToken(KotlinParser.COMMA, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_catchBlock




    def catchBlock(self):

        localctx = KotlinParser.CatchBlockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 264, self.RULE_catchBlock)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3169
            self.match(KotlinParser.CATCH)
            self.state = 3173
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3170
                self.match(KotlinParser.NL)
                self.state = 3175
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3176
            self.match(KotlinParser.LPAREN)
            self.state = 3180
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS:
                self.state = 3177
                self.annotation()
                self.state = 3182
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3183
            self.simpleIdentifier()
            self.state = 3184
            self.match(KotlinParser.COLON)
            self.state = 3185
            self.type()
            self.state = 3193
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==KotlinParser.NL or _la==KotlinParser.COMMA:
                self.state = 3189
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3186
                    self.match(KotlinParser.NL)
                    self.state = 3191
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3192
                self.match(KotlinParser.COMMA)


            self.state = 3195
            self.match(KotlinParser.RPAREN)
            self.state = 3199
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3196
                self.match(KotlinParser.NL)
                self.state = 3201
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3202
            self.block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FinallyBlockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def FINALLY(self):
            return self.getToken(KotlinParser.FINALLY, 0)

        def block(self):
            return self.getTypedRuleContext(KotlinParser.BlockContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_finallyBlock




    def finallyBlock(self):

        localctx = KotlinParser.FinallyBlockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 266, self.RULE_finallyBlock)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3204
            self.match(KotlinParser.FINALLY)
            self.state = 3208
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3205
                self.match(KotlinParser.NL)
                self.state = 3210
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3211
            self.block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class JumpExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def THROW(self):
            return self.getToken(KotlinParser.THROW, 0)

        def expression(self):
            return self.getTypedRuleContext(KotlinParser.ExpressionContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def RETURN(self):
            return self.getToken(KotlinParser.RETURN, 0)

        def RETURN_AT(self):
            return self.getToken(KotlinParser.RETURN_AT, 0)

        def CONTINUE(self):
            return self.getToken(KotlinParser.CONTINUE, 0)

        def CONTINUE_AT(self):
            return self.getToken(KotlinParser.CONTINUE_AT, 0)

        def BREAK(self):
            return self.getToken(KotlinParser.BREAK, 0)

        def BREAK_AT(self):
            return self.getToken(KotlinParser.BREAK_AT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_jumpExpression




    def jumpExpression(self):

        localctx = KotlinParser.JumpExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 268, self.RULE_jumpExpression)
        self._la = 0 # Token type
        try:
            self.state = 3229
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.THROW]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3213
                self.match(KotlinParser.THROW)
                self.state = 3217
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3214
                    self.match(KotlinParser.NL)
                    self.state = 3219
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3220
                self.expression()
                pass
            elif token in [KotlinParser.RETURN_AT, KotlinParser.RETURN]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3221
                _la = self._input.LA(1)
                if not(_la==KotlinParser.RETURN_AT or _la==KotlinParser.RETURN):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 3223
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,496,self._ctx)
                if la_ == 1:
                    self.state = 3222
                    self.expression()


                pass
            elif token in [KotlinParser.CONTINUE]:
                self.enterOuterAlt(localctx, 3)
                self.state = 3225
                self.match(KotlinParser.CONTINUE)
                pass
            elif token in [KotlinParser.CONTINUE_AT]:
                self.enterOuterAlt(localctx, 4)
                self.state = 3226
                self.match(KotlinParser.CONTINUE_AT)
                pass
            elif token in [KotlinParser.BREAK]:
                self.enterOuterAlt(localctx, 5)
                self.state = 3227
                self.match(KotlinParser.BREAK)
                pass
            elif token in [KotlinParser.BREAK_AT]:
                self.enterOuterAlt(localctx, 6)
                self.state = 3228
                self.match(KotlinParser.BREAK_AT)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CallableReferenceContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COLONCOLON(self):
            return self.getToken(KotlinParser.COLONCOLON, 0)

        def simpleIdentifier(self):
            return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,0)


        def CLASS(self):
            return self.getToken(KotlinParser.CLASS, 0)

        def receiverType(self):
            return self.getTypedRuleContext(KotlinParser.ReceiverTypeContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_callableReference




    def callableReference(self):

        localctx = KotlinParser.CallableReferenceContext(self, self._ctx, self.state)
        self.enterRule(localctx, 270, self.RULE_callableReference)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3232
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LPAREN) | (1 << KotlinParser.AT_NO_WS) | (1 << KotlinParser.AT_PRE_WS) | (1 << KotlinParser.FILE) | (1 << KotlinParser.FIELD) | (1 << KotlinParser.PROPERTY))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (KotlinParser.GET - 64)) | (1 << (KotlinParser.SET - 64)) | (1 << (KotlinParser.RECEIVER - 64)) | (1 << (KotlinParser.PARAM - 64)) | (1 << (KotlinParser.SETPARAM - 64)) | (1 << (KotlinParser.DELEGATE - 64)) | (1 << (KotlinParser.IMPORT - 64)) | (1 << (KotlinParser.CONSTRUCTOR - 64)) | (1 << (KotlinParser.BY - 64)) | (1 << (KotlinParser.COMPANION - 64)) | (1 << (KotlinParser.INIT - 64)) | (1 << (KotlinParser.WHERE - 64)) | (1 << (KotlinParser.CATCH - 64)) | (1 << (KotlinParser.FINALLY - 64)) | (1 << (KotlinParser.OUT - 64)) | (1 << (KotlinParser.DYNAMIC - 64)) | (1 << (KotlinParser.PUBLIC - 64)) | (1 << (KotlinParser.PRIVATE - 64)) | (1 << (KotlinParser.PROTECTED - 64)) | (1 << (KotlinParser.INTERNAL - 64)) | (1 << (KotlinParser.ENUM - 64)) | (1 << (KotlinParser.SEALED - 64)) | (1 << (KotlinParser.ANNOTATION - 64)) | (1 << (KotlinParser.DATA - 64)) | (1 << (KotlinParser.INNER - 64)) | (1 << (KotlinParser.VALUE - 64)) | (1 << (KotlinParser.TAILREC - 64)) | (1 << (KotlinParser.OPERATOR - 64)) | (1 << (KotlinParser.INLINE - 64)) | (1 << (KotlinParser.INFIX - 64)) | (1 << (KotlinParser.EXTERNAL - 64)) | (1 << (KotlinParser.SUSPEND - 64)) | (1 << (KotlinParser.OVERRIDE - 64)) | (1 << (KotlinParser.ABSTRACT - 64)) | (1 << (KotlinParser.FINAL - 64)) | (1 << (KotlinParser.OPEN - 64)) | (1 << (KotlinParser.CONST - 64)))) != 0) or ((((_la - 128)) & ~0x3f) == 0 and ((1 << (_la - 128)) & ((1 << (KotlinParser.LATEINIT - 128)) | (1 << (KotlinParser.VARARG - 128)) | (1 << (KotlinParser.NOINLINE - 128)) | (1 << (KotlinParser.CROSSINLINE - 128)) | (1 << (KotlinParser.REIFIED - 128)) | (1 << (KotlinParser.EXPECT - 128)) | (1 << (KotlinParser.ACTUAL - 128)) | (1 << (KotlinParser.Identifier - 128)))) != 0):
                self.state = 3231
                self.receiverType()


            self.state = 3234
            self.match(KotlinParser.COLONCOLON)
            self.state = 3238
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3235
                self.match(KotlinParser.NL)
                self.state = 3240
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3243
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.FILE, KotlinParser.FIELD, KotlinParser.PROPERTY, KotlinParser.GET, KotlinParser.SET, KotlinParser.RECEIVER, KotlinParser.PARAM, KotlinParser.SETPARAM, KotlinParser.DELEGATE, KotlinParser.IMPORT, KotlinParser.CONSTRUCTOR, KotlinParser.BY, KotlinParser.COMPANION, KotlinParser.INIT, KotlinParser.WHERE, KotlinParser.CATCH, KotlinParser.FINALLY, KotlinParser.OUT, KotlinParser.DYNAMIC, KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.REIFIED, KotlinParser.EXPECT, KotlinParser.ACTUAL, KotlinParser.Identifier]:
                self.state = 3241
                self.simpleIdentifier()
                pass
            elif token in [KotlinParser.CLASS]:
                self.state = 3242
                self.match(KotlinParser.CLASS)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignmentAndOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ADD_ASSIGNMENT(self):
            return self.getToken(KotlinParser.ADD_ASSIGNMENT, 0)

        def SUB_ASSIGNMENT(self):
            return self.getToken(KotlinParser.SUB_ASSIGNMENT, 0)

        def MULT_ASSIGNMENT(self):
            return self.getToken(KotlinParser.MULT_ASSIGNMENT, 0)

        def DIV_ASSIGNMENT(self):
            return self.getToken(KotlinParser.DIV_ASSIGNMENT, 0)

        def MOD_ASSIGNMENT(self):
            return self.getToken(KotlinParser.MOD_ASSIGNMENT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_assignmentAndOperator




    def assignmentAndOperator(self):

        localctx = KotlinParser.AssignmentAndOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 272, self.RULE_assignmentAndOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3245
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.ADD_ASSIGNMENT) | (1 << KotlinParser.SUB_ASSIGNMENT) | (1 << KotlinParser.MULT_ASSIGNMENT) | (1 << KotlinParser.DIV_ASSIGNMENT) | (1 << KotlinParser.MOD_ASSIGNMENT))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EqualityOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EXCL_EQ(self):
            return self.getToken(KotlinParser.EXCL_EQ, 0)

        def EXCL_EQEQ(self):
            return self.getToken(KotlinParser.EXCL_EQEQ, 0)

        def EQEQ(self):
            return self.getToken(KotlinParser.EQEQ, 0)

        def EQEQEQ(self):
            return self.getToken(KotlinParser.EQEQEQ, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_equalityOperator




    def equalityOperator(self):

        localctx = KotlinParser.EqualityOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 274, self.RULE_equalityOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3247
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.EXCL_EQ) | (1 << KotlinParser.EXCL_EQEQ) | (1 << KotlinParser.EQEQ) | (1 << KotlinParser.EQEQEQ))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ComparisonOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LANGLE(self):
            return self.getToken(KotlinParser.LANGLE, 0)

        def RANGLE(self):
            return self.getToken(KotlinParser.RANGLE, 0)

        def LE(self):
            return self.getToken(KotlinParser.LE, 0)

        def GE(self):
            return self.getToken(KotlinParser.GE, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_comparisonOperator




    def comparisonOperator(self):

        localctx = KotlinParser.ComparisonOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 276, self.RULE_comparisonOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3249
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.LANGLE) | (1 << KotlinParser.RANGLE) | (1 << KotlinParser.LE) | (1 << KotlinParser.GE))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class InOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IN(self):
            return self.getToken(KotlinParser.IN, 0)

        def NOT_IN(self):
            return self.getToken(KotlinParser.NOT_IN, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_inOperator




    def inOperator(self):

        localctx = KotlinParser.InOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 278, self.RULE_inOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3251
            _la = self._input.LA(1)
            if not(_la==KotlinParser.IN or _la==KotlinParser.NOT_IN):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IsOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IS(self):
            return self.getToken(KotlinParser.IS, 0)

        def NOT_IS(self):
            return self.getToken(KotlinParser.NOT_IS, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_isOperator




    def isOperator(self):

        localctx = KotlinParser.IsOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 280, self.RULE_isOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3253
            _la = self._input.LA(1)
            if not(_la==KotlinParser.IS or _la==KotlinParser.NOT_IS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AdditiveOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ADD(self):
            return self.getToken(KotlinParser.ADD, 0)

        def SUB(self):
            return self.getToken(KotlinParser.SUB, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_additiveOperator




    def additiveOperator(self):

        localctx = KotlinParser.AdditiveOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 282, self.RULE_additiveOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3255
            _la = self._input.LA(1)
            if not(_la==KotlinParser.ADD or _la==KotlinParser.SUB):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiplicativeOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def MULT(self):
            return self.getToken(KotlinParser.MULT, 0)

        def DIV(self):
            return self.getToken(KotlinParser.DIV, 0)

        def MOD(self):
            return self.getToken(KotlinParser.MOD, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiplicativeOperator




    def multiplicativeOperator(self):

        localctx = KotlinParser.MultiplicativeOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 284, self.RULE_multiplicativeOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3257
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << KotlinParser.MULT) | (1 << KotlinParser.MOD) | (1 << KotlinParser.DIV))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AsOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AS(self):
            return self.getToken(KotlinParser.AS, 0)

        def AS_SAFE(self):
            return self.getToken(KotlinParser.AS_SAFE, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_asOperator




    def asOperator(self):

        localctx = KotlinParser.AsOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 286, self.RULE_asOperator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3259
            _la = self._input.LA(1)
            if not(_la==KotlinParser.AS_SAFE or _la==KotlinParser.AS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PrefixUnaryOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def INCR(self):
            return self.getToken(KotlinParser.INCR, 0)

        def DECR(self):
            return self.getToken(KotlinParser.DECR, 0)

        def SUB(self):
            return self.getToken(KotlinParser.SUB, 0)

        def ADD(self):
            return self.getToken(KotlinParser.ADD, 0)

        def excl(self):
            return self.getTypedRuleContext(KotlinParser.ExclContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_prefixUnaryOperator




    def prefixUnaryOperator(self):

        localctx = KotlinParser.PrefixUnaryOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 288, self.RULE_prefixUnaryOperator)
        try:
            self.state = 3266
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.INCR]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3261
                self.match(KotlinParser.INCR)
                pass
            elif token in [KotlinParser.DECR]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3262
                self.match(KotlinParser.DECR)
                pass
            elif token in [KotlinParser.SUB]:
                self.enterOuterAlt(localctx, 3)
                self.state = 3263
                self.match(KotlinParser.SUB)
                pass
            elif token in [KotlinParser.ADD]:
                self.enterOuterAlt(localctx, 4)
                self.state = 3264
                self.match(KotlinParser.ADD)
                pass
            elif token in [KotlinParser.EXCL_WS, KotlinParser.EXCL_NO_WS]:
                self.enterOuterAlt(localctx, 5)
                self.state = 3265
                self.excl()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PostfixUnaryOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def INCR(self):
            return self.getToken(KotlinParser.INCR, 0)

        def DECR(self):
            return self.getToken(KotlinParser.DECR, 0)

        def EXCL_NO_WS(self):
            return self.getToken(KotlinParser.EXCL_NO_WS, 0)

        def excl(self):
            return self.getTypedRuleContext(KotlinParser.ExclContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_postfixUnaryOperator




    def postfixUnaryOperator(self):

        localctx = KotlinParser.PostfixUnaryOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 290, self.RULE_postfixUnaryOperator)
        try:
            self.state = 3272
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.INCR]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3268
                self.match(KotlinParser.INCR)
                pass
            elif token in [KotlinParser.DECR]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3269
                self.match(KotlinParser.DECR)
                pass
            elif token in [KotlinParser.EXCL_NO_WS]:
                self.enterOuterAlt(localctx, 3)
                self.state = 3270
                self.match(KotlinParser.EXCL_NO_WS)
                self.state = 3271
                self.excl()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ExclContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EXCL_NO_WS(self):
            return self.getToken(KotlinParser.EXCL_NO_WS, 0)

        def EXCL_WS(self):
            return self.getToken(KotlinParser.EXCL_WS, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_excl




    def excl(self):

        localctx = KotlinParser.ExclContext(self, self._ctx, self.state)
        self.enterRule(localctx, 292, self.RULE_excl)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3274
            _la = self._input.LA(1)
            if not(_la==KotlinParser.EXCL_WS or _la==KotlinParser.EXCL_NO_WS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MemberAccessOperatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def safeNav(self):
            return self.getTypedRuleContext(KotlinParser.SafeNavContext,0)


        def COLONCOLON(self):
            return self.getToken(KotlinParser.COLONCOLON, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_memberAccessOperator




    def memberAccessOperator(self):

        localctx = KotlinParser.MemberAccessOperatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 294, self.RULE_memberAccessOperator)
        self._la = 0 # Token type
        try:
            self.state = 3291
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,505,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 3279
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3276
                    self.match(KotlinParser.NL)
                    self.state = 3281
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3282
                self.match(KotlinParser.DOT)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 3286
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3283
                    self.match(KotlinParser.NL)
                    self.state = 3288
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 3289
                self.safeNav()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 3290
                self.match(KotlinParser.COLONCOLON)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SafeNavContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def QUEST_NO_WS(self):
            return self.getToken(KotlinParser.QUEST_NO_WS, 0)

        def DOT(self):
            return self.getToken(KotlinParser.DOT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_safeNav




    def safeNav(self):

        localctx = KotlinParser.SafeNavContext(self, self._ctx, self.state)
        self.enterRule(localctx, 296, self.RULE_safeNav)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3293
            self.match(KotlinParser.QUEST_NO_WS)
            self.state = 3294
            self.match(KotlinParser.DOT)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ModifiersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def modifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ModifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ModifierContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_modifiers




    def modifiers(self):

        localctx = KotlinParser.ModifiersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 298, self.RULE_modifiers)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3298 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 3298
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                        self.state = 3296
                        self.annotation()
                        pass
                    elif token in [KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL, KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE, KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND, KotlinParser.OVERRIDE, KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN, KotlinParser.CONST, KotlinParser.LATEINIT, KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE, KotlinParser.EXPECT, KotlinParser.ACTUAL]:
                        self.state = 3297
                        self.modifier()
                        pass
                    else:
                        raise NoViableAltException(self)


                else:
                    raise NoViableAltException(self)
                self.state = 3300 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,507,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParameterModifiersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def annotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.AnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.AnnotationContext,i)


        def parameterModifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.ParameterModifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.ParameterModifierContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_parameterModifiers




    def parameterModifiers(self):

        localctx = KotlinParser.ParameterModifiersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 300, self.RULE_parameterModifiers)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3304 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 3304
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                        self.state = 3302
                        self.annotation()
                        pass
                    elif token in [KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE]:
                        self.state = 3303
                        self.parameterModifier()
                        pass
                    else:
                        raise NoViableAltException(self)


                else:
                    raise NoViableAltException(self)
                self.state = 3306 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,509,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def classModifier(self):
            return self.getTypedRuleContext(KotlinParser.ClassModifierContext,0)


        def memberModifier(self):
            return self.getTypedRuleContext(KotlinParser.MemberModifierContext,0)


        def visibilityModifier(self):
            return self.getTypedRuleContext(KotlinParser.VisibilityModifierContext,0)


        def functionModifier(self):
            return self.getTypedRuleContext(KotlinParser.FunctionModifierContext,0)


        def propertyModifier(self):
            return self.getTypedRuleContext(KotlinParser.PropertyModifierContext,0)


        def inheritanceModifier(self):
            return self.getTypedRuleContext(KotlinParser.InheritanceModifierContext,0)


        def parameterModifier(self):
            return self.getTypedRuleContext(KotlinParser.ParameterModifierContext,0)


        def platformModifier(self):
            return self.getTypedRuleContext(KotlinParser.PlatformModifierContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_modifier




    def modifier(self):

        localctx = KotlinParser.ModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 302, self.RULE_modifier)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3316
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.ENUM, KotlinParser.SEALED, KotlinParser.ANNOTATION, KotlinParser.DATA, KotlinParser.INNER, KotlinParser.VALUE]:
                self.state = 3308
                self.classModifier()
                pass
            elif token in [KotlinParser.OVERRIDE, KotlinParser.LATEINIT]:
                self.state = 3309
                self.memberModifier()
                pass
            elif token in [KotlinParser.PUBLIC, KotlinParser.PRIVATE, KotlinParser.PROTECTED, KotlinParser.INTERNAL]:
                self.state = 3310
                self.visibilityModifier()
                pass
            elif token in [KotlinParser.TAILREC, KotlinParser.OPERATOR, KotlinParser.INLINE, KotlinParser.INFIX, KotlinParser.EXTERNAL, KotlinParser.SUSPEND]:
                self.state = 3311
                self.functionModifier()
                pass
            elif token in [KotlinParser.CONST]:
                self.state = 3312
                self.propertyModifier()
                pass
            elif token in [KotlinParser.ABSTRACT, KotlinParser.FINAL, KotlinParser.OPEN]:
                self.state = 3313
                self.inheritanceModifier()
                pass
            elif token in [KotlinParser.VARARG, KotlinParser.NOINLINE, KotlinParser.CROSSINLINE]:
                self.state = 3314
                self.parameterModifier()
                pass
            elif token in [KotlinParser.EXPECT, KotlinParser.ACTUAL]:
                self.state = 3315
                self.platformModifier()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 3321
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,511,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 3318
                    self.match(KotlinParser.NL) 
                self.state = 3323
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,511,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeModifiersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typeModifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeModifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeModifierContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeModifiers




    def typeModifiers(self):

        localctx = KotlinParser.TypeModifiersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 304, self.RULE_typeModifiers)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3325 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 3324
                    self.typeModifier()

                else:
                    raise NoViableAltException(self)
                self.state = 3327 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,512,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def annotation(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationContext,0)


        def SUSPEND(self):
            return self.getToken(KotlinParser.SUSPEND, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_typeModifier




    def typeModifier(self):

        localctx = KotlinParser.TypeModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 306, self.RULE_typeModifier)
        self._la = 0 # Token type
        try:
            self.state = 3337
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3329
                self.annotation()
                pass
            elif token in [KotlinParser.SUSPEND]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3330
                self.match(KotlinParser.SUSPEND)
                self.state = 3334
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3331
                    self.match(KotlinParser.NL)
                    self.state = 3336
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClassModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENUM(self):
            return self.getToken(KotlinParser.ENUM, 0)

        def SEALED(self):
            return self.getToken(KotlinParser.SEALED, 0)

        def ANNOTATION(self):
            return self.getToken(KotlinParser.ANNOTATION, 0)

        def DATA(self):
            return self.getToken(KotlinParser.DATA, 0)

        def INNER(self):
            return self.getToken(KotlinParser.INNER, 0)

        def VALUE(self):
            return self.getToken(KotlinParser.VALUE, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_classModifier




    def classModifier(self):

        localctx = KotlinParser.ClassModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 308, self.RULE_classModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3339
            _la = self._input.LA(1)
            if not(((((_la - 111)) & ~0x3f) == 0 and ((1 << (_la - 111)) & ((1 << (KotlinParser.ENUM - 111)) | (1 << (KotlinParser.SEALED - 111)) | (1 << (KotlinParser.ANNOTATION - 111)) | (1 << (KotlinParser.DATA - 111)) | (1 << (KotlinParser.INNER - 111)) | (1 << (KotlinParser.VALUE - 111)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MemberModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def OVERRIDE(self):
            return self.getToken(KotlinParser.OVERRIDE, 0)

        def LATEINIT(self):
            return self.getToken(KotlinParser.LATEINIT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_memberModifier




    def memberModifier(self):

        localctx = KotlinParser.MemberModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 310, self.RULE_memberModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3341
            _la = self._input.LA(1)
            if not(_la==KotlinParser.OVERRIDE or _la==KotlinParser.LATEINIT):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class VisibilityModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PUBLIC(self):
            return self.getToken(KotlinParser.PUBLIC, 0)

        def PRIVATE(self):
            return self.getToken(KotlinParser.PRIVATE, 0)

        def INTERNAL(self):
            return self.getToken(KotlinParser.INTERNAL, 0)

        def PROTECTED(self):
            return self.getToken(KotlinParser.PROTECTED, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_visibilityModifier




    def visibilityModifier(self):

        localctx = KotlinParser.VisibilityModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 312, self.RULE_visibilityModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3343
            _la = self._input.LA(1)
            if not(((((_la - 107)) & ~0x3f) == 0 and ((1 << (_la - 107)) & ((1 << (KotlinParser.PUBLIC - 107)) | (1 << (KotlinParser.PRIVATE - 107)) | (1 << (KotlinParser.PROTECTED - 107)) | (1 << (KotlinParser.INTERNAL - 107)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class VarianceModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IN(self):
            return self.getToken(KotlinParser.IN, 0)

        def OUT(self):
            return self.getToken(KotlinParser.OUT, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_varianceModifier




    def varianceModifier(self):

        localctx = KotlinParser.VarianceModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 314, self.RULE_varianceModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3345
            _la = self._input.LA(1)
            if not(_la==KotlinParser.IN or _la==KotlinParser.OUT):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeParameterModifiersContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typeParameterModifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.TypeParameterModifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.TypeParameterModifierContext,i)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeParameterModifiers




    def typeParameterModifiers(self):

        localctx = KotlinParser.TypeParameterModifiersContext(self, self._ctx, self.state)
        self.enterRule(localctx, 316, self.RULE_typeParameterModifiers)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3348 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 3347
                    self.typeParameterModifier()

                else:
                    raise NoViableAltException(self)
                self.state = 3350 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,515,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeParameterModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def reificationModifier(self):
            return self.getTypedRuleContext(KotlinParser.ReificationModifierContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def varianceModifier(self):
            return self.getTypedRuleContext(KotlinParser.VarianceModifierContext,0)


        def annotation(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_typeParameterModifier




    def typeParameterModifier(self):

        localctx = KotlinParser.TypeParameterModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 318, self.RULE_typeParameterModifier)
        try:
            self.state = 3367
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [KotlinParser.REIFIED]:
                self.enterOuterAlt(localctx, 1)
                self.state = 3352
                self.reificationModifier()
                self.state = 3356
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,516,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 3353
                        self.match(KotlinParser.NL) 
                    self.state = 3358
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,516,self._ctx)

                pass
            elif token in [KotlinParser.IN, KotlinParser.OUT]:
                self.enterOuterAlt(localctx, 2)
                self.state = 3359
                self.varianceModifier()
                self.state = 3363
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,517,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 3360
                        self.match(KotlinParser.NL) 
                    self.state = 3365
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,517,self._ctx)

                pass
            elif token in [KotlinParser.AT_NO_WS, KotlinParser.AT_PRE_WS]:
                self.enterOuterAlt(localctx, 3)
                self.state = 3366
                self.annotation()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FunctionModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TAILREC(self):
            return self.getToken(KotlinParser.TAILREC, 0)

        def OPERATOR(self):
            return self.getToken(KotlinParser.OPERATOR, 0)

        def INFIX(self):
            return self.getToken(KotlinParser.INFIX, 0)

        def INLINE(self):
            return self.getToken(KotlinParser.INLINE, 0)

        def EXTERNAL(self):
            return self.getToken(KotlinParser.EXTERNAL, 0)

        def SUSPEND(self):
            return self.getToken(KotlinParser.SUSPEND, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_functionModifier




    def functionModifier(self):

        localctx = KotlinParser.FunctionModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 320, self.RULE_functionModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3369
            _la = self._input.LA(1)
            if not(((((_la - 117)) & ~0x3f) == 0 and ((1 << (_la - 117)) & ((1 << (KotlinParser.TAILREC - 117)) | (1 << (KotlinParser.OPERATOR - 117)) | (1 << (KotlinParser.INLINE - 117)) | (1 << (KotlinParser.INFIX - 117)) | (1 << (KotlinParser.EXTERNAL - 117)) | (1 << (KotlinParser.SUSPEND - 117)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropertyModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def CONST(self):
            return self.getToken(KotlinParser.CONST, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_propertyModifier




    def propertyModifier(self):

        localctx = KotlinParser.PropertyModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 322, self.RULE_propertyModifier)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3371
            self.match(KotlinParser.CONST)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class InheritanceModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ABSTRACT(self):
            return self.getToken(KotlinParser.ABSTRACT, 0)

        def FINAL(self):
            return self.getToken(KotlinParser.FINAL, 0)

        def OPEN(self):
            return self.getToken(KotlinParser.OPEN, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_inheritanceModifier




    def inheritanceModifier(self):

        localctx = KotlinParser.InheritanceModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 324, self.RULE_inheritanceModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3373
            _la = self._input.LA(1)
            if not(((((_la - 124)) & ~0x3f) == 0 and ((1 << (_la - 124)) & ((1 << (KotlinParser.ABSTRACT - 124)) | (1 << (KotlinParser.FINAL - 124)) | (1 << (KotlinParser.OPEN - 124)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParameterModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def VARARG(self):
            return self.getToken(KotlinParser.VARARG, 0)

        def NOINLINE(self):
            return self.getToken(KotlinParser.NOINLINE, 0)

        def CROSSINLINE(self):
            return self.getToken(KotlinParser.CROSSINLINE, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_parameterModifier




    def parameterModifier(self):

        localctx = KotlinParser.ParameterModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 326, self.RULE_parameterModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3375
            _la = self._input.LA(1)
            if not(((((_la - 129)) & ~0x3f) == 0 and ((1 << (_la - 129)) & ((1 << (KotlinParser.VARARG - 129)) | (1 << (KotlinParser.NOINLINE - 129)) | (1 << (KotlinParser.CROSSINLINE - 129)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ReificationModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def REIFIED(self):
            return self.getToken(KotlinParser.REIFIED, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_reificationModifier




    def reificationModifier(self):

        localctx = KotlinParser.ReificationModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 328, self.RULE_reificationModifier)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3377
            self.match(KotlinParser.REIFIED)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PlatformModifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EXPECT(self):
            return self.getToken(KotlinParser.EXPECT, 0)

        def ACTUAL(self):
            return self.getToken(KotlinParser.ACTUAL, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_platformModifier




    def platformModifier(self):

        localctx = KotlinParser.PlatformModifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 330, self.RULE_platformModifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3379
            _la = self._input.LA(1)
            if not(_la==KotlinParser.EXPECT or _la==KotlinParser.ACTUAL):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AnnotationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def singleAnnotation(self):
            return self.getTypedRuleContext(KotlinParser.SingleAnnotationContext,0)


        def multiAnnotation(self):
            return self.getTypedRuleContext(KotlinParser.MultiAnnotationContext,0)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_annotation




    def annotation(self):

        localctx = KotlinParser.AnnotationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 332, self.RULE_annotation)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3383
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,519,self._ctx)
            if la_ == 1:
                self.state = 3381
                self.singleAnnotation()
                pass

            elif la_ == 2:
                self.state = 3382
                self.multiAnnotation()
                pass


            self.state = 3388
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,520,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 3385
                    self.match(KotlinParser.NL) 
                self.state = 3390
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,520,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SingleAnnotationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def unescapedAnnotation(self):
            return self.getTypedRuleContext(KotlinParser.UnescapedAnnotationContext,0)


        def annotationUseSiteTarget(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationUseSiteTargetContext,0)


        def AT_NO_WS(self):
            return self.getToken(KotlinParser.AT_NO_WS, 0)

        def AT_PRE_WS(self):
            return self.getToken(KotlinParser.AT_PRE_WS, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_singleAnnotation




    def singleAnnotation(self):

        localctx = KotlinParser.SingleAnnotationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 334, self.RULE_singleAnnotation)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3400
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,522,self._ctx)
            if la_ == 1:
                self.state = 3391
                self.annotationUseSiteTarget()
                self.state = 3395
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3392
                    self.match(KotlinParser.NL)
                    self.state = 3397
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                self.state = 3398
                self.match(KotlinParser.AT_NO_WS)
                pass

            elif la_ == 3:
                self.state = 3399
                self.match(KotlinParser.AT_PRE_WS)
                pass


            self.state = 3402
            self.unescapedAnnotation()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultiAnnotationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self):
            return self.getToken(KotlinParser.LSQUARE, 0)

        def RSQUARE(self):
            return self.getToken(KotlinParser.RSQUARE, 0)

        def annotationUseSiteTarget(self):
            return self.getTypedRuleContext(KotlinParser.AnnotationUseSiteTargetContext,0)


        def AT_NO_WS(self):
            return self.getToken(KotlinParser.AT_NO_WS, 0)

        def AT_PRE_WS(self):
            return self.getToken(KotlinParser.AT_PRE_WS, 0)

        def unescapedAnnotation(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.UnescapedAnnotationContext)
            else:
                return self.getTypedRuleContext(KotlinParser.UnescapedAnnotationContext,i)


        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_multiAnnotation




    def multiAnnotation(self):

        localctx = KotlinParser.MultiAnnotationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 336, self.RULE_multiAnnotation)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3413
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,524,self._ctx)
            if la_ == 1:
                self.state = 3404
                self.annotationUseSiteTarget()
                self.state = 3408
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==KotlinParser.NL:
                    self.state = 3405
                    self.match(KotlinParser.NL)
                    self.state = 3410
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                self.state = 3411
                self.match(KotlinParser.AT_NO_WS)
                pass

            elif la_ == 3:
                self.state = 3412
                self.match(KotlinParser.AT_PRE_WS)
                pass


            self.state = 3415
            self.match(KotlinParser.LSQUARE)
            self.state = 3417 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 3416
                self.unescapedAnnotation()
                self.state = 3419 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (((((_la - 61)) & ~0x3f) == 0 and ((1 << (_la - 61)) & ((1 << (KotlinParser.FILE - 61)) | (1 << (KotlinParser.FIELD - 61)) | (1 << (KotlinParser.PROPERTY - 61)) | (1 << (KotlinParser.GET - 61)) | (1 << (KotlinParser.SET - 61)) | (1 << (KotlinParser.RECEIVER - 61)) | (1 << (KotlinParser.PARAM - 61)) | (1 << (KotlinParser.SETPARAM - 61)) | (1 << (KotlinParser.DELEGATE - 61)) | (1 << (KotlinParser.IMPORT - 61)) | (1 << (KotlinParser.CONSTRUCTOR - 61)) | (1 << (KotlinParser.BY - 61)) | (1 << (KotlinParser.COMPANION - 61)) | (1 << (KotlinParser.INIT - 61)) | (1 << (KotlinParser.WHERE - 61)) | (1 << (KotlinParser.CATCH - 61)) | (1 << (KotlinParser.FINALLY - 61)) | (1 << (KotlinParser.OUT - 61)) | (1 << (KotlinParser.DYNAMIC - 61)) | (1 << (KotlinParser.PUBLIC - 61)) | (1 << (KotlinParser.PRIVATE - 61)) | (1 << (KotlinParser.PROTECTED - 61)) | (1 << (KotlinParser.INTERNAL - 61)) | (1 << (KotlinParser.ENUM - 61)) | (1 << (KotlinParser.SEALED - 61)) | (1 << (KotlinParser.ANNOTATION - 61)) | (1 << (KotlinParser.DATA - 61)) | (1 << (KotlinParser.INNER - 61)) | (1 << (KotlinParser.VALUE - 61)) | (1 << (KotlinParser.TAILREC - 61)) | (1 << (KotlinParser.OPERATOR - 61)) | (1 << (KotlinParser.INLINE - 61)) | (1 << (KotlinParser.INFIX - 61)) | (1 << (KotlinParser.EXTERNAL - 61)) | (1 << (KotlinParser.SUSPEND - 61)) | (1 << (KotlinParser.OVERRIDE - 61)) | (1 << (KotlinParser.ABSTRACT - 61)))) != 0) or ((((_la - 125)) & ~0x3f) == 0 and ((1 << (_la - 125)) & ((1 << (KotlinParser.FINAL - 125)) | (1 << (KotlinParser.OPEN - 125)) | (1 << (KotlinParser.CONST - 125)) | (1 << (KotlinParser.LATEINIT - 125)) | (1 << (KotlinParser.VARARG - 125)) | (1 << (KotlinParser.NOINLINE - 125)) | (1 << (KotlinParser.CROSSINLINE - 125)) | (1 << (KotlinParser.REIFIED - 125)) | (1 << (KotlinParser.EXPECT - 125)) | (1 << (KotlinParser.ACTUAL - 125)) | (1 << (KotlinParser.Identifier - 125)))) != 0)):
                    break

            self.state = 3421
            self.match(KotlinParser.RSQUARE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AnnotationUseSiteTargetContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COLON(self):
            return self.getToken(KotlinParser.COLON, 0)

        def AT_NO_WS(self):
            return self.getToken(KotlinParser.AT_NO_WS, 0)

        def AT_PRE_WS(self):
            return self.getToken(KotlinParser.AT_PRE_WS, 0)

        def FIELD(self):
            return self.getToken(KotlinParser.FIELD, 0)

        def PROPERTY(self):
            return self.getToken(KotlinParser.PROPERTY, 0)

        def GET(self):
            return self.getToken(KotlinParser.GET, 0)

        def SET(self):
            return self.getToken(KotlinParser.SET, 0)

        def RECEIVER(self):
            return self.getToken(KotlinParser.RECEIVER, 0)

        def PARAM(self):
            return self.getToken(KotlinParser.PARAM, 0)

        def SETPARAM(self):
            return self.getToken(KotlinParser.SETPARAM, 0)

        def DELEGATE(self):
            return self.getToken(KotlinParser.DELEGATE, 0)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_annotationUseSiteTarget




    def annotationUseSiteTarget(self):

        localctx = KotlinParser.AnnotationUseSiteTargetContext(self, self._ctx, self.state)
        self.enterRule(localctx, 338, self.RULE_annotationUseSiteTarget)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3423
            _la = self._input.LA(1)
            if not(_la==KotlinParser.AT_NO_WS or _la==KotlinParser.AT_PRE_WS):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 3424
            _la = self._input.LA(1)
            if not(((((_la - 62)) & ~0x3f) == 0 and ((1 << (_la - 62)) & ((1 << (KotlinParser.FIELD - 62)) | (1 << (KotlinParser.PROPERTY - 62)) | (1 << (KotlinParser.GET - 62)) | (1 << (KotlinParser.SET - 62)) | (1 << (KotlinParser.RECEIVER - 62)) | (1 << (KotlinParser.PARAM - 62)) | (1 << (KotlinParser.SETPARAM - 62)) | (1 << (KotlinParser.DELEGATE - 62)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 3428
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==KotlinParser.NL:
                self.state = 3425
                self.match(KotlinParser.NL)
                self.state = 3430
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 3431
            self.match(KotlinParser.COLON)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class UnescapedAnnotationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def constructorInvocation(self):
            return self.getTypedRuleContext(KotlinParser.ConstructorInvocationContext,0)


        def userType(self):
            return self.getTypedRuleContext(KotlinParser.UserTypeContext,0)


        def getRuleIndex(self):
            return KotlinParser.RULE_unescapedAnnotation




    def unescapedAnnotation(self):

        localctx = KotlinParser.UnescapedAnnotationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 340, self.RULE_unescapedAnnotation)
        try:
            self.state = 3435
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,527,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 3433
                self.constructorInvocation()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 3434
                self.userType()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SimpleIdentifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Identifier(self):
            return self.getToken(KotlinParser.Identifier, 0)

        def ABSTRACT(self):
            return self.getToken(KotlinParser.ABSTRACT, 0)

        def ANNOTATION(self):
            return self.getToken(KotlinParser.ANNOTATION, 0)

        def BY(self):
            return self.getToken(KotlinParser.BY, 0)

        def CATCH(self):
            return self.getToken(KotlinParser.CATCH, 0)

        def COMPANION(self):
            return self.getToken(KotlinParser.COMPANION, 0)

        def CONSTRUCTOR(self):
            return self.getToken(KotlinParser.CONSTRUCTOR, 0)

        def CROSSINLINE(self):
            return self.getToken(KotlinParser.CROSSINLINE, 0)

        def DATA(self):
            return self.getToken(KotlinParser.DATA, 0)

        def DYNAMIC(self):
            return self.getToken(KotlinParser.DYNAMIC, 0)

        def ENUM(self):
            return self.getToken(KotlinParser.ENUM, 0)

        def EXTERNAL(self):
            return self.getToken(KotlinParser.EXTERNAL, 0)

        def FINAL(self):
            return self.getToken(KotlinParser.FINAL, 0)

        def FINALLY(self):
            return self.getToken(KotlinParser.FINALLY, 0)

        def GET(self):
            return self.getToken(KotlinParser.GET, 0)

        def IMPORT(self):
            return self.getToken(KotlinParser.IMPORT, 0)

        def INFIX(self):
            return self.getToken(KotlinParser.INFIX, 0)

        def INIT(self):
            return self.getToken(KotlinParser.INIT, 0)

        def INLINE(self):
            return self.getToken(KotlinParser.INLINE, 0)

        def INNER(self):
            return self.getToken(KotlinParser.INNER, 0)

        def INTERNAL(self):
            return self.getToken(KotlinParser.INTERNAL, 0)

        def LATEINIT(self):
            return self.getToken(KotlinParser.LATEINIT, 0)

        def NOINLINE(self):
            return self.getToken(KotlinParser.NOINLINE, 0)

        def OPEN(self):
            return self.getToken(KotlinParser.OPEN, 0)

        def OPERATOR(self):
            return self.getToken(KotlinParser.OPERATOR, 0)

        def OUT(self):
            return self.getToken(KotlinParser.OUT, 0)

        def OVERRIDE(self):
            return self.getToken(KotlinParser.OVERRIDE, 0)

        def PRIVATE(self):
            return self.getToken(KotlinParser.PRIVATE, 0)

        def PROTECTED(self):
            return self.getToken(KotlinParser.PROTECTED, 0)

        def PUBLIC(self):
            return self.getToken(KotlinParser.PUBLIC, 0)

        def REIFIED(self):
            return self.getToken(KotlinParser.REIFIED, 0)

        def SEALED(self):
            return self.getToken(KotlinParser.SEALED, 0)

        def TAILREC(self):
            return self.getToken(KotlinParser.TAILREC, 0)

        def SET(self):
            return self.getToken(KotlinParser.SET, 0)

        def VARARG(self):
            return self.getToken(KotlinParser.VARARG, 0)

        def WHERE(self):
            return self.getToken(KotlinParser.WHERE, 0)

        def FIELD(self):
            return self.getToken(KotlinParser.FIELD, 0)

        def PROPERTY(self):
            return self.getToken(KotlinParser.PROPERTY, 0)

        def RECEIVER(self):
            return self.getToken(KotlinParser.RECEIVER, 0)

        def PARAM(self):
            return self.getToken(KotlinParser.PARAM, 0)

        def SETPARAM(self):
            return self.getToken(KotlinParser.SETPARAM, 0)

        def DELEGATE(self):
            return self.getToken(KotlinParser.DELEGATE, 0)

        def FILE(self):
            return self.getToken(KotlinParser.FILE, 0)

        def EXPECT(self):
            return self.getToken(KotlinParser.EXPECT, 0)

        def ACTUAL(self):
            return self.getToken(KotlinParser.ACTUAL, 0)

        def CONST(self):
            return self.getToken(KotlinParser.CONST, 0)

        def SUSPEND(self):
            return self.getToken(KotlinParser.SUSPEND, 0)

        def VALUE(self):
            return self.getToken(KotlinParser.VALUE, 0)

        def getRuleIndex(self):
            return KotlinParser.RULE_simpleIdentifier




    def simpleIdentifier(self):

        localctx = KotlinParser.SimpleIdentifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 342, self.RULE_simpleIdentifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3437
            _la = self._input.LA(1)
            if not(((((_la - 61)) & ~0x3f) == 0 and ((1 << (_la - 61)) & ((1 << (KotlinParser.FILE - 61)) | (1 << (KotlinParser.FIELD - 61)) | (1 << (KotlinParser.PROPERTY - 61)) | (1 << (KotlinParser.GET - 61)) | (1 << (KotlinParser.SET - 61)) | (1 << (KotlinParser.RECEIVER - 61)) | (1 << (KotlinParser.PARAM - 61)) | (1 << (KotlinParser.SETPARAM - 61)) | (1 << (KotlinParser.DELEGATE - 61)) | (1 << (KotlinParser.IMPORT - 61)) | (1 << (KotlinParser.CONSTRUCTOR - 61)) | (1 << (KotlinParser.BY - 61)) | (1 << (KotlinParser.COMPANION - 61)) | (1 << (KotlinParser.INIT - 61)) | (1 << (KotlinParser.WHERE - 61)) | (1 << (KotlinParser.CATCH - 61)) | (1 << (KotlinParser.FINALLY - 61)) | (1 << (KotlinParser.OUT - 61)) | (1 << (KotlinParser.DYNAMIC - 61)) | (1 << (KotlinParser.PUBLIC - 61)) | (1 << (KotlinParser.PRIVATE - 61)) | (1 << (KotlinParser.PROTECTED - 61)) | (1 << (KotlinParser.INTERNAL - 61)) | (1 << (KotlinParser.ENUM - 61)) | (1 << (KotlinParser.SEALED - 61)) | (1 << (KotlinParser.ANNOTATION - 61)) | (1 << (KotlinParser.DATA - 61)) | (1 << (KotlinParser.INNER - 61)) | (1 << (KotlinParser.VALUE - 61)) | (1 << (KotlinParser.TAILREC - 61)) | (1 << (KotlinParser.OPERATOR - 61)) | (1 << (KotlinParser.INLINE - 61)) | (1 << (KotlinParser.INFIX - 61)) | (1 << (KotlinParser.EXTERNAL - 61)) | (1 << (KotlinParser.SUSPEND - 61)) | (1 << (KotlinParser.OVERRIDE - 61)) | (1 << (KotlinParser.ABSTRACT - 61)))) != 0) or ((((_la - 125)) & ~0x3f) == 0 and ((1 << (_la - 125)) & ((1 << (KotlinParser.FINAL - 125)) | (1 << (KotlinParser.OPEN - 125)) | (1 << (KotlinParser.CONST - 125)) | (1 << (KotlinParser.LATEINIT - 125)) | (1 << (KotlinParser.VARARG - 125)) | (1 << (KotlinParser.NOINLINE - 125)) | (1 << (KotlinParser.CROSSINLINE - 125)) | (1 << (KotlinParser.REIFIED - 125)) | (1 << (KotlinParser.EXPECT - 125)) | (1 << (KotlinParser.ACTUAL - 125)) | (1 << (KotlinParser.Identifier - 125)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IdentifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def simpleIdentifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(KotlinParser.SimpleIdentifierContext)
            else:
                return self.getTypedRuleContext(KotlinParser.SimpleIdentifierContext,i)


        def DOT(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.DOT)
            else:
                return self.getToken(KotlinParser.DOT, i)

        def NL(self, i:int=None):
            if i is None:
                return self.getTokens(KotlinParser.NL)
            else:
                return self.getToken(KotlinParser.NL, i)

        def getRuleIndex(self):
            return KotlinParser.RULE_identifier




    def identifier(self):

        localctx = KotlinParser.IdentifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 344, self.RULE_identifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 3439
            self.simpleIdentifier()
            self.state = 3450
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,529,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 3443
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==KotlinParser.NL:
                        self.state = 3440
                        self.match(KotlinParser.NL)
                        self.state = 3445
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    self.state = 3446
                    self.match(KotlinParser.DOT)
                    self.state = 3447
                    self.simpleIdentifier() 
                self.state = 3452
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,529,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx





